/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _619MP7JRSCONTENTCOLLECTIONTYPE_LOCALTYPE_H
#define _619MP7JRSCONTENTCOLLECTIONTYPE_LOCALTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsContentCollectionType_LocalTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtInclude.h


class IMp7JrsAudioDType;
class IMp7JrsContentCollectionType_LocalType;
class IMp7JrsGofGopFeatureType;
class IMp7JrsVisualDType;
typedef Mp7JrsPtr< IMp7JrsAudioDType > Mp7JrsAudioDPtr;
typedef Mp7JrsPtr< IMp7JrsContentCollectionType_LocalType> Mp7JrsContentCollectionType_LocalPtr;
typedef Mp7JrsPtr< IMp7JrsGofGopFeatureType > Mp7JrsGofGopFeaturePtr;
typedef Mp7JrsPtr< IMp7JrsVisualDType > Mp7JrsVisualDPtr;

/** 
 * Generated interface IMp7JrsContentCollectionType_LocalType for class Mp7JrsContentCollectionType_LocalType<br>
 */
class MP7JRS_EXPORT IMp7JrsContentCollectionType_LocalType 
 :
		public Mp7JrsNode
{
public:
	// TODO: make these protected?
	IMp7JrsContentCollectionType_LocalType();
	virtual ~IMp7JrsContentCollectionType_LocalType();
		/** @name Choice

		 */
		//@{
	/**
	 * Get VisualFeature element.
	 * @return Mp7JrsPtr< Mp7JrsNode >
	 * Possible types (within this library):
	 * <ul>
	 * <li>IMp7JrsAdvancedFaceRecognitionPtr</li>
	 * <li>IMp7JrsCameraMotionPtr</li>
	 * <li>IMp7JrsColorLayoutPtr</li>
	 * <li>IMp7JrsColorStructurePtr</li>
	 * <li>IMp7JrsColorTemperaturePtr</li>
	 * <li>IMp7JrsContourShapePtr</li>
	 * <li>IMp7JrsDominantColorPtr</li>
	 * <li>IMp7JrsEdgeHistogramPtr</li>
	 * <li>IMp7JrsFaceRecognitionPtr</li>
	 * <li>IMp7JrsGoFGoPColorPtr</li>
	 * <li>IMp7JrsHomogeneousTexturePtr</li>
	 * <li>IMp7JrsMotionActivityPtr</li>
	 * <li>IMp7JrsMotionTrajectoryPtr</li>
	 * <li>IMp7JrsParametricMotionPtr</li>
	 * <li>IMp7JrsRegionShapePtr</li>
	 * <li>IMp7JrsScalableColorPtr</li>
	 * <li>IMp7JrsShape3DPtr</li>
	 * <li>IMp7JrsShapeVariationPtr</li>
	 * <li>IMp7JrsTextureBrowsingPtr</li>
	 * </ul>
	 */
	virtual Mp7JrsPtr< Mp7JrsNode > GetVisualFeature() const = 0;

	/**
	 * Set VisualFeature element.
	 * @param item const Mp7JrsPtr< Mp7JrsNode > &
	 * Valid types for item:
	 * <ul>
	 * <li>IMp7JrsAdvancedFaceRecognitionPtr</li>
	 * <li>IMp7JrsCameraMotionPtr</li>
	 * <li>IMp7JrsColorLayoutPtr</li>
	 * <li>IMp7JrsColorStructurePtr</li>
	 * <li>IMp7JrsColorTemperaturePtr</li>
	 * <li>IMp7JrsContourShapePtr</li>
	 * <li>IMp7JrsDominantColorPtr</li>
	 * <li>IMp7JrsEdgeHistogramPtr</li>
	 * <li>IMp7JrsFaceRecognitionPtr</li>
	 * <li>IMp7JrsGoFGoPColorPtr</li>
	 * <li>IMp7JrsHomogeneousTexturePtr</li>
	 * <li>IMp7JrsMotionActivityPtr</li>
	 * <li>IMp7JrsMotionTrajectoryPtr</li>
	 * <li>IMp7JrsParametricMotionPtr</li>
	 * <li>IMp7JrsRegionShapePtr</li>
	 * <li>IMp7JrsScalableColorPtr</li>
	 * <li>IMp7JrsShape3DPtr</li>
	 * <li>IMp7JrsShapeVariationPtr</li>
	 * <li>IMp7JrsTextureBrowsingPtr</li>
	 * </ul>
	 */
	virtual void SetVisualFeature(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	/**
	 * Get GofGopFeature element.
	 * @return Mp7JrsGofGopFeaturePtr
	 */
	virtual Mp7JrsGofGopFeaturePtr GetGofGopFeature() const = 0;

	/**
	 * Set GofGopFeature element.
	 * @param item const Mp7JrsGofGopFeaturePtr &
	 */
	virtual void SetGofGopFeature(const Mp7JrsGofGopFeaturePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	/**
	 * Get AudioFeature element.
	 * @return Mp7JrsPtr< Mp7JrsNode >
	 * Possible types (within this library):
	 * <ul>
	 * <li>IMp7JrsAudioBPMPtr</li>
	 * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
	 * <li>IMp7JrsAudioHarmonicityPtr</li>
	 * <li>IMp7JrsAudioPowerPtr</li>
	 * <li>IMp7JrsAudioSpectrumBasisPtr</li>
	 * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
	 * <li>IMp7JrsAudioSpectrumEnvelopePtr</li>
	 * <li>IMp7JrsAudioSpectrumFlatnessPtr</li>
	 * <li>IMp7JrsAudioSpectrumProjectionPtr</li>
	 * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
	 * <li>IMp7JrsAudioWaveformPtr</li>
	 * <li>IMp7JrsBackgroundNoiseLevelPtr</li>
	 * <li>IMp7JrsBalancePtr</li>
	 * <li>IMp7JrsBandwidthPtr</li>
	 * <li>IMp7JrsCrossChannelCorrelationPtr</li>
	 * <li>IMp7JrsDcOffsetPtr</li>
	 * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
	 * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
	 * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
	 * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
	 * <li>IMp7JrsLogAttackTimePtr</li>
	 * <li>IMp7JrsMeterPtr</li>
	 * <li>IMp7JrsRelativeDelayPtr</li>
	 * <li>IMp7JrsSilencePtr</li>
	 * <li>IMp7JrsSoundModelStateHistogramPtr</li>
	 * <li>IMp7JrsSpectralCentroidPtr</li>
	 * <li>IMp7JrsTemporalCentroidPtr</li>
	 * </ul>
	 */
	virtual Mp7JrsPtr< Mp7JrsNode > GetAudioFeature() const = 0;

	/**
	 * Set AudioFeature element.
	 * @param item const Mp7JrsPtr< Mp7JrsNode > &
	 * Valid types for item:
	 * <ul>
	 * <li>IMp7JrsAudioBPMPtr</li>
	 * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
	 * <li>IMp7JrsAudioHarmonicityPtr</li>
	 * <li>IMp7JrsAudioPowerPtr</li>
	 * <li>IMp7JrsAudioSpectrumBasisPtr</li>
	 * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
	 * <li>IMp7JrsAudioSpectrumEnvelopePtr</li>
	 * <li>IMp7JrsAudioSpectrumFlatnessPtr</li>
	 * <li>IMp7JrsAudioSpectrumProjectionPtr</li>
	 * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
	 * <li>IMp7JrsAudioWaveformPtr</li>
	 * <li>IMp7JrsBackgroundNoiseLevelPtr</li>
	 * <li>IMp7JrsBalancePtr</li>
	 * <li>IMp7JrsBandwidthPtr</li>
	 * <li>IMp7JrsCrossChannelCorrelationPtr</li>
	 * <li>IMp7JrsDcOffsetPtr</li>
	 * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
	 * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
	 * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
	 * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
	 * <li>IMp7JrsLogAttackTimePtr</li>
	 * <li>IMp7JrsMeterPtr</li>
	 * <li>IMp7JrsRelativeDelayPtr</li>
	 * <li>IMp7JrsSilencePtr</li>
	 * <li>IMp7JrsSoundModelStateHistogramPtr</li>
	 * <li>IMp7JrsSpectralCentroidPtr</li>
	 * <li>IMp7JrsTemporalCentroidPtr</li>
	 * </ul>
	 */
	virtual void SetAudioFeature(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

		//@}
// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsContentCollectionType_LocalType<br>
 * Defined in mpeg7-v2.xsd, line 8106.<br>
 */
class MP7JRS_EXPORT Mp7JrsContentCollectionType_LocalType :
		public IMp7JrsContentCollectionType_LocalType
{
	friend class Mp7JrsContentCollectionType_LocalTypeFactory; // constructs objects

public:
		/* @name Choice

		 */
		//@{
	/*
	 * Get VisualFeature element.
	 * @return Mp7JrsPtr< Mp7JrsNode >
	 * Possible types (within this library):
	 * <ul>
	 * <li>IMp7JrsAdvancedFaceRecognitionPtr</li>
	 * <li>IMp7JrsCameraMotionPtr</li>
	 * <li>IMp7JrsColorLayoutPtr</li>
	 * <li>IMp7JrsColorStructurePtr</li>
	 * <li>IMp7JrsColorTemperaturePtr</li>
	 * <li>IMp7JrsContourShapePtr</li>
	 * <li>IMp7JrsDominantColorPtr</li>
	 * <li>IMp7JrsEdgeHistogramPtr</li>
	 * <li>IMp7JrsFaceRecognitionPtr</li>
	 * <li>IMp7JrsGoFGoPColorPtr</li>
	 * <li>IMp7JrsHomogeneousTexturePtr</li>
	 * <li>IMp7JrsMotionActivityPtr</li>
	 * <li>IMp7JrsMotionTrajectoryPtr</li>
	 * <li>IMp7JrsParametricMotionPtr</li>
	 * <li>IMp7JrsRegionShapePtr</li>
	 * <li>IMp7JrsScalableColorPtr</li>
	 * <li>IMp7JrsShape3DPtr</li>
	 * <li>IMp7JrsShapeVariationPtr</li>
	 * <li>IMp7JrsTextureBrowsingPtr</li>
	 * </ul>
	 */
	virtual Mp7JrsPtr< Mp7JrsNode > GetVisualFeature() const;

	/*
	 * Set VisualFeature element.
	 * @param item const Mp7JrsPtr< Mp7JrsNode > &
	 * Valid types for item:
	 * <ul>
	 * <li>IMp7JrsAdvancedFaceRecognitionPtr</li>
	 * <li>IMp7JrsCameraMotionPtr</li>
	 * <li>IMp7JrsColorLayoutPtr</li>
	 * <li>IMp7JrsColorStructurePtr</li>
	 * <li>IMp7JrsColorTemperaturePtr</li>
	 * <li>IMp7JrsContourShapePtr</li>
	 * <li>IMp7JrsDominantColorPtr</li>
	 * <li>IMp7JrsEdgeHistogramPtr</li>
	 * <li>IMp7JrsFaceRecognitionPtr</li>
	 * <li>IMp7JrsGoFGoPColorPtr</li>
	 * <li>IMp7JrsHomogeneousTexturePtr</li>
	 * <li>IMp7JrsMotionActivityPtr</li>
	 * <li>IMp7JrsMotionTrajectoryPtr</li>
	 * <li>IMp7JrsParametricMotionPtr</li>
	 * <li>IMp7JrsRegionShapePtr</li>
	 * <li>IMp7JrsScalableColorPtr</li>
	 * <li>IMp7JrsShape3DPtr</li>
	 * <li>IMp7JrsShapeVariationPtr</li>
	 * <li>IMp7JrsTextureBrowsingPtr</li>
	 * </ul>
	 */
	virtual void SetVisualFeature(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	/*
	 * Get GofGopFeature element.
	 * @return Mp7JrsGofGopFeaturePtr
	 */
	virtual Mp7JrsGofGopFeaturePtr GetGofGopFeature() const;

	/*
	 * Set GofGopFeature element.
	 * @param item const Mp7JrsGofGopFeaturePtr &
	 */
	virtual void SetGofGopFeature(const Mp7JrsGofGopFeaturePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	/*
	 * Get AudioFeature element.
	 * @return Mp7JrsPtr< Mp7JrsNode >
	 * Possible types (within this library):
	 * <ul>
	 * <li>IMp7JrsAudioBPMPtr</li>
	 * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
	 * <li>IMp7JrsAudioHarmonicityPtr</li>
	 * <li>IMp7JrsAudioPowerPtr</li>
	 * <li>IMp7JrsAudioSpectrumBasisPtr</li>
	 * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
	 * <li>IMp7JrsAudioSpectrumEnvelopePtr</li>
	 * <li>IMp7JrsAudioSpectrumFlatnessPtr</li>
	 * <li>IMp7JrsAudioSpectrumProjectionPtr</li>
	 * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
	 * <li>IMp7JrsAudioWaveformPtr</li>
	 * <li>IMp7JrsBackgroundNoiseLevelPtr</li>
	 * <li>IMp7JrsBalancePtr</li>
	 * <li>IMp7JrsBandwidthPtr</li>
	 * <li>IMp7JrsCrossChannelCorrelationPtr</li>
	 * <li>IMp7JrsDcOffsetPtr</li>
	 * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
	 * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
	 * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
	 * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
	 * <li>IMp7JrsLogAttackTimePtr</li>
	 * <li>IMp7JrsMeterPtr</li>
	 * <li>IMp7JrsRelativeDelayPtr</li>
	 * <li>IMp7JrsSilencePtr</li>
	 * <li>IMp7JrsSoundModelStateHistogramPtr</li>
	 * <li>IMp7JrsSpectralCentroidPtr</li>
	 * <li>IMp7JrsTemporalCentroidPtr</li>
	 * </ul>
	 */
	virtual Mp7JrsPtr< Mp7JrsNode > GetAudioFeature() const;

	/*
	 * Set AudioFeature element.
	 * @param item const Mp7JrsPtr< Mp7JrsNode > &
	 * Valid types for item:
	 * <ul>
	 * <li>IMp7JrsAudioBPMPtr</li>
	 * <li>IMp7JrsAudioFundamentalFrequencyPtr</li>
	 * <li>IMp7JrsAudioHarmonicityPtr</li>
	 * <li>IMp7JrsAudioPowerPtr</li>
	 * <li>IMp7JrsAudioSpectrumBasisPtr</li>
	 * <li>IMp7JrsAudioSpectrumCentroidPtr</li>
	 * <li>IMp7JrsAudioSpectrumEnvelopePtr</li>
	 * <li>IMp7JrsAudioSpectrumFlatnessPtr</li>
	 * <li>IMp7JrsAudioSpectrumProjectionPtr</li>
	 * <li>IMp7JrsAudioSpectrumSpreadPtr</li>
	 * <li>IMp7JrsAudioWaveformPtr</li>
	 * <li>IMp7JrsBackgroundNoiseLevelPtr</li>
	 * <li>IMp7JrsBalancePtr</li>
	 * <li>IMp7JrsBandwidthPtr</li>
	 * <li>IMp7JrsCrossChannelCorrelationPtr</li>
	 * <li>IMp7JrsDcOffsetPtr</li>
	 * <li>IMp7JrsHarmonicSpectralCentroidPtr</li>
	 * <li>IMp7JrsHarmonicSpectralDeviationPtr</li>
	 * <li>IMp7JrsHarmonicSpectralSpreadPtr</li>
	 * <li>IMp7JrsHarmonicSpectralVariationPtr</li>
	 * <li>IMp7JrsLogAttackTimePtr</li>
	 * <li>IMp7JrsMeterPtr</li>
	 * <li>IMp7JrsRelativeDelayPtr</li>
	 * <li>IMp7JrsSilencePtr</li>
	 * <li>IMp7JrsSoundModelStateHistogramPtr</li>
	 * <li>IMp7JrsSpectralCentroidPtr</li>
	 * <li>IMp7JrsTemporalCentroidPtr</li>
	 * </ul>
	 */
	virtual void SetAudioFeature(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

		//@}

	/* Get the class name.
	 * @return The name of this class.
	 */
	virtual XMLCh * ToText() const;

	/* Parse this type.
	 * @param txt buffer to parse from.
	 * @return true if parsing was successful, false otherwise.
	 */
	virtual bool Parse(const XMLCh * const txt);

	/* Serialialize this object to a DOM tree.
	 * @param doc DOM document.
	 * @param parent Parent node in DOM tree.
	 */
	virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

	/* Deserialize this object from a DOM tree.
	 * @param doc DOM document.
	 * @param parent Parent element in DOM tree.
	 * @param current Current element in DOM tree.
	 */
	virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

	/* Creates and inserts a Childnode of the specified Type
	 * @param type The tag type of the child to be created (tag name).
	 * @param xsitype The xsi type of the child to be created if available.
	 * @return the created child with the correct dynamic type for the specified type.
	 */
	virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
		 
// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtMyMethodDef.h


public:

	virtual ~Mp7JrsContentCollectionType_LocalType();

protected:
	Mp7JrsContentCollectionType_LocalType();
	virtual void DeepCopy(const Mp7JrsNodePtr &original);

	virtual Mp7JrsNodeEnum GetAllChildElements() const;

	virtual Mp7JrsNodePtr GetBaseRoot();
	virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:


// DefinionChoice
	Mp7JrsPtr< Mp7JrsNode > m_VisualFeature;
	Mp7JrsGofGopFeaturePtr m_GofGopFeature;
	Mp7JrsPtr< Mp7JrsNode > m_AudioFeature;
// End DefinionChoice

// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtMyPropDef.h


public:
	virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
	void Cleanup();
};

#endif // _619MP7JRSCONTENTCOLLECTIONTYPE_LOCALTYPE_H

