/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#ifndef MP7JRS_DEFINES_H
#define MP7JRS_DEFINES_H

#ifndef MP7JRS_VERSION
#	define MP7JRS_VERSION 0207
#endif // MP7JRS_VERSION

// use DLL version, if static version is not explicitely forced
#ifndef MP7JRS_STATIC
#	ifndef MP7JRS_DLL
#		define MP7JRS_DLL
#	endif
#endif

#if defined(WIN32)
// For DLL you must define MP7JRS_DLL and MP7JRS_DECLSPEC_EXPORT
#	ifdef MP7JRS_DLL
#		ifdef MP7JRS_DECLSPEC_EXPORT
#			define MP7JRS_EXPORT __declspec(dllexport)
#		else
#			define MP7JRS_EXPORT __declspec(dllimport)
#		endif
#	else
#		define MP7JRS_EXPORT
#	endif // MP7JRS_DLL
#elif defined(__APPLE__)
#	define MP7JRS_EXPORT
#elif defined(__linux__)
#	define MP7JRS_EXPORT
#else
#	error("TODO: Adjust platform specific export methods.")
#endif



// From WinDef.h
#ifndef NULL
#ifdef __cplusplus
#define NULL    0
#else
#define NULL    ((void *)0)
#endif
#endif

#include <xercesc/util/XercesDefs.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>

#if defined(WIN32) // FTT XML_VISUALCPP is not defined in Xerces  >= 3.0.
// sat 2003-08-06
// Shut up compiler about long identifiers
#pragma warning( disable : 4786 )

// suppress warning about missing DLL interface of ptr template
#pragma warning( disable : 4251 )
#endif // WIN32 // XML_VISUALCPP


// baw 12 02 2003
// modification to support Xerces 2.2.0 and later
// uses the Xerces namespace if it has been defined
XERCES_CPP_NAMESPACE_USE


// baw 07 07 2003
// definitions for lock status and client identification
typedef enum {
	MP7JRS_NO_LOCKS_USED,
	MP7JRS_UNLOCKED,
	MP7JRS_LOCKED,
	MP7JRS_LOCKED4ADD
} Mp7JrsLockType;

typedef long Mp7JrsClientID;

#define MP7JRS_UNDEFINED_CLIENT ((Mp7JrsClientID) -1)

// string conversion class

class MP7JRS_EXPORT Mp7JrsXStr
{
public :
    Mp7JrsXStr(const char* const toTranscode)
    {
        // Call the private transcoding method
        fUnicodeForm = XMLString::transcode(toTranscode);
		fNonUnicodeForm = (char*)NULL;
    }

	Mp7JrsXStr(const XMLCh* const toTranscode) 
	{
		fNonUnicodeForm = XMLString::transcode(toTranscode);
		fUnicodeForm = (XMLCh*)NULL;
	}

    ~Mp7JrsXStr()
    {
        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&fUnicodeForm);
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&fNonUnicodeForm);
    }

    XMLCh* unicodeForm() const
    {
        return fUnicodeForm;
    }

    char* nonUnicodeForm() const
    {
        return fNonUnicodeForm;
    }

private :
    XMLCh*   fUnicodeForm;
	char*	 fNonUnicodeForm;
};

#define X(str) Mp7JrsXStr(str).unicodeForm()
#define Xinv(str) Mp7JrsXStr(str).nonUnicodeForm()

typedef enum {
	MP7JRS_NOTIF_LOCK,
	MP7JRS_NOTIF_UPDATE
} Mp7JrsNotificationType;

typedef enum {
	MP7JRS_NA_LOCK_REQUESTED,
	MP7JRS_NA_LOCK_GRANTED,
	MP7JRS_NA_UNLOCK,
	MP7JRS_NA_NODE_ADDED,
	MP7JRS_NA_NODE_INSERTED,
	MP7JRS_NA_NODE_UPDATED
} Mp7JrsNotificationAction;


#endif // MP7JRS_DEFINES_H
