/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#ifndef MP7JRSMISSINGELEMENTEXCEPTION_H
#define MP7JRSMISSINGELEMENTEXCEPTION_H

#include "Mp7JrsException.h"

/** Exception class for <code>GetOrCreateFromXPath</code> if the requested index was too high. */
class MP7JRS_EXPORT Mp7JrsMissingElementException : public Mp7JrsException
{
public:
	Mp7JrsMissingElementException(Mp7JrsExSeverity sev, const XMLCh* msg, unsigned int lastvalidindex, Mp7JrsMsgMemory mode = MP7JRS_AUTODELETE) 
		: Mp7JrsException(sev, msg, mode), LastValidIndex(lastvalidindex)
	{ 
	}

	Mp7JrsMissingElementException(Mp7JrsExSeverity sev, const char* msg, unsigned int lastvalidindex, Mp7JrsMsgMemory mode = MP7JRS_AUTODELETE) 
		: Mp7JrsException(sev, msg, mode), LastValidIndex(lastvalidindex)
	{
	}

	// Copy ctor, otherwise we have a memory leak!
	Mp7JrsMissingElementException(const Mp7JrsMissingElementException & other)
		: Mp7JrsException(other), LastValidIndex(other.LastValidIndex)
	{
	}
	
	virtual ~Mp7JrsMissingElementException()
	{
	}
	
	/** Correct index should be the value of GetLastValidIndex() plus 1. */
	unsigned int GetLastValidIndex() { return LastValidIndex; };

private:
	unsigned int LastValidIndex;
};

#endif // MP7JRSMISSINGELEMENTEXCEPTION_H
