/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _1609MP7JRSORDERINGKEYTYPE_H
#define _1609MP7JRSORDERINGKEYTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsOrderingKeyTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtInclude.h


#include "Mp7JrsHeaderType.h"
#include "Mp7JrsMpeg7BaseType.h"
#include "Mp7JrsOrderingKeyType_direction_LocalType.h"
class IMp7JrsHeaderType;
class IMp7JrsOrderingKeyType;
class IMp7JrsOrderingKeyType_Field_CollectionType;
class IMp7JrsOrderingKeyType_Selector_LocalType;
typedef Mp7JrsPtr< IMp7JrsHeaderType > Mp7JrsHeaderPtr;
typedef Mp7JrsPtr< IMp7JrsOrderingKeyType_Field_CollectionType > Mp7JrsOrderingKeyType_Field_CollectionPtr;
typedef Mp7JrsPtr< IMp7JrsOrderingKeyType_Selector_LocalType > Mp7JrsOrderingKeyType_Selector_LocalPtr;
typedef Mp7JrsPtr< IMp7JrsOrderingKeyType> Mp7JrsOrderingKeyPtr;

/** 
 * Generated interface IMp7JrsOrderingKeyType for class Mp7JrsOrderingKeyType<br>
 * Located at: mpeg7-v2.xsd, line 4791<br>
 * Classified: Class<br>
 * Derived from: HeaderType (Class)<br>
 */
class MP7JRS_EXPORT IMp7JrsOrderingKeyType 
 :
		public IMp7JrsHeaderType
{
public:
	// TODO: make these protected?
	IMp7JrsOrderingKeyType();
	virtual ~IMp7JrsOrderingKeyType();
	/**
	 * Get name attribute.
	 * @return XMLCh *
	 */
	virtual XMLCh * Getname() const = 0;

	/**
	 * Get name attribute validity information.
	 * @return true if attribute value is valid, false if not.
	 */
	virtual bool Existname() const = 0;

	/**
	 * Set name attribute.
	 * @param item XMLCh *
	 */
	virtual void Setname(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


	/**
	 * Invalidate name attribute.
	 
	 * (Validate it again by setting a value)
	 */
	virtual void Invalidatename() = 0;

	/**
	 * Get semantics attribute.
	 * @return XMLCh *
	 */
	virtual XMLCh * Getsemantics() const = 0;

	/**
	 * Get semantics attribute validity information.
	 * @return true if attribute value is valid, false if not.
	 */
	virtual bool Existsemantics() const = 0;

	/**
	 * Set semantics attribute.
	 * @param item XMLCh *
	 */
	virtual void Setsemantics(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


	/**
	 * Invalidate semantics attribute.
	 
	 * (Validate it again by setting a value)
	 */
	virtual void Invalidatesemantics() = 0;

	/**
	 * Get direction attribute.
	 * @return Mp7JrsOrderingKeyType_direction_LocalType::Enumeration
	 */
	virtual Mp7JrsOrderingKeyType_direction_LocalType::Enumeration Getdirection() const = 0;

	/**
	 * Get direction attribute validity information.
	 * @return true if attribute value is valid, false if not.
	 */
	virtual bool Existdirection() const = 0;

	/**
	 * Set direction attribute.
	 * @param item Mp7JrsOrderingKeyType_direction_LocalType::Enumeration
	 */
	virtual void Setdirection(Mp7JrsOrderingKeyType_direction_LocalType::Enumeration item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


	/**
	 * Invalidate direction attribute.
	 
	 * (Validate it again by setting a value)
	 */
	virtual void Invalidatedirection() = 0;

		/** @name Sequence

		 */
		//@{
	/**
	 * Get Selector element.
	 * @return Mp7JrsOrderingKeyType_Selector_LocalPtr
	 */
	virtual Mp7JrsOrderingKeyType_Selector_LocalPtr GetSelector() const = 0;

	/**
	 * Set Selector element.
	 * @param item const Mp7JrsOrderingKeyType_Selector_LocalPtr &
	 */
	// Mandatory			
	virtual void SetSelector(const Mp7JrsOrderingKeyType_Selector_LocalPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	/**
	 * Get Field element.
	 * @return Mp7JrsOrderingKeyType_Field_CollectionPtr
	 */
	virtual Mp7JrsOrderingKeyType_Field_CollectionPtr GetField() const = 0;

	/**
	 * Set Field element.
	 * @param item const Mp7JrsOrderingKeyType_Field_CollectionPtr &
	 */
	// Mandatory collection
	virtual void SetField(const Mp7JrsOrderingKeyType_Field_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

		//@}
	/**
	 * Get id attribute.
<br>
	 * (Inherited from Mp7JrsHeaderType)
	 * @return XMLCh *
	 */
	virtual XMLCh * Getid() const = 0;

	/**
	 * Get id attribute validity information.
	 * (Inherited from Mp7JrsHeaderType)
	 * @return true if attribute value is valid, false if not.
	 */
	virtual bool Existid() const = 0;

	/**
	 * Set id attribute.
<br>
	 * (Inherited from Mp7JrsHeaderType)
	 * @param item XMLCh *
	 */
	virtual void Setid(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


	/**
	 * Invalidate id attribute.
	 * (Inherited from Mp7JrsHeaderType)
	 
	 * (Validate it again by setting a value)
	 */
	virtual void Invalidateid() = 0;

	/**
	 * Gets or creates Mp7JrsNodePtr child elements of OrderingKeyType.
	 * Currently this type contains 2 child elements of type "Mp7JrsNodePtr".

	 * <ul>
	 * <li>Selector</li>
	 * <li>urn:mpeg:mpeg7:schema:2004:Field</li>
	 * </ul>
	 * 
	 * Supported XPath expressions for GetOrCreateFromXPath
	 * 
	 * \par<code>/</code>
	 *  - Get the root of this node.
	 * \par<code>..</code>
	 *  - This feature is experimental.
	 *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
	 * 
	 * \par <code>\@A</code>
	 * - Force creation of a child attribute named <code>A</code>.
	 * - This is valid only, if the type of attribute is derived from Mp7JrsNodePtr.
	 * - Evaluating this expression for a plain type like int or string results in an exception.
	 *
	 * \par<code>A</code> 
	 *  - Select all child elements named <code>A</code>.
	 *  - In case of no element, create a new one.
	 *  - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
	 * \par<code>A[i]</code>
	 *  - Select the <code>i</code>th child element of all <code>A</code>.
	 *  - Please note, that XPath indexes start at 1.
	 *  - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
	 *  - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
	 *  - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
	 *  - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
	 *
	 * \par <code>A[\@xsi:type="T"]</code>
	 *  - Select all child elements named <code>A</code> of type <code>T</code>.
	 *  - Currently only typenames without namespace are recognised.
	 *  - If there is no such element, create and append a new one.
	 * \par <code>A[i][\@xsi:type="T"]</code>
	 *  - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
	 *  - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
	 * \par <code>A[\@xsi:type="T"][i]</code>
	 *  - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
	 *  - If there is no such element at index <code>i</code>, create and append one.
	 *
	 * \par <code>A[last()]</code>
	 *  - Select the last child element named <code>A</code>.
	 *  - In case of no element, create a new one.
	 * \par <code>A[last()][\@xsi:type="T"]</code>
	 *  - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
	 *  - For collections: in case of no such type, create and append a new one.
	 *  - For single elements: if an element with different type exists, return the empty list.
	 * 
	 * \par <code>A[\@xsi:type="T"][last()]</code>
	 *  - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
	 *  - For collections: in case of no such type, create and append a new one.
	 *  - For single elements: if an element with different type exists, return the empty list.
	 * 
	 * \par <code>A[position() \> last()]</code>
	 *  - Force creation of a child element named <code>A</code> with its default type.
	 *  - For collections: create and append the default element.
	 *  - For single elements: if any element exists, an exception is thrown.
	 *
	 * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
	 *  - Force creation of a child element named <code>A</code> of type <code>T</code>.
	 *  - For collections: create and append the element.
	 *  - For single elements: if any element exists, an exception is thrown.
	 *
	 * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
	 *  - Force creation of a child element named <code>A</code> of type <code>T</code>.
	 *  - Behaves the same way as the previous expression.
	 * 
	 * Supported XPath expressions for GetFromXPath<br>
	 * The same expressions like GetOrCreateFromXPath plus
	 * \par <code>A[position() \< i] </code>
	 *  - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
	 *  - <code>i</code> must be greater than -1, otherwise an exception is thrown.
	 * \par <code>A[position() \> i] </code>
	 *  - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
	 *  - <code>i</code> must be greater than -1, otherwise an exception is thrown.
	 * \par <code>A[position() \> i | position() \< j] </code>
	 *  - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
	 *  - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
	 *  - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
	 *
	 * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
	 *
	 * @param xpath  the xpath expression to get or create.
	 * @param client the ID in case of locking.
	 * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
	 */
	// xpath: mandatory
	// client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

	
// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsOrderingKeyType<br>
 * Located at: mpeg7-v2.xsd, line 4791<br>
 * Classified: Class<br>
 * Derived from: HeaderType (Class)<br>
 * Defined in mpeg7-v2.xsd, line 4791.<br>
 */
class MP7JRS_EXPORT Mp7JrsOrderingKeyType :
		public IMp7JrsOrderingKeyType
{
	friend class Mp7JrsOrderingKeyTypeFactory; // constructs objects

public:
	/*
	 * Retrieves the base class of Mp7JrsOrderingKeyType
	 
	 * @return Mp7JrsPtr< Mp7JrsHeaderType >
	 */
	virtual Mp7JrsPtr< Mp7JrsHeaderType > GetBase() const;
	/*
	 * Get name attribute.
	 * @return XMLCh *
	 */
	virtual XMLCh * Getname() const;

	/*
	 * Get name attribute validity information.
	 * @return true if attribute value is valid, false if not.
	 */
	virtual bool Existname() const;

	/*
	 * Set name attribute.
	 * @param item XMLCh *
	 */
	virtual void Setname(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


	/*
	 * Invalidate name attribute.
	 
	 * (Validate it again by setting a value)
	 */
	virtual void Invalidatename();

	/*
	 * Get semantics attribute.
	 * @return XMLCh *
	 */
	virtual XMLCh * Getsemantics() const;

	/*
	 * Get semantics attribute validity information.
	 * @return true if attribute value is valid, false if not.
	 */
	virtual bool Existsemantics() const;

	/*
	 * Set semantics attribute.
	 * @param item XMLCh *
	 */
	virtual void Setsemantics(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


	/*
	 * Invalidate semantics attribute.
	 
	 * (Validate it again by setting a value)
	 */
	virtual void Invalidatesemantics();

	/*
	 * Get direction attribute.
	 * @return Mp7JrsOrderingKeyType_direction_LocalType::Enumeration
	 */
	virtual Mp7JrsOrderingKeyType_direction_LocalType::Enumeration Getdirection() const;

	/*
	 * Get direction attribute validity information.
	 * @return true if attribute value is valid, false if not.
	 */
	virtual bool Existdirection() const;

	/*
	 * Set direction attribute.
	 * @param item Mp7JrsOrderingKeyType_direction_LocalType::Enumeration
	 */
	virtual void Setdirection(Mp7JrsOrderingKeyType_direction_LocalType::Enumeration item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


	/*
	 * Invalidate direction attribute.
	 
	 * (Validate it again by setting a value)
	 */
	virtual void Invalidatedirection();

		/* @name Sequence

		 */
		//@{
	/*
	 * Get Selector element.
	 * @return Mp7JrsOrderingKeyType_Selector_LocalPtr
	 */
	virtual Mp7JrsOrderingKeyType_Selector_LocalPtr GetSelector() const;

	/*
	 * Set Selector element.
	 * @param item const Mp7JrsOrderingKeyType_Selector_LocalPtr &
	 */
	// Mandatory			
	virtual void SetSelector(const Mp7JrsOrderingKeyType_Selector_LocalPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	/*
	 * Get Field element.
	 * @return Mp7JrsOrderingKeyType_Field_CollectionPtr
	 */
	virtual Mp7JrsOrderingKeyType_Field_CollectionPtr GetField() const;

	/*
	 * Set Field element.
	 * @param item const Mp7JrsOrderingKeyType_Field_CollectionPtr &
	 */
	// Mandatory collection
	virtual void SetField(const Mp7JrsOrderingKeyType_Field_CollectionPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

		//@}
	/*
	 * Get id attribute.
<br>
	 * (Inherited from Mp7JrsHeaderType)
	 * @return XMLCh *
	 */
	virtual XMLCh * Getid() const;

	/*
	 * Get id attribute validity information.
	 * (Inherited from Mp7JrsHeaderType)
	 * @return true if attribute value is valid, false if not.
	 */
	virtual bool Existid() const;

	/*
	 * Set id attribute.
<br>
	 * (Inherited from Mp7JrsHeaderType)
	 * @param item XMLCh *
	 */
	virtual void Setid(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


	/*
	 * Invalidate id attribute.
	 * (Inherited from Mp7JrsHeaderType)
	 
	 * (Validate it again by setting a value)
	 */
	virtual void Invalidateid();

	/*
	 * Gets or creates Mp7JrsNodePtr child elements of OrderingKeyType.
	 * Currently this type contains 2 child elements of type "Mp7JrsNodePtr".

	 * <ul>
	 * <li>Selector</li>
	 * <li>urn:mpeg:mpeg7:schema:2004:Field</li>
	 * </ul>
	 * 
	 * Supported XPath expressions for GetOrCreateFromXPath
	 * 
	 * \par<code>/</code>
	 *  - Get the root of this node.
	 * \par<code>..</code>
	 *  - This feature is experimental.
	 *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
	 * 
	 * \par <code>\@A</code>
	 * - Force creation of a child attribute named <code>A</code>.
	 * - This is valid only, if the type of attribute is derived from Mp7JrsNodePtr.
	 * - Evaluating this expression for a plain type like int or string results in an exception.
	 *
	 * \par<code>A</code> 
	 *  - Select all child elements named <code>A</code>.
	 *  - In case of no element, create a new one.
	 *  - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
	 * \par<code>A[i]</code>
	 *  - Select the <code>i</code>th child element of all <code>A</code>.
	 *  - Please note, that XPath indexes start at 1.
	 *  - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
	 *  - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
	 *  - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
	 *  - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
	 *
	 * \par <code>A[\@xsi:type="T"]</code>
	 *  - Select all child elements named <code>A</code> of type <code>T</code>.
	 *  - Currently only typenames without namespace are recognised.
	 *  - If there is no such element, create and append a new one.
	 * \par <code>A[i][\@xsi:type="T"]</code>
	 *  - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
	 *  - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
	 * \par <code>A[\@xsi:type="T"][i]</code>
	 *  - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
	 *  - If there is no such element at index <code>i</code>, create and append one.
	 *
	 * \par <code>A[last()]</code>
	 *  - Select the last child element named <code>A</code>.
	 *  - In case of no element, create a new one.
	 * \par <code>A[last()][\@xsi:type="T"]</code>
	 *  - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
	 *  - For collections: in case of no such type, create and append a new one.
	 *  - For single elements: if an element with different type exists, return the empty list.
	 * 
	 * \par <code>A[\@xsi:type="T"][last()]</code>
	 *  - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
	 *  - For collections: in case of no such type, create and append a new one.
	 *  - For single elements: if an element with different type exists, return the empty list.
	 * 
	 * \par <code>A[position() \> last()]</code>
	 *  - Force creation of a child element named <code>A</code> with its default type.
	 *  - For collections: create and append the default element.
	 *  - For single elements: if any element exists, an exception is thrown.
	 *
	 * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
	 *  - Force creation of a child element named <code>A</code> of type <code>T</code>.
	 *  - For collections: create and append the element.
	 *  - For single elements: if any element exists, an exception is thrown.
	 *
	 * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
	 *  - Force creation of a child element named <code>A</code> of type <code>T</code>.
	 *  - Behaves the same way as the previous expression.
	 * 
	 * Supported XPath expressions for GetFromXPath<br>
	 * The same expressions like GetOrCreateFromXPath plus
	 * \par <code>A[position() \< i] </code>
	 *  - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
	 *  - <code>i</code> must be greater than -1, otherwise an exception is thrown.
	 * \par <code>A[position() \> i] </code>
	 *  - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
	 *  - <code>i</code> must be greater than -1, otherwise an exception is thrown.
	 * \par <code>A[position() \> i | position() \< j] </code>
	 *  - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
	 *  - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
	 *  - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
	 *
	 * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
	 *
	 * @param xpath  the xpath expression to get or create.
	 * @param client the ID in case of locking.
	 * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
	 */
	// xpath: mandatory
	// client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

	

	/* Get the class name.
	 * @return The name of this class.
	 */
	virtual XMLCh * ToText() const;

	/* Parse this type.
	 * @param txt buffer to parse from.
	 * @return true if parsing was successful, false otherwise.
	 */
	virtual bool Parse(const XMLCh * const txt);

	/* Serialialize this object to a DOM tree.
	 * @param doc DOM document.
	 * @param parent Parent node in DOM tree.
	 */
	virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

	/* Deserialize this object from a DOM tree.
	 * @param doc DOM document.
	 * @param parent Parent element in DOM tree.
	 * @param current Current element in DOM tree.
	 */
	virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

	/* Creates and inserts a Childnode of the specified Type
	 * @param type The tag type of the child to be created (tag name).
	 * @param xsitype The xsi type of the child to be created if available.
	 * @return the created child with the correct dynamic type for the specified type.
	 */
	virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
		 
// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtMyMethodDef.h


public:

	virtual ~Mp7JrsOrderingKeyType();

protected:
	Mp7JrsOrderingKeyType();
	virtual void DeepCopy(const Mp7JrsNodePtr &original);

	virtual Mp7JrsNodeEnum GetAllChildElements() const;

	virtual Mp7JrsNodePtr GetBaseRoot();
	virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
	XMLCh * m_name;
	bool m_name_Exist;
	XMLCh * m_semantics;
	bool m_semantics_Exist;
	Mp7JrsOrderingKeyType_direction_LocalType::Enumeration m_direction;
	bool m_direction_Exist;
	Mp7JrsOrderingKeyType_direction_LocalType::Enumeration m_direction_Default;

	// Base Class
	Mp7JrsPtr< Mp7JrsHeaderType > m_Base;

	Mp7JrsOrderingKeyType_Selector_LocalPtr m_Selector;
	Mp7JrsOrderingKeyType_Field_CollectionPtr m_Field;

// no includefile for extension defined 
// file Mp7JrsOrderingKeyType_ExtMyPropDef.h


public:
	virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
	void Cleanup();
};

#endif // _1609MP7JRSORDERINGKEYTYPE_H

