/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#ifndef MP7JRSSETTINGS_H
#define MP7JRSSETTINGS_H

#pragma once

#include "Mp7JrsDefines.h"

/** Mp7Jrs settings.
  * A singleton class which contains all global settings.
  * This class is NOT threadsafe.
  */
class MP7JRS_EXPORT Mp7JrsSettings
{

public:
	/** @name Constructors and Destructors
	 */
	//@{

	static Mp7JrsSettings & GetInstance()
	{
		static Mp7JrsSettings instance; // Guaranteed to be destroyed. Instantiated on first use.
		return instance;
	}
	//@}


	/** Triggers special treatment of MediaTime in AVDP.
		In MPEG-7 V2 MediaTime offers a choice of MediaTimePoint, MediaRelTimePoint and MediaRelIncrTimePoint.
		In AVDP we only have MediaRelIncrTimePoint (+ and optional MediaIncrDuration)

		Since we don't want an extra lib for AVDP
		the MediaTimePoint is used to retrieve the absolute timepoint
		calculated from the MediaRelIncrTimePoint and the mediaTimeUnit and mediaTimeBase
		taken from an outer DOM element (like AudioVisual).
		mediaTimeBase points to a MediaLocator, that now contains an absolute TimePoint
		so that MediaTimePoint (absolute) = MediaRelIncrTimePoint * mediaTimeUnit + mediaTimeBase (from MediaLocator)
		This behaviour is only available if 
		Mp7JrsSettings.instance().TreatRelIncrTimeAsAbsolute is set to true.

		Default setting: false.
	*/
	bool TreatRelIncrTimeAsAbsolute;


private:
	Mp7JrsSettings(void)
	{
		TreatRelIncrTimeAsAbsolute = false;
	}

	Mp7JrsSettings(Mp7JrsSettings const&);
	void operator=(Mp7JrsSettings const&);
};

#endif
