/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// PatternType_H

#ifndef _1263MP7JRSMEDIADURATIONTYPE_H
#define _1263MP7JRSMEDIADURATIONTYPE_H


#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsmediaDurationTypeFactory.h"
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>
#include "Mp7JrsConvert.h"

// no includefile for extension defined 
// file Mp7JrsmediaDurationType_ExtInclude.h



/**
 * Generated interface IMp7JrsmediaDurationType for Pattern Mp7JrsmediaDurationType
 */
class MP7JRS_EXPORT IMp7JrsmediaDurationType :
	public Mp7JrsNode
{
public:
	virtual int GetDurationSign() const = 0;

	virtual void SetDurationSign(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetDays() const = 0;

	virtual void SetDays(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetHours() const = 0;

	virtual void SetHours(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetMinutes() const = 0;

	virtual void SetMinutes(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetSeconds() const = 0;

	virtual void SetSeconds(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetNumberOfFractions() const = 0;

	virtual void SetNumberOfFractions(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual bool GetTime_Exist() const = 0;

	virtual void SetTime_Exist(bool val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

	virtual int GetFractionsPerSecond() const = 0;

	virtual void SetFractionsPerSecond(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


// begin extension included
// file Mp7JrsmediaDurationType_ExtMethodDef.h


public:

	/** Get total number of fractions represented by this timepoint.
	  * @return Total number of fractions represented with respect to the number of fractions/second of this duration. 
	  */
	virtual int GetTotalNrFractions();

	/** Set the total number of fractions of this duration.
	  * @return Total number of fractions represented with respect to the number of fractions/second of this duration. 
	  */
	virtual void SetTotalNrFractions(int fractions, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);
// end extension included


};

/*
 * Generated pattern Mp7JrsmediaDurationType<br>
 * Located at: mpeg7-v2.xsd, line 3707<br>
 * Regular expression: <br>
 * \-?P(\d+D)?(T(\d+H)?(\d+M)?(\d+S)?(\d+N)?)?(\d+F)?<br>
 */
class MP7JRS_EXPORT Mp7JrsmediaDurationType :
	public IMp7JrsmediaDurationType
{
	friend class Mp7JrsmediaDurationTypeFactory;
	
public: 
	virtual int GetDurationSign() const;

	virtual void SetDurationSign(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetDays() const;

	virtual void SetDays(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetHours() const;

	virtual void SetHours(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetMinutes() const;

	virtual void SetMinutes(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetSeconds() const;

	virtual void SetSeconds(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetNumberOfFractions() const;

	virtual void SetNumberOfFractions(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual bool GetTime_Exist() const;

	virtual void SetTime_Exist(bool val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

	virtual int GetFractionsPerSecond() const;

	virtual void SetFractionsPerSecond(int val, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


	virtual XMLCh * ToText() const;
	virtual bool Parse(const XMLCh * const txt);
	virtual bool ContentFromString(const XMLCh * const txt);

	virtual XMLCh * ContentToString() const;

	virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);
	virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent,  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

// no includefile for extension defined 
// file Mp7JrsmediaDurationType_ExtMyMethodDef.h


	virtual ~Mp7JrsmediaDurationType();

protected:
	Mp7JrsmediaDurationType();
	virtual void DeepCopy(const Mp7JrsNodePtr &original);
	virtual Mp7JrsNodeEnum GetAllChildElements() const { return Mp7JrsNodeEnum(); }

	virtual Mp7JrsNodePtr GetBaseRoot();
	virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
	int m_DurationSign;
	int m_Days;
	int m_Hours;
	int m_Minutes;
	int m_Seconds;
	int m_NumberOfFractions;
	bool m_Time_Exist;
	int m_FractionsPerSecond;

// no includefile for extension defined 
// file Mp7JrsmediaDurationType_ExtPropDef.h


protected:
	void Init();
	void Cleanup();
	
protected:
    const XMLCh * const m_nsURI; // For namespace handling
};

#endif // _1263MP7JRSMEDIADURATIONTYPE_H

