
/* Simple MPEG-7 JRS sample program, Objective-C++ version.
 * 
 * (c) 2007 by Werner Bailer <werner@wbailer.com>
 *     based on the C++ sample (c) 2003 - 2009 JOANNEUM RESEARCH
 */



#import <Foundation/Foundation.h>


/// #include <stdio.h>

// Xerces utilities
#include <xercesc/util/PlatformUtils.hpp>

// archive class used for (de-) serializing
#include "Mp7Jrs/Mp7JrsArchive.h"

// factory registry and defines
#include "Mp7Jrs/Mp7JrsFactory.h"
#include "Mp7Jrs/Mp7JrsFactoryDefines.h"


int main (int argc, const char * argv[]) {
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

	try {
	
		// initializes Xerces
		XMLPlatformUtils::Initialize();
		// initialize MPEG-7 JRS
		Mp7JrsFactory::Initialize();

		// the following code goes into its own block to make sure 
		// that the smart pointer fall out of scope and are destructed
		// before the factory needed for destruction is deleted
		{
			// create archive instance
			Mp7JrsArchive archive;

			// set encoding and validation mode
			archive.SetEncoding("UTF-8");
			archive.SetDoValidation(false);

			Mp7JrsNodePtr node = CreateVideoSegmentType;

			// load and validate MPEG-7 XML file
			// returns the root node of the document
 			Mp7JrsNodePtr root = archive.FromFile("../../test.xml");

			// serialize the root node again
			archive.ToFile("out.xml",root);

		}
		

		// cleanup MPEG-7 JRS
		Mp7JrsFactory::Terminate();
		// cleanup Xerces
		XMLPlatformUtils::Terminate();
	}
	catch (Mp7JrsException& e) 
	{
	
		NSLog([NSString stringWithUTF8String:e.GetMessage() ] );
	}

    [pool release];
    return 0;
}
