 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtImplInclude.h



#include "Mp7JrsAudioDType.h"
#include "Mp7JrsAudioLLDScalarType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsAudioHarmonicityType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsAudioHarmonicityType::IMp7JrsAudioHarmonicityType()
{

// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtPropInit.h

}

IMp7JrsAudioHarmonicityType::~IMp7JrsAudioHarmonicityType()
{
// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtPropCleanup.h

}

Mp7JrsAudioHarmonicityType::Mp7JrsAudioHarmonicityType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsAudioHarmonicityType::~Mp7JrsAudioHarmonicityType()
{
	Cleanup();
}

void Mp7JrsAudioHarmonicityType::Init()
{
	// Init base
	m_Base = CreateAudioDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_HarmonicRatio = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_UpperLimitOfHarmonicity = Mp7JrsPtr< Mp7JrsNode >(); // Class


// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtMyPropInit.h

}

void Mp7JrsAudioHarmonicityType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_HarmonicRatio);
	// Mp7JrsFactory::DeleteObject(m_UpperLimitOfHarmonicity);
}

void Mp7JrsAudioHarmonicityType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use AudioHarmonicityTypePtr, since we
	// might need GetBase(), which isn't defined in IAudioHarmonicityType
	const Mp7JrsPtr< Mp7JrsAudioHarmonicityType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsAudioDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsAudioHarmonicityType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_HarmonicRatio);
		this->SetHarmonicRatio(Mp7JrsFactory::CloneObject(tmp->GetHarmonicRatio()));
		// Mp7JrsFactory::DeleteObject(m_UpperLimitOfHarmonicity);
		this->SetUpperLimitOfHarmonicity(Mp7JrsFactory::CloneObject(tmp->GetUpperLimitOfHarmonicity()));
}

Mp7JrsNodePtr Mp7JrsAudioHarmonicityType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsAudioHarmonicityType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAudioDType > Mp7JrsAudioHarmonicityType::GetBase() const
{
	return m_Base;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsAudioHarmonicityType::GetHarmonicRatio() const
{
		return m_HarmonicRatio;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsAudioHarmonicityType::GetUpperLimitOfHarmonicity() const
{
		return m_UpperLimitOfHarmonicity;
}

void Mp7JrsAudioHarmonicityType::SetHarmonicRatio(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioHarmonicityType::SetHarmonicRatio().");
	}
	if (m_HarmonicRatio != item)
	{
		// Mp7JrsFactory::DeleteObject(m_HarmonicRatio);
		m_HarmonicRatio = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_HarmonicRatio) m_HarmonicRatio->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_HarmonicRatio) m_HarmonicRatio->UseTypeAttribute = true;
		if(m_HarmonicRatio != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_HarmonicRatio->SetContentName(XMLString::transcode("HarmonicRatio"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioHarmonicityType::SetUpperLimitOfHarmonicity(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioHarmonicityType::SetUpperLimitOfHarmonicity().");
	}
	if (m_UpperLimitOfHarmonicity != item)
	{
		// Mp7JrsFactory::DeleteObject(m_UpperLimitOfHarmonicity);
		m_UpperLimitOfHarmonicity = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_UpperLimitOfHarmonicity) m_UpperLimitOfHarmonicity->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_UpperLimitOfHarmonicity) m_UpperLimitOfHarmonicity->UseTypeAttribute = true;
		if(m_UpperLimitOfHarmonicity != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_UpperLimitOfHarmonicity->SetContentName(XMLString::transcode("UpperLimitOfHarmonicity"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsintegerVectorPtr Mp7JrsAudioHarmonicityType::Getchannels() const
{
	return GetBase()->Getchannels();
}

bool Mp7JrsAudioHarmonicityType::Existchannels() const
{
	return GetBase()->Existchannels();
}
void Mp7JrsAudioHarmonicityType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioHarmonicityType::Setchannels().");
	}
	GetBase()->Setchannels(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioHarmonicityType::Invalidatechannels()
{
	GetBase()->Invalidatechannels();
}

Mp7JrsNodeEnum Mp7JrsAudioHarmonicityType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("HarmonicRatio")) == 0)
	{
		// HarmonicRatio is simple abstract element AudioLLDScalarType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetHarmonicRatio()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsAudioLLDScalarPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetHarmonicRatio(p, client);
					if((p = GetHarmonicRatio()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("UpperLimitOfHarmonicity")) == 0)
	{
		// UpperLimitOfHarmonicity is simple abstract element AudioLLDScalarType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetUpperLimitOfHarmonicity()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsAudioLLDScalarPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetUpperLimitOfHarmonicity(p, client);
					if((p = GetUpperLimitOfHarmonicity()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for AudioHarmonicityType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "AudioHarmonicityType");
	}
	return result;
}

XMLCh * Mp7JrsAudioHarmonicityType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsAudioHarmonicityType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsAudioHarmonicityType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("AudioHarmonicityType"));
	// Element serialization:
	// Class
	
	if (m_HarmonicRatio != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_HarmonicRatio->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("HarmonicRatio"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_HarmonicRatio->UseTypeAttribute = true;
		}
		m_HarmonicRatio->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_UpperLimitOfHarmonicity != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_UpperLimitOfHarmonicity->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("UpperLimitOfHarmonicity"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_UpperLimitOfHarmonicity->UseTypeAttribute = true;
		}
		m_UpperLimitOfHarmonicity->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsAudioHarmonicityType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsAudioDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("HarmonicRatio"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("HarmonicRatio")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateAudioLLDScalarType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetHarmonicRatio(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("UpperLimitOfHarmonicity"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("UpperLimitOfHarmonicity")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateAudioLLDScalarType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetUpperLimitOfHarmonicity(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsAudioHarmonicityType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("HarmonicRatio")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateAudioLLDScalarType;
	}
	this->SetHarmonicRatio(child);
  }
  if (XMLString::compareString(elementname, X("UpperLimitOfHarmonicity")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateAudioLLDScalarType;
	}
	this->SetUpperLimitOfHarmonicity(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsAudioHarmonicityType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetHarmonicRatio() != Mp7JrsNodePtr())
		result.Insert(GetHarmonicRatio());
	if (GetUpperLimitOfHarmonicity() != Mp7JrsNodePtr())
		result.Insert(GetUpperLimitOfHarmonicity());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsAudioHarmonicityType_ExtMethodImpl.h


