 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsAudioLLDScalarType_ExtImplInclude.h



#include "Mp7JrsAudioDType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsAudioLLDScalarType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsAudioLLDScalarType_SeriesOfScalar_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:SeriesOfScalar

#include <assert.h>
IMp7JrsAudioLLDScalarType::IMp7JrsAudioLLDScalarType()
{

// no includefile for extension defined 
// file Mp7JrsAudioLLDScalarType_ExtPropInit.h

}

IMp7JrsAudioLLDScalarType::~IMp7JrsAudioLLDScalarType()
{
// no includefile for extension defined 
// file Mp7JrsAudioLLDScalarType_ExtPropCleanup.h

}

Mp7JrsAudioLLDScalarType::Mp7JrsAudioLLDScalarType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsAudioLLDScalarType::~Mp7JrsAudioLLDScalarType()
{
	Cleanup();
}

void Mp7JrsAudioLLDScalarType::Init()
{
	// Init base
	m_Base = CreateAudioDType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_confidence = CreatezeroToOneType; // Create a valid class
	m_confidence->SetContent(0.0); // Use Value initialiser (xxx2)
	m_confidence_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Scalar = (0.0); // Value

	m_SeriesOfScalar = Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsAudioLLDScalarType_ExtMyPropInit.h

}

void Mp7JrsAudioLLDScalarType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsAudioLLDScalarType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_confidence);
	// Mp7JrsFactory::DeleteObject(m_SeriesOfScalar);
}

void Mp7JrsAudioLLDScalarType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use AudioLLDScalarTypePtr, since we
	// might need GetBase(), which isn't defined in IAudioLLDScalarType
	const Mp7JrsPtr< Mp7JrsAudioLLDScalarType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsAudioDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsAudioLLDScalarType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_confidence);
	if (tmp->Existconfidence())
	{
		this->Setconfidence(Mp7JrsFactory::CloneObject(tmp->Getconfidence()));
	}
	else
	{
		Invalidateconfidence();
	}
	}
		this->SetScalar(tmp->GetScalar());
		// Mp7JrsFactory::DeleteObject(m_SeriesOfScalar);
		this->SetSeriesOfScalar(Mp7JrsFactory::CloneObject(tmp->GetSeriesOfScalar()));
}

Mp7JrsNodePtr Mp7JrsAudioLLDScalarType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsAudioLLDScalarType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAudioDType > Mp7JrsAudioLLDScalarType::GetBase() const
{
	return m_Base;
}

Mp7JrszeroToOnePtr Mp7JrsAudioLLDScalarType::Getconfidence() const
{
	return m_confidence;
}

bool Mp7JrsAudioLLDScalarType::Existconfidence() const
{
	return m_confidence_Exist;
}
void Mp7JrsAudioLLDScalarType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioLLDScalarType::Setconfidence().");
	}
	m_confidence = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_confidence) m_confidence->SetParent(m_myself.getPointer());
	m_confidence_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioLLDScalarType::Invalidateconfidence()
{
	m_confidence_Exist = false;
}
float Mp7JrsAudioLLDScalarType::GetScalar() const
{
		return m_Scalar;
}

Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr Mp7JrsAudioLLDScalarType::GetSeriesOfScalar() const
{
		return m_SeriesOfScalar;
}

// implementing setter for choice 
/* element */
void Mp7JrsAudioLLDScalarType::SetScalar(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioLLDScalarType::SetScalar().");
	}
	if (m_Scalar != item)
	{
		m_Scalar = item;
		m_Scalar_Valid = true;
	// Mp7JrsFactory::DeleteObject(m_SeriesOfScalar);
	m_SeriesOfScalar = Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsAudioLLDScalarType::SetSeriesOfScalar(const Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioLLDScalarType::SetSeriesOfScalar().");
	}
	if (m_SeriesOfScalar != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SeriesOfScalar);
		m_SeriesOfScalar = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SeriesOfScalar) m_SeriesOfScalar->SetParent(m_myself.getPointer());
		if(m_SeriesOfScalar != Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SeriesOfScalar->SetContentName(XMLString::transcode("SeriesOfScalar"));
		}
	
	m_Scalar_Valid = false;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsintegerVectorPtr Mp7JrsAudioLLDScalarType::Getchannels() const
{
	return GetBase()->Getchannels();
}

bool Mp7JrsAudioLLDScalarType::Existchannels() const
{
	return GetBase()->Existchannels();
}
void Mp7JrsAudioLLDScalarType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioLLDScalarType::Setchannels().");
	}
	GetBase()->Setchannels(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioLLDScalarType::Invalidatechannels()
{
	GetBase()->Invalidatechannels();
}

Mp7JrsNodeEnum Mp7JrsAudioLLDScalarType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 2 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; channels,  T:urn:mpeg:mpeg7:schema:2004:integerVector, C:Collection 
//- N; confidence,  T:urn:mpeg:mpeg7:schema:2004:zeroToOneType, C:Class 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("confidence")) == 0)
	{
		// confidence is simple attribute zeroToOneType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = Getconfidence()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:zeroToOneType")))) != empty)
			{
				// Is type allowed
				Mp7JrszeroToOnePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					Setconfidence(p, client);
					if((p = Getconfidence()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("SeriesOfScalar")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:SeriesOfScalar is item of type AudioLLDScalarType_SeriesOfScalar_LocalType
		// in element collection AudioLLDScalarType_SeriesOfScalar_CollectionType
		
		context->Found = true;
		Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr coll = GetSeriesOfScalar();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateAudioLLDScalarType_SeriesOfScalar_CollectionType;
				SetSeriesOfScalar(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:AudioLLDScalarType_SeriesOfScalar_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsAudioLLDScalarType_SeriesOfScalar_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for AudioLLDScalarType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "AudioLLDScalarType");
	}
	return result;
}

XMLCh * Mp7JrsAudioLLDScalarType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsAudioLLDScalarType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsAudioLLDScalarType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsAudioLLDScalarType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("AudioLLDScalarType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_confidence_Exist)
	{
	// Class
	if (m_confidence != Mp7JrszeroToOnePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_confidence->ToText();
		element->setAttributeNS(X(""), X("confidence"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if(m_Scalar_Valid) // Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_Scalar);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("Scalar"), false);
		XMLString::release(&tmp);
	}
	if (m_SeriesOfScalar != Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_SeriesOfScalar->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsAudioLLDScalarType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("confidence")))
	{
		// Deserialize class type
		Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
		tmp->Parse(parent->getAttribute(X("confidence")));
		this->Setconfidence(tmp);
		* current = parent;
	}

	// Extensionbase is Mp7JrsAudioDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
	{
	// Deserialize value element
	if(Mp7JrsUtil::HasNodeName(parent, X("Scalar"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		XMLCh * tmp = Mp7JrsUtil::GetElementText(parent);
		this->SetScalar(Mp7JrsConvert::TextToFloat(tmp));
		XMLString::release(&tmp);
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		continue;	   // For choices
	}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("SeriesOfScalar"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("SeriesOfScalar")) == 0))
		{
			// Deserialize factory type
			Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr tmp = CreateAudioLLDScalarType_SeriesOfScalar_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetSeriesOfScalar(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	} while(false);
// no includefile for extension defined 
// file Mp7JrsAudioLLDScalarType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsAudioLLDScalarType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("SeriesOfScalar")) == 0))
  {
	Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr tmp = CreateAudioLLDScalarType_SeriesOfScalar_CollectionType;
	this->SetSeriesOfScalar(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsAudioLLDScalarType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSeriesOfScalar() != Mp7JrsNodePtr())
		result.Insert(GetSeriesOfScalar());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsAudioLLDScalarType_ExtMethodImpl.h


