 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsAudioWaveformType_ExtImplInclude.h



#include "Mp7JrsAudioLLDScalarType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsAudioWaveformType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsAudioLLDScalarType_SeriesOfScalar_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:SeriesOfScalar

#include <assert.h>
IMp7JrsAudioWaveformType::IMp7JrsAudioWaveformType()
{

// no includefile for extension defined 
// file Mp7JrsAudioWaveformType_ExtPropInit.h

}

IMp7JrsAudioWaveformType::~IMp7JrsAudioWaveformType()
{
// no includefile for extension defined 
// file Mp7JrsAudioWaveformType_ExtPropCleanup.h

}

Mp7JrsAudioWaveformType::Mp7JrsAudioWaveformType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsAudioWaveformType::~Mp7JrsAudioWaveformType()
{
	Cleanup();
}

void Mp7JrsAudioWaveformType::Init()
{
	// Init base
	m_Base = CreateAudioLLDScalarType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_minRange = 0.0; // Value
	m_minRange_Exist = false;
	m_maxRange = 0.0; // Value
	m_maxRange_Exist = false;



// no includefile for extension defined 
// file Mp7JrsAudioWaveformType_ExtMyPropInit.h

}

void Mp7JrsAudioWaveformType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsAudioWaveformType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
}

void Mp7JrsAudioWaveformType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use AudioWaveformTypePtr, since we
	// might need GetBase(), which isn't defined in IAudioWaveformType
	const Mp7JrsPtr< Mp7JrsAudioWaveformType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsAudioLLDScalarPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsAudioWaveformType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->ExistminRange())
	{
		this->SetminRange(tmp->GetminRange());
	}
	else
	{
		InvalidateminRange();
	}
	}
	{
	if (tmp->ExistmaxRange())
	{
		this->SetmaxRange(tmp->GetmaxRange());
	}
	else
	{
		InvalidatemaxRange();
	}
	}
}

Mp7JrsNodePtr Mp7JrsAudioWaveformType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsAudioWaveformType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAudioLLDScalarType > Mp7JrsAudioWaveformType::GetBase() const
{
	return m_Base;
}

float Mp7JrsAudioWaveformType::GetminRange() const
{
	return m_minRange;
}

bool Mp7JrsAudioWaveformType::ExistminRange() const
{
	return m_minRange_Exist;
}
void Mp7JrsAudioWaveformType::SetminRange(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioWaveformType::SetminRange().");
	}
	m_minRange = item;
	m_minRange_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioWaveformType::InvalidateminRange()
{
	m_minRange_Exist = false;
}
float Mp7JrsAudioWaveformType::GetmaxRange() const
{
	return m_maxRange;
}

bool Mp7JrsAudioWaveformType::ExistmaxRange() const
{
	return m_maxRange_Exist;
}
void Mp7JrsAudioWaveformType::SetmaxRange(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioWaveformType::SetmaxRange().");
	}
	m_maxRange = item;
	m_maxRange_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioWaveformType::InvalidatemaxRange()
{
	m_maxRange_Exist = false;
}
Mp7JrszeroToOnePtr Mp7JrsAudioWaveformType::Getconfidence() const
{
	return GetBase()->Getconfidence();
}

bool Mp7JrsAudioWaveformType::Existconfidence() const
{
	return GetBase()->Existconfidence();
}
void Mp7JrsAudioWaveformType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioWaveformType::Setconfidence().");
	}
	GetBase()->Setconfidence(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioWaveformType::Invalidateconfidence()
{
	GetBase()->Invalidateconfidence();
}
float Mp7JrsAudioWaveformType::GetScalar() const
{
	return GetBase()->GetScalar();
}

Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr Mp7JrsAudioWaveformType::GetSeriesOfScalar() const
{
	return GetBase()->GetSeriesOfScalar();
}

// implementing setter for choice 
/* element */
void Mp7JrsAudioWaveformType::SetScalar(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioWaveformType::SetScalar().");
	}
	GetBase()->SetScalar(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsAudioWaveformType::SetSeriesOfScalar(const Mp7JrsAudioLLDScalarType_SeriesOfScalar_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioWaveformType::SetSeriesOfScalar().");
	}
	GetBase()->SetSeriesOfScalar(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsintegerVectorPtr Mp7JrsAudioWaveformType::Getchannels() const
{
	return GetBase()->GetBase()->Getchannels();
}

bool Mp7JrsAudioWaveformType::Existchannels() const
{
	return GetBase()->GetBase()->Existchannels();
}
void Mp7JrsAudioWaveformType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsAudioWaveformType::Setchannels().");
	}
	GetBase()->GetBase()->Setchannels(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsAudioWaveformType::Invalidatechannels()
{
	GetBase()->GetBase()->Invalidatechannels();
}

Mp7JrsNodeEnum Mp7JrsAudioWaveformType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  2, 4 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 2
//- N; channels,  T:urn:mpeg:mpeg7:schema:2004:integerVector, C:Collection 
//- N; confidence,  T:urn:mpeg:mpeg7:schema:2004:zeroToOneType, C:Class 
//- N; minRange,  T:http://www.w3.org/2001/XMLSchema:float, C:Value 
//- N; maxRange,  T:http://www.w3.org/2001/XMLSchema:float, C:Value 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("minRange")) == 0)
	{
		// minRange is simple attribute float
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "float");
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("maxRange")) == 0)
	{
		// maxRange is simple attribute float
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "float");
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for AudioWaveformType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "AudioWaveformType");
	}
	return result;
}

XMLCh * Mp7JrsAudioWaveformType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsAudioWaveformType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsAudioWaveformType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsAudioWaveformType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("AudioWaveformType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_minRange_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_minRange);
		element->setAttributeNS(X(""), X("minRange"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Attribute Serialization:
		

	// Optional attriute
	if(m_maxRange_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_maxRange);
		element->setAttributeNS(X(""), X("maxRange"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsAudioWaveformType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("minRange")))
	{
		// deserialize value type
		this->SetminRange(Mp7JrsConvert::TextToFloat(parent->getAttribute(X("minRange"))));
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("maxRange")))
	{
		// deserialize value type
		this->SetmaxRange(Mp7JrsConvert::TextToFloat(parent->getAttribute(X("maxRange"))));
		* current = parent;
	}

	// Extensionbase is Mp7JrsAudioLLDScalarType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsAudioWaveformType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsAudioWaveformType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsAudioWaveformType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSeriesOfScalar() != Mp7JrsNodePtr())
		result.Insert(GetSeriesOfScalar());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsAudioWaveformType_ExtMethodImpl.h


