 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsClassificationSchemeBaseType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsClassificationSchemeBaseType_domain_CollectionType.h"
#include "Mp7JrsClassificationSchemeBaseType_Import_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsClassificationSchemeBaseType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsReferenceType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Import

#include <assert.h>
IMp7JrsClassificationSchemeBaseType::IMp7JrsClassificationSchemeBaseType()
{

// no includefile for extension defined 
// file Mp7JrsClassificationSchemeBaseType_ExtPropInit.h

}

IMp7JrsClassificationSchemeBaseType::~IMp7JrsClassificationSchemeBaseType()
{
// no includefile for extension defined 
// file Mp7JrsClassificationSchemeBaseType_ExtPropCleanup.h

}

Mp7JrsClassificationSchemeBaseType::Mp7JrsClassificationSchemeBaseType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsClassificationSchemeBaseType::~Mp7JrsClassificationSchemeBaseType()
{
	Cleanup();
}

void Mp7JrsClassificationSchemeBaseType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_uri = NULL; // String
	m_domain = Mp7JrsClassificationSchemeBaseType_domain_CollectionPtr(); // Collection
	m_domain_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Import = Mp7JrsClassificationSchemeBaseType_Import_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsClassificationSchemeBaseType_ExtMyPropInit.h

}

void Mp7JrsClassificationSchemeBaseType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsClassificationSchemeBaseType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_uri); // String
	// Mp7JrsFactory::DeleteObject(m_domain);
	// Mp7JrsFactory::DeleteObject(m_Import);
}

void Mp7JrsClassificationSchemeBaseType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ClassificationSchemeBaseTypePtr, since we
	// might need GetBase(), which isn't defined in IClassificationSchemeBaseType
	const Mp7JrsPtr< Mp7JrsClassificationSchemeBaseType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsClassificationSchemeBaseType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_uri); // String
		this->Seturi(XMLString::replicate(tmp->Geturi()));
	}
	{
	// Mp7JrsFactory::DeleteObject(m_domain);
	if (tmp->Existdomain())
	{
		this->Setdomain(Mp7JrsFactory::CloneObject(tmp->Getdomain()));
	}
	else
	{
		Invalidatedomain();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Import);
		this->SetImport(Mp7JrsFactory::CloneObject(tmp->GetImport()));
}

Mp7JrsNodePtr Mp7JrsClassificationSchemeBaseType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsClassificationSchemeBaseType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsClassificationSchemeBaseType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsClassificationSchemeBaseType::Geturi() const
{
	return m_uri;
}

void Mp7JrsClassificationSchemeBaseType::Seturi(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsClassificationSchemeBaseType::Seturi().");
	}
	m_uri = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsClassificationSchemeBaseType_domain_CollectionPtr Mp7JrsClassificationSchemeBaseType::Getdomain() const
{
	return m_domain;
}

bool Mp7JrsClassificationSchemeBaseType::Existdomain() const
{
	return m_domain_Exist;
}
void Mp7JrsClassificationSchemeBaseType::Setdomain(const Mp7JrsClassificationSchemeBaseType_domain_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsClassificationSchemeBaseType::Setdomain().");
	}
	m_domain = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_domain) m_domain->SetParent(m_myself.getPointer());
	m_domain_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsClassificationSchemeBaseType::Invalidatedomain()
{
	m_domain_Exist = false;
}
Mp7JrsClassificationSchemeBaseType_Import_CollectionPtr Mp7JrsClassificationSchemeBaseType::GetImport() const
{
		return m_Import;
}

void Mp7JrsClassificationSchemeBaseType::SetImport(const Mp7JrsClassificationSchemeBaseType_Import_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsClassificationSchemeBaseType::SetImport().");
	}
	if (m_Import != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Import);
		m_Import = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Import) m_Import->SetParent(m_myself.getPointer());
		if(m_Import != Mp7JrsClassificationSchemeBaseType_Import_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Import->SetContentName(XMLString::transcode("Import"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsClassificationSchemeBaseType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsClassificationSchemeBaseType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsClassificationSchemeBaseType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsClassificationSchemeBaseType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsClassificationSchemeBaseType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsClassificationSchemeBaseType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsClassificationSchemeBaseType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsClassificationSchemeBaseType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsClassificationSchemeBaseType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsClassificationSchemeBaseType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsClassificationSchemeBaseType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsClassificationSchemeBaseType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsClassificationSchemeBaseType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsClassificationSchemeBaseType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsClassificationSchemeBaseType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsClassificationSchemeBaseType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsClassificationSchemeBaseType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsClassificationSchemeBaseType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsClassificationSchemeBaseType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsClassificationSchemeBaseType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsClassificationSchemeBaseType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsClassificationSchemeBaseType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsClassificationSchemeBaseType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsClassificationSchemeBaseType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsClassificationSchemeBaseType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsClassificationSchemeBaseType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsClassificationSchemeBaseType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsClassificationSchemeBaseType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsClassificationSchemeBaseType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  2, 7 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 2
//- N; id,  T:http://www.w3.org/2001/XMLSchema:ID, C:String 
//- N; timeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; timeUnit,  T:urn:mpeg:mpeg7:schema:2004:durationType, C:Pattern 
//- N; mediaTimeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; mediaTimeUnit,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
//- N; uri,  T:http://www.w3.org/2001/XMLSchema:anyURI, C:String 
//- N; domain,  T:urn:mpeg:mpeg7:schema:2004:ClassificationSchemeBaseType_domain_CollectionType, C:Collection 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("uri")) == 0)
	{
		// uri is simple attribute anyURI
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "XMLCh *");
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("domain")) == 0)
	{
		// domain is simple attribute ClassificationSchemeBaseType_domain_CollectionType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = Getdomain()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ClassificationSchemeBaseType_domain_CollectionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsClassificationSchemeBaseType_domain_CollectionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					Setdomain(p, client);
					if((p = Getdomain()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Import")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Import is item of type ReferenceType
		// in element collection ClassificationSchemeBaseType_Import_CollectionType
		
		context->Found = true;
		Mp7JrsClassificationSchemeBaseType_Import_CollectionPtr coll = GetImport();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateClassificationSchemeBaseType_Import_CollectionType;
				SetImport(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ClassificationSchemeBaseType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ClassificationSchemeBaseType");
	}
	return result;
}

XMLCh * Mp7JrsClassificationSchemeBaseType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsClassificationSchemeBaseType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsClassificationSchemeBaseType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsClassificationSchemeBaseType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ClassificationSchemeBaseType"));
	// Attribute Serialization:
	// String
	if(m_uri != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("uri"), m_uri);
	}
	// Attribute Serialization:
		

	// Optional attriute
	if(m_domain_Exist)
	{
	// Collection
	if(m_domain != Mp7JrsClassificationSchemeBaseType_domain_CollectionPtr())
	{
		XMLCh * tmp = m_domain->ToText();
		element->setAttributeNS(X(""), X("domain"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_Import != Mp7JrsClassificationSchemeBaseType_Import_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Import->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsClassificationSchemeBaseType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("uri")))
	{
		// Deserialize string type
		this->Seturi(Mp7JrsConvert::TextToString(parent->getAttribute(X("uri"))));
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("domain")))
	{
		// Deserialize collection type
		Mp7JrsClassificationSchemeBaseType_domain_CollectionPtr tmp = CreateClassificationSchemeBaseType_domain_CollectionType;
		tmp->Parse(parent->getAttribute(X("domain")));
		this->Setdomain(tmp);
		* current = parent;
	}

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("Import"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("Import")) == 0))
		{
			// Deserialize factory type
			Mp7JrsClassificationSchemeBaseType_Import_CollectionPtr tmp = CreateClassificationSchemeBaseType_Import_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetImport(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsClassificationSchemeBaseType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsClassificationSchemeBaseType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Import")) == 0))
  {
	Mp7JrsClassificationSchemeBaseType_Import_CollectionPtr tmp = CreateClassificationSchemeBaseType_Import_CollectionType;
	this->SetImport(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsClassificationSchemeBaseType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetImport() != Mp7JrsNodePtr())
		result.Insert(GetImport());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsClassificationSchemeBaseType_ExtMethodImpl.h


