 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsConfusionCountType_ExtImplInclude.h



#include "Mp7JrsHeaderType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsIntegerMatrixType.h"
#include "Mp7JrsConfusionCountType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsConfusionCountType::IMp7JrsConfusionCountType()
{

// no includefile for extension defined 
// file Mp7JrsConfusionCountType_ExtPropInit.h

}

IMp7JrsConfusionCountType::~IMp7JrsConfusionCountType()
{
// no includefile for extension defined 
// file Mp7JrsConfusionCountType_ExtPropCleanup.h

}

Mp7JrsConfusionCountType::Mp7JrsConfusionCountType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsConfusionCountType::~Mp7JrsConfusionCountType()
{
	Cleanup();
}

void Mp7JrsConfusionCountType::Init()
{
	// Init base
	m_Base = CreateHeaderType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_numOfDimensions = 1; // Value

	// Init elements (element, union sequence choice all any)
	
	m_Insertion = Mp7JrsintegerVectorPtr(); // Collection
	m_Deletion = Mp7JrsintegerVectorPtr(); // Collection
	m_Substitution = Mp7JrsIntegerMatrixPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsConfusionCountType_ExtMyPropInit.h

}

void Mp7JrsConfusionCountType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsConfusionCountType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Insertion);
	// Mp7JrsFactory::DeleteObject(m_Deletion);
	// Mp7JrsFactory::DeleteObject(m_Substitution);
}

void Mp7JrsConfusionCountType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ConfusionCountTypePtr, since we
	// might need GetBase(), which isn't defined in IConfusionCountType
	const Mp7JrsPtr< Mp7JrsConfusionCountType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsHeaderPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsConfusionCountType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
		this->SetnumOfDimensions(tmp->GetnumOfDimensions());
	}
		// Mp7JrsFactory::DeleteObject(m_Insertion);
		this->SetInsertion(Mp7JrsFactory::CloneObject(tmp->GetInsertion()));
		// Mp7JrsFactory::DeleteObject(m_Deletion);
		this->SetDeletion(Mp7JrsFactory::CloneObject(tmp->GetDeletion()));
		// Mp7JrsFactory::DeleteObject(m_Substitution);
		this->SetSubstitution(Mp7JrsFactory::CloneObject(tmp->GetSubstitution()));
}

Mp7JrsNodePtr Mp7JrsConfusionCountType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsConfusionCountType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsHeaderType > Mp7JrsConfusionCountType::GetBase() const
{
	return m_Base;
}

unsigned Mp7JrsConfusionCountType::GetnumOfDimensions() const
{
	return m_numOfDimensions;
}

void Mp7JrsConfusionCountType::SetnumOfDimensions(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsConfusionCountType::SetnumOfDimensions().");
	}
	m_numOfDimensions = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsintegerVectorPtr Mp7JrsConfusionCountType::GetInsertion() const
{
		return m_Insertion;
}

Mp7JrsintegerVectorPtr Mp7JrsConfusionCountType::GetDeletion() const
{
		return m_Deletion;
}

Mp7JrsIntegerMatrixPtr Mp7JrsConfusionCountType::GetSubstitution() const
{
		return m_Substitution;
}

void Mp7JrsConfusionCountType::SetInsertion(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsConfusionCountType::SetInsertion().");
	}
	if (m_Insertion != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Insertion);
		m_Insertion = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Insertion) m_Insertion->SetParent(m_myself.getPointer());
		if(m_Insertion != Mp7JrsintegerVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Insertion->SetContentName(XMLString::transcode("Insertion"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsConfusionCountType::SetDeletion(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsConfusionCountType::SetDeletion().");
	}
	if (m_Deletion != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Deletion);
		m_Deletion = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Deletion) m_Deletion->SetParent(m_myself.getPointer());
		if(m_Deletion != Mp7JrsintegerVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Deletion->SetContentName(XMLString::transcode("Deletion"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsConfusionCountType::SetSubstitution(const Mp7JrsIntegerMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsConfusionCountType::SetSubstitution().");
	}
	if (m_Substitution != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Substitution);
		m_Substitution = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Substitution) m_Substitution->SetParent(m_myself.getPointer());
		if (m_Substitution && m_Substitution->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:IntegerMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Substitution->UseTypeAttribute = true;
		}
		if(m_Substitution != Mp7JrsIntegerMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Substitution->SetContentName(XMLString::transcode("Substitution"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsConfusionCountType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsConfusionCountType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsConfusionCountType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsConfusionCountType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsConfusionCountType::Invalidateid()
{
	GetBase()->Invalidateid();
}

Mp7JrsNodeEnum Mp7JrsConfusionCountType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 2 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; id,  T:http://www.w3.org/2001/XMLSchema:ID, C:String 
//- N; numOfDimensions,  T:http://www.w3.org/2001/XMLSchema:positiveInteger, C:Value 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("numOfDimensions")) == 0)
	{
		// numOfDimensions is simple attribute positiveInteger
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "unsigned");
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Insertion")) == 0)
	{
		// Insertion is simple element integerVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetInsertion()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:integerVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsintegerVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetInsertion(p, client);
					if((p = GetInsertion()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Deletion")) == 0)
	{
		// Deletion is simple element integerVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDeletion()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:integerVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsintegerVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDeletion(p, client);
					if((p = GetDeletion()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Substitution")) == 0)
	{
		// Substitution is simple element IntegerMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSubstitution()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:IntegerMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsIntegerMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSubstitution(p, client);
					if((p = GetSubstitution()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ConfusionCountType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ConfusionCountType");
	}
	return result;
}

XMLCh * Mp7JrsConfusionCountType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsConfusionCountType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsConfusionCountType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsConfusionCountType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ConfusionCountType"));
	// Attribute Serialization:
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_numOfDimensions);
		element->setAttributeNS(X(""), X("numOfDimensions"), tmp);
		XMLString::release(&tmp);
	}
	// Element serialization:
	if (m_Insertion != Mp7JrsintegerVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Insertion->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Deletion != Mp7JrsintegerVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Deletion->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_Substitution != Mp7JrsIntegerMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Substitution->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Substitution"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Substitution->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsConfusionCountType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("numOfDimensions")))
	{
		// deserialize value type
		this->SetnumOfDimensions(Mp7JrsConvert::TextToUnsignedInt(parent->getAttribute(X("numOfDimensions"))));
		* current = parent;
	}

	// Extensionbase is Mp7JrsHeaderType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialise Element ValCollectionType
		if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Insertion")) == 0))
		{
			// Deserialize factory type
			Mp7JrsintegerVectorPtr tmp = CreateintegerVector;
			tmp->Deserialize(doc, parent, current);
			this->SetInsertion(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialise Element ValCollectionType
		if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Deletion")) == 0))
		{
			// Deserialize factory type
			Mp7JrsintegerVectorPtr tmp = CreateintegerVector;
			tmp->Deserialize(doc, parent, current);
			this->SetDeletion(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Substitution"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Substitution")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateIntegerMatrixType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSubstitution(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsConfusionCountType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsConfusionCountType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Insertion")) == 0))
  {
	Mp7JrsintegerVectorPtr tmp = CreateintegerVector;
	this->SetInsertion(tmp);
	child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Deletion")) == 0))
  {
	Mp7JrsintegerVectorPtr tmp = CreateintegerVector;
	this->SetDeletion(tmp);
	child = tmp;
  }
  if (XMLString::compareString(elementname, X("Substitution")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateIntegerMatrixType;
	}
	this->SetSubstitution(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsConfusionCountType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetInsertion() != Mp7JrsNodePtr())
		result.Insert(GetInsertion());
	if (GetDeletion() != Mp7JrsNodePtr())
		result.Insert(GetDeletion());
	if (GetSubstitution() != Mp7JrsNodePtr())
		result.Insert(GetSubstitution());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsConfusionCountType_ExtMethodImpl.h


