 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsDecoderInitType_ExtImplInclude.h



#include "Mp7JrsDecoderInitType_SchemaReference_CollectionType.h"
#include "Mp7JrsAccessUnitType.h"
#include "Mp7JrsDecoderInitType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsSchemaReferenceType.h" // Element collection urn:mpeg:mpeg7:schema:2004:SchemaReference

#include <assert.h>
IMp7JrsDecoderInitType::IMp7JrsDecoderInitType()
{

// no includefile for extension defined 
// file Mp7JrsDecoderInitType_ExtPropInit.h

}

IMp7JrsDecoderInitType::~IMp7JrsDecoderInitType()
{
// no includefile for extension defined 
// file Mp7JrsDecoderInitType_ExtPropCleanup.h

}

Mp7JrsDecoderInitType::Mp7JrsDecoderInitType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsDecoderInitType::~Mp7JrsDecoderInitType()
{
	Cleanup();
}

void Mp7JrsDecoderInitType::Init()
{

	// Init attributes
	m_systemsProfileLevelIndication = 0.0; // Value

	// Init elements (element, union sequence choice all any)
	
	m_SchemaReference = Mp7JrsDecoderInitType_SchemaReference_CollectionPtr(); // Collection
	m_InitialDescription = Mp7JrsAccessUnitPtr(); // Class
	m_InitialDescription_Exist = false;


// no includefile for extension defined 
// file Mp7JrsDecoderInitType_ExtMyPropInit.h

}

void Mp7JrsDecoderInitType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsDecoderInitType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_SchemaReference);
	// Mp7JrsFactory::DeleteObject(m_InitialDescription);
}

void Mp7JrsDecoderInitType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use DecoderInitTypePtr, since we
	// might need GetBase(), which isn't defined in IDecoderInitType
	const Mp7JrsPtr< Mp7JrsDecoderInitType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
		this->SetsystemsProfileLevelIndication(tmp->GetsystemsProfileLevelIndication());
	}
		// Mp7JrsFactory::DeleteObject(m_SchemaReference);
		this->SetSchemaReference(Mp7JrsFactory::CloneObject(tmp->GetSchemaReference()));
	if (tmp->IsValidInitialDescription())
	{
		// Mp7JrsFactory::DeleteObject(m_InitialDescription);
		this->SetInitialDescription(Mp7JrsFactory::CloneObject(tmp->GetInitialDescription()));
	}
	else
	{
		InvalidateInitialDescription();
	}
}

Mp7JrsNodePtr Mp7JrsDecoderInitType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsDecoderInitType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


double Mp7JrsDecoderInitType::GetsystemsProfileLevelIndication() const
{
	return m_systemsProfileLevelIndication;
}

void Mp7JrsDecoderInitType::SetsystemsProfileLevelIndication(double item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDecoderInitType::SetsystemsProfileLevelIndication().");
	}
	m_systemsProfileLevelIndication = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsDecoderInitType_SchemaReference_CollectionPtr Mp7JrsDecoderInitType::GetSchemaReference() const
{
		return m_SchemaReference;
}

Mp7JrsAccessUnitPtr Mp7JrsDecoderInitType::GetInitialDescription() const
{
		return m_InitialDescription;
}

// Element is optional
bool Mp7JrsDecoderInitType::IsValidInitialDescription() const
{
	return m_InitialDescription_Exist;
}

void Mp7JrsDecoderInitType::SetSchemaReference(const Mp7JrsDecoderInitType_SchemaReference_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDecoderInitType::SetSchemaReference().");
	}
	if (m_SchemaReference != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SchemaReference);
		m_SchemaReference = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SchemaReference) m_SchemaReference->SetParent(m_myself.getPointer());
		if(m_SchemaReference != Mp7JrsDecoderInitType_SchemaReference_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SchemaReference->SetContentName(XMLString::transcode("SchemaReference"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDecoderInitType::SetInitialDescription(const Mp7JrsAccessUnitPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDecoderInitType::SetInitialDescription().");
	}
	if (m_InitialDescription != item || m_InitialDescription_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_InitialDescription);
		m_InitialDescription = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_InitialDescription) m_InitialDescription->SetParent(m_myself.getPointer());
		if (m_InitialDescription && m_InitialDescription->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:AccessUnitType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_InitialDescription->UseTypeAttribute = true;
		}
		m_InitialDescription_Exist = true;
		if(m_InitialDescription != Mp7JrsAccessUnitPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_InitialDescription->SetContentName(XMLString::transcode("InitialDescription"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDecoderInitType::InvalidateInitialDescription()
{
	m_InitialDescription_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsDecoderInitType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; systemsProfileLevelIndication,  T:http://www.w3.org/2001/XMLSchema:decimal, C:Value 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("systemsProfileLevelIndication")) == 0)
	{
		// systemsProfileLevelIndication is simple attribute decimal
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "double");
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SchemaReference")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:SchemaReference is item of type SchemaReferenceType
		// in element collection DecoderInitType_SchemaReference_CollectionType
		
		context->Found = true;
		Mp7JrsDecoderInitType_SchemaReference_CollectionPtr coll = GetSchemaReference();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateDecoderInitType_SchemaReference_CollectionType;
				SetSchemaReference(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SchemaReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSchemaReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("InitialDescription")) == 0)
	{
		// InitialDescription is simple element AccessUnitType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetInitialDescription()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:AccessUnitType")))) != empty)
			{
				// Is type allowed
				Mp7JrsAccessUnitPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetInitialDescription(p, client);
					if((p = GetInitialDescription()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for DecoderInitType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "DecoderInitType");
	}
	return result;
}

XMLCh * Mp7JrsDecoderInitType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsDecoderInitType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsDecoderInitType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("DecoderInitType"));
	// Attribute Serialization:
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_systemsProfileLevelIndication);
		element->setAttributeNS(X(""), X("systemsProfileLevelIndication"), tmp);
		XMLString::release(&tmp);
	}
	// Element serialization:
	if (m_SchemaReference != Mp7JrsDecoderInitType_SchemaReference_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_SchemaReference->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_InitialDescription != Mp7JrsAccessUnitPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_InitialDescription->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("InitialDescription"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_InitialDescription->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsDecoderInitType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("systemsProfileLevelIndication")))
	{
		// deserialize value type
		this->SetsystemsProfileLevelIndication(Mp7JrsConvert::TextToDouble(parent->getAttribute(X("systemsProfileLevelIndication"))));
		* current = parent;
	}

	parent = Mp7JrsUtil::GetNextChild(parent);
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("SchemaReference"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("SchemaReference")) == 0))
		{
			// Deserialize factory type
			Mp7JrsDecoderInitType_SchemaReference_CollectionPtr tmp = CreateDecoderInitType_SchemaReference_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetSchemaReference(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("InitialDescription"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("InitialDescription")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateAccessUnitType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetInitialDescription(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsDecoderInitType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsDecoderInitType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("SchemaReference")) == 0))
  {
	Mp7JrsDecoderInitType_SchemaReference_CollectionPtr tmp = CreateDecoderInitType_SchemaReference_CollectionType;
	this->SetSchemaReference(tmp);
	child = tmp;
  }
  if (XMLString::compareString(elementname, X("InitialDescription")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateAccessUnitType;
	}
	this->SetInitialDescription(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsDecoderInitType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSchemaReference() != Mp7JrsNodePtr())
		result.Insert(GetSchemaReference());
	if (GetInitialDescription() != Mp7JrsNodePtr())
		result.Insert(GetInitialDescription());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsDecoderInitType_ExtMethodImpl.h


