 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsDescriptorModelType_ExtImplInclude.h



#include "Mp7JrsModelType.h"
#include "Mp7JrsDType.h"
#include "Mp7JrsDescriptorModelType_Field_CollectionType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsDescriptorModelType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsxPathFieldType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Field

#include <assert.h>
IMp7JrsDescriptorModelType::IMp7JrsDescriptorModelType()
{

// no includefile for extension defined 
// file Mp7JrsDescriptorModelType_ExtPropInit.h

}

IMp7JrsDescriptorModelType::~IMp7JrsDescriptorModelType()
{
// no includefile for extension defined 
// file Mp7JrsDescriptorModelType_ExtPropCleanup.h

}

Mp7JrsDescriptorModelType::Mp7JrsDescriptorModelType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsDescriptorModelType::~Mp7JrsDescriptorModelType()
{
	Cleanup();
}

void Mp7JrsDescriptorModelType::Init()
{
	// Init base
	m_Base = CreateModelType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Descriptor = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_Field = Mp7JrsDescriptorModelType_Field_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsDescriptorModelType_ExtMyPropInit.h

}

void Mp7JrsDescriptorModelType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsDescriptorModelType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Descriptor);
	// Mp7JrsFactory::DeleteObject(m_Field);
}

void Mp7JrsDescriptorModelType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use DescriptorModelTypePtr, since we
	// might need GetBase(), which isn't defined in IDescriptorModelType
	const Mp7JrsPtr< Mp7JrsDescriptorModelType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsModelPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsDescriptorModelType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Descriptor);
		this->SetDescriptor(Mp7JrsFactory::CloneObject(tmp->GetDescriptor()));
		// Mp7JrsFactory::DeleteObject(m_Field);
		this->SetField(Mp7JrsFactory::CloneObject(tmp->GetField()));
}

Mp7JrsNodePtr Mp7JrsDescriptorModelType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsDescriptorModelType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsModelType > Mp7JrsDescriptorModelType::GetBase() const
{
	return m_Base;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsDescriptorModelType::GetDescriptor() const
{
		return m_Descriptor;
}

Mp7JrsDescriptorModelType_Field_CollectionPtr Mp7JrsDescriptorModelType::GetField() const
{
		return m_Field;
}

void Mp7JrsDescriptorModelType::SetDescriptor(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDescriptorModelType::SetDescriptor().");
	}
	if (m_Descriptor != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Descriptor);
		m_Descriptor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Descriptor) m_Descriptor->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_Descriptor) m_Descriptor->UseTypeAttribute = true;
		if(m_Descriptor != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Descriptor->SetContentName(XMLString::transcode("Descriptor"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDescriptorModelType::SetField(const Mp7JrsDescriptorModelType_Field_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDescriptorModelType::SetField().");
	}
	if (m_Field != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Field);
		m_Field = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Field) m_Field->SetParent(m_myself.getPointer());
		if(m_Field != Mp7JrsDescriptorModelType_Field_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Field->SetContentName(XMLString::transcode("Field"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrszeroToOnePtr Mp7JrsDescriptorModelType::Getconfidence() const
{
	return GetBase()->Getconfidence();
}

bool Mp7JrsDescriptorModelType::Existconfidence() const
{
	return GetBase()->Existconfidence();
}
void Mp7JrsDescriptorModelType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDescriptorModelType::Setconfidence().");
	}
	GetBase()->Setconfidence(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDescriptorModelType::Invalidateconfidence()
{
	GetBase()->Invalidateconfidence();
}
Mp7JrszeroToOnePtr Mp7JrsDescriptorModelType::Getreliability() const
{
	return GetBase()->Getreliability();
}

bool Mp7JrsDescriptorModelType::Existreliability() const
{
	return GetBase()->Existreliability();
}
void Mp7JrsDescriptorModelType::Setreliability(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDescriptorModelType::Setreliability().");
	}
	GetBase()->Setreliability(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDescriptorModelType::Invalidatereliability()
{
	GetBase()->Invalidatereliability();
}
XMLCh * Mp7JrsDescriptorModelType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsDescriptorModelType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsDescriptorModelType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDescriptorModelType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDescriptorModelType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsDescriptorModelType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsDescriptorModelType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsDescriptorModelType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDescriptorModelType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDescriptorModelType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsDescriptorModelType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsDescriptorModelType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsDescriptorModelType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDescriptorModelType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDescriptorModelType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsDescriptorModelType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsDescriptorModelType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsDescriptorModelType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDescriptorModelType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDescriptorModelType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsDescriptorModelType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsDescriptorModelType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsDescriptorModelType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDescriptorModelType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDescriptorModelType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsDescriptorModelType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsDescriptorModelType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDescriptorModelType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsDescriptorModelType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 7 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Descriptor")) == 0)
	{
		// Descriptor is simple abstract element DType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDescriptor()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsDPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDescriptor(p, client);
					if((p = GetDescriptor()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Field")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Field is item of type xPathFieldType
		// in element collection DescriptorModelType_Field_CollectionType
		
		context->Found = true;
		Mp7JrsDescriptorModelType_Field_CollectionPtr coll = GetField();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateDescriptorModelType_Field_CollectionType;
				SetField(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:xPathFieldType")))) != empty)
			{
				// Is type allowed
				Mp7JrsxPathFieldPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for DescriptorModelType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "DescriptorModelType");
	}
	return result;
}

XMLCh * Mp7JrsDescriptorModelType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsDescriptorModelType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsDescriptorModelType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsDescriptorModelType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("DescriptorModelType"));
	// Element serialization:
	// Class
	
	if (m_Descriptor != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Descriptor->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Descriptor"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_Descriptor->UseTypeAttribute = true;
		}
		m_Descriptor->Serialize(doc, element, &elem);
	}
	if (m_Field != Mp7JrsDescriptorModelType_Field_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Field->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsDescriptorModelType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsModelType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Descriptor"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Descriptor")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateDType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetDescriptor(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("Field"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("Field")) == 0))
		{
			// Deserialize factory type
			Mp7JrsDescriptorModelType_Field_CollectionPtr tmp = CreateDescriptorModelType_Field_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetField(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsDescriptorModelType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsDescriptorModelType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("Descriptor")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateDType;
	}
	this->SetDescriptor(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Field")) == 0))
  {
	Mp7JrsDescriptorModelType_Field_CollectionPtr tmp = CreateDescriptorModelType_Field_CollectionType;
	this->SetField(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsDescriptorModelType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetDescriptor() != Mp7JrsNodePtr())
		result.Insert(GetDescriptor());
	if (GetField() != Mp7JrsNodePtr())
		result.Insert(GetField());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsDescriptorModelType_ExtMethodImpl.h


