 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsExtendedLanguageType_ExtImplInclude.h



#include "Mp7JrsExtendedLanguageType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsExtendedLanguageType::IMp7JrsExtendedLanguageType()
{

// no includefile for extension defined 
// file Mp7JrsExtendedLanguageType_ExtPropInit.h

}

IMp7JrsExtendedLanguageType::~IMp7JrsExtendedLanguageType()
{
// no includefile for extension defined 
// file Mp7JrsExtendedLanguageType_ExtPropCleanup.h

}

Mp7JrsExtendedLanguageType::Mp7JrsExtendedLanguageType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsExtendedLanguageType::~Mp7JrsExtendedLanguageType()
{
	Cleanup();
}

void Mp7JrsExtendedLanguageType::Init()
{
	// Init base
	m_Base = XMLString::transcode("");

	// Init attributes
	m_type = Mp7JrsExtendedLanguageType_type_LocalType::UninitializedEnumeration;
	m_type_Default = Mp7JrsExtendedLanguageType_type_LocalType::original; // Default enumeration
	m_type_Exist = false;
	m_supplemental = false; // Value
	m_supplemental_Default = false; // Default value
	m_supplemental_Exist = false;



// no includefile for extension defined 
// file Mp7JrsExtendedLanguageType_ExtMyPropInit.h

}

void Mp7JrsExtendedLanguageType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsExtendedLanguageType_ExtMyPropCleanup.h


	XMLString::release(&m_Base);
}

void Mp7JrsExtendedLanguageType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ExtendedLanguageTypePtr, since we
	// might need GetBase(), which isn't defined in IExtendedLanguageType
	const Mp7JrsPtr< Mp7JrsExtendedLanguageType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	XMLString::release(&m_Base);
	m_Base = XMLString::replicate(tmp->GetContent());
	{
	if (tmp->Existtype())
	{
		this->Settype(tmp->Gettype());
	}
	else
	{
		Invalidatetype();
	}
	}
	{
	if (tmp->Existsupplemental())
	{
		this->Setsupplemental(tmp->Getsupplemental());
	}
	else
	{
		Invalidatesupplemental();
	}
	}
}

Mp7JrsNodePtr Mp7JrsExtendedLanguageType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsExtendedLanguageType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


void Mp7JrsExtendedLanguageType::SetContent(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsExtendedLanguageType::SetContent().");
	}
	if (m_Base != item)
	{
		XMLString::release(&m_Base);
		m_Base = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsExtendedLanguageType::GetContent() const
{
	return m_Base;
}
Mp7JrsExtendedLanguageType_type_LocalType::Enumeration Mp7JrsExtendedLanguageType::Gettype() const
{
	if (this->Existtype()) {
		return m_type;
	} else {
		return m_type_Default;
	}
}

bool Mp7JrsExtendedLanguageType::Existtype() const
{
	return m_type_Exist;
}
void Mp7JrsExtendedLanguageType::Settype(Mp7JrsExtendedLanguageType_type_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsExtendedLanguageType::Settype().");
	}
	m_type = item;
	m_type_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsExtendedLanguageType::Invalidatetype()
{
	m_type_Exist = false;
}
bool Mp7JrsExtendedLanguageType::Getsupplemental() const
{
	if (this->Existsupplemental()) {
		return m_supplemental;
	} else {
		return m_supplemental_Default;
	}
}

bool Mp7JrsExtendedLanguageType::Existsupplemental() const
{
	return m_supplemental_Exist;
}
void Mp7JrsExtendedLanguageType::Setsupplemental(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsExtendedLanguageType::Setsupplemental().");
	}
	m_supplemental = item;
	m_supplemental_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsExtendedLanguageType::Invalidatesupplemental()
{
	m_supplemental_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsExtendedLanguageType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  2, 2 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 2
//- N; type,  T:urn:mpeg:mpeg7:schema:2004:ExtendedLanguageType_type_LocalType, C:Enumeration 
//- N; supplemental,  T:http://www.w3.org/2001/XMLSchema:boolean, C:Value 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("type")) == 0)
	{
		// type is simple attribute ExtendedLanguageType_type_LocalType
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "Mp7JrsExtendedLanguageType_type_LocalType::Enumeration");
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("supplemental")) == 0)
	{
		// supplemental is simple attribute boolean
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "bool");
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ExtendedLanguageType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ExtendedLanguageType");
	}
	return result;
}

XMLCh * Mp7JrsExtendedLanguageType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsExtendedLanguageType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsExtendedLanguageType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsExtendedLanguageType with workaround via Deserialise()
	return Mp7JrsXPathParseContext::ContentFromString(this, txt);
}
XMLCh* Mp7JrsExtendedLanguageType::ContentToString() const
{
	// Mp7JrsExtendedLanguageType with workaround via Serialise()
	return Mp7JrsXPathParseContext::ContentToString(this);
}

void Mp7JrsExtendedLanguageType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	element->appendChild(doc->createTextNode(m_Base));

	assert(element);
// no includefile for extension defined 
// file Mp7JrsExtendedLanguageType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ExtendedLanguageType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_type_Exist)
	{
	// Enumeration
	if(m_type != Mp7JrsExtendedLanguageType_type_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsExtendedLanguageType_type_LocalType::ToText(m_type);
		element->setAttributeNS(X(""), X("type"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Attribute Serialization:
		

	// Optional attriute
	if(m_supplemental_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_supplemental);
		element->setAttributeNS(X(""), X("supplemental"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsExtendedLanguageType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("type")))
	{
		this->Settype(Mp7JrsExtendedLanguageType_type_LocalType::Parse(parent->getAttribute(X("type"))));
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("supplemental")))
	{
		// deserialize value type
		this->Setsupplemental(Mp7JrsConvert::TextToBool(parent->getAttribute(X("supplemental"))));
		* current = parent;
	}

	XMLCh * tmp = Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent));
	if(tmp != NULL)
	{
		XMLString::release(&m_Base);
		m_Base = tmp;
		found = true;
	}
// no includefile for extension defined 
// file Mp7JrsExtendedLanguageType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsExtendedLanguageType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsExtendedLanguageType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsExtendedLanguageType_ExtMethodImpl.h


