 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsFocusOfExpansionType_ExtImplInclude.h



#include "Mp7JrsFocusOfExpansionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsFocusOfExpansionType::IMp7JrsFocusOfExpansionType()
{

// no includefile for extension defined 
// file Mp7JrsFocusOfExpansionType_ExtPropInit.h

}

IMp7JrsFocusOfExpansionType::~IMp7JrsFocusOfExpansionType()
{
// no includefile for extension defined 
// file Mp7JrsFocusOfExpansionType_ExtPropCleanup.h

}

Mp7JrsFocusOfExpansionType::Mp7JrsFocusOfExpansionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsFocusOfExpansionType::~Mp7JrsFocusOfExpansionType()
{
	Cleanup();
}

void Mp7JrsFocusOfExpansionType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_HorizontalPosition = (0.0); // Value

	m_VerticalPosition = (0.0); // Value



// no includefile for extension defined 
// file Mp7JrsFocusOfExpansionType_ExtMyPropInit.h

}

void Mp7JrsFocusOfExpansionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsFocusOfExpansionType_ExtMyPropCleanup.h


}

void Mp7JrsFocusOfExpansionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use FocusOfExpansionTypePtr, since we
	// might need GetBase(), which isn't defined in IFocusOfExpansionType
	const Mp7JrsPtr< Mp7JrsFocusOfExpansionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		this->SetHorizontalPosition(tmp->GetHorizontalPosition());
		this->SetVerticalPosition(tmp->GetVerticalPosition());
}

Mp7JrsNodePtr Mp7JrsFocusOfExpansionType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsFocusOfExpansionType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


float Mp7JrsFocusOfExpansionType::GetHorizontalPosition() const
{
		return m_HorizontalPosition;
}

float Mp7JrsFocusOfExpansionType::GetVerticalPosition() const
{
		return m_VerticalPosition;
}

void Mp7JrsFocusOfExpansionType::SetHorizontalPosition(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFocusOfExpansionType::SetHorizontalPosition().");
	}
	if (m_HorizontalPosition != item)
	{
		m_HorizontalPosition = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFocusOfExpansionType::SetVerticalPosition(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFocusOfExpansionType::SetVerticalPosition().");
	}
	if (m_VerticalPosition != item)
	{
		m_VerticalPosition = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsFocusOfExpansionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	// Only rudimentary expressions allowed for this type with no child elements
	return result;
}	

XMLCh * Mp7JrsFocusOfExpansionType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsFocusOfExpansionType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsFocusOfExpansionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("FocusOfExpansionType"));
	// Element serialization:
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_HorizontalPosition);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("HorizontalPosition"), false);
		XMLString::release(&tmp);
	}
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_VerticalPosition);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("VerticalPosition"), false);
		XMLString::release(&tmp);
	}

}

bool Mp7JrsFocusOfExpansionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
	// Deserialize value element
	if(Mp7JrsUtil::HasNodeName(parent, X("HorizontalPosition"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		XMLCh * tmp = Mp7JrsUtil::GetElementText(parent);
		this->SetHorizontalPosition(Mp7JrsConvert::TextToFloat(tmp));
		XMLString::release(&tmp);
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	// Deserialize value element
	if(Mp7JrsUtil::HasNodeName(parent, X("VerticalPosition"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		XMLCh * tmp = Mp7JrsUtil::GetElementText(parent);
		this->SetVerticalPosition(Mp7JrsConvert::TextToFloat(tmp));
		XMLString::release(&tmp);
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsFocusOfExpansionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsFocusOfExpansionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsFocusOfExpansionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsFocusOfExpansionType_ExtMethodImpl.h


