 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsFragmentUpdateContextType_ExtImplInclude.h



#include "Mp7JrsFragmentUpdateContextType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsFragmentUpdateContextType::IMp7JrsFragmentUpdateContextType()
{

// no includefile for extension defined 
// file Mp7JrsFragmentUpdateContextType_ExtPropInit.h

}

IMp7JrsFragmentUpdateContextType::~IMp7JrsFragmentUpdateContextType()
{
// no includefile for extension defined 
// file Mp7JrsFragmentUpdateContextType_ExtPropCleanup.h

}

Mp7JrsFragmentUpdateContextType::Mp7JrsFragmentUpdateContextType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsFragmentUpdateContextType::~Mp7JrsFragmentUpdateContextType()
{
	Cleanup();
}

void Mp7JrsFragmentUpdateContextType::Init()
{
	// Init base
	m_Base = CreateFragmentUpdateContextTypeBase;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_position = Mp7JrsFragmentUpdateContextType_position_LocalType::UninitializedEnumeration;
	m_position_Default = Mp7JrsFragmentUpdateContextType_position_LocalType::lastChild; // Default enumeration
	m_position_Exist = false;



// no includefile for extension defined 
// file Mp7JrsFragmentUpdateContextType_ExtMyPropInit.h

}

void Mp7JrsFragmentUpdateContextType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsFragmentUpdateContextType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
}

void Mp7JrsFragmentUpdateContextType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use FragmentUpdateContextTypePtr, since we
	// might need GetBase(), which isn't defined in IFragmentUpdateContextType
	const Mp7JrsPtr< Mp7JrsFragmentUpdateContextType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsFragmentUpdateContextTypeBasePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsFragmentUpdateContextType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->Existposition())
	{
		this->Setposition(tmp->Getposition());
	}
	else
	{
		Invalidateposition();
	}
	}
}

Mp7JrsNodePtr Mp7JrsFragmentUpdateContextType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsFragmentUpdateContextType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsFragmentUpdateContextTypeBase > Mp7JrsFragmentUpdateContextType::GetBase() const
{
	return m_Base;
}

Mp7JrsFragmentUpdateContextType_position_LocalType::Enumeration Mp7JrsFragmentUpdateContextType::Getposition() const
{
	if (this->Existposition()) {
		return m_position;
	} else {
		return m_position_Default;
	}
}

bool Mp7JrsFragmentUpdateContextType::Existposition() const
{
	return m_position_Exist;
}
void Mp7JrsFragmentUpdateContextType::Setposition(Mp7JrsFragmentUpdateContextType_position_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFragmentUpdateContextType::Setposition().");
	}
	m_position = item;
	m_position_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFragmentUpdateContextType::Invalidateposition()
{
	m_position_Exist = false;
}
XMLCh * Mp7JrsFragmentUpdateContextType::GetContent() const
{
	return GetBase()->GetContent();
}

void Mp7JrsFragmentUpdateContextType::SetContent(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFragmentUpdateContextType::SetContent().");
	}
	GetBase()->SetContent(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsFragmentUpdateContextType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; position,  T:urn:mpeg:mpeg7:schema:2004:FragmentUpdateContextType_position_LocalType, C:Enumeration 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("position")) == 0)
	{
		// position is simple attribute FragmentUpdateContextType_position_LocalType
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "Mp7JrsFragmentUpdateContextType_position_LocalType::Enumeration");
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for FragmentUpdateContextType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "FragmentUpdateContextType");
	}
	return result;
}

XMLCh * Mp7JrsFragmentUpdateContextType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsFragmentUpdateContextType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsFragmentUpdateContextType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsFragmentUpdateContextType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsFragmentUpdateContextType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	m_Base->Serialize(doc, element, newElem);

	assert(element);
// no includefile for extension defined 
// file Mp7JrsFragmentUpdateContextType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("FragmentUpdateContextType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_position_Exist)
	{
	// Enumeration
	if(m_position != Mp7JrsFragmentUpdateContextType_position_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsFragmentUpdateContextType_position_LocalType::ToText(m_position);
		element->setAttributeNS(X(""), X("position"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsFragmentUpdateContextType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("position")))
	{
		this->Setposition(Mp7JrsFragmentUpdateContextType_position_LocalType::Parse(parent->getAttribute(X("position"))));
		* current = parent;
	}

  // Extensionbase is Mp7JrsFragmentUpdateContextTypeBase
  // Deserialize cce factory type
  if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
  }
// no includefile for extension defined 
// file Mp7JrsFragmentUpdateContextType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsFragmentUpdateContextType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsFragmentUpdateContextType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsFragmentUpdateContextType_ExtMethodImpl.h


