 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsFragmentUpdateUnitType_ExtImplInclude.h



#include "Mp7JrsFragmentUpdateCommandType.h"
#include "Mp7JrsFragmentUpdateContextType.h"
#include "Mp7JrsFragmentUpdatePayloadType.h"
#include "Mp7JrsFragmentUpdateUnitType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsFragmentUpdateUnitType::IMp7JrsFragmentUpdateUnitType()
{

// no includefile for extension defined 
// file Mp7JrsFragmentUpdateUnitType_ExtPropInit.h

}

IMp7JrsFragmentUpdateUnitType::~IMp7JrsFragmentUpdateUnitType()
{
// no includefile for extension defined 
// file Mp7JrsFragmentUpdateUnitType_ExtPropCleanup.h

}

Mp7JrsFragmentUpdateUnitType::Mp7JrsFragmentUpdateUnitType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsFragmentUpdateUnitType::~Mp7JrsFragmentUpdateUnitType()
{
	Cleanup();
}

void Mp7JrsFragmentUpdateUnitType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_FUCommand = Mp7JrsFragmentUpdateCommandPtr(); // Class
	m_FUContext = Mp7JrsFragmentUpdateContextPtr(); // Class
	m_FUContext_Exist = false;
	m_FUPayload = Mp7JrsFragmentUpdatePayloadPtr(); // Class
	m_FUPayload_Exist = false;


// no includefile for extension defined 
// file Mp7JrsFragmentUpdateUnitType_ExtMyPropInit.h

}

void Mp7JrsFragmentUpdateUnitType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsFragmentUpdateUnitType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_FUCommand);
	// Mp7JrsFactory::DeleteObject(m_FUContext);
	// Mp7JrsFactory::DeleteObject(m_FUPayload);
}

void Mp7JrsFragmentUpdateUnitType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use FragmentUpdateUnitTypePtr, since we
	// might need GetBase(), which isn't defined in IFragmentUpdateUnitType
	const Mp7JrsPtr< Mp7JrsFragmentUpdateUnitType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_FUCommand);
		this->SetFUCommand(Mp7JrsFactory::CloneObject(tmp->GetFUCommand()));
	if (tmp->IsValidFUContext())
	{
		// Mp7JrsFactory::DeleteObject(m_FUContext);
		this->SetFUContext(Mp7JrsFactory::CloneObject(tmp->GetFUContext()));
	}
	else
	{
		InvalidateFUContext();
	}
	if (tmp->IsValidFUPayload())
	{
		// Mp7JrsFactory::DeleteObject(m_FUPayload);
		this->SetFUPayload(Mp7JrsFactory::CloneObject(tmp->GetFUPayload()));
	}
	else
	{
		InvalidateFUPayload();
	}
}

Mp7JrsNodePtr Mp7JrsFragmentUpdateUnitType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsFragmentUpdateUnitType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsFragmentUpdateCommandPtr Mp7JrsFragmentUpdateUnitType::GetFUCommand() const
{
		return m_FUCommand;
}

Mp7JrsFragmentUpdateContextPtr Mp7JrsFragmentUpdateUnitType::GetFUContext() const
{
		return m_FUContext;
}

// Element is optional
bool Mp7JrsFragmentUpdateUnitType::IsValidFUContext() const
{
	return m_FUContext_Exist;
}

Mp7JrsFragmentUpdatePayloadPtr Mp7JrsFragmentUpdateUnitType::GetFUPayload() const
{
		return m_FUPayload;
}

// Element is optional
bool Mp7JrsFragmentUpdateUnitType::IsValidFUPayload() const
{
	return m_FUPayload_Exist;
}

void Mp7JrsFragmentUpdateUnitType::SetFUCommand(const Mp7JrsFragmentUpdateCommandPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFragmentUpdateUnitType::SetFUCommand().");
	}
	if (m_FUCommand != item)
	{
		// Mp7JrsFactory::DeleteObject(m_FUCommand);
		m_FUCommand = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FUCommand) m_FUCommand->SetParent(m_myself.getPointer());
		if (m_FUCommand && m_FUCommand->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FragmentUpdateCommandType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FUCommand->UseTypeAttribute = true;
		}
		if(m_FUCommand != Mp7JrsFragmentUpdateCommandPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FUCommand->SetContentName(XMLString::transcode("FUCommand"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFragmentUpdateUnitType::SetFUContext(const Mp7JrsFragmentUpdateContextPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFragmentUpdateUnitType::SetFUContext().");
	}
	if (m_FUContext != item || m_FUContext_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_FUContext);
		m_FUContext = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FUContext) m_FUContext->SetParent(m_myself.getPointer());
		if (m_FUContext && m_FUContext->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FragmentUpdateContextType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FUContext->UseTypeAttribute = true;
		}
		m_FUContext_Exist = true;
		if(m_FUContext != Mp7JrsFragmentUpdateContextPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FUContext->SetContentName(XMLString::transcode("FUContext"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFragmentUpdateUnitType::InvalidateFUContext()
{
	m_FUContext_Exist = false;
}
void Mp7JrsFragmentUpdateUnitType::SetFUPayload(const Mp7JrsFragmentUpdatePayloadPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFragmentUpdateUnitType::SetFUPayload().");
	}
	if (m_FUPayload != item || m_FUPayload_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_FUPayload);
		m_FUPayload = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FUPayload) m_FUPayload->SetParent(m_myself.getPointer());
		if (m_FUPayload && m_FUPayload->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FragmentUpdatePayloadType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FUPayload->UseTypeAttribute = true;
		}
		m_FUPayload_Exist = true;
		if(m_FUPayload != Mp7JrsFragmentUpdatePayloadPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FUPayload->SetContentName(XMLString::transcode("FUPayload"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFragmentUpdateUnitType::InvalidateFUPayload()
{
	m_FUPayload_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsFragmentUpdateUnitType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("FUCommand")) == 0)
	{
		// FUCommand is simple element FragmentUpdateCommandType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFUCommand()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FragmentUpdateCommandType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFragmentUpdateCommandPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFUCommand(p, client);
					if((p = GetFUCommand()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("FUContext")) == 0)
	{
		// FUContext is simple element FragmentUpdateContextType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFUContext()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FragmentUpdateContextType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFragmentUpdateContextPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFUContext(p, client);
					if((p = GetFUContext()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("FUPayload")) == 0)
	{
		// FUPayload is simple element FragmentUpdatePayloadType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFUPayload()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FragmentUpdatePayloadType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFragmentUpdatePayloadPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFUPayload(p, client);
					if((p = GetFUPayload()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for FragmentUpdateUnitType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "FragmentUpdateUnitType");
	}
	return result;
}

XMLCh * Mp7JrsFragmentUpdateUnitType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsFragmentUpdateUnitType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsFragmentUpdateUnitType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("FragmentUpdateUnitType"));
	// Element serialization:
	// Class
	
	if (m_FUCommand != Mp7JrsFragmentUpdateCommandPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FUCommand->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FUCommand"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FUCommand->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_FUContext != Mp7JrsFragmentUpdateContextPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FUContext->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FUContext"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FUContext->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_FUPayload != Mp7JrsFragmentUpdatePayloadPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FUPayload->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FUPayload"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FUPayload->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsFragmentUpdateUnitType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FUCommand"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FUCommand")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFragmentUpdateCommandType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetFUCommand(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FUContext"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FUContext")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFragmentUpdateContextType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetFUContext(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FUPayload"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FUPayload")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFragmentUpdatePayloadType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetFUPayload(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsFragmentUpdateUnitType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsFragmentUpdateUnitType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("FUCommand")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFragmentUpdateCommandType;
	}
	this->SetFUCommand(child);
  }
  if (XMLString::compareString(elementname, X("FUContext")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFragmentUpdateContextType;
	}
	this->SetFUContext(child);
  }
  if (XMLString::compareString(elementname, X("FUPayload")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFragmentUpdatePayloadType;
	}
	this->SetFUPayload(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsFragmentUpdateUnitType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetFUCommand() != Mp7JrsNodePtr())
		result.Insert(GetFUCommand());
	if (GetFUContext() != Mp7JrsNodePtr())
		result.Insert(GetFUContext());
	if (GetFUPayload() != Mp7JrsNodePtr())
		result.Insert(GetFUPayload());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsFragmentUpdateUnitType_ExtMethodImpl.h


