 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsFrequencyTreeType_ExtImplInclude.h



#include "Mp7JrsViewDecompositionType.h"
#include "Mp7JrsFrequencyViewType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsFrequencyTreeType_CollectionType.h"
#include "Mp7JrsSignalType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsFrequencyTreeType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsFrequencyTreeType_LocalType.h" // Choice collection Child
#include "Mp7JrsFrequencyTreeType.h" // Choice collection element Child

#include <assert.h>
IMp7JrsFrequencyTreeType::IMp7JrsFrequencyTreeType()
{

// no includefile for extension defined 
// file Mp7JrsFrequencyTreeType_ExtPropInit.h

}

IMp7JrsFrequencyTreeType::~IMp7JrsFrequencyTreeType()
{
// no includefile for extension defined 
// file Mp7JrsFrequencyTreeType_ExtPropCleanup.h

}

Mp7JrsFrequencyTreeType::Mp7JrsFrequencyTreeType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsFrequencyTreeType::~Mp7JrsFrequencyTreeType()
{
	Cleanup();
}

void Mp7JrsFrequencyTreeType::Init()
{
	// Init base
	m_Base = CreateViewDecompositionType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_branching = 0; // Value

	// Init elements (element, union sequence choice all any)
	
	m_View = Mp7JrsFrequencyViewPtr(); // Class
	m_ViewRef = Mp7JrsReferencePtr(); // Class
	m_FrequencyTreeType_LocalType = Mp7JrsFrequencyTreeType_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsFrequencyTreeType_ExtMyPropInit.h

}

void Mp7JrsFrequencyTreeType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsFrequencyTreeType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_View);
	// Mp7JrsFactory::DeleteObject(m_ViewRef);
	// Mp7JrsFactory::DeleteObject(m_FrequencyTreeType_LocalType);
}

void Mp7JrsFrequencyTreeType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use FrequencyTreeTypePtr, since we
	// might need GetBase(), which isn't defined in IFrequencyTreeType
	const Mp7JrsPtr< Mp7JrsFrequencyTreeType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsViewDecompositionPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsFrequencyTreeType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
		this->Setbranching(tmp->Getbranching());
	}
		// Mp7JrsFactory::DeleteObject(m_View);
		this->SetView(Mp7JrsFactory::CloneObject(tmp->GetView()));
		// Mp7JrsFactory::DeleteObject(m_ViewRef);
		this->SetViewRef(Mp7JrsFactory::CloneObject(tmp->GetViewRef()));
		// Mp7JrsFactory::DeleteObject(m_FrequencyTreeType_LocalType);
		this->SetFrequencyTreeType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetFrequencyTreeType_LocalType()));
}

Mp7JrsNodePtr Mp7JrsFrequencyTreeType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsFrequencyTreeType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsViewDecompositionType > Mp7JrsFrequencyTreeType::GetBase() const
{
	return m_Base;
}

unsigned Mp7JrsFrequencyTreeType::Getbranching() const
{
	return m_branching;
}

void Mp7JrsFrequencyTreeType::Setbranching(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::Setbranching().");
	}
	m_branching = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsFrequencyViewPtr Mp7JrsFrequencyTreeType::GetView() const
{
		return m_View;
}

Mp7JrsReferencePtr Mp7JrsFrequencyTreeType::GetViewRef() const
{
		return m_ViewRef;
}

Mp7JrsFrequencyTreeType_CollectionPtr Mp7JrsFrequencyTreeType::GetFrequencyTreeType_LocalType() const
{
		return m_FrequencyTreeType_LocalType;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsFrequencyTreeType::SetView(const Mp7JrsFrequencyViewPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::SetView().");
	}
	if (m_View != item)
	{
		// Mp7JrsFactory::DeleteObject(m_View);
		m_View = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_View) m_View->SetParent(m_myself.getPointer());
		if (m_View && m_View->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FrequencyViewType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_View->UseTypeAttribute = true;
		}
		if(m_View != Mp7JrsFrequencyViewPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_View->SetContentName(XMLString::transcode("View"));
		}
	// Mp7JrsFactory::DeleteObject(m_ViewRef);
	m_ViewRef = Mp7JrsReferencePtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsFrequencyTreeType::SetViewRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::SetViewRef().");
	}
	if (m_ViewRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ViewRef);
		m_ViewRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ViewRef) m_ViewRef->SetParent(m_myself.getPointer());
		if (m_ViewRef && m_ViewRef->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ViewRef->UseTypeAttribute = true;
		}
		if(m_ViewRef != Mp7JrsReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ViewRef->SetContentName(XMLString::transcode("ViewRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_View);
	m_View = Mp7JrsFrequencyViewPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyTreeType::SetFrequencyTreeType_LocalType(const Mp7JrsFrequencyTreeType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::SetFrequencyTreeType_LocalType().");
	}
	if (m_FrequencyTreeType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_FrequencyTreeType_LocalType);
		m_FrequencyTreeType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FrequencyTreeType_LocalType) m_FrequencyTreeType_LocalType->SetParent(m_myself.getPointer());
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsFrequencyTreeType::Getcomplete() const
{
	return GetBase()->Getcomplete();
}

bool Mp7JrsFrequencyTreeType::Existcomplete() const
{
	return GetBase()->Existcomplete();
}
void Mp7JrsFrequencyTreeType::Setcomplete(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::Setcomplete().");
	}
	GetBase()->Setcomplete(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyTreeType::Invalidatecomplete()
{
	GetBase()->Invalidatecomplete();
}
bool Mp7JrsFrequencyTreeType::GetnonRedundant() const
{
	return GetBase()->GetnonRedundant();
}

bool Mp7JrsFrequencyTreeType::ExistnonRedundant() const
{
	return GetBase()->ExistnonRedundant();
}
void Mp7JrsFrequencyTreeType::SetnonRedundant(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::SetnonRedundant().");
	}
	GetBase()->SetnonRedundant(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyTreeType::InvalidatenonRedundant()
{
	GetBase()->InvalidatenonRedundant();
}
Mp7JrsSignalPtr Mp7JrsFrequencyTreeType::GetSource() const
{
	return GetBase()->GetSource();
}

// Element is optional
bool Mp7JrsFrequencyTreeType::IsValidSource() const
{
	return GetBase()->IsValidSource();
}

void Mp7JrsFrequencyTreeType::SetSource(const Mp7JrsSignalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::SetSource().");
	}
	GetBase()->SetSource(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyTreeType::InvalidateSource()
{
	GetBase()->InvalidateSource();
}
XMLCh * Mp7JrsFrequencyTreeType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsFrequencyTreeType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsFrequencyTreeType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyTreeType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsFrequencyTreeType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsFrequencyTreeType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsFrequencyTreeType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyTreeType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsFrequencyTreeType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsFrequencyTreeType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsFrequencyTreeType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyTreeType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsFrequencyTreeType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsFrequencyTreeType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsFrequencyTreeType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyTreeType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsFrequencyTreeType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsFrequencyTreeType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsFrequencyTreeType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsFrequencyTreeType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsFrequencyTreeType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsFrequencyTreeType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsFrequencyTreeType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsFrequencyTreeType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 8 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; id,  T:http://www.w3.org/2001/XMLSchema:ID, C:String 
//- N; timeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; timeUnit,  T:urn:mpeg:mpeg7:schema:2004:durationType, C:Pattern 
//- N; mediaTimeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; mediaTimeUnit,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
//- N; complete,  T:http://www.w3.org/2001/XMLSchema:boolean, C:Value 
//- N; nonRedundant,  T:http://www.w3.org/2001/XMLSchema:boolean, C:Value 
//- N; branching,  T:http://www.w3.org/2001/XMLSchema:nonNegativeInteger, C:Value 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("branching")) == 0)
	{
		// branching is simple attribute nonNegativeInteger
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "unsigned");
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("View")) == 0)
	{
		// View is simple element FrequencyViewType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetView()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FrequencyViewType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFrequencyViewPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetView(p, client);
					if((p = GetView()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ViewRef")) == 0)
	{
		// ViewRef is simple element ReferenceType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetViewRef()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetViewRef(p, client);
					if((p = GetViewRef()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Child")) == 0)
	{
		// Child is contained in itemtype FrequencyTreeType_LocalType
		// in choice collection FrequencyTreeType_CollectionType
		
		context->Found = true;
		Mp7JrsFrequencyTreeType_CollectionPtr coll = GetFrequencyTreeType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateFrequencyTreeType_CollectionType;
				SetFrequencyTreeType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsFrequencyTreeType_LocalPtr)coll->elementAt(i))->GetChild()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsFrequencyTreeType_LocalPtr item = CreateFrequencyTreeType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FrequencyTreeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFrequencyTreePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsFrequencyTreeType_LocalPtr)coll->elementAt(i))->SetChild(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsFrequencyTreeType_LocalPtr)coll->elementAt(i))->GetChild()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ChildRef")) == 0)
	{
		// ChildRef is contained in itemtype FrequencyTreeType_LocalType
		// in choice collection FrequencyTreeType_CollectionType
		
		context->Found = true;
		Mp7JrsFrequencyTreeType_CollectionPtr coll = GetFrequencyTreeType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateFrequencyTreeType_CollectionType;
				SetFrequencyTreeType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsFrequencyTreeType_LocalPtr)coll->elementAt(i))->GetChildRef()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsFrequencyTreeType_LocalPtr item = CreateFrequencyTreeType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsFrequencyTreeType_LocalPtr)coll->elementAt(i))->SetChildRef(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsFrequencyTreeType_LocalPtr)coll->elementAt(i))->GetChildRef()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for FrequencyTreeType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "FrequencyTreeType");
	}
	return result;
}

XMLCh * Mp7JrsFrequencyTreeType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsFrequencyTreeType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsFrequencyTreeType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsFrequencyTreeType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("FrequencyTreeType"));
	// Attribute Serialization:
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_branching);
		element->setAttributeNS(X(""), X("branching"), tmp);
		XMLString::release(&tmp);
	}
	// Element serialization:
	// Class
	
	if (m_View != Mp7JrsFrequencyViewPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_View->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("View"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_View->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ViewRef != Mp7JrsReferencePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ViewRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ViewRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ViewRef->Serialize(doc, element, &elem);
	}
	if (m_FrequencyTreeType_LocalType != Mp7JrsFrequencyTreeType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_FrequencyTreeType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsFrequencyTreeType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("branching")))
	{
		// deserialize value type
		this->Setbranching(Mp7JrsConvert::TextToUnsignedInt(parent->getAttribute(X("branching"))));
		* current = parent;
	}

	// Extensionbase is Mp7JrsViewDecompositionType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
	{
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("View"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("View")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFrequencyViewType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetView(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ViewRef"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ViewRef")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateReferenceType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetViewRef(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	} while(false);
		// Deserialise Choice Collection
		if((parent != NULL) && (
			// urn:mpeg:mpeg7:schema:2004:Child
			Mp7JrsUtil::HasNodeName(parent, X("Child"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD			(XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Child")) == 0)
				||
			// urn:mpeg:mpeg7:schema:2004:ChildRef
			Mp7JrsUtil::HasNodeName(parent, X("ChildRef"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD			(XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:ChildRef")) == 0)
				))
		{
			// Deserialize factory type
			Mp7JrsFrequencyTreeType_CollectionPtr tmp = CreateFrequencyTreeType_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetFrequencyTreeType_LocalType(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsFrequencyTreeType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsFrequencyTreeType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("View")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFrequencyViewType;
	}
	this->SetView(child);
  }
  if (XMLString::compareString(elementname, X("ViewRef")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateReferenceType;
	}
	this->SetViewRef(child);
  }
  // Choice Collection
  if(
// TODO better use this	
//  (Mp7JrsUtil::HasNodeName(parent, X("Child"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
	(XMLString::compareString(elementname, X("Child")) == 0)
	||
// TODO better use this	
//  (Mp7JrsUtil::HasNodeName(parent, X("ChildRef"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
	(XMLString::compareString(elementname, X("ChildRef")) == 0)
	)
  {
	Mp7JrsFrequencyTreeType_CollectionPtr tmp = CreateFrequencyTreeType_CollectionType;
	this->SetFrequencyTreeType_LocalType(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsFrequencyTreeType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetFrequencyTreeType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetFrequencyTreeType_LocalType());
	if (GetSource() != Mp7JrsNodePtr())
		result.Insert(GetSource());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	if (GetView() != Mp7JrsNodePtr())
		result.Insert(GetView());
	if (GetViewRef() != Mp7JrsNodePtr())
		result.Insert(GetViewRef());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsFrequencyTreeType_ExtMethodImpl.h


