 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsGaussianMixtureModelType_LocalType_ExtImplInclude.h



#include "Mp7JrsnonNegativeReal.h"
#include "Mp7JrsGaussianDistributionType.h"
#include "Mp7JrsGaussianMixtureModelType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsGaussianMixtureModelType_LocalType::IMp7JrsGaussianMixtureModelType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsGaussianMixtureModelType_LocalType_ExtPropInit.h

}

IMp7JrsGaussianMixtureModelType_LocalType::~IMp7JrsGaussianMixtureModelType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsGaussianMixtureModelType_LocalType_ExtPropCleanup.h

}

Mp7JrsGaussianMixtureModelType_LocalType::Mp7JrsGaussianMixtureModelType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsGaussianMixtureModelType_LocalType::~Mp7JrsGaussianMixtureModelType_LocalType()
{
	Cleanup();
}

void Mp7JrsGaussianMixtureModelType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_Weight = Mp7JrsnonNegativeRealPtr(); // Class with content 
	m_GaussianDistribution = Mp7JrsGaussianDistributionPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsGaussianMixtureModelType_LocalType_ExtMyPropInit.h

}

void Mp7JrsGaussianMixtureModelType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsGaussianMixtureModelType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Weight);
	// Mp7JrsFactory::DeleteObject(m_GaussianDistribution);
}

void Mp7JrsGaussianMixtureModelType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use GaussianMixtureModelType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IGaussianMixtureModelType_LocalType
	const Mp7JrsPtr< Mp7JrsGaussianMixtureModelType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_Weight);
		this->SetWeight(Mp7JrsFactory::CloneObject(tmp->GetWeight()));
		// Mp7JrsFactory::DeleteObject(m_GaussianDistribution);
		this->SetGaussianDistribution(Mp7JrsFactory::CloneObject(tmp->GetGaussianDistribution()));
}

Mp7JrsNodePtr Mp7JrsGaussianMixtureModelType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsGaussianMixtureModelType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsnonNegativeRealPtr Mp7JrsGaussianMixtureModelType_LocalType::GetWeight() const
{
		return m_Weight;
}

Mp7JrsGaussianDistributionPtr Mp7JrsGaussianMixtureModelType_LocalType::GetGaussianDistribution() const
{
		return m_GaussianDistribution;
}

void Mp7JrsGaussianMixtureModelType_LocalType::SetWeight(const Mp7JrsnonNegativeRealPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGaussianMixtureModelType_LocalType::SetWeight().");
	}
	if (m_Weight != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Weight);
		m_Weight = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Weight) m_Weight->SetParent(m_myself.getPointer());
		if (m_Weight && m_Weight->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:nonNegativeReal"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Weight->UseTypeAttribute = true;
		}
		if(m_Weight != Mp7JrsnonNegativeRealPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Weight->SetContentName(XMLString::transcode("Weight"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGaussianMixtureModelType_LocalType::SetGaussianDistribution(const Mp7JrsGaussianDistributionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGaussianMixtureModelType_LocalType::SetGaussianDistribution().");
	}
	if (m_GaussianDistribution != item)
	{
		// Mp7JrsFactory::DeleteObject(m_GaussianDistribution);
		m_GaussianDistribution = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_GaussianDistribution) m_GaussianDistribution->SetParent(m_myself.getPointer());
		if (m_GaussianDistribution && m_GaussianDistribution->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:GaussianDistributionType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_GaussianDistribution->UseTypeAttribute = true;
		}
		if(m_GaussianDistribution != Mp7JrsGaussianDistributionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_GaussianDistribution->SetContentName(XMLString::transcode("GaussianDistribution"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: GaussianMixtureModelType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsGaussianMixtureModelType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsGaussianMixtureModelType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsGaussianMixtureModelType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsGaussianMixtureModelType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("GaussianMixtureModelType_LocalType"));
	// Element serialization:
	// Class with content		
	
	if (m_Weight != Mp7JrsnonNegativeRealPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Weight->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Weight"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Weight->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_GaussianDistribution != Mp7JrsGaussianDistributionPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_GaussianDistribution->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("GaussianDistribution"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_GaussianDistribution->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsGaussianMixtureModelType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Weight"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Weight")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreatenonNegativeReal;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetWeight(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("GaussianDistribution"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("GaussianDistribution")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateGaussianDistributionType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetGaussianDistribution(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsGaussianMixtureModelType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsGaussianMixtureModelType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Weight")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreatenonNegativeReal;
	}
	this->SetWeight(child);
  }
  if (XMLString::compareString(elementname, X("GaussianDistribution")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateGaussianDistributionType;
	}
	this->SetGaussianDistribution(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsGaussianMixtureModelType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetWeight() != Mp7JrsNodePtr())
		result.Insert(GetWeight());
	if (GetGaussianDistribution() != Mp7JrsNodePtr())
		result.Insert(GetGaussianDistribution());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsGaussianMixtureModelType_LocalType_ExtMethodImpl.h


