 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsGofGopFeatureType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7JrsGofGopFeatureType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsGofGopFeatureType::IMp7JrsGofGopFeatureType()
{

// no includefile for extension defined 
// file Mp7JrsGofGopFeatureType_ExtPropInit.h

}

IMp7JrsGofGopFeatureType::~IMp7JrsGofGopFeatureType()
{
// no includefile for extension defined 
// file Mp7JrsGofGopFeatureType_ExtPropCleanup.h

}

Mp7JrsGofGopFeatureType::Mp7JrsGofGopFeatureType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsGofGopFeatureType::~Mp7JrsGofGopFeatureType()
{
	Cleanup();
}

void Mp7JrsGofGopFeatureType::Init()
{

	// Init attributes
	m_aggregation = Mp7JrsGofGopFeatureType_aggregation_LocalType::UninitializedEnumeration;
	m_aggregation_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Descriptor = Mp7JrsPtr< Mp7JrsNode >(); // Class


// no includefile for extension defined 
// file Mp7JrsGofGopFeatureType_ExtMyPropInit.h

}

void Mp7JrsGofGopFeatureType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsGofGopFeatureType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Descriptor);
}

void Mp7JrsGofGopFeatureType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use GofGopFeatureTypePtr, since we
	// might need GetBase(), which isn't defined in IGofGopFeatureType
	const Mp7JrsPtr< Mp7JrsGofGopFeatureType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	if (tmp->Existaggregation())
	{
		this->Setaggregation(tmp->Getaggregation());
	}
	else
	{
		Invalidateaggregation();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Descriptor);
		this->SetDescriptor(Mp7JrsFactory::CloneObject(tmp->GetDescriptor()));
}

Mp7JrsNodePtr Mp7JrsGofGopFeatureType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsGofGopFeatureType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsGofGopFeatureType_aggregation_LocalType::Enumeration Mp7JrsGofGopFeatureType::Getaggregation() const
{
	return m_aggregation;
}

bool Mp7JrsGofGopFeatureType::Existaggregation() const
{
	return m_aggregation_Exist;
}
void Mp7JrsGofGopFeatureType::Setaggregation(Mp7JrsGofGopFeatureType_aggregation_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGofGopFeatureType::Setaggregation().");
	}
	m_aggregation = item;
	m_aggregation_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGofGopFeatureType::Invalidateaggregation()
{
	m_aggregation_Exist = false;
}
Mp7JrsPtr< Mp7JrsNode > Mp7JrsGofGopFeatureType::GetDescriptor() const
{
		return m_Descriptor;
}

void Mp7JrsGofGopFeatureType::SetDescriptor(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGofGopFeatureType::SetDescriptor().");
	}
	if (m_Descriptor != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Descriptor);
		m_Descriptor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Descriptor) m_Descriptor->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_Descriptor) m_Descriptor->UseTypeAttribute = true;
		if(m_Descriptor != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Descriptor->SetContentName(XMLString::transcode("Descriptor"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsGofGopFeatureType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; aggregation,  T:urn:mpeg:mpeg7:schema:2004:GofGopFeatureType_aggregation_LocalType, C:Enumeration 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("aggregation")) == 0)
	{
		// aggregation is simple attribute GofGopFeatureType_aggregation_LocalType
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "Mp7JrsGofGopFeatureType_aggregation_LocalType::Enumeration");
	}
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Descriptor")) == 0)
	{
		// Descriptor is simple abstract element VisualDType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDescriptor()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsVisualDPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDescriptor(p, client);
					if((p = GetDescriptor()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for GofGopFeatureType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "GofGopFeatureType");
	}
	return result;
}

XMLCh * Mp7JrsGofGopFeatureType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsGofGopFeatureType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsGofGopFeatureType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("GofGopFeatureType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_aggregation_Exist)
	{
	// Enumeration
	if(m_aggregation != Mp7JrsGofGopFeatureType_aggregation_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsGofGopFeatureType_aggregation_LocalType::ToText(m_aggregation);
		element->setAttributeNS(X(""), X("aggregation"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Descriptor != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Descriptor->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Descriptor"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_Descriptor->UseTypeAttribute = true;
		}
		m_Descriptor->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsGofGopFeatureType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("aggregation")))
	{
		this->Setaggregation(Mp7JrsGofGopFeatureType_aggregation_LocalType::Parse(parent->getAttribute(X("aggregation"))));
		* current = parent;
	}

	parent = Mp7JrsUtil::GetNextChild(parent);
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Descriptor"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Descriptor")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateVisualDType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetDescriptor(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsGofGopFeatureType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsGofGopFeatureType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Descriptor")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateVisualDType;
	}
	this->SetDescriptor(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsGofGopFeatureType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetDescriptor() != Mp7JrsNodePtr())
		result.Insert(GetDescriptor());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsGofGopFeatureType_ExtMethodImpl.h


