 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsGraphicalClassificationSchemeType_ExtImplInclude.h



#include "Mp7JrsClassificationSchemeBaseType.h"
#include "Mp7JrsGraphicalClassificationSchemeType_GraphicalTerm_CollectionType.h"
#include "Mp7JrsClassificationSchemeBaseType_domain_CollectionType.h"
#include "Mp7JrsClassificationSchemeBaseType_Import_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsGraphicalClassificationSchemeType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsReferenceType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Import
#include "Mp7JrsGraphicalTermDefinitionType.h" // Element collection urn:mpeg:mpeg7:schema:2004:GraphicalTerm

#include <assert.h>
IMp7JrsGraphicalClassificationSchemeType::IMp7JrsGraphicalClassificationSchemeType()
{

// no includefile for extension defined 
// file Mp7JrsGraphicalClassificationSchemeType_ExtPropInit.h

}

IMp7JrsGraphicalClassificationSchemeType::~IMp7JrsGraphicalClassificationSchemeType()
{
// no includefile for extension defined 
// file Mp7JrsGraphicalClassificationSchemeType_ExtPropCleanup.h

}

Mp7JrsGraphicalClassificationSchemeType::Mp7JrsGraphicalClassificationSchemeType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsGraphicalClassificationSchemeType::~Mp7JrsGraphicalClassificationSchemeType()
{
	Cleanup();
}

void Mp7JrsGraphicalClassificationSchemeType::Init()
{
	// Init base
	m_Base = CreateClassificationSchemeBaseType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_strict = false; // Value
	m_strict_Default = false; // Default value
	m_strict_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_GraphicalTerm = Mp7JrsGraphicalClassificationSchemeType_GraphicalTerm_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsGraphicalClassificationSchemeType_ExtMyPropInit.h

}

void Mp7JrsGraphicalClassificationSchemeType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsGraphicalClassificationSchemeType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_GraphicalTerm);
}

void Mp7JrsGraphicalClassificationSchemeType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use GraphicalClassificationSchemeTypePtr, since we
	// might need GetBase(), which isn't defined in IGraphicalClassificationSchemeType
	const Mp7JrsPtr< Mp7JrsGraphicalClassificationSchemeType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsClassificationSchemeBasePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsGraphicalClassificationSchemeType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->Existstrict())
	{
		this->Setstrict(tmp->Getstrict());
	}
	else
	{
		Invalidatestrict();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_GraphicalTerm);
		this->SetGraphicalTerm(Mp7JrsFactory::CloneObject(tmp->GetGraphicalTerm()));
}

Mp7JrsNodePtr Mp7JrsGraphicalClassificationSchemeType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsGraphicalClassificationSchemeType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsClassificationSchemeBaseType > Mp7JrsGraphicalClassificationSchemeType::GetBase() const
{
	return m_Base;
}

bool Mp7JrsGraphicalClassificationSchemeType::Getstrict() const
{
	if (this->Existstrict()) {
		return m_strict;
	} else {
		return m_strict_Default;
	}
}

bool Mp7JrsGraphicalClassificationSchemeType::Existstrict() const
{
	return m_strict_Exist;
}
void Mp7JrsGraphicalClassificationSchemeType::Setstrict(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::Setstrict().");
	}
	m_strict = item;
	m_strict_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphicalClassificationSchemeType::Invalidatestrict()
{
	m_strict_Exist = false;
}
Mp7JrsGraphicalClassificationSchemeType_GraphicalTerm_CollectionPtr Mp7JrsGraphicalClassificationSchemeType::GetGraphicalTerm() const
{
		return m_GraphicalTerm;
}

void Mp7JrsGraphicalClassificationSchemeType::SetGraphicalTerm(const Mp7JrsGraphicalClassificationSchemeType_GraphicalTerm_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::SetGraphicalTerm().");
	}
	if (m_GraphicalTerm != item)
	{
		// Mp7JrsFactory::DeleteObject(m_GraphicalTerm);
		m_GraphicalTerm = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_GraphicalTerm) m_GraphicalTerm->SetParent(m_myself.getPointer());
		if(m_GraphicalTerm != Mp7JrsGraphicalClassificationSchemeType_GraphicalTerm_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_GraphicalTerm->SetContentName(XMLString::transcode("GraphicalTerm"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsGraphicalClassificationSchemeType::Geturi() const
{
	return GetBase()->Geturi();
}

void Mp7JrsGraphicalClassificationSchemeType::Seturi(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::Seturi().");
	}
	GetBase()->Seturi(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsClassificationSchemeBaseType_domain_CollectionPtr Mp7JrsGraphicalClassificationSchemeType::Getdomain() const
{
	return GetBase()->Getdomain();
}

bool Mp7JrsGraphicalClassificationSchemeType::Existdomain() const
{
	return GetBase()->Existdomain();
}
void Mp7JrsGraphicalClassificationSchemeType::Setdomain(const Mp7JrsClassificationSchemeBaseType_domain_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::Setdomain().");
	}
	GetBase()->Setdomain(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphicalClassificationSchemeType::Invalidatedomain()
{
	GetBase()->Invalidatedomain();
}
Mp7JrsClassificationSchemeBaseType_Import_CollectionPtr Mp7JrsGraphicalClassificationSchemeType::GetImport() const
{
	return GetBase()->GetImport();
}

void Mp7JrsGraphicalClassificationSchemeType::SetImport(const Mp7JrsClassificationSchemeBaseType_Import_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::SetImport().");
	}
	GetBase()->SetImport(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsGraphicalClassificationSchemeType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsGraphicalClassificationSchemeType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsGraphicalClassificationSchemeType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphicalClassificationSchemeType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsGraphicalClassificationSchemeType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsGraphicalClassificationSchemeType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsGraphicalClassificationSchemeType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphicalClassificationSchemeType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsGraphicalClassificationSchemeType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsGraphicalClassificationSchemeType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsGraphicalClassificationSchemeType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphicalClassificationSchemeType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsGraphicalClassificationSchemeType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsGraphicalClassificationSchemeType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsGraphicalClassificationSchemeType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphicalClassificationSchemeType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsGraphicalClassificationSchemeType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsGraphicalClassificationSchemeType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsGraphicalClassificationSchemeType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphicalClassificationSchemeType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsGraphicalClassificationSchemeType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsGraphicalClassificationSchemeType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphicalClassificationSchemeType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsGraphicalClassificationSchemeType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 8 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; id,  T:http://www.w3.org/2001/XMLSchema:ID, C:String 
//- N; timeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; timeUnit,  T:urn:mpeg:mpeg7:schema:2004:durationType, C:Pattern 
//- N; mediaTimeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; mediaTimeUnit,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
//- N; uri,  T:http://www.w3.org/2001/XMLSchema:anyURI, C:String 
//- N; domain,  T:urn:mpeg:mpeg7:schema:2004:ClassificationSchemeBaseType_domain_CollectionType, C:Collection 
//- N; strict,  T:http://www.w3.org/2001/XMLSchema:boolean, C:Value 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("strict")) == 0)
	{
		// strict is simple attribute boolean
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "bool");
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("GraphicalTerm")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:GraphicalTerm is item of type GraphicalTermDefinitionType
		// in element collection GraphicalClassificationSchemeType_GraphicalTerm_CollectionType
		
		context->Found = true;
		Mp7JrsGraphicalClassificationSchemeType_GraphicalTerm_CollectionPtr coll = GetGraphicalTerm();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateGraphicalClassificationSchemeType_GraphicalTerm_CollectionType;
				SetGraphicalTerm(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:GraphicalTermDefinitionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsGraphicalTermDefinitionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for GraphicalClassificationSchemeType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "GraphicalClassificationSchemeType");
	}
	return result;
}

XMLCh * Mp7JrsGraphicalClassificationSchemeType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsGraphicalClassificationSchemeType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsGraphicalClassificationSchemeType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsGraphicalClassificationSchemeType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("GraphicalClassificationSchemeType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_strict_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_strict);
		element->setAttributeNS(X(""), X("strict"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_GraphicalTerm != Mp7JrsGraphicalClassificationSchemeType_GraphicalTerm_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_GraphicalTerm->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsGraphicalClassificationSchemeType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("strict")))
	{
		// deserialize value type
		this->Setstrict(Mp7JrsConvert::TextToBool(parent->getAttribute(X("strict"))));
		* current = parent;
	}

	// Extensionbase is Mp7JrsClassificationSchemeBaseType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("GraphicalTerm"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("GraphicalTerm")) == 0))
		{
			// Deserialize factory type
			Mp7JrsGraphicalClassificationSchemeType_GraphicalTerm_CollectionPtr tmp = CreateGraphicalClassificationSchemeType_GraphicalTerm_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetGraphicalTerm(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsGraphicalClassificationSchemeType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsGraphicalClassificationSchemeType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("GraphicalTerm")) == 0))
  {
	Mp7JrsGraphicalClassificationSchemeType_GraphicalTerm_CollectionPtr tmp = CreateGraphicalClassificationSchemeType_GraphicalTerm_CollectionType;
	this->SetGraphicalTerm(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsGraphicalClassificationSchemeType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetGraphicalTerm() != Mp7JrsNodePtr())
		result.Insert(GetGraphicalTerm());
	if (GetImport() != Mp7JrsNodePtr())
		result.Insert(GetImport());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsGraphicalClassificationSchemeType_ExtMethodImpl.h


