 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsGridLayoutType_ExtImplInclude.h



#include "Mp7Jrsunsigned8.h"
#include "Mp7JrsGridLayoutType_descriptorMask_LocalType.h"
#include "Mp7JrsGridLayoutType_Descriptor_CollectionType.h"
#include "Mp7JrsGridLayoutType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsVisualDType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Descriptor

#include <assert.h>
IMp7JrsGridLayoutType::IMp7JrsGridLayoutType()
{

// no includefile for extension defined 
// file Mp7JrsGridLayoutType_ExtPropInit.h

}

IMp7JrsGridLayoutType::~IMp7JrsGridLayoutType()
{
// no includefile for extension defined 
// file Mp7JrsGridLayoutType_ExtPropCleanup.h

}

Mp7JrsGridLayoutType::Mp7JrsGridLayoutType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsGridLayoutType::~Mp7JrsGridLayoutType()
{
	Cleanup();
}

void Mp7JrsGridLayoutType::Init()
{

	// Init attributes
	m_numOfPartX = Createunsigned8; // Create a valid class
	m_numOfPartX->SetContent(0); // Use Value initialiser (xxx2)
	m_numOfPartY = Createunsigned8; // Create a valid class
	m_numOfPartY->SetContent(0); // Use Value initialiser (xxx2)
	m_descriptorMask = Mp7JrsGridLayoutType_descriptorMask_LocalPtr(); // Pattern
	m_descriptorMask_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Descriptor = Mp7JrsGridLayoutType_Descriptor_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsGridLayoutType_ExtMyPropInit.h

}

void Mp7JrsGridLayoutType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsGridLayoutType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_numOfPartX);
	// Mp7JrsFactory::DeleteObject(m_numOfPartY);
	// Mp7JrsFactory::DeleteObject(m_descriptorMask); // Pattern
	// Mp7JrsFactory::DeleteObject(m_Descriptor);
}

void Mp7JrsGridLayoutType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use GridLayoutTypePtr, since we
	// might need GetBase(), which isn't defined in IGridLayoutType
	const Mp7JrsPtr< Mp7JrsGridLayoutType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	// Mp7JrsFactory::DeleteObject(m_numOfPartX);
		this->SetnumOfPartX(Mp7JrsFactory::CloneObject(tmp->GetnumOfPartX()));
	}
	{
	// Mp7JrsFactory::DeleteObject(m_numOfPartY);
		this->SetnumOfPartY(Mp7JrsFactory::CloneObject(tmp->GetnumOfPartY()));
	}
	{
	// Mp7JrsFactory::DeleteObject(m_descriptorMask); // Pattern
	if (tmp->ExistdescriptorMask())
	{
		this->SetdescriptorMask(Mp7JrsFactory::CloneObject(tmp->GetdescriptorMask()));
	}
	else
	{
		InvalidatedescriptorMask();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Descriptor);
		this->SetDescriptor(Mp7JrsFactory::CloneObject(tmp->GetDescriptor()));
}

Mp7JrsNodePtr Mp7JrsGridLayoutType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsGridLayoutType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7Jrsunsigned8Ptr Mp7JrsGridLayoutType::GetnumOfPartX() const
{
	return m_numOfPartX;
}

void Mp7JrsGridLayoutType::SetnumOfPartX(const Mp7Jrsunsigned8Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGridLayoutType::SetnumOfPartX().");
	}
	m_numOfPartX = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_numOfPartX) m_numOfPartX->SetParent(m_myself.getPointer());
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7Jrsunsigned8Ptr Mp7JrsGridLayoutType::GetnumOfPartY() const
{
	return m_numOfPartY;
}

void Mp7JrsGridLayoutType::SetnumOfPartY(const Mp7Jrsunsigned8Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGridLayoutType::SetnumOfPartY().");
	}
	m_numOfPartY = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_numOfPartY) m_numOfPartY->SetParent(m_myself.getPointer());
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsGridLayoutType_descriptorMask_LocalPtr Mp7JrsGridLayoutType::GetdescriptorMask() const
{
	return m_descriptorMask;
}

bool Mp7JrsGridLayoutType::ExistdescriptorMask() const
{
	return m_descriptorMask_Exist;
}
void Mp7JrsGridLayoutType::SetdescriptorMask(const Mp7JrsGridLayoutType_descriptorMask_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGridLayoutType::SetdescriptorMask().");
	}
	m_descriptorMask = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_descriptorMask) m_descriptorMask->SetParent(m_myself.getPointer());
	m_descriptorMask_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGridLayoutType::InvalidatedescriptorMask()
{
	m_descriptorMask_Exist = false;
}
Mp7JrsGridLayoutType_Descriptor_CollectionPtr Mp7JrsGridLayoutType::GetDescriptor() const
{
		return m_Descriptor;
}

void Mp7JrsGridLayoutType::SetDescriptor(const Mp7JrsGridLayoutType_Descriptor_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGridLayoutType::SetDescriptor().");
	}
	if (m_Descriptor != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Descriptor);
		m_Descriptor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Descriptor) m_Descriptor->SetParent(m_myself.getPointer());
		if(m_Descriptor != Mp7JrsGridLayoutType_Descriptor_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Descriptor->SetContentName(XMLString::transcode("Descriptor"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsGridLayoutType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  3, 3 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 3
//- N; numOfPartX,  T:urn:mpeg:mpeg7:schema:2004:unsigned8, C:Class 
//- N; numOfPartY,  T:urn:mpeg:mpeg7:schema:2004:unsigned8, C:Class 
//- N; descriptorMask,  T:urn:mpeg:mpeg7:schema:2004:GridLayoutType_descriptorMask_LocalType, C:Pattern 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("numOfPartX")) == 0)
	{
		// numOfPartX is simple attribute unsigned8
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GetnumOfPartX()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned8")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned8Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetnumOfPartX(p, client);
					if((p = GetnumOfPartX()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("numOfPartY")) == 0)
	{
		// numOfPartY is simple attribute unsigned8
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GetnumOfPartY()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned8")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned8Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetnumOfPartY(p, client);
					if((p = GetnumOfPartY()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("descriptorMask")) == 0)
	{
		// descriptorMask is simple attribute GridLayoutType_descriptorMask_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GetdescriptorMask()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:GridLayoutType_descriptorMask_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsGridLayoutType_descriptorMask_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetdescriptorMask(p, client);
					if((p = GetdescriptorMask()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Descriptor")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Descriptor is item of abstract type VisualDType
		// in element collection GridLayoutType_Descriptor_CollectionType
		
		context->Found = true;
		Mp7JrsGridLayoutType_Descriptor_CollectionPtr coll = GetDescriptor();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateGridLayoutType_Descriptor_CollectionType;
				SetDescriptor(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), 65025))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsVisualDPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for GridLayoutType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "GridLayoutType");
	}
	return result;
}

XMLCh * Mp7JrsGridLayoutType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsGridLayoutType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsGridLayoutType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("GridLayoutType"));
	// Attribute Serialization:
	// Class
	if (m_numOfPartX != Mp7Jrsunsigned8Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_numOfPartX->ToText();
		element->setAttributeNS(X(""), X("numOfPartX"), tmp);
		XMLString::release(&tmp);
	}
	// Attribute Serialization:
	// Class
	if (m_numOfPartY != Mp7Jrsunsigned8Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_numOfPartY->ToText();
		element->setAttributeNS(X(""), X("numOfPartY"), tmp);
		XMLString::release(&tmp);
	}
	// Attribute Serialization:
		

	// Optional attriute
	if(m_descriptorMask_Exist)
	{
	// Pattern
	if (m_descriptorMask != Mp7JrsGridLayoutType_descriptorMask_LocalPtr())
	{
		XMLCh * tmp = m_descriptorMask->ToText();
		element->setAttributeNS(X(""), X("descriptorMask"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_Descriptor != Mp7JrsGridLayoutType_Descriptor_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Descriptor->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsGridLayoutType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("numOfPartX")))
	{
		// Deserialize class type
		Mp7Jrsunsigned8Ptr tmp = Createunsigned8;
		tmp->Parse(parent->getAttribute(X("numOfPartX")));
		this->SetnumOfPartX(tmp);
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("numOfPartY")))
	{
		// Deserialize class type
		Mp7Jrsunsigned8Ptr tmp = Createunsigned8;
		tmp->Parse(parent->getAttribute(X("numOfPartY")));
		this->SetnumOfPartY(tmp);
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("descriptorMask")))
	{
		Mp7JrsGridLayoutType_descriptorMask_LocalPtr tmp = CreateGridLayoutType_descriptorMask_LocalType;
		tmp->Parse(parent->getAttribute(X("descriptorMask")));
		this->SetdescriptorMask(tmp);
		* current = parent;
	}

	parent = Mp7JrsUtil::GetNextChild(parent);
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("Descriptor"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("Descriptor")) == 0))
		{
			// Deserialize factory type
			Mp7JrsGridLayoutType_Descriptor_CollectionPtr tmp = CreateGridLayoutType_Descriptor_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetDescriptor(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsGridLayoutType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsGridLayoutType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Descriptor")) == 0))
  {
	Mp7JrsGridLayoutType_Descriptor_CollectionPtr tmp = CreateGridLayoutType_Descriptor_CollectionType;
	this->SetDescriptor(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsGridLayoutType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetDescriptor() != Mp7JrsNodePtr())
		result.Insert(GetDescriptor());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsGridLayoutType_ExtMethodImpl.h


