 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMatchingHintType_ExtImplInclude.h



#include "Mp7JrsDType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsMatchingHintType_Hint_CollectionType.h"
#include "Mp7JrsMatchingHintType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsMatchingHintType_Hint_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Hint

#include <assert.h>
IMp7JrsMatchingHintType::IMp7JrsMatchingHintType()
{

// no includefile for extension defined 
// file Mp7JrsMatchingHintType_ExtPropInit.h

}

IMp7JrsMatchingHintType::~IMp7JrsMatchingHintType()
{
// no includefile for extension defined 
// file Mp7JrsMatchingHintType_ExtPropCleanup.h

}

Mp7JrsMatchingHintType::Mp7JrsMatchingHintType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMatchingHintType::~Mp7JrsMatchingHintType()
{
	Cleanup();
}

void Mp7JrsMatchingHintType::Init()
{
	// Init base
	m_Base = CreateDType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_reliability = CreatezeroToOneType; // Create a valid class
	m_reliability->SetContent(0.0); // Use Value initialiser (xxx2)
	m_reliability_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Hint = Mp7JrsMatchingHintType_Hint_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsMatchingHintType_ExtMyPropInit.h

}

void Mp7JrsMatchingHintType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMatchingHintType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_reliability);
	// Mp7JrsFactory::DeleteObject(m_Hint);
}

void Mp7JrsMatchingHintType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MatchingHintTypePtr, since we
	// might need GetBase(), which isn't defined in IMatchingHintType
	const Mp7JrsPtr< Mp7JrsMatchingHintType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMatchingHintType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_reliability);
	if (tmp->Existreliability())
	{
		this->Setreliability(Mp7JrsFactory::CloneObject(tmp->Getreliability()));
	}
	else
	{
		Invalidatereliability();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Hint);
		this->SetHint(Mp7JrsFactory::CloneObject(tmp->GetHint()));
}

Mp7JrsNodePtr Mp7JrsMatchingHintType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsMatchingHintType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDType > Mp7JrsMatchingHintType::GetBase() const
{
	return m_Base;
}

Mp7JrszeroToOnePtr Mp7JrsMatchingHintType::Getreliability() const
{
	if (this->Existreliability()) {
		return m_reliability;
	} else {
		return m_reliability_Default;
	}
}

bool Mp7JrsMatchingHintType::Existreliability() const
{
	return m_reliability_Exist;
}
void Mp7JrsMatchingHintType::Setreliability(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMatchingHintType::Setreliability().");
	}
	m_reliability = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_reliability) m_reliability->SetParent(m_myself.getPointer());
	m_reliability_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMatchingHintType::Invalidatereliability()
{
	m_reliability_Exist = false;
}
Mp7JrsMatchingHintType_Hint_CollectionPtr Mp7JrsMatchingHintType::GetHint() const
{
		return m_Hint;
}

void Mp7JrsMatchingHintType::SetHint(const Mp7JrsMatchingHintType_Hint_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMatchingHintType::SetHint().");
	}
	if (m_Hint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Hint);
		m_Hint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Hint) m_Hint->SetParent(m_myself.getPointer());
		if(m_Hint != Mp7JrsMatchingHintType_Hint_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Hint->SetContentName(XMLString::transcode("Hint"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsMatchingHintType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; reliability,  T:urn:mpeg:mpeg7:schema:2004:zeroToOneType, C:Class 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("reliability")) == 0)
	{
		// reliability is simple attribute zeroToOneType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = Getreliability()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:zeroToOneType")))) != empty)
			{
				// Is type allowed
				Mp7JrszeroToOnePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					Setreliability(p, client);
					if((p = Getreliability()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Hint")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Hint is item of type MatchingHintType_Hint_LocalType
		// in element collection MatchingHintType_Hint_CollectionType
		
		context->Found = true;
		Mp7JrsMatchingHintType_Hint_CollectionPtr coll = GetHint();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMatchingHintType_Hint_CollectionType;
				SetHint(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MatchingHintType_Hint_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMatchingHintType_Hint_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MatchingHintType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MatchingHintType");
	}
	return result;
}

XMLCh * Mp7JrsMatchingHintType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMatchingHintType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsMatchingHintType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMatchingHintType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MatchingHintType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_reliability_Exist)
	{
	// Class
	if (m_reliability != Mp7JrszeroToOnePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_reliability->ToText();
		element->setAttributeNS(X(""), X("reliability"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_Hint != Mp7JrsMatchingHintType_Hint_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Hint->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsMatchingHintType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("reliability")))
	{
		// Deserialize class type
		Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
		tmp->Parse(parent->getAttribute(X("reliability")));
		this->Setreliability(tmp);
		* current = parent;
	}

	// Extensionbase is Mp7JrsDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("Hint"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("Hint")) == 0))
		{
			// Deserialize factory type
			Mp7JrsMatchingHintType_Hint_CollectionPtr tmp = CreateMatchingHintType_Hint_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetHint(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsMatchingHintType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMatchingHintType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Hint")) == 0))
  {
	Mp7JrsMatchingHintType_Hint_CollectionPtr tmp = CreateMatchingHintType_Hint_CollectionType;
	this->SetHint(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMatchingHintType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetHint() != Mp7JrsNodePtr())
		result.Insert(GetHint());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMatchingHintType_ExtMethodImpl.h


