 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMediaIdentificationType_ExtImplInclude.h



#include "Mp7JrsDType.h"
#include "Mp7JrsUniqueIDType.h"
#include "Mp7JrsMediaIdentificationType_AudioDomain_CollectionType.h"
#include "Mp7JrsMediaIdentificationType_VideoDomain_CollectionType.h"
#include "Mp7JrsMediaIdentificationType_ImageDomain_CollectionType.h"
#include "Mp7JrsMediaIdentificationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsControlledTermUseType.h" // Element collection urn:mpeg:mpeg7:schema:2004:AudioDomain

#include <assert.h>
IMp7JrsMediaIdentificationType::IMp7JrsMediaIdentificationType()
{

// no includefile for extension defined 
// file Mp7JrsMediaIdentificationType_ExtPropInit.h

}

IMp7JrsMediaIdentificationType::~IMp7JrsMediaIdentificationType()
{
// no includefile for extension defined 
// file Mp7JrsMediaIdentificationType_ExtPropCleanup.h

}

Mp7JrsMediaIdentificationType::Mp7JrsMediaIdentificationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMediaIdentificationType::~Mp7JrsMediaIdentificationType()
{
	Cleanup();
}

void Mp7JrsMediaIdentificationType::Init()
{
	// Init base
	m_Base = CreateDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_EntityIdentifier = Mp7JrsUniqueIDPtr(); // Class
	m_AudioDomain = Mp7JrsMediaIdentificationType_AudioDomain_CollectionPtr(); // Collection
	m_VideoDomain = Mp7JrsMediaIdentificationType_VideoDomain_CollectionPtr(); // Collection
	m_ImageDomain = Mp7JrsMediaIdentificationType_ImageDomain_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsMediaIdentificationType_ExtMyPropInit.h

}

void Mp7JrsMediaIdentificationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMediaIdentificationType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_EntityIdentifier);
	// Mp7JrsFactory::DeleteObject(m_AudioDomain);
	// Mp7JrsFactory::DeleteObject(m_VideoDomain);
	// Mp7JrsFactory::DeleteObject(m_ImageDomain);
}

void Mp7JrsMediaIdentificationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MediaIdentificationTypePtr, since we
	// might need GetBase(), which isn't defined in IMediaIdentificationType
	const Mp7JrsPtr< Mp7JrsMediaIdentificationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMediaIdentificationType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_EntityIdentifier);
		this->SetEntityIdentifier(Mp7JrsFactory::CloneObject(tmp->GetEntityIdentifier()));
		// Mp7JrsFactory::DeleteObject(m_AudioDomain);
		this->SetAudioDomain(Mp7JrsFactory::CloneObject(tmp->GetAudioDomain()));
		// Mp7JrsFactory::DeleteObject(m_VideoDomain);
		this->SetVideoDomain(Mp7JrsFactory::CloneObject(tmp->GetVideoDomain()));
		// Mp7JrsFactory::DeleteObject(m_ImageDomain);
		this->SetImageDomain(Mp7JrsFactory::CloneObject(tmp->GetImageDomain()));
}

Mp7JrsNodePtr Mp7JrsMediaIdentificationType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsMediaIdentificationType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDType > Mp7JrsMediaIdentificationType::GetBase() const
{
	return m_Base;
}

Mp7JrsUniqueIDPtr Mp7JrsMediaIdentificationType::GetEntityIdentifier() const
{
		return m_EntityIdentifier;
}

Mp7JrsMediaIdentificationType_AudioDomain_CollectionPtr Mp7JrsMediaIdentificationType::GetAudioDomain() const
{
		return m_AudioDomain;
}

Mp7JrsMediaIdentificationType_VideoDomain_CollectionPtr Mp7JrsMediaIdentificationType::GetVideoDomain() const
{
		return m_VideoDomain;
}

Mp7JrsMediaIdentificationType_ImageDomain_CollectionPtr Mp7JrsMediaIdentificationType::GetImageDomain() const
{
		return m_ImageDomain;
}

void Mp7JrsMediaIdentificationType::SetEntityIdentifier(const Mp7JrsUniqueIDPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaIdentificationType::SetEntityIdentifier().");
	}
	if (m_EntityIdentifier != item)
	{
		// Mp7JrsFactory::DeleteObject(m_EntityIdentifier);
		m_EntityIdentifier = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_EntityIdentifier) m_EntityIdentifier->SetParent(m_myself.getPointer());
		if (m_EntityIdentifier && m_EntityIdentifier->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_EntityIdentifier->UseTypeAttribute = true;
		}
		if(m_EntityIdentifier != Mp7JrsUniqueIDPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_EntityIdentifier->SetContentName(XMLString::transcode("EntityIdentifier"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaIdentificationType::SetAudioDomain(const Mp7JrsMediaIdentificationType_AudioDomain_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaIdentificationType::SetAudioDomain().");
	}
	if (m_AudioDomain != item)
	{
		// Mp7JrsFactory::DeleteObject(m_AudioDomain);
		m_AudioDomain = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AudioDomain) m_AudioDomain->SetParent(m_myself.getPointer());
		if(m_AudioDomain != Mp7JrsMediaIdentificationType_AudioDomain_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AudioDomain->SetContentName(XMLString::transcode("AudioDomain"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaIdentificationType::SetVideoDomain(const Mp7JrsMediaIdentificationType_VideoDomain_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaIdentificationType::SetVideoDomain().");
	}
	if (m_VideoDomain != item)
	{
		// Mp7JrsFactory::DeleteObject(m_VideoDomain);
		m_VideoDomain = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VideoDomain) m_VideoDomain->SetParent(m_myself.getPointer());
		if(m_VideoDomain != Mp7JrsMediaIdentificationType_VideoDomain_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VideoDomain->SetContentName(XMLString::transcode("VideoDomain"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaIdentificationType::SetImageDomain(const Mp7JrsMediaIdentificationType_ImageDomain_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaIdentificationType::SetImageDomain().");
	}
	if (m_ImageDomain != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ImageDomain);
		m_ImageDomain = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ImageDomain) m_ImageDomain->SetParent(m_myself.getPointer());
		if(m_ImageDomain != Mp7JrsMediaIdentificationType_ImageDomain_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ImageDomain->SetContentName(XMLString::transcode("ImageDomain"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsMediaIdentificationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("EntityIdentifier")) == 0)
	{
		// EntityIdentifier is simple element UniqueIDType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetEntityIdentifier()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUniqueIDPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetEntityIdentifier(p, client);
					if((p = GetEntityIdentifier()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("AudioDomain")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:AudioDomain is item of type ControlledTermUseType
		// in element collection MediaIdentificationType_AudioDomain_CollectionType
		
		context->Found = true;
		Mp7JrsMediaIdentificationType_AudioDomain_CollectionPtr coll = GetAudioDomain();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMediaIdentificationType_AudioDomain_CollectionType;
				SetAudioDomain(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("VideoDomain")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:VideoDomain is item of type ControlledTermUseType
		// in element collection MediaIdentificationType_VideoDomain_CollectionType
		
		context->Found = true;
		Mp7JrsMediaIdentificationType_VideoDomain_CollectionPtr coll = GetVideoDomain();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMediaIdentificationType_VideoDomain_CollectionType;
				SetVideoDomain(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ImageDomain")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:ImageDomain is item of type ControlledTermUseType
		// in element collection MediaIdentificationType_ImageDomain_CollectionType
		
		context->Found = true;
		Mp7JrsMediaIdentificationType_ImageDomain_CollectionPtr coll = GetImageDomain();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMediaIdentificationType_ImageDomain_CollectionType;
				SetImageDomain(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MediaIdentificationType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MediaIdentificationType");
	}
	return result;
}

XMLCh * Mp7JrsMediaIdentificationType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsMediaIdentificationType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsMediaIdentificationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMediaIdentificationType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MediaIdentificationType"));
	// Element serialization:
	// Class
	
	if (m_EntityIdentifier != Mp7JrsUniqueIDPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_EntityIdentifier->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("EntityIdentifier"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_EntityIdentifier->Serialize(doc, element, &elem);
	}
	if (m_AudioDomain != Mp7JrsMediaIdentificationType_AudioDomain_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_AudioDomain->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_VideoDomain != Mp7JrsMediaIdentificationType_VideoDomain_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_VideoDomain->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_ImageDomain != Mp7JrsMediaIdentificationType_ImageDomain_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_ImageDomain->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsMediaIdentificationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("EntityIdentifier"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("EntityIdentifier")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateUniqueIDType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetEntityIdentifier(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("AudioDomain"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("AudioDomain")) == 0))
		{
			// Deserialize factory type
			Mp7JrsMediaIdentificationType_AudioDomain_CollectionPtr tmp = CreateMediaIdentificationType_AudioDomain_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetAudioDomain(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("VideoDomain"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("VideoDomain")) == 0))
		{
			// Deserialize factory type
			Mp7JrsMediaIdentificationType_VideoDomain_CollectionPtr tmp = CreateMediaIdentificationType_VideoDomain_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetVideoDomain(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("ImageDomain"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("ImageDomain")) == 0))
		{
			// Deserialize factory type
			Mp7JrsMediaIdentificationType_ImageDomain_CollectionPtr tmp = CreateMediaIdentificationType_ImageDomain_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetImageDomain(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsMediaIdentificationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMediaIdentificationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("EntityIdentifier")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateUniqueIDType;
	}
	this->SetEntityIdentifier(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("AudioDomain")) == 0))
  {
	Mp7JrsMediaIdentificationType_AudioDomain_CollectionPtr tmp = CreateMediaIdentificationType_AudioDomain_CollectionType;
	this->SetAudioDomain(tmp);
	child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("VideoDomain")) == 0))
  {
	Mp7JrsMediaIdentificationType_VideoDomain_CollectionPtr tmp = CreateMediaIdentificationType_VideoDomain_CollectionType;
	this->SetVideoDomain(tmp);
	child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("ImageDomain")) == 0))
  {
	Mp7JrsMediaIdentificationType_ImageDomain_CollectionPtr tmp = CreateMediaIdentificationType_ImageDomain_CollectionType;
	this->SetImageDomain(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMediaIdentificationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetEntityIdentifier() != Mp7JrsNodePtr())
		result.Insert(GetEntityIdentifier());
	if (GetAudioDomain() != Mp7JrsNodePtr())
		result.Insert(GetAudioDomain());
	if (GetVideoDomain() != Mp7JrsNodePtr())
		result.Insert(GetVideoDomain());
	if (GetImageDomain() != Mp7JrsNodePtr())
		result.Insert(GetImageDomain());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMediaIdentificationType_ExtMethodImpl.h


