 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// begin extension included
// file Mp7JrsMediaLocatorType_ExtImplInclude.h

#include "Mp7JrsSettings.h" // For TreatRelIncrTimeAsAbsolute flag


// end extension included



#include "Mp7JrsInlineMediaType.h"
#include "Mp7JrsMediaLocatorType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsMediaLocatorType::IMp7JrsMediaLocatorType()
{

// no includefile for extension defined 
// file Mp7JrsMediaLocatorType_ExtPropInit.h

}

IMp7JrsMediaLocatorType::~IMp7JrsMediaLocatorType()
{
// no includefile for extension defined 
// file Mp7JrsMediaLocatorType_ExtPropCleanup.h

}

Mp7JrsMediaLocatorType::Mp7JrsMediaLocatorType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMediaLocatorType::~Mp7JrsMediaLocatorType()
{
	Cleanup();
}

void Mp7JrsMediaLocatorType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_MediaUri = XMLString::transcode(""); //  Mandatory String
	m_InlineMedia = Mp7JrsInlineMediaPtr(); // Class
	m_StreamID = (0); // Value

	m_StreamID_Exist = false;


// no includefile for extension defined 
// file Mp7JrsMediaLocatorType_ExtMyPropInit.h

}

void Mp7JrsMediaLocatorType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMediaLocatorType_ExtMyPropCleanup.h


	XMLString::release(&this->m_MediaUri);
	this->m_MediaUri = NULL;
	// Mp7JrsFactory::DeleteObject(m_InlineMedia);
}

void Mp7JrsMediaLocatorType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MediaLocatorTypePtr, since we
	// might need GetBase(), which isn't defined in IMediaLocatorType
	const Mp7JrsPtr< Mp7JrsMediaLocatorType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		this->SetMediaUri(XMLString::replicate(tmp->GetMediaUri()));
		// Mp7JrsFactory::DeleteObject(m_InlineMedia);
		this->SetInlineMedia(Mp7JrsFactory::CloneObject(tmp->GetInlineMedia()));
	if (tmp->IsValidStreamID())
	{
		this->SetStreamID(tmp->GetStreamID());
	}
	else
	{
		InvalidateStreamID();
	}
}

Mp7JrsNodePtr Mp7JrsMediaLocatorType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsMediaLocatorType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


XMLCh * Mp7JrsMediaLocatorType::GetMediaUri() const
{
		return m_MediaUri;
}

Mp7JrsInlineMediaPtr Mp7JrsMediaLocatorType::GetInlineMedia() const
{
		return m_InlineMedia;
}

unsigned Mp7JrsMediaLocatorType::GetStreamID() const
{
		return m_StreamID;
}

// Element is optional
bool Mp7JrsMediaLocatorType::IsValidStreamID() const
{
	return m_StreamID_Exist;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsMediaLocatorType::SetMediaUri(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaLocatorType::SetMediaUri().");
	}
	if (m_MediaUri != item)
	{
		XMLString::release(&m_MediaUri);
		m_MediaUri = item;
	// Mp7JrsFactory::DeleteObject(m_InlineMedia);
	m_InlineMedia = Mp7JrsInlineMediaPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMediaLocatorType::SetInlineMedia(const Mp7JrsInlineMediaPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaLocatorType::SetInlineMedia().");
	}
	if (m_InlineMedia != item)
	{
		// Mp7JrsFactory::DeleteObject(m_InlineMedia);
		m_InlineMedia = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_InlineMedia) m_InlineMedia->SetParent(m_myself.getPointer());
		if (m_InlineMedia && m_InlineMedia->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:InlineMediaType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_InlineMedia->UseTypeAttribute = true;
		}
		if(m_InlineMedia != Mp7JrsInlineMediaPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_InlineMedia->SetContentName(XMLString::transcode("InlineMedia"));
		}
	XMLString::release(&this->m_MediaUri);
	m_MediaUri = NULL; // FTT Shouldn't it be XMLString::release()?
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaLocatorType::SetStreamID(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaLocatorType::SetStreamID().");
	}
	if (m_StreamID != item || m_StreamID_Exist == false)
	{
		m_StreamID = item;
		m_StreamID_Exist = true;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaLocatorType::InvalidateStreamID()
{
	m_StreamID_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsMediaLocatorType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("InlineMedia")) == 0)
	{
		// InlineMedia is simple element InlineMediaType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetInlineMedia()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:InlineMediaType")))) != empty)
			{
				// Is type allowed
				Mp7JrsInlineMediaPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetInlineMedia(p, client);
					if((p = GetInlineMedia()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MediaLocatorType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MediaLocatorType");
	}
	return result;
}

XMLCh * Mp7JrsMediaLocatorType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMediaLocatorType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsMediaLocatorType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MediaLocatorType"));
	// Element serialization:
	 // String
	if(m_MediaUri != (XMLCh*)NULL) Mp7JrsUtil::SerializeTextNode(element, doc, m_MediaUri, X("urn:mpeg:mpeg7:schema:2004"), X("MediaUri"), false);
	// Class
	
	if (m_InlineMedia != Mp7JrsInlineMediaPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_InlineMedia->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("InlineMedia"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_InlineMedia->Serialize(doc, element, &elem);
	}
	if(m_StreamID_Exist)
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_StreamID);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("StreamID"), true);
		XMLString::release(&tmp);
	}

}

bool Mp7JrsMediaLocatorType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
	do // Deserialize choice just one time!
	{
	// Deserialize string (XMLCh*) element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaUri"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		// FTT memleaking		this->SetMediaUri(Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent)));
		this->SetMediaUri(Mp7JrsUtil::GetElementText(parent));
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		continue;	   // For choices
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("InlineMedia"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("InlineMedia")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateInlineMediaType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetInlineMedia(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	} while(false);
	// Deserialize value element
	if(Mp7JrsUtil::HasNodeName(parent, X("StreamID"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		XMLCh * tmp = Mp7JrsUtil::GetElementText(parent);
		this->SetStreamID(Mp7JrsConvert::TextToUnsignedInt(tmp));
		XMLString::release(&tmp);
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsMediaLocatorType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMediaLocatorType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("InlineMedia")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateInlineMediaType;
	}
	this->SetInlineMedia(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMediaLocatorType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetInlineMedia() != Mp7JrsNodePtr())
		result.Insert(GetInlineMedia());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// begin extension included
// file Mp7JrsMediaLocatorType_ExtMethodImpl.h
Mp7JrsPtr<class IMp7JrsmediaTimePointType>  Mp7JrsMediaLocatorType::GetMediaTimeBase() const {
	if(!Mp7JrsSettings::GetInstance().TreatRelIncrTimeAsAbsolute)
		throw Mp7JrsException(MP7JRS_ERROR, X("Mp7JrsMediaLocatorType::GetMediaTimeBase(): You must enable extension by setting Mp7JrsSettings.TreatRelIncrTimeAsAbsolute to true."));
	return _mediaTimeBase;
}

void Mp7JrsMediaLocatorType::SetMediaTimeBase(const Mp7JrsmediaTimePointPtr &timePoint, Mp7JrsClientID client)
{
	if(!Mp7JrsSettings::GetInstance().TreatRelIncrTimeAsAbsolute)
		throw Mp7JrsException(MP7JRS_ERROR, X("Mp7JrsMediaLocatorType::SetMediaTimeBase(): You must enable extension by setting Mp7JrsSettings.TreatRelIncrTimeAsAbsolute to true."));

	if (!IsLocked(client))
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaLocatorType::SetMediaTimeBase().");

	_mediaTimeBase = timePoint;

	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}
// end extension included


