 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// begin extension included
// file Mp7JrsMediaRelIncrTimePointType_ExtImplInclude.h
#include "Mp7JrsmediaTimePointType.h"
#include "Mp7JrsMediaTimeType.h"
// end extension included



#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsMediaRelIncrTimePointType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsMediaRelIncrTimePointType::IMp7JrsMediaRelIncrTimePointType()
{

// no includefile for extension defined 
// file Mp7JrsMediaRelIncrTimePointType_ExtPropInit.h

}

IMp7JrsMediaRelIncrTimePointType::~IMp7JrsMediaRelIncrTimePointType()
{
// no includefile for extension defined 
// file Mp7JrsMediaRelIncrTimePointType_ExtPropCleanup.h

}

Mp7JrsMediaRelIncrTimePointType::Mp7JrsMediaRelIncrTimePointType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMediaRelIncrTimePointType::~Mp7JrsMediaRelIncrTimePointType()
{
	Cleanup();
}

void Mp7JrsMediaRelIncrTimePointType::Init()
{
	// Init base
	m_Base = 0;

	// Init attributes
	m_mediaTimeUnit = Mp7JrsmediaDurationPtr(); // Pattern
	m_mediaTimeUnit_Exist = false;
	m_mediaTimeBase = Mp7JrsxPathRefPtr(); // Pattern
	m_mediaTimeBase_Exist = false;



// begin extension included
// file Mp7JrsMediaRelIncrTimePointType_ExtMyPropInit.h
	m_AbsTimePointValid = false;
	m_ContentValid = true;
// end extension included

}

void Mp7JrsMediaRelIncrTimePointType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMediaRelIncrTimePointType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_mediaTimeUnit); // Pattern
	// Mp7JrsFactory::DeleteObject(m_mediaTimeBase); // Pattern
}

void Mp7JrsMediaRelIncrTimePointType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MediaRelIncrTimePointTypePtr, since we
	// might need GetBase(), which isn't defined in IMediaRelIncrTimePointType
	const Mp7JrsPtr< Mp7JrsMediaRelIncrTimePointType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	m_Base = tmp->GetContent();
	{
	// Mp7JrsFactory::DeleteObject(m_mediaTimeUnit); // Pattern
	if (tmp->ExistmediaTimeUnit())
	{
		this->SetmediaTimeUnit(Mp7JrsFactory::CloneObject(tmp->GetmediaTimeUnit()));
	}
	else
	{
		InvalidatemediaTimeUnit();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_mediaTimeBase); // Pattern
	if (tmp->ExistmediaTimeBase())
	{
		this->SetmediaTimeBase(Mp7JrsFactory::CloneObject(tmp->GetmediaTimeBase()));
	}
	else
	{
		InvalidatemediaTimeBase();
	}
	}
}

Mp7JrsNodePtr Mp7JrsMediaRelIncrTimePointType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsMediaRelIncrTimePointType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


void Mp7JrsMediaRelIncrTimePointType::SetContent(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelIncrTimePointType::SetContent().");
	}
	if (m_Base != item)
	{
		m_Base = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
	// Special behavior:
	m_AbsTimePointValid = false;
	m_ContentValid = true;
}

int Mp7JrsMediaRelIncrTimePointType::GetContent() const
{
	return m_Base;
}
Mp7JrsmediaDurationPtr Mp7JrsMediaRelIncrTimePointType::GetmediaTimeUnit() const
{
	return m_mediaTimeUnit;
}

bool Mp7JrsMediaRelIncrTimePointType::ExistmediaTimeUnit() const
{
	return m_mediaTimeUnit_Exist;
}
void Mp7JrsMediaRelIncrTimePointType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelIncrTimePointType::SetmediaTimeUnit().");
	}
	m_mediaTimeUnit = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_mediaTimeUnit) m_mediaTimeUnit->SetParent(m_myself.getPointer());
	m_mediaTimeUnit_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaRelIncrTimePointType::InvalidatemediaTimeUnit()
{
	m_mediaTimeUnit_Exist = false;
}
Mp7JrsxPathRefPtr Mp7JrsMediaRelIncrTimePointType::GetmediaTimeBase() const
{
	return m_mediaTimeBase;
}

bool Mp7JrsMediaRelIncrTimePointType::ExistmediaTimeBase() const
{
	return m_mediaTimeBase_Exist;
}
void Mp7JrsMediaRelIncrTimePointType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelIncrTimePointType::SetmediaTimeBase().");
	}
	m_mediaTimeBase = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_mediaTimeBase) m_mediaTimeBase->SetParent(m_myself.getPointer());
	m_mediaTimeBase_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaRelIncrTimePointType::InvalidatemediaTimeBase()
{
	m_mediaTimeBase_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsMediaRelIncrTimePointType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  2, 2 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 2
//- N; mediaTimeUnit,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
//- N; mediaTimeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("mediaTimeUnit")) == 0)
	{
		// mediaTimeUnit is simple attribute mediaDurationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GetmediaTimeUnit()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaDurationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsmediaDurationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetmediaTimeUnit(p, client);
					if((p = GetmediaTimeUnit()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("mediaTimeBase")) == 0)
	{
		// mediaTimeBase is simple attribute xPathRefType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GetmediaTimeBase()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:xPathRefType")))) != empty)
			{
				// Is type allowed
				Mp7JrsxPathRefPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetmediaTimeBase(p, client);
					if((p = GetmediaTimeBase()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MediaRelIncrTimePointType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MediaRelIncrTimePointType");
	}
	return result;
}

XMLCh * Mp7JrsMediaRelIncrTimePointType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMediaRelIncrTimePointType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsMediaRelIncrTimePointType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsMediaRelIncrTimePointType with workaround via Deserialise()
	return Mp7JrsXPathParseContext::ContentFromString(this, txt);
}
XMLCh* Mp7JrsMediaRelIncrTimePointType::ContentToString() const
{
	// Mp7JrsMediaRelIncrTimePointType with workaround via Serialise()
	return Mp7JrsXPathParseContext::ContentToString(this);
}

void Mp7JrsMediaRelIncrTimePointType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// Special behavior:
	// At this point, the node should aready be embeded in the MPEG-7 document.
	// If the content was not evaluated correctly at the beginning it should be possible now.
	if (m_AbsTimePointValid && !m_ContentValid)
		SetMediaTimePoint(m_AbsTimePoint);

	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_Base);
		element->appendChild(doc->createTextNode(tmp));
		XMLString::release(&tmp);
	}

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMediaRelIncrTimePointType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MediaRelIncrTimePointType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_mediaTimeUnit_Exist)
	{
	// Pattern
	if (m_mediaTimeUnit != Mp7JrsmediaDurationPtr())
	{
		XMLCh * tmp = m_mediaTimeUnit->ToText();
		element->setAttributeNS(X(""), X("mediaTimeUnit"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Attribute Serialization:
		

	// Optional attriute
	if(m_mediaTimeBase_Exist)
	{
	// Pattern
	if (m_mediaTimeBase != Mp7JrsxPathRefPtr())
	{
		XMLCh * tmp = m_mediaTimeBase->ToText();
		element->setAttributeNS(X(""), X("mediaTimeBase"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsMediaRelIncrTimePointType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("mediaTimeUnit")))
	{
		Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
		tmp->Parse(parent->getAttribute(X("mediaTimeUnit")));
		this->SetmediaTimeUnit(tmp);
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("mediaTimeBase")))
	{
		Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
		tmp->Parse(parent->getAttribute(X("mediaTimeBase")));
		this->SetmediaTimeBase(tmp);
		* current = parent;
	}

  m_Base = Mp7JrsConvert::TextToInt(Mp7JrsUtil::GetElementText(parent));
  found = true;
// no includefile for extension defined 
// file Mp7JrsMediaRelIncrTimePointType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMediaRelIncrTimePointType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMediaRelIncrTimePointType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// begin extension included
// file Mp7JrsMediaRelIncrTimePointType_ExtMethodImpl.h
void Mp7JrsMediaRelIncrTimePointType::SetMediaTimePoint(Mp7JrsmediaTimePointPtr timePoint, Mp7JrsClientID client) {
	if (timePoint == 0)
		return;
	int value = timePoint->GetTotalNrFractions();
	bool valueCorrect = false;
	// check if the node is already embeded in the MPEG-7 document. In this case we can evaluate the correct Content
	Mp7JrsMediaTimePtr mediaTime = GetParent();
	if (mediaTime != 0) {
		Mp7JrsmediaTimePointPtr base;
		Mp7JrsmediaDurationPtr unit;
		mediaTime->resolveBaseAndUnit(base, unit);
		if (base != 0 && base->GetFractionsPerSecond() != 0 && unit != 0 && unit->GetFractionsPerSecond() != 0) {
			value = (timePoint->GetTotalNrFractions() - base->GetTotalNrFractions()) * (double)base->GetFractionsPerSecond() / (double)timePoint->GetFractionsPerSecond() / (double)unit->GetTotalNrFractions();
			valueCorrect = true;
		}
	}
	SetContent(value, client);
	m_ContentValid = valueCorrect;
	m_AbsTimePoint = timePoint;
	m_AbsTimePointValid = true;
}

Mp7JrsmediaTimePointPtr Mp7JrsMediaRelIncrTimePointType::GetMediaTimePoint() {
	if (!m_AbsTimePointValid) {
		Mp7JrsMediaTimePtr mediaTime = GetParent();
		if (mediaTime != 0) {
			Mp7JrsmediaTimePointPtr base;
			Mp7JrsmediaDurationPtr unit;
			mediaTime->resolveBaseAndUnit(base, unit);
			if (base != 0 && base->GetFractionsPerSecond() != 0 && unit != 0 && unit->GetFractionsPerSecond() != 0) {
				int result = base->GetTotalNrFractions() + GetContent() * unit->GetTotalNrFractions();
				Mp7JrsmediaTimePointPtr absoluteTimePoint = CreatemediaTimePointType;
				absoluteTimePoint->SetFractionsPerSecond(unit->GetFractionsPerSecond());
				absoluteTimePoint->SetTotalNrFractions(result);
				m_AbsTimePoint = absoluteTimePoint;
				m_AbsTimePointValid = true;
			}
			else
				throw Mp7JrsException(MP7JRS_ERROR, "Cannot resolve mediaTimeUnit and mediaTimeBase to create an absolute TimePoint from a MediaRelIncrTimePoint");
		}
		else
			throw Mp7JrsException(MP7JRS_ERROR, "Cannot resolve mediaTimeUnit and mediaTimeBase to create a TimePoint from a MediaRelIncrTimePoint. Element is not within the document structure.");
	}
	return m_AbsTimePoint;
}
// end extension included


