 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMediaRelTimePointType_ExtImplInclude.h



#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsMediaRelTimePointType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsMediaRelTimePointType::IMp7JrsMediaRelTimePointType()
{

// no includefile for extension defined 
// file Mp7JrsMediaRelTimePointType_ExtPropInit.h

}

IMp7JrsMediaRelTimePointType::~IMp7JrsMediaRelTimePointType()
{
// no includefile for extension defined 
// file Mp7JrsMediaRelTimePointType_ExtPropCleanup.h

}

Mp7JrsMediaRelTimePointType::Mp7JrsMediaRelTimePointType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMediaRelTimePointType::~Mp7JrsMediaRelTimePointType()
{
	Cleanup();
}

void Mp7JrsMediaRelTimePointType::Init()
{
	// Init base
	m_Base = CreatemediaTimeOffsetType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_mediaTimeBase = Mp7JrsxPathRefPtr(); // Pattern
	m_mediaTimeBase_Exist = false;



// no includefile for extension defined 
// file Mp7JrsMediaRelTimePointType_ExtMyPropInit.h

}

void Mp7JrsMediaRelTimePointType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMediaRelTimePointType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_mediaTimeBase); // Pattern
}

void Mp7JrsMediaRelTimePointType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MediaRelTimePointTypePtr, since we
	// might need GetBase(), which isn't defined in IMediaRelTimePointType
	const Mp7JrsPtr< Mp7JrsMediaRelTimePointType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsmediaTimeOffsetPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMediaRelTimePointType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_mediaTimeBase); // Pattern
	if (tmp->ExistmediaTimeBase())
	{
		this->SetmediaTimeBase(Mp7JrsFactory::CloneObject(tmp->GetmediaTimeBase()));
	}
	else
	{
		InvalidatemediaTimeBase();
	}
	}
}

Mp7JrsNodePtr Mp7JrsMediaRelTimePointType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsMediaRelTimePointType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsmediaTimeOffsetType > Mp7JrsMediaRelTimePointType::GetBase() const
{
	return m_Base;
}

Mp7JrsxPathRefPtr Mp7JrsMediaRelTimePointType::GetmediaTimeBase() const
{
	return m_mediaTimeBase;
}

bool Mp7JrsMediaRelTimePointType::ExistmediaTimeBase() const
{
	return m_mediaTimeBase_Exist;
}
void Mp7JrsMediaRelTimePointType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelTimePointType::SetmediaTimeBase().");
	}
	m_mediaTimeBase = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_mediaTimeBase) m_mediaTimeBase->SetParent(m_myself.getPointer());
	m_mediaTimeBase_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMediaRelTimePointType::InvalidatemediaTimeBase()
{
	m_mediaTimeBase_Exist = false;
}
int Mp7JrsMediaRelTimePointType::GetOffsetSign() const
{
	return GetBase()->GetOffsetSign();
}

void Mp7JrsMediaRelTimePointType::SetOffsetSign(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelTimePointType::SetOffsetSign().");
	}
	GetBase()->SetOffsetSign(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsMediaRelTimePointType::GetDays() const
{
	return GetBase()->GetDays();
}

void Mp7JrsMediaRelTimePointType::SetDays(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelTimePointType::SetDays().");
	}
	GetBase()->SetDays(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsMediaRelTimePointType::GetHours() const
{
	return GetBase()->GetHours();
}

void Mp7JrsMediaRelTimePointType::SetHours(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelTimePointType::SetHours().");
	}
	GetBase()->SetHours(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsMediaRelTimePointType::GetMinutes() const
{
	return GetBase()->GetMinutes();
}

void Mp7JrsMediaRelTimePointType::SetMinutes(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelTimePointType::SetMinutes().");
	}
	GetBase()->SetMinutes(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsMediaRelTimePointType::GetSeconds() const
{
	return GetBase()->GetSeconds();
}

void Mp7JrsMediaRelTimePointType::SetSeconds(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelTimePointType::SetSeconds().");
	}
	GetBase()->SetSeconds(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsMediaRelTimePointType::GetNumberOfFractions() const
{
	return GetBase()->GetNumberOfFractions();
}

void Mp7JrsMediaRelTimePointType::SetNumberOfFractions(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelTimePointType::SetNumberOfFractions().");
	}
	GetBase()->SetNumberOfFractions(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsMediaRelTimePointType::GetTime_Exist() const
{
	return GetBase()->GetTime_Exist();
}

void Mp7JrsMediaRelTimePointType::SetTime_Exist(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelTimePointType::SetTime_Exist().");
	}
	GetBase()->SetTime_Exist(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsMediaRelTimePointType::GetFractionsPerSecond() const
{
	return GetBase()->GetFractionsPerSecond();
}

void Mp7JrsMediaRelTimePointType::SetFractionsPerSecond(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaRelTimePointType::SetFractionsPerSecond().");
	}
	GetBase()->SetFractionsPerSecond(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsMediaRelTimePointType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; mediaTimeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("mediaTimeBase")) == 0)
	{
		// mediaTimeBase is simple attribute xPathRefType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GetmediaTimeBase()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:xPathRefType")))) != empty)
			{
				// Is type allowed
				Mp7JrsxPathRefPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetmediaTimeBase(p, client);
					if((p = GetmediaTimeBase()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MediaRelTimePointType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MediaRelTimePointType");
	}
	return result;
}

XMLCh * Mp7JrsMediaRelTimePointType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMediaRelTimePointType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsMediaRelTimePointType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsMediaRelTimePointType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsMediaRelTimePointType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	m_Base->Serialize(doc, element, newElem);

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMediaRelTimePointType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MediaRelTimePointType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_mediaTimeBase_Exist)
	{
	// Pattern
	if (m_mediaTimeBase != Mp7JrsxPathRefPtr())
	{
		XMLCh * tmp = m_mediaTimeBase->ToText();
		element->setAttributeNS(X(""), X("mediaTimeBase"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsMediaRelTimePointType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("mediaTimeBase")))
	{
		Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
		tmp->Parse(parent->getAttribute(X("mediaTimeBase")));
		this->SetmediaTimeBase(tmp);
		* current = parent;
	}

  // Extensionbase is Mp7JrsmediaTimeOffsetType
  // Deserialize cce factory type
  if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
  }
// no includefile for extension defined 
// file Mp7JrsMediaRelTimePointType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMediaRelTimePointType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMediaRelTimePointType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// begin extension included
// file Mp7JrsMediaRelTimePointType_ExtMethodImpl.h
#include "Mp7JrsmediaTimePointType.h"
#include "Mp7JrsMediaTimeType.h"
#include "Mp7JrsMediaLocatorType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsDSType.h"

int IMp7JrsMediaRelTimePointType::GetTotalNrFractions() {
	int sum = 0;
	// days missing!!!!!
	sum = 60 * (GetHours() + sum);
	sum = 60 * (GetMinutes() + sum);
	sum = GetFractionsPerSecond() * (GetSeconds() + sum);
	sum += GetNumberOfFractions(); 
	return sum;
}

void IMp7JrsMediaRelTimePointType::SetTotalNrFractions(int fractions, Mp7JrsClientID client) {
	int remainder = fractions;
	SetNumberOfFractions(remainder % GetFractionsPerSecond(),client);
	remainder /= GetFractionsPerSecond();
	SetSeconds (remainder % 60,client);
	remainder /= 60;
	SetMinutes (remainder % 60,client);
	remainder /= 60;
	SetHours (remainder,client);
	// days ignored
}



Mp7JrsmediaTimePointPtr IMp7JrsMediaRelTimePointType::GetAbsMediaTimePoint() {
	Mp7JrsmediaTimePointPtr absTp = CreatemediaTimePointType;
	Mp7JrsmediaTimePointPtr relTp = _GetMediaTimeBase();
	absTp->SetFractionsPerSecond(GetFractionsPerSecond());
	int nFractions = (int)(GetTotalNrFractions()+(relTp->GetTotalNrFractions()*1.0/relTp->GetFractionsPerSecond())*GetFractionsPerSecond());
	absTp->SetTotalNrFractions(nFractions);
	return absTp;
}

void IMp7JrsMediaRelTimePointType::SetAbsMediaTimePoint(Mp7JrsmediaTimePointPtr tp, Mp7JrsClientID client) {
	Mp7JrsmediaTimePointPtr relTp = _GetMediaTimeBase();
	SetFractionsPerSecond(tp->GetFractionsPerSecond(),client);
	int nFractions = (int)(tp->GetTotalNrFractions()+(relTp->GetTotalNrFractions()*1.0/relTp->GetFractionsPerSecond())*tp->GetFractionsPerSecond());
	SetTotalNrFractions(nFractions,client);	
}

Mp7JrsmediaTimePointPtr IMp7JrsMediaRelTimePointType::_GetMediaTimeBase() {
	Mp7JrsxPathRefPtr mtbPath;
	Mp7JrsNodePtr parent;
	if (ExistmediaTimeBase()) mtbPath = GetmediaTimeBase();
	else {
		parent = GetParent();
		while (parent) {
			Mp7JrsDSPtr dstype = parent;
			if (dstype) {
				if (dstype->ExistmediaTimeBase()) {
					mtbPath = dstype->GetmediaTimeBase();
					break;
				}
			}
			parent = parent->GetParent();
		}
	}

	
	Mp7JrsmediaTimePointPtr nullTp;
	// if no path found return empty time point
	if (mtbPath==NULL) return nullTp;

	// try to resolve XPath
	Mp7JrsNodePtr target;
	if (parent) target = parent->NodeFromXPath(mtbPath->GetContent());
	else target = NodeFromXPath(mtbPath->GetContent());

	// no target found
	if (target==NULL) return nullTp;

	// check if target is a MediaTimeType
	Mp7JrsMediaTimePtr mediaTime = target;
	if (mediaTime) {
		// try to get time point
		Mp7JrsmediaTimePointPtr mtp = mediaTime->GetMediaTimePoint();
		// if not, try to get absolute value of relative tp
		if (mtp!=NULL) return mtp;

		Mp7JrsMediaRelTimePointPtr mrtp = mediaTime->GetMediaRelTimePoint();
		if (mrtp==NULL) return nullTp;

		mtp = mrtp->GetAbsMediaTimePoint();
		return mtp;
	}
	// check if target is a MediaLocator
	Mp7JrsMediaLocatorPtr mediaLoc = target;
	if (mediaLoc) {
		// not well defined in schema, has to be done in own extension
		return nullTp;
	}

	return nullTp;
}
// end extension included


