 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMediaTimeType_ExtImplInclude.h



#include "Mp7JrsmediaTimePointType.h"
#include "Mp7JrsMediaRelTimePointType.h"
#include "Mp7JrsMediaRelIncrTimePointType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsMediaIncrDurationType.h"
#include "Mp7JrsMediaTimeType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include "Mp7JrsSettings.h" // For TreatRelIncrTimeAsAbsolute
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsDSType.h"
#include "Mp7JrsMediaLocatorType.h"
#include "Mp7JrsTemporalSegmentLocatorType.h" // FTT Maybe not neccessary for AVDP
#include "Mp7JrsImageLocatorType.h" // FTT Maybe not neccessary for AVDP
#include "Mp7JrsStreamLocatorType.h" // FTT Maybe not neccessary for AVDP
#include "Mp7JrsLogicalUnitLocatorType.h" // FTT Maybe not neccessary for AVDP

#include <assert.h>
IMp7JrsMediaTimeType::IMp7JrsMediaTimeType()
{

// begin extension included
// file Mp7JrsMediaTimeType_ExtPropInit.h

	_endTime = CreatemediaTimePointType;
// end extension included

}

IMp7JrsMediaTimeType::~IMp7JrsMediaTimeType()
{
// no includefile for extension defined 
// file Mp7JrsMediaTimeType_ExtPropCleanup.h

}

Mp7JrsMediaTimeType::Mp7JrsMediaTimeType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMediaTimeType::~Mp7JrsMediaTimeType()
{
	Cleanup();
}

void Mp7JrsMediaTimeType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_MediaTimePoint = Mp7JrsmediaTimePointPtr(); // Pattern
	m_MediaRelTimePoint = Mp7JrsMediaRelTimePointPtr(); // Class
	m_MediaRelIncrTimePoint = Mp7JrsMediaRelIncrTimePointPtr(); // Class with content 
	m_MediaDuration = Mp7JrsmediaDurationPtr(); // Pattern
	m_MediaIncrDuration = Mp7JrsMediaIncrDurationPtr(); // Class with content 


// begin extension included
// file Mp7JrsMediaTimeType_ExtMyPropInit.h


// end extension included

}

void Mp7JrsMediaTimeType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMediaTimeType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
	// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
	// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
	// Mp7JrsFactory::DeleteObject(m_MediaDuration);
	// Mp7JrsFactory::DeleteObject(m_MediaIncrDuration);
}

void Mp7JrsMediaTimeType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MediaTimeTypePtr, since we
	// might need GetBase(), which isn't defined in IMediaTimeType
	const Mp7JrsPtr< Mp7JrsMediaTimeType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
		this->SetMediaTimePoint(Mp7JrsFactory::CloneObject(tmp->GetMediaTimePoint()));
		// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
		this->SetMediaRelTimePoint(Mp7JrsFactory::CloneObject(tmp->GetMediaRelTimePoint()));
		// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
		this->SetMediaRelIncrTimePoint(Mp7JrsFactory::CloneObject(tmp->GetMediaRelIncrTimePoint()));
		// Mp7JrsFactory::DeleteObject(m_MediaDuration);
		this->SetMediaDuration(Mp7JrsFactory::CloneObject(tmp->GetMediaDuration()));
		// Mp7JrsFactory::DeleteObject(m_MediaIncrDuration);
		this->SetMediaIncrDuration(Mp7JrsFactory::CloneObject(tmp->GetMediaIncrDuration()));
}

Mp7JrsNodePtr Mp7JrsMediaTimeType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsMediaTimeType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsmediaTimePointPtr Mp7JrsMediaTimeType::GetMediaTimePoint() const
{

	// GetMediaTimePoint() behaves special
	if (Mp7JrsSettings::GetInstance().TreatRelIncrTimeAsAbsolute) {

		// FTT
		// This is AVDP code
		// We do a special treatment for MediaTime in AVDP
		// In MPEG-7 V2 MediaTime offers a choice of MediaTimePoint, MediaRelTimePoint and MediaRelIncrTimePoint
		// In AVDP we only have MediaRelIncrTimePoint (+ and optional MediaIncrDuration)

		// Since we don't want an extra lib for AVDP
		// the MediaTimePoint is used to retrieve the absolute timepoint
		// calculated from the MediaRelIncrTimePoint and the referenced mediaTimeUnit and mediaTimeBase.
		// mediaTimeBase points to a MediaLocator, that now contains an absolute TimePointType
		// so that MediaTimePoint (absolute) = MediaRelIncrTimePoint * mediaTimeUnit + mediaTimeBase (from MediaLocator)
		// This behaviour is only available if 
		// Mp7JrsSettings::GetInstance().TreatRelIncrTimeAsAbsolute is set to true.


		// Find mediaTimeUnit and mediaTimeBase - which is a MediaLocator -
		// get the final media time base from the medialocator
		// and map the MediaRelIncrTime to absolute time point.

		// Is there an AVDP-relative time value to be converted to absolute media time?
		if(GetMediaRelIncrTimePoint() != NULL) {
			return GetMediaRelIncrTimePoint()->GetMediaTimePoint();
		}
	}
		return m_MediaTimePoint;
}

Mp7JrsMediaRelTimePointPtr Mp7JrsMediaTimeType::GetMediaRelTimePoint() const
{
		return m_MediaRelTimePoint;
}

Mp7JrsMediaRelIncrTimePointPtr Mp7JrsMediaTimeType::GetMediaRelIncrTimePoint() const
{
		return m_MediaRelIncrTimePoint;
}

Mp7JrsmediaDurationPtr Mp7JrsMediaTimeType::GetMediaDuration() const
{
	// GetMediaDuration() behaves special
	if (Mp7JrsSettings::GetInstance().TreatRelIncrTimeAsAbsolute) {
		if (GetMediaIncrDuration() != 0) {
			return GetMediaIncrDuration()->GetMediaDuration();
		}
	}
		return m_MediaDuration;
}

Mp7JrsMediaIncrDurationPtr Mp7JrsMediaTimeType::GetMediaIncrDuration() const
{
		return m_MediaIncrDuration;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsMediaTimeType::SetMediaTimePoint(const Mp7JrsmediaTimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaTimeType::SetMediaTimePoint().");
	}

	if (Mp7JrsSettings::GetInstance().TreatRelIncrTimeAsAbsolute) {

		// FTT
		// This is AVDP code
		// We do a special treatment for MediaTime in AVDP
		// In MPEG-7 V2 MediaTime offers a choice of MediaTimePoint, MediaRelTimePoint and MediaRelIncrTimePoint
		// In AVDP we only have MediaRelIncrTimePoint (+ and optional MediaIncrDuration)

		// Since we don't want an extra lib for AVDP
		// the MediaTimePoint is used to set the absolute timepoint
		// but converted to AVDP expected RelIncrTimePointType
		// For conversion we need the mediaTimeUnit and mediaTimeBase
		// that may come from a MediaLocator in the DOM above

		// so MediaRelIncrTimePoint and the referenced mediaTimeUnit and mediaTimeBase.
		// mediaTimeBase points to a MediaLocator, that now contains an absolute TimePointType
		// so that MediaTimePoint (absolute) = MediaRelIncrTimePoint * mediaTimeUnit + mediaTimeBase (from MediaLocator)
		// This behaviour is only available if 
		// Mp7JrsSettings::GetInstance().TreatRelIncrTimeAsAbsolute is set to true.


		// Find mediaTimeUnit and mediaTimeBase - which is a MediaLocator -
		// get the final media time base from the medialocator
		// and map the MediaRelIncrTime to absolute time point.
		if (item == 0) // setting an empty item means to clear the node.
			SetMediaRelIncrTimePoint(Mp7JrsNodePtr());
		else {
			if (GetMediaRelIncrTimePoint() == 0)
				SetMediaRelIncrTimePoint(CreateMediaRelIncrTimePointType);
			GetMediaRelIncrTimePoint()->SetMediaTimePoint(item);
		}
		return;
	}
	if (m_MediaTimePoint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
		m_MediaTimePoint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaTimePoint) m_MediaTimePoint->SetParent(m_myself.getPointer());
		if (m_MediaTimePoint && m_MediaTimePoint->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaTimePointType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaTimePoint->UseTypeAttribute = true;
		}
		if(m_MediaTimePoint != Mp7JrsmediaTimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaTimePoint->SetContentName(XMLString::transcode("MediaTimePoint"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
	m_MediaRelTimePoint = Mp7JrsMediaRelTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
	m_MediaRelIncrTimePoint = Mp7JrsMediaRelIncrTimePointPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMediaTimeType::SetMediaRelTimePoint(const Mp7JrsMediaRelTimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaTimeType::SetMediaRelTimePoint().");
	}
	if (m_MediaRelTimePoint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
		m_MediaRelTimePoint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaRelTimePoint) m_MediaRelTimePoint->SetParent(m_myself.getPointer());
		if (m_MediaRelTimePoint && m_MediaRelTimePoint->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaRelTimePointType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaRelTimePoint->UseTypeAttribute = true;
		}
		if(m_MediaRelTimePoint != Mp7JrsMediaRelTimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaRelTimePoint->SetContentName(XMLString::transcode("MediaRelTimePoint"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
	m_MediaTimePoint = Mp7JrsmediaTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
	m_MediaRelIncrTimePoint = Mp7JrsMediaRelIncrTimePointPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMediaTimeType::SetMediaRelIncrTimePoint(const Mp7JrsMediaRelIncrTimePointPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaTimeType::SetMediaRelIncrTimePoint().");
	}
	if (m_MediaRelIncrTimePoint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaRelIncrTimePoint);
		m_MediaRelIncrTimePoint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaRelIncrTimePoint) m_MediaRelIncrTimePoint->SetParent(m_myself.getPointer());
		if (m_MediaRelIncrTimePoint && m_MediaRelIncrTimePoint->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaRelIncrTimePointType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaRelIncrTimePoint->UseTypeAttribute = true;
		}
		if(m_MediaRelIncrTimePoint != Mp7JrsMediaRelIncrTimePointPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaRelIncrTimePoint->SetContentName(XMLString::transcode("MediaRelIncrTimePoint"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaTimePoint);
	m_MediaTimePoint = Mp7JrsmediaTimePointPtr();
	// Mp7JrsFactory::DeleteObject(m_MediaRelTimePoint);
	m_MediaRelTimePoint = Mp7JrsMediaRelTimePointPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsMediaTimeType::SetMediaDuration(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaTimeType::SetMediaDuration().");
	}

	if (Mp7JrsSettings::GetInstance().TreatRelIncrTimeAsAbsolute) {
		if (item == 0) // setting an empty item means to clear the node.
			SetMediaIncrDuration(Mp7JrsNodePtr());
		else {
			if (GetMediaIncrDuration() == 0)
				SetMediaIncrDuration(CreateMediaIncrDurationType);
			GetMediaIncrDuration()->SetMediaDuration(item);
		}
		return;
	}
	if (m_MediaDuration != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaDuration);
		m_MediaDuration = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaDuration) m_MediaDuration->SetParent(m_myself.getPointer());
		if (m_MediaDuration && m_MediaDuration->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaDurationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaDuration->UseTypeAttribute = true;
		}
		if(m_MediaDuration != Mp7JrsmediaDurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaDuration->SetContentName(XMLString::transcode("MediaDuration"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaIncrDuration);
	m_MediaIncrDuration = Mp7JrsMediaIncrDurationPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMediaTimeType::SetMediaIncrDuration(const Mp7JrsMediaIncrDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMediaTimeType::SetMediaIncrDuration().");
	}
	if (m_MediaIncrDuration != item)
	{
		// Mp7JrsFactory::DeleteObject(m_MediaIncrDuration);
		m_MediaIncrDuration = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MediaIncrDuration) m_MediaIncrDuration->SetParent(m_myself.getPointer());
		if (m_MediaIncrDuration && m_MediaIncrDuration->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaIncrDurationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_MediaIncrDuration->UseTypeAttribute = true;
		}
		if(m_MediaIncrDuration != Mp7JrsMediaIncrDurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MediaIncrDuration->SetContentName(XMLString::transcode("MediaIncrDuration"));
		}
	// Mp7JrsFactory::DeleteObject(m_MediaDuration);
	m_MediaDuration = Mp7JrsmediaDurationPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsMediaTimeType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("MediaTimePoint")) == 0)
	{
		// MediaTimePoint is simple element mediaTimePointType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaTimePoint()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaTimePointType")))) != empty)
			{
				// Is type allowed
				Mp7JrsmediaTimePointPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaTimePoint(p, client);
					if((p = GetMediaTimePoint()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaRelTimePoint")) == 0)
	{
		// MediaRelTimePoint is simple element MediaRelTimePointType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaRelTimePoint()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaRelTimePointType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaRelTimePointPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaRelTimePoint(p, client);
					if((p = GetMediaRelTimePoint()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaRelIncrTimePoint")) == 0)
	{
		// MediaRelIncrTimePoint is simple element MediaRelIncrTimePointType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaRelIncrTimePoint()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaRelIncrTimePointType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaRelIncrTimePointPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaRelIncrTimePoint(p, client);
					if((p = GetMediaRelIncrTimePoint()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaDuration")) == 0)
	{
		// MediaDuration is simple element mediaDurationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaDuration()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaDurationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsmediaDurationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaDuration(p, client);
					if((p = GetMediaDuration()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MediaIncrDuration")) == 0)
	{
		// MediaIncrDuration is simple element MediaIncrDurationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMediaIncrDuration()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaIncrDurationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaIncrDurationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMediaIncrDuration(p, client);
					if((p = GetMediaIncrDuration()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MediaTimeType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MediaTimeType");
	}
	return result;
}

XMLCh * Mp7JrsMediaTimeType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMediaTimeType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsMediaTimeType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MediaTimeType"));
	// Element serialization:
	if(m_MediaTimePoint != Mp7JrsmediaTimePointPtr()) // Pattern FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Pattern
		// CON contentname already set in element setter -- not needed here
//		XMLCh * contentname = XMLString::transcode("MediaTimePoint");
//		m_MediaTimePoint->SetContentName(contentname);
		m_MediaTimePoint->UseTypeAttribute = this->UseTypeAttribute;
		m_MediaTimePoint->Serialize(doc, element);
	}
	// Class
	
	if (m_MediaRelTimePoint != Mp7JrsMediaRelTimePointPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaRelTimePoint->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaRelTimePoint"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaRelTimePoint->Serialize(doc, element, &elem);
	}
	// Class with content		
	
	if (m_MediaRelIncrTimePoint != Mp7JrsMediaRelIncrTimePointPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaRelIncrTimePoint->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaRelIncrTimePoint"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaRelIncrTimePoint->Serialize(doc, element, &elem);
	}
	if(m_MediaDuration != Mp7JrsmediaDurationPtr()) // Pattern FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Pattern
		// CON contentname already set in element setter -- not needed here
//		XMLCh * contentname = XMLString::transcode("MediaDuration");
//		m_MediaDuration->SetContentName(contentname);
		m_MediaDuration->UseTypeAttribute = this->UseTypeAttribute;
		m_MediaDuration->Serialize(doc, element);
	}
	// Class with content		
	
	if (m_MediaIncrDuration != Mp7JrsMediaIncrDurationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_MediaIncrDuration->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("MediaIncrDuration"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_MediaIncrDuration->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsMediaTimeType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
	do // Deserialize choice just one time!
	{
		// Deserialize pattern element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaTimePoint"), X("urn:mpeg:mpeg7:schema:2004")))
//			if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaTimePoint")) == 0))
		{
			Mp7JrsmediaTimePointPtr tmp = CreatemediaTimePointType;
			tmp->Deserialize(doc, parent, current);
			this->SetMediaTimePoint(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaRelTimePoint"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaRelTimePoint")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMediaRelTimePointType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetMediaRelTimePoint(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaRelIncrTimePoint"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaRelIncrTimePoint")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMediaRelIncrTimePointType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetMediaRelIncrTimePoint(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	} while(false);
	do // Deserialize choice just one time!
	{
		// Deserialize pattern element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaDuration"), X("urn:mpeg:mpeg7:schema:2004")))
//			if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaDuration")) == 0))
		{
			Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
			tmp->Deserialize(doc, parent, current);
			this->SetMediaDuration(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("MediaIncrDuration"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("MediaIncrDuration")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMediaIncrDurationType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetMediaIncrDuration(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	} while(false);
// no includefile for extension defined 
// file Mp7JrsMediaTimeType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMediaTimeType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("MediaTimePoint")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreatemediaTimePointType;
	}
	this->SetMediaTimePoint(child);
  }
  if (XMLString::compareString(elementname, X("MediaRelTimePoint")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMediaRelTimePointType;
	}
	this->SetMediaRelTimePoint(child);
  }
  if (XMLString::compareString(elementname, X("MediaRelIncrTimePoint")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMediaRelIncrTimePointType;
	}
	this->SetMediaRelIncrTimePoint(child);
  }
  if (XMLString::compareString(elementname, X("MediaDuration")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreatemediaDurationType;
	}
	this->SetMediaDuration(child);
  }
  if (XMLString::compareString(elementname, X("MediaIncrDuration")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMediaIncrDurationType;
	}
	this->SetMediaIncrDuration(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMediaTimeType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetMediaTimePoint() != Mp7JrsNodePtr())
		result.Insert(GetMediaTimePoint());
	if (GetMediaRelTimePoint() != Mp7JrsNodePtr())
		result.Insert(GetMediaRelTimePoint());
	if (GetMediaRelIncrTimePoint() != Mp7JrsNodePtr())
		result.Insert(GetMediaRelIncrTimePoint());
	if (GetMediaDuration() != Mp7JrsNodePtr())
		result.Insert(GetMediaDuration());
	if (GetMediaIncrDuration() != Mp7JrsNodePtr())
		result.Insert(GetMediaIncrDuration());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// begin extension included
// file Mp7JrsMediaTimeType_ExtMethodImpl.h

Mp7JrsmediaTimePointPtr IMp7JrsMediaTimeType::GetBeginTime() {
	return GetMediaTimePoint();
}

void IMp7JrsMediaTimeType::SetBeginTime(const Mp7JrsmediaTimePointPtr &beginTime, Mp7JrsClientID client)
{
	// FIXME: generated Setter methods store the passed object, not a
	// clone of it.  Should we do the same here?
	SetMediaTimePoint(Mp7JrsmediaTimePointPtr(Mp7JrsFactory::CloneObject(beginTime)),client);
}

Mp7JrsmediaTimePointPtr IMp7JrsMediaTimeType::GetEndTime() {

	Mp7JrsFactory::DeleteObject(_endTime);

	_endTime = (Mp7JrsmediaTimePointPtr) Mp7JrsFactory::CloneObject(GetBeginTime());
	
	if (GetMediaDuration()) {
		_endTime->Add(GetMediaDuration());
	}
	return _endTime;
}

void IMp7JrsMediaTimeType::SetEndTime(const Mp7JrsmediaTimePointPtr &endTime, Mp7JrsClientID client)
{
	Mp7JrsFactory::DeleteObject(_endTime);
	_endTime = Mp7JrsmediaTimePointPtr(Mp7JrsFactory::CloneObject(endTime));
	SetMediaDuration(_endTime->Subtract(GetMediaTimePoint()),client);
}

void IMp7JrsMediaTimeType::GetTime(Mp7JrsmediaTimePointPtr& beginTime, Mp7JrsmediaTimePointPtr& endTime) {
	beginTime = GetBeginTime();
	endTime = GetEndTime();
}

void IMp7JrsMediaTimeType::SetTime(const Mp7JrsmediaTimePointPtr &beginTime, const Mp7JrsmediaTimePointPtr &endTime, Mp7JrsClientID client) {
	SetBeginTime(beginTime,client);
	SetEndTime(endTime,client);
}

bool IMp7JrsMediaTimeType::IsInside(const Mp7JrsmediaTimePointPtr &timePoint){

	Mp7JrsmediaTimePointPtr endTime = GetEndTime();
	Mp7JrsmediaTimePointPtr beginTime = GetBeginTime();

	if ((timePoint->GetTotalNrFractions() >= beginTime->GetTotalNrFractions())
		&& (timePoint->GetTotalNrFractions() <= endTime->GetTotalNrFractions())) {
		return true;
	}
	else {
		return false;
	}

}

void Mp7JrsMediaTimeType::resolveBaseAndUnit(Mp7JrsmediaTimePointPtr &base, Mp7JrsmediaDurationPtr &unit) {
	// Is there a related mediaTimeBase out there?
	Mp7JrsxPathRefPtr pathToMediaLocator;
	Mp7JrsNodePtr parent;
	Mp7JrsNodePtr targetForBase; // Target has to be a MediaLocator (customized to contain a MediaTimePoint)

	bool resolvedBase = false;
	bool resolvedUnit = false;
	// Is there a related mediaTimeUnit out there?
	Mp7JrsmediaDurationPtr mtu; // NULL 
	if (GetMediaRelIncrTimePoint() != 0 && GetMediaRelIncrTimePoint()->ExistmediaTimeUnit()) {
		mtu = GetMediaRelIncrTimePoint()->GetmediaTimeUnit();
		resolvedUnit = true;
	}
	if (GetMediaRelIncrTimePoint() != 0 && GetMediaRelIncrTimePoint()->ExistmediaTimeBase()) {
		pathToMediaLocator = GetMediaRelIncrTimePoint()->GetmediaTimeBase();
		XMLCh* path = pathToMediaLocator->GetContent();

		Mp7JrsNodeEnum e1 = GetMediaRelIncrTimePoint()->GetFromXPath(path);
		if(e1.HasNext()) {
			targetForBase = e1.GetNext();
			resolvedBase = true;
		}
	}
	if (!resolvedBase || !resolvedUnit) { // Search in DOM
		parent = GetParent();
		while (parent && (!resolvedBase || !resolvedUnit)) {
			Mp7JrsDSPtr dstype = parent;
			if (dstype != 0) {
				if (!resolvedBase && dstype->ExistmediaTimeBase()) {
					pathToMediaLocator = dstype->GetmediaTimeBase();
					XMLCh* path = pathToMediaLocator->GetContent();
					Mp7JrsNodeEnum e1 = dstype->GetFromXPath(path);
					if(e1.HasNext()) {
						targetForBase = e1.GetNext();
						resolvedBase = true;
					}
				}
				if (!resolvedUnit && dstype->ExistmediaTimeUnit()) {
					mtu = dstype->GetmediaTimeUnit();
					resolvedUnit = true;
				}
			}
			parent = parent->GetParent();
		}
	}

	// Check existence of mediaTimeUnit
	if(mtu != 0) {
		unit = mtu;
		base = CreatemediaTimePointType;
		base->SetFractionsPerSecond(unit->GetFractionsPerSecond());
		base->SetNumberOfFractions(0);
	}

	// Resolve MediaLocator which contains the mediatimepoint - the absolute timebase
	// If not found, assume timebase starts with 0.
	if (targetForBase != 0) {
		// Check if target has a MediaTimeType
		{
			Mp7JrsMediaLocatorPtr mediaLocator = targetForBase;
			if(mediaLocator != 0 && mediaLocator->GetMediaTimeBase() != 0) 
				base = mediaLocator->GetMediaTimeBase();
		}
		{
			Mp7JrsTemporalSegmentLocatorPtr mediaLocator = targetForBase;
			if(mediaLocator != 0 && mediaLocator->GetMediaTimeBase() != 0) 
				base = mediaLocator->GetMediaTimeBase();
		}
		{
			Mp7JrsImageLocatorPtr mediaLocator = targetForBase;
			if(mediaLocator != 0 && mediaLocator->GetMediaTimeBase() != 0) 
				base = mediaLocator->GetMediaTimeBase();
		}
		{
			Mp7JrsStreamLocatorPtr mediaLocator = targetForBase;
			if(mediaLocator != 0 && mediaLocator->GetMediaTimeBase() != 0) 
				base = mediaLocator->GetMediaTimeBase();
		}
		{
			Mp7JrsLogicalUnitLocatorPtr mediaLocator = targetForBase;
			if(mediaLocator != 0 && mediaLocator->GetMediaTimeBase() != 0) 
				base = mediaLocator->GetMediaTimeBase();
		}
	}
}
// end extension included


