 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMixedCollectionType_LocalType_ExtImplInclude.h



#include "Mp7JrsMultimediaContentType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsMixedCollectionType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsMixedCollectionType_LocalType::IMp7JrsMixedCollectionType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsMixedCollectionType_LocalType_ExtPropInit.h

}

IMp7JrsMixedCollectionType_LocalType::~IMp7JrsMixedCollectionType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsMixedCollectionType_LocalType_ExtPropCleanup.h

}

Mp7JrsMixedCollectionType_LocalType::Mp7JrsMixedCollectionType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMixedCollectionType_LocalType::~Mp7JrsMixedCollectionType_LocalType()
{
	Cleanup();
}

void Mp7JrsMixedCollectionType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_Content = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_ContentRef = Mp7JrsReferencePtr(); // Class


// no includefile for extension defined 
// file Mp7JrsMixedCollectionType_LocalType_ExtMyPropInit.h

}

void Mp7JrsMixedCollectionType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMixedCollectionType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Content);
	// Mp7JrsFactory::DeleteObject(m_ContentRef);
}

void Mp7JrsMixedCollectionType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MixedCollectionType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IMixedCollectionType_LocalType
	const Mp7JrsPtr< Mp7JrsMixedCollectionType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_Content);
		this->SetContent(Mp7JrsFactory::CloneObject(tmp->GetContent()));
		// Mp7JrsFactory::DeleteObject(m_ContentRef);
		this->SetContentRef(Mp7JrsFactory::CloneObject(tmp->GetContentRef()));
}

Mp7JrsNodePtr Mp7JrsMixedCollectionType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsMixedCollectionType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsPtr< Mp7JrsNode > Mp7JrsMixedCollectionType_LocalType::GetContent() const
{
		return m_Content;
}

Mp7JrsReferencePtr Mp7JrsMixedCollectionType_LocalType::GetContentRef() const
{
		return m_ContentRef;
}

// implementing setter for choice 
/* element */
void Mp7JrsMixedCollectionType_LocalType::SetContent(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMixedCollectionType_LocalType::SetContent().");
	}
	if (m_Content != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Content);
		m_Content = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Content) m_Content->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_Content) m_Content->UseTypeAttribute = true;
		if(m_Content != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Content->SetContentName(XMLString::transcode("Content"));
		}
	// Mp7JrsFactory::DeleteObject(m_ContentRef);
	m_ContentRef = Mp7JrsReferencePtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMixedCollectionType_LocalType::SetContentRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMixedCollectionType_LocalType::SetContentRef().");
	}
	if (m_ContentRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ContentRef);
		m_ContentRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ContentRef) m_ContentRef->SetParent(m_myself.getPointer());
		if (m_ContentRef && m_ContentRef->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ContentRef->UseTypeAttribute = true;
		}
		if(m_ContentRef != Mp7JrsReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ContentRef->SetContentName(XMLString::transcode("ContentRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_Content);
	m_Content = Mp7JrsPtr< Mp7JrsNode >();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: MixedCollectionType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsMixedCollectionType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsMixedCollectionType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsMixedCollectionType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsMixedCollectionType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_Content != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Content->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Content"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_Content->UseTypeAttribute = true;
		}
		m_Content->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ContentRef != Mp7JrsReferencePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ContentRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ContentRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ContentRef->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsMixedCollectionType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
	{
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Content"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Content")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMultimediaContentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetContent(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ContentRef"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ContentRef")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateReferenceType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetContentRef(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	} while(false);
// no includefile for extension defined 
// file Mp7JrsMixedCollectionType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMixedCollectionType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Content")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMultimediaContentType;
	}
	this->SetContent(child);
  }
  if (XMLString::compareString(elementname, X("ContentRef")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateReferenceType;
	}
	this->SetContentRef(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMixedCollectionType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetContent() != Mp7JrsNodePtr())
		result.Insert(GetContent());
	if (GetContentRef() != Mp7JrsNodePtr())
		result.Insert(GetContentRef());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMixedCollectionType_LocalType_ExtMethodImpl.h


