 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMotionActivityType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7JrsMotionActivityType_Intensity_LocalType.h"
#include "Mp7Jrsunsigned3.h"
#include "Mp7JrsMotionActivityType_SpatialDistributionParams_LocalType.h"
#include "Mp7JrsMotionActivityType_SpatialLocalizationParams_LocalType.h"
#include "Mp7JrsMotionActivityType_TemporalParams_LocalType.h"
#include "Mp7JrsMotionActivityType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsMotionActivityType::IMp7JrsMotionActivityType()
{

// no includefile for extension defined 
// file Mp7JrsMotionActivityType_ExtPropInit.h

}

IMp7JrsMotionActivityType::~IMp7JrsMotionActivityType()
{
// no includefile for extension defined 
// file Mp7JrsMotionActivityType_ExtPropCleanup.h

}

Mp7JrsMotionActivityType::Mp7JrsMotionActivityType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMotionActivityType::~Mp7JrsMotionActivityType()
{
	Cleanup();
}

void Mp7JrsMotionActivityType::Init()
{
	// Init base
	m_Base = CreateVisualDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Intensity = Mp7JrsMotionActivityType_Intensity_LocalPtr(); // Class with content 
	m_DominantDirection = Mp7Jrsunsigned3Ptr(); // Class with content 
	m_DominantDirection_Exist = false;
	m_SpatialDistributionParams = Mp7JrsMotionActivityType_SpatialDistributionParams_LocalPtr(); // Class
	m_SpatialDistributionParams_Exist = false;
	m_SpatialLocalizationParams = Mp7JrsMotionActivityType_SpatialLocalizationParams_LocalPtr(); // Class
	m_SpatialLocalizationParams_Exist = false;
	m_TemporalParams = Mp7JrsMotionActivityType_TemporalParams_LocalPtr(); // Class
	m_TemporalParams_Exist = false;


// no includefile for extension defined 
// file Mp7JrsMotionActivityType_ExtMyPropInit.h

}

void Mp7JrsMotionActivityType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMotionActivityType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Intensity);
	// Mp7JrsFactory::DeleteObject(m_DominantDirection);
	// Mp7JrsFactory::DeleteObject(m_SpatialDistributionParams);
	// Mp7JrsFactory::DeleteObject(m_SpatialLocalizationParams);
	// Mp7JrsFactory::DeleteObject(m_TemporalParams);
}

void Mp7JrsMotionActivityType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MotionActivityTypePtr, since we
	// might need GetBase(), which isn't defined in IMotionActivityType
	const Mp7JrsPtr< Mp7JrsMotionActivityType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsVisualDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMotionActivityType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Intensity);
		this->SetIntensity(Mp7JrsFactory::CloneObject(tmp->GetIntensity()));
	if (tmp->IsValidDominantDirection())
	{
		// Mp7JrsFactory::DeleteObject(m_DominantDirection);
		this->SetDominantDirection(Mp7JrsFactory::CloneObject(tmp->GetDominantDirection()));
	}
	else
	{
		InvalidateDominantDirection();
	}
	if (tmp->IsValidSpatialDistributionParams())
	{
		// Mp7JrsFactory::DeleteObject(m_SpatialDistributionParams);
		this->SetSpatialDistributionParams(Mp7JrsFactory::CloneObject(tmp->GetSpatialDistributionParams()));
	}
	else
	{
		InvalidateSpatialDistributionParams();
	}
	if (tmp->IsValidSpatialLocalizationParams())
	{
		// Mp7JrsFactory::DeleteObject(m_SpatialLocalizationParams);
		this->SetSpatialLocalizationParams(Mp7JrsFactory::CloneObject(tmp->GetSpatialLocalizationParams()));
	}
	else
	{
		InvalidateSpatialLocalizationParams();
	}
	if (tmp->IsValidTemporalParams())
	{
		// Mp7JrsFactory::DeleteObject(m_TemporalParams);
		this->SetTemporalParams(Mp7JrsFactory::CloneObject(tmp->GetTemporalParams()));
	}
	else
	{
		InvalidateTemporalParams();
	}
}

Mp7JrsNodePtr Mp7JrsMotionActivityType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsMotionActivityType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDType > Mp7JrsMotionActivityType::GetBase() const
{
	return m_Base;
}

Mp7JrsMotionActivityType_Intensity_LocalPtr Mp7JrsMotionActivityType::GetIntensity() const
{
		return m_Intensity;
}

Mp7Jrsunsigned3Ptr Mp7JrsMotionActivityType::GetDominantDirection() const
{
		return m_DominantDirection;
}

// Element is optional
bool Mp7JrsMotionActivityType::IsValidDominantDirection() const
{
	return m_DominantDirection_Exist;
}

Mp7JrsMotionActivityType_SpatialDistributionParams_LocalPtr Mp7JrsMotionActivityType::GetSpatialDistributionParams() const
{
		return m_SpatialDistributionParams;
}

// Element is optional
bool Mp7JrsMotionActivityType::IsValidSpatialDistributionParams() const
{
	return m_SpatialDistributionParams_Exist;
}

Mp7JrsMotionActivityType_SpatialLocalizationParams_LocalPtr Mp7JrsMotionActivityType::GetSpatialLocalizationParams() const
{
		return m_SpatialLocalizationParams;
}

// Element is optional
bool Mp7JrsMotionActivityType::IsValidSpatialLocalizationParams() const
{
	return m_SpatialLocalizationParams_Exist;
}

Mp7JrsMotionActivityType_TemporalParams_LocalPtr Mp7JrsMotionActivityType::GetTemporalParams() const
{
		return m_TemporalParams;
}

// Element is optional
bool Mp7JrsMotionActivityType::IsValidTemporalParams() const
{
	return m_TemporalParams_Exist;
}

void Mp7JrsMotionActivityType::SetIntensity(const Mp7JrsMotionActivityType_Intensity_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMotionActivityType::SetIntensity().");
	}
	if (m_Intensity != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Intensity);
		m_Intensity = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Intensity) m_Intensity->SetParent(m_myself.getPointer());
		if (m_Intensity && m_Intensity->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionActivityType_Intensity_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Intensity->UseTypeAttribute = true;
		}
		if(m_Intensity != Mp7JrsMotionActivityType_Intensity_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Intensity->SetContentName(XMLString::transcode("Intensity"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMotionActivityType::SetDominantDirection(const Mp7Jrsunsigned3Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMotionActivityType::SetDominantDirection().");
	}
	if (m_DominantDirection != item || m_DominantDirection_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_DominantDirection);
		m_DominantDirection = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DominantDirection) m_DominantDirection->SetParent(m_myself.getPointer());
		if (m_DominantDirection && m_DominantDirection->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned3"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DominantDirection->UseTypeAttribute = true;
		}
		m_DominantDirection_Exist = true;
		if(m_DominantDirection != Mp7Jrsunsigned3Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DominantDirection->SetContentName(XMLString::transcode("DominantDirection"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMotionActivityType::InvalidateDominantDirection()
{
	m_DominantDirection_Exist = false;
}
void Mp7JrsMotionActivityType::SetSpatialDistributionParams(const Mp7JrsMotionActivityType_SpatialDistributionParams_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMotionActivityType::SetSpatialDistributionParams().");
	}
	if (m_SpatialDistributionParams != item || m_SpatialDistributionParams_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SpatialDistributionParams);
		m_SpatialDistributionParams = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SpatialDistributionParams) m_SpatialDistributionParams->SetParent(m_myself.getPointer());
		if (m_SpatialDistributionParams && m_SpatialDistributionParams->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionActivityType_SpatialDistributionParams_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SpatialDistributionParams->UseTypeAttribute = true;
		}
		m_SpatialDistributionParams_Exist = true;
		if(m_SpatialDistributionParams != Mp7JrsMotionActivityType_SpatialDistributionParams_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SpatialDistributionParams->SetContentName(XMLString::transcode("SpatialDistributionParams"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMotionActivityType::InvalidateSpatialDistributionParams()
{
	m_SpatialDistributionParams_Exist = false;
}
void Mp7JrsMotionActivityType::SetSpatialLocalizationParams(const Mp7JrsMotionActivityType_SpatialLocalizationParams_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMotionActivityType::SetSpatialLocalizationParams().");
	}
	if (m_SpatialLocalizationParams != item || m_SpatialLocalizationParams_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SpatialLocalizationParams);
		m_SpatialLocalizationParams = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SpatialLocalizationParams) m_SpatialLocalizationParams->SetParent(m_myself.getPointer());
		if (m_SpatialLocalizationParams && m_SpatialLocalizationParams->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionActivityType_SpatialLocalizationParams_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SpatialLocalizationParams->UseTypeAttribute = true;
		}
		m_SpatialLocalizationParams_Exist = true;
		if(m_SpatialLocalizationParams != Mp7JrsMotionActivityType_SpatialLocalizationParams_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SpatialLocalizationParams->SetContentName(XMLString::transcode("SpatialLocalizationParams"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMotionActivityType::InvalidateSpatialLocalizationParams()
{
	m_SpatialLocalizationParams_Exist = false;
}
void Mp7JrsMotionActivityType::SetTemporalParams(const Mp7JrsMotionActivityType_TemporalParams_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMotionActivityType::SetTemporalParams().");
	}
	if (m_TemporalParams != item || m_TemporalParams_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_TemporalParams);
		m_TemporalParams = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TemporalParams) m_TemporalParams->SetParent(m_myself.getPointer());
		if (m_TemporalParams && m_TemporalParams->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionActivityType_TemporalParams_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TemporalParams->UseTypeAttribute = true;
		}
		m_TemporalParams_Exist = true;
		if(m_TemporalParams != Mp7JrsMotionActivityType_TemporalParams_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TemporalParams->SetContentName(XMLString::transcode("TemporalParams"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMotionActivityType::InvalidateTemporalParams()
{
	m_TemporalParams_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsMotionActivityType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Intensity")) == 0)
	{
		// Intensity is simple element MotionActivityType_Intensity_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetIntensity()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionActivityType_Intensity_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMotionActivityType_Intensity_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetIntensity(p, client);
					if((p = GetIntensity()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("DominantDirection")) == 0)
	{
		// DominantDirection is simple element unsigned3
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDominantDirection()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned3")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned3Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDominantDirection(p, client);
					if((p = GetDominantDirection()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SpatialDistributionParams")) == 0)
	{
		// SpatialDistributionParams is simple element MotionActivityType_SpatialDistributionParams_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSpatialDistributionParams()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionActivityType_SpatialDistributionParams_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMotionActivityType_SpatialDistributionParams_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSpatialDistributionParams(p, client);
					if((p = GetSpatialDistributionParams()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SpatialLocalizationParams")) == 0)
	{
		// SpatialLocalizationParams is simple element MotionActivityType_SpatialLocalizationParams_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSpatialLocalizationParams()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionActivityType_SpatialLocalizationParams_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMotionActivityType_SpatialLocalizationParams_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSpatialLocalizationParams(p, client);
					if((p = GetSpatialLocalizationParams()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("TemporalParams")) == 0)
	{
		// TemporalParams is simple element MotionActivityType_TemporalParams_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTemporalParams()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MotionActivityType_TemporalParams_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMotionActivityType_TemporalParams_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTemporalParams(p, client);
					if((p = GetTemporalParams()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MotionActivityType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MotionActivityType");
	}
	return result;
}

XMLCh * Mp7JrsMotionActivityType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsMotionActivityType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsMotionActivityType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMotionActivityType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MotionActivityType"));
	// Element serialization:
	// Class with content		
	
	if (m_Intensity != Mp7JrsMotionActivityType_Intensity_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Intensity->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Intensity"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Intensity->Serialize(doc, element, &elem);
	}
	// Class with content		
	
	if (m_DominantDirection != Mp7Jrsunsigned3Ptr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DominantDirection->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DominantDirection"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DominantDirection->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_SpatialDistributionParams != Mp7JrsMotionActivityType_SpatialDistributionParams_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SpatialDistributionParams->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SpatialDistributionParams"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SpatialDistributionParams->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_SpatialLocalizationParams != Mp7JrsMotionActivityType_SpatialLocalizationParams_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SpatialLocalizationParams->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SpatialLocalizationParams"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SpatialLocalizationParams->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_TemporalParams != Mp7JrsMotionActivityType_TemporalParams_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TemporalParams->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TemporalParams"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TemporalParams->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsMotionActivityType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVisualDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Intensity"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Intensity")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMotionActivityType_Intensity_LocalType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetIntensity(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DominantDirection"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DominantDirection")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = Createunsigned3;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetDominantDirection(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SpatialDistributionParams"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SpatialDistributionParams")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMotionActivityType_SpatialDistributionParams_LocalType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSpatialDistributionParams(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SpatialLocalizationParams"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SpatialLocalizationParams")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMotionActivityType_SpatialLocalizationParams_LocalType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSpatialLocalizationParams(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TemporalParams"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TemporalParams")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMotionActivityType_TemporalParams_LocalType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetTemporalParams(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsMotionActivityType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMotionActivityType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("Intensity")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMotionActivityType_Intensity_LocalType;
	}
	this->SetIntensity(child);
  }
  if (XMLString::compareString(elementname, X("DominantDirection")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = Createunsigned3;
	}
	this->SetDominantDirection(child);
  }
  if (XMLString::compareString(elementname, X("SpatialDistributionParams")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMotionActivityType_SpatialDistributionParams_LocalType;
	}
	this->SetSpatialDistributionParams(child);
  }
  if (XMLString::compareString(elementname, X("SpatialLocalizationParams")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMotionActivityType_SpatialLocalizationParams_LocalType;
	}
	this->SetSpatialLocalizationParams(child);
  }
  if (XMLString::compareString(elementname, X("TemporalParams")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMotionActivityType_TemporalParams_LocalType;
	}
	this->SetTemporalParams(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMotionActivityType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetIntensity() != Mp7JrsNodePtr())
		result.Insert(GetIntensity());
	if (GetDominantDirection() != Mp7JrsNodePtr())
		result.Insert(GetDominantDirection());
	if (GetSpatialDistributionParams() != Mp7JrsNodePtr())
		result.Insert(GetSpatialDistributionParams());
	if (GetSpatialLocalizationParams() != Mp7JrsNodePtr())
		result.Insert(GetSpatialLocalizationParams());
	if (GetTemporalParams() != Mp7JrsNodePtr())
		result.Insert(GetTemporalParams());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMotionActivityType_ExtMethodImpl.h


