 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsPersonNameType_LocalType_ExtImplInclude.h



#include "Mp7JrsNameComponentType.h"
#include "Mp7JrsPersonNameType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsPersonNameType_LocalType::IMp7JrsPersonNameType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsPersonNameType_LocalType_ExtPropInit.h

}

IMp7JrsPersonNameType_LocalType::~IMp7JrsPersonNameType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsPersonNameType_LocalType_ExtPropCleanup.h

}

Mp7JrsPersonNameType_LocalType::Mp7JrsPersonNameType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsPersonNameType_LocalType::~Mp7JrsPersonNameType_LocalType()
{
	Cleanup();
}

void Mp7JrsPersonNameType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_GivenName = Mp7JrsNameComponentPtr(); // Class
	m_LinkingName = Mp7JrsNameComponentPtr(); // Class
	m_LinkingName_Exist = false;
	m_FamilyName = Mp7JrsNameComponentPtr(); // Class
	m_FamilyName_Exist = false;
	m_Title = Mp7JrsNameComponentPtr(); // Class
	m_Title_Exist = false;
	m_Salutation = Mp7JrsNameComponentPtr(); // Class
	m_Salutation_Exist = false;
	m_Numeration = NULL; // Optional String
	m_Numeration_Exist = false;


// no includefile for extension defined 
// file Mp7JrsPersonNameType_LocalType_ExtMyPropInit.h

}

void Mp7JrsPersonNameType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsPersonNameType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_GivenName);
	// Mp7JrsFactory::DeleteObject(m_LinkingName);
	// Mp7JrsFactory::DeleteObject(m_FamilyName);
	// Mp7JrsFactory::DeleteObject(m_Title);
	// Mp7JrsFactory::DeleteObject(m_Salutation);
	XMLString::release(&this->m_Numeration);
	this->m_Numeration = NULL;
}

void Mp7JrsPersonNameType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use PersonNameType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IPersonNameType_LocalType
	const Mp7JrsPtr< Mp7JrsPersonNameType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_GivenName);
		this->SetGivenName(Mp7JrsFactory::CloneObject(tmp->GetGivenName()));
	if (tmp->IsValidLinkingName())
	{
		// Mp7JrsFactory::DeleteObject(m_LinkingName);
		this->SetLinkingName(Mp7JrsFactory::CloneObject(tmp->GetLinkingName()));
	}
	else
	{
		InvalidateLinkingName();
	}
	if (tmp->IsValidFamilyName())
	{
		// Mp7JrsFactory::DeleteObject(m_FamilyName);
		this->SetFamilyName(Mp7JrsFactory::CloneObject(tmp->GetFamilyName()));
	}
	else
	{
		InvalidateFamilyName();
	}
	if (tmp->IsValidTitle())
	{
		// Mp7JrsFactory::DeleteObject(m_Title);
		this->SetTitle(Mp7JrsFactory::CloneObject(tmp->GetTitle()));
	}
	else
	{
		InvalidateTitle();
	}
	if (tmp->IsValidSalutation())
	{
		// Mp7JrsFactory::DeleteObject(m_Salutation);
		this->SetSalutation(Mp7JrsFactory::CloneObject(tmp->GetSalutation()));
	}
	else
	{
		InvalidateSalutation();
	}
	if (tmp->IsValidNumeration())
	{
		this->SetNumeration(XMLString::replicate(tmp->GetNumeration()));
	}
	else
	{
		InvalidateNumeration();
	}
}

Mp7JrsNodePtr Mp7JrsPersonNameType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsPersonNameType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsNameComponentPtr Mp7JrsPersonNameType_LocalType::GetGivenName() const
{
		return m_GivenName;
}

Mp7JrsNameComponentPtr Mp7JrsPersonNameType_LocalType::GetLinkingName() const
{
		return m_LinkingName;
}

// Element is optional
bool Mp7JrsPersonNameType_LocalType::IsValidLinkingName() const
{
	return m_LinkingName_Exist;
}

Mp7JrsNameComponentPtr Mp7JrsPersonNameType_LocalType::GetFamilyName() const
{
		return m_FamilyName;
}

// Element is optional
bool Mp7JrsPersonNameType_LocalType::IsValidFamilyName() const
{
	return m_FamilyName_Exist;
}

Mp7JrsNameComponentPtr Mp7JrsPersonNameType_LocalType::GetTitle() const
{
		return m_Title;
}

// Element is optional
bool Mp7JrsPersonNameType_LocalType::IsValidTitle() const
{
	return m_Title_Exist;
}

Mp7JrsNameComponentPtr Mp7JrsPersonNameType_LocalType::GetSalutation() const
{
		return m_Salutation;
}

// Element is optional
bool Mp7JrsPersonNameType_LocalType::IsValidSalutation() const
{
	return m_Salutation_Exist;
}

XMLCh * Mp7JrsPersonNameType_LocalType::GetNumeration() const
{
		return m_Numeration;
}

// Element is optional
bool Mp7JrsPersonNameType_LocalType::IsValidNumeration() const
{
	return m_Numeration_Exist;
}

// implementing setter for choice 
/* element */
void Mp7JrsPersonNameType_LocalType::SetGivenName(const Mp7JrsNameComponentPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType_LocalType::SetGivenName().");
	}
	if (m_GivenName != item)
	{
		// Mp7JrsFactory::DeleteObject(m_GivenName);
		m_GivenName = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_GivenName) m_GivenName->SetParent(m_myself.getPointer());
		if (m_GivenName && m_GivenName->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:NameComponentType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_GivenName->UseTypeAttribute = true;
		}
		if(m_GivenName != Mp7JrsNameComponentPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_GivenName->SetContentName(XMLString::transcode("GivenName"));
		}
	// Mp7JrsFactory::DeleteObject(m_LinkingName);
	m_LinkingName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_FamilyName);
	m_FamilyName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_Title);
	m_Title = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_Salutation);
	m_Salutation = Mp7JrsNameComponentPtr();
	XMLString::release(&this->m_Numeration);
	m_Numeration = NULL; // FTT Shouldn't it be XMLString::release()?
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsPersonNameType_LocalType::SetLinkingName(const Mp7JrsNameComponentPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType_LocalType::SetLinkingName().");
	}
	if (m_LinkingName != item || m_LinkingName_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_LinkingName);
		m_LinkingName = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_LinkingName) m_LinkingName->SetParent(m_myself.getPointer());
		if (m_LinkingName && m_LinkingName->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:NameComponentType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_LinkingName->UseTypeAttribute = true;
		}
		m_LinkingName_Exist = true;
		if(m_LinkingName != Mp7JrsNameComponentPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_LinkingName->SetContentName(XMLString::transcode("LinkingName"));
		}
	// Mp7JrsFactory::DeleteObject(m_GivenName);
	m_GivenName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_FamilyName);
	m_FamilyName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_Title);
	m_Title = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_Salutation);
	m_Salutation = Mp7JrsNameComponentPtr();
	XMLString::release(&this->m_Numeration);
	m_Numeration = NULL; // FTT Shouldn't it be XMLString::release()?
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPersonNameType_LocalType::InvalidateLinkingName()
{
	m_LinkingName_Exist = false;
}
/* element */
void Mp7JrsPersonNameType_LocalType::SetFamilyName(const Mp7JrsNameComponentPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType_LocalType::SetFamilyName().");
	}
	if (m_FamilyName != item || m_FamilyName_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_FamilyName);
		m_FamilyName = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FamilyName) m_FamilyName->SetParent(m_myself.getPointer());
		if (m_FamilyName && m_FamilyName->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:NameComponentType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FamilyName->UseTypeAttribute = true;
		}
		m_FamilyName_Exist = true;
		if(m_FamilyName != Mp7JrsNameComponentPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FamilyName->SetContentName(XMLString::transcode("FamilyName"));
		}
	// Mp7JrsFactory::DeleteObject(m_GivenName);
	m_GivenName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_LinkingName);
	m_LinkingName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_Title);
	m_Title = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_Salutation);
	m_Salutation = Mp7JrsNameComponentPtr();
	XMLString::release(&this->m_Numeration);
	m_Numeration = NULL; // FTT Shouldn't it be XMLString::release()?
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPersonNameType_LocalType::InvalidateFamilyName()
{
	m_FamilyName_Exist = false;
}
/* element */
void Mp7JrsPersonNameType_LocalType::SetTitle(const Mp7JrsNameComponentPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType_LocalType::SetTitle().");
	}
	if (m_Title != item || m_Title_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Title);
		m_Title = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Title) m_Title->SetParent(m_myself.getPointer());
		if (m_Title && m_Title->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:NameComponentType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Title->UseTypeAttribute = true;
		}
		m_Title_Exist = true;
		if(m_Title != Mp7JrsNameComponentPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Title->SetContentName(XMLString::transcode("Title"));
		}
	// Mp7JrsFactory::DeleteObject(m_GivenName);
	m_GivenName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_LinkingName);
	m_LinkingName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_FamilyName);
	m_FamilyName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_Salutation);
	m_Salutation = Mp7JrsNameComponentPtr();
	XMLString::release(&this->m_Numeration);
	m_Numeration = NULL; // FTT Shouldn't it be XMLString::release()?
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPersonNameType_LocalType::InvalidateTitle()
{
	m_Title_Exist = false;
}
/* element */
void Mp7JrsPersonNameType_LocalType::SetSalutation(const Mp7JrsNameComponentPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType_LocalType::SetSalutation().");
	}
	if (m_Salutation != item || m_Salutation_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Salutation);
		m_Salutation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Salutation) m_Salutation->SetParent(m_myself.getPointer());
		if (m_Salutation && m_Salutation->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:NameComponentType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Salutation->UseTypeAttribute = true;
		}
		m_Salutation_Exist = true;
		if(m_Salutation != Mp7JrsNameComponentPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Salutation->SetContentName(XMLString::transcode("Salutation"));
		}
	// Mp7JrsFactory::DeleteObject(m_GivenName);
	m_GivenName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_LinkingName);
	m_LinkingName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_FamilyName);
	m_FamilyName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_Title);
	m_Title = Mp7JrsNameComponentPtr();
	XMLString::release(&this->m_Numeration);
	m_Numeration = NULL; // FTT Shouldn't it be XMLString::release()?
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPersonNameType_LocalType::InvalidateSalutation()
{
	m_Salutation_Exist = false;
}
/* element */
void Mp7JrsPersonNameType_LocalType::SetNumeration(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonNameType_LocalType::SetNumeration().");
	}
	if (m_Numeration != item || m_Numeration_Exist == false)
	{
		XMLString::release(&m_Numeration);
		m_Numeration = item;
		m_Numeration_Exist = true;
	// Mp7JrsFactory::DeleteObject(m_GivenName);
	m_GivenName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_LinkingName);
	m_LinkingName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_FamilyName);
	m_FamilyName = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_Title);
	m_Title = Mp7JrsNameComponentPtr();
	// Mp7JrsFactory::DeleteObject(m_Salutation);
	m_Salutation = Mp7JrsNameComponentPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPersonNameType_LocalType::InvalidateNumeration()
{
	m_Numeration_Exist = false;
}
// Type: PersonNameType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsPersonNameType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsPersonNameType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsPersonNameType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsPersonNameType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_GivenName != Mp7JrsNameComponentPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_GivenName->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("GivenName"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_GivenName->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_LinkingName != Mp7JrsNameComponentPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_LinkingName->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("LinkingName"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_LinkingName->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_FamilyName != Mp7JrsNameComponentPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FamilyName->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FamilyName"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FamilyName->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Title != Mp7JrsNameComponentPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Title->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Title"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Title->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Salutation != Mp7JrsNameComponentPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Salutation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Salutation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Salutation->Serialize(doc, element, &elem);
	}
	 // String
	if(m_Numeration != (XMLCh*)NULL) Mp7JrsUtil::SerializeTextNode(element, doc, m_Numeration, X("urn:mpeg:mpeg7:schema:2004"), X("Numeration"), true);

}

bool Mp7JrsPersonNameType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
	{
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("GivenName"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("GivenName")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateNameComponentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetGivenName(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("LinkingName"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("LinkingName")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateNameComponentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetLinkingName(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FamilyName"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FamilyName")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateNameComponentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetFamilyName(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Title"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Title")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateNameComponentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetTitle(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Salutation"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Salutation")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateNameComponentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSalutation(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	// Deserialize string (XMLCh*) element
	if(Mp7JrsUtil::HasNodeName(parent, X("Numeration"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		// FTT memleaking		this->SetNumeration(Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent)));
		this->SetNumeration(Mp7JrsUtil::GetElementText(parent));
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		continue;	   // For choices
	}
	} while(false);
// no includefile for extension defined 
// file Mp7JrsPersonNameType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsPersonNameType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("GivenName")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateNameComponentType;
	}
	this->SetGivenName(child);
  }
  if (XMLString::compareString(elementname, X("LinkingName")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateNameComponentType;
	}
	this->SetLinkingName(child);
  }
  if (XMLString::compareString(elementname, X("FamilyName")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateNameComponentType;
	}
	this->SetFamilyName(child);
  }
  if (XMLString::compareString(elementname, X("Title")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateNameComponentType;
	}
	this->SetTitle(child);
  }
  if (XMLString::compareString(elementname, X("Salutation")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateNameComponentType;
	}
	this->SetSalutation(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsPersonNameType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetGivenName() != Mp7JrsNodePtr())
		result.Insert(GetGivenName());
	if (GetLinkingName() != Mp7JrsNodePtr())
		result.Insert(GetLinkingName());
	if (GetFamilyName() != Mp7JrsNodePtr())
		result.Insert(GetFamilyName());
	if (GetTitle() != Mp7JrsNodePtr())
		result.Insert(GetTitle());
	if (GetSalutation() != Mp7JrsNodePtr())
		result.Insert(GetSalutation());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsPersonNameType_LocalType_ExtMethodImpl.h


