 

// Core library class
// based on NodeRefCollectionType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


#include <stdio.h>
#include "Mp7JrsPersonType_CollectionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrsPersonType_CollectionType_ExtImplInclude.h


#include "Mp7JrsPersonType_LocalType.h" // Collection itemtype
// Choice item type PersonNameType is concrete
// Choice item type ControlledTermUseType is concrete
#include "Mp7JrsControlledTermUseType.h"
#include "Mp7JrsPersonNameType.h"



Mp7JrsPersonType_CollectionType::Mp7JrsPersonType_CollectionType()
{
	m_Item = new Mp7JrsValueVectorOf<Mp7JrsNodePtr>(0); // XXX
	dontSerializeWithContentName = false; // default -- (see Customize.template::DontSetContentNameFor)

// no includefile for extension defined 
// file Mp7JrsPersonType_CollectionType_ExtPropInit.h


}

Mp7JrsPersonType_CollectionType::~Mp7JrsPersonType_CollectionType()
{
// no includefile for extension defined 
// file Mp7JrsPersonType_CollectionType_ExtPropCleanup.h


	delete m_Item;
}

void Mp7JrsPersonType_CollectionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	const Mp7JrsPersonType_CollectionPtr tmp = original;
	if (tmp == NULL) return;  // EXIT: passed argument of wrong type

	removeAllElements();
	for(unsigned int i = 0; i < tmp->size(); i++)
	{
		Mp7JrsNodePtr dummy = Mp7JrsFactory::CloneObject(tmp->elementAt(i));
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (dummy) dummy->SetParent(m_myself.getPointer());
		m_Item->addElement(dummy);
	}
}

// no includefile for extension defined 
// file Mp7JrsPersonType_CollectionType_ExtMethodImpl.h


void Mp7JrsPersonType_CollectionType::Initialize(unsigned int maxElems)
{
	m_Item->ensureExtraCapacity(maxElems);
}

void Mp7JrsPersonType_CollectionType::addElement(const Mp7JrsNodePtr &toAdd, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonType_CollectionType::addElement().");
	}
	m_Item->addElement(toAdd);
	if (toAdd) {
		// sat 2004-05-05: factor this out into method when there's a spare hour to compile ...
		toAdd->SetParent(m_myself.getPointer());
		if (!toAdd->GetContentName()) {
			// It's a LocalType (presumably) -- the content name isn't used.
			toAdd->SetContentName(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(""));
		}
		
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_ADDED,
		client, toAdd);
}

void Mp7JrsPersonType_CollectionType::setElementAt(const Mp7JrsNodePtr &toSet, unsigned int setAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonType_CollectionType::setElementAt().");
	}
	Mp7JrsNodePtr previous = m_Item->elementAt(setAt);
	if (previous) previous->SetParent(Mp7JrsNodePtr());
	m_Item->setElementAt(toSet, setAt);

	if (toSet) {
		// sat 2004-05-05: factor this out into method when there's a spare hour to compile ...
		toSet->SetParent(m_myself.getPointer());
		if (!toSet->GetContentName()) {
			// It's a LocalType (presumably) -- the content name isn't used.
			toSet->SetContentName(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(""));
		}
		
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, toSet, setAt);
}

void Mp7JrsPersonType_CollectionType::insertElementAt(const Mp7JrsNodePtr &toInsert, unsigned int insertAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonType_CollectionType::insertElementAt().");
	}
	m_Item->insertElementAt(toInsert, insertAt);
	if (toInsert) {
		// sat 2004-05-05: factor this out into method when there's a spare hour to compile ...
		toInsert->SetParent(m_myself.getPointer());
		if (!toInsert->GetContentName()) {
			// It's a LocalType (presumably) -- the content name isn't used.
			toInsert->SetContentName(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(""));
		}
		
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_INSERTED,
		client, toInsert, insertAt);
}

/*
Mp7JrsNodePtr Mp7JrsPersonType_CollectionType::orphanElementAt(unsigned int orphanAt)
{
	return m_Item->orphanElementAt(orphanAt);
}
*/

void Mp7JrsPersonType_CollectionType::removeAllElements(Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonType_CollectionType::removeAllElements().");
	}

	for (unsigned int i = 0; i < m_Item->size(); i++) {
		// FIXME: handle elementAt(removeAt) == NULL
		m_Item->elementAt(i)->SetParent(Mp7JrsNodePtr());
	}

	m_Item->removeAllElements();
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPersonType_CollectionType::removeElementAt(unsigned int removeAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPersonType_CollectionType::removeElementAt().");
	}
	// FIXME: handle elementAt(removeAt) == NULL
	m_Item->elementAt(removeAt)->SetParent(Mp7JrsNodePtr());
	m_Item->removeElementAt(removeAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsPersonType_CollectionType::containsElement(const Mp7JrsNodePtr &toCheck)
{
	return m_Item->containsElement(toCheck);
}

unsigned int Mp7JrsPersonType_CollectionType::curCapacity() const
{
	return (unsigned int)m_Item->curCapacity();
}

const Mp7JrsNodePtr Mp7JrsPersonType_CollectionType::elementAt(unsigned int getAt) const
{
	return m_Item->elementAt(getAt);
}

Mp7JrsNodePtr Mp7JrsPersonType_CollectionType::elementAt(unsigned int getAt)
{
	return m_Item->elementAt(getAt);
}

Mp7JrsNodeEnum Mp7JrsPersonType_CollectionType::GetElementsContainingType(unsigned int id) const
{
	Mp7JrsEnumerator< Mp7JrsNodePtr > result;
	for (unsigned int i = 0; i < size(); i++)
	{
		Mp7JrsPersonType_LocalPtr tmp = elementAt(i);
		if (tmp != Mp7JrsPersonType_LocalPtr() && tmp->GetClassId() == Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PersonType_LocalType"))) {
			// FTT this is better!
			if (tmp->GetName() != Mp7JrsPersonNamePtr()
				&& tmp->GetName()->GetClassId() == id)
				{
					result.Insert(tmp->GetName());
					continue;
				}
			if (tmp->GetNameTerm() != Mp7JrsControlledTermUsePtr()
				&& tmp->GetNameTerm()->GetClassId() == id)
				{
					result.Insert(tmp->GetNameTerm());
					continue;
				}
		}
	}
	return result;
}
Mp7JrsNodeEnum Mp7JrsPersonType_CollectionType::GetElementsOfType(unsigned int id) const
{
	Mp7JrsEnumerator< Mp7JrsNodePtr > result;
	for (unsigned int i = 0; i < size(); i++)
	{
		if (elementAt(i)->GetClassId() == id)
		{
			result.Insert(elementAt(i));
		}
	}
	return result;
}

Mp7JrsNodeEnum Mp7JrsPersonType_CollectionType::GetElements() const
{
	Mp7JrsEnumerator< Mp7JrsPtr<Mp7JrsNode> > result;
	for (unsigned int i = 0; i < size(); i++)
	{
		result.Insert(elementAt(i));
	}
	return result;
}

unsigned int Mp7JrsPersonType_CollectionType::size() const
{
	return (unsigned int)m_Item->size();
}

void Mp7JrsPersonType_CollectionType::ensureExtraCapacity(unsigned int length)
{
	m_Item->ensureExtraCapacity(length);
}

// JRS: addition to Xerces collection
int Mp7JrsPersonType_CollectionType::elementIndexOf(const Mp7JrsNodePtr &toCheck) const
{
	for(unsigned int i = 0; i < size(); i++)
	{
		if(elementAt(i) == toCheck)
		{
			return (int) i;
		}
	}
	return -1; // not found
}

XMLCh * Mp7JrsPersonType_CollectionType::ToText() const
{
	XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
	XMLCh * delim = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(" ");
	for(unsigned int i = 0; i < size(); i++)
	{
		XMLCh * tmp = (elementAt(i))->ToText();
		Mp7JrsUtil::CatString(&buf, tmp);
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
		if(i < size() - 1)
		{
			Mp7JrsUtil::CatString(&buf, delim);
		}
	}
	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&delim);
	// Note: You must release this buffer
	return buf;
}

bool Mp7JrsPersonType_CollectionType::Parse(const XMLCh * const txt)
{
	if(txt == NULL)
	{
		return false;
	}
		
	// This is not a simpletype list ItemType and intentionally unimplemented for collections
	return false;
}

/** 
 * Item type is ElementOnly. 
 */
void Mp7JrsPersonType_CollectionType::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
	// The element we serialize into
	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
	if (!newElem) newElem = &localNewElem;

	if (*newElem) {
		element = *newElem;
	} else if(parent) {
		element = parent;
	} else {
		element = doc->getDocumentElement();
	}


// no includefile for extension defined 
// file Mp7JrsPersonType_CollectionType_ExtPreSerialize.h


	for(unsigned int i = 0; i < size(); i++)
	{
		// Collection is not SimpleTypeList -> serialize elements
		if(this->GetContentName() != NULL && !dontSerializeWithContentName)
		{
// OLD			XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
      DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			*newElem = elem;
			(elementAt(i))->Serialize(doc, elem, newElem);
			element->appendChild(elem);
		}
		else
		{
			// Serialize into current node
			(elementAt(i))->Serialize(doc, element, &element);
		}
		if(i < size() - 1)
		{
			element->appendChild(doc->createTextNode(X(" ")));
		}
	}
}



bool Mp7JrsPersonType_CollectionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(parent == NULL)
	{
		return false;
	}
	* current = parent; // Init with first element - check this!

	// Deserialize collection
	// Itemtype = Empty | ElementOnly
	DOMElement * ignore = NULL;
	while((parent != NULL) && (
		Mp7JrsUtil::HasNodeName(parent, X("Name"), X("urn:mpeg:mpeg7:schema:2004"))
// OLD3		(XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:Name")) == 0)
		||
		Mp7JrsUtil::HasNodeName(parent, X("NameTerm"), X("urn:mpeg:mpeg7:schema:2004"))
// OLD3		(XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:NameTerm")) == 0)
	))
	{
		Mp7JrsNodePtr item = Mp7JrsPersonType_LocalPtr();

		item = CreatePersonType_LocalType;
		// Sequence or Choice -- don't use "own" node
		item->Deserialize(doc, parent, current);
		// baw 2004-11-15 begin: do not set content name on local type
		// XMLCh * contentname = XMLString::replicate(parent->getNodeName());
		// item->SetContentName(contentname);
		item->SetContentName(NULL);
		// baw 2004-11-15 end
		addElement(item);
		parent = Mp7JrsUtil::GetNextAkin(* current); // Previously GetNextSibling
	}


// no includefile for extension defined 
// file Mp7JrsPersonType_CollectionType_ExtPostDeserialize.h


	return true;
}

inline
bool am_in_collection(int next_slash, int next_bracket, int next_close_bracket)
{
	return next_bracket >= 0
		&& next_close_bracket > next_bracket + 1 // at least one digit
		&& (next_slash < 0		// either at end of string ...
			|| next_slash > next_close_bracket); // ... or collection
												 // is in current part
}

Mp7JrsNodePtr Mp7JrsPersonType_CollectionType::NodeFromXPath(const XMLCh *xpath) const
{
	const XMLCh *work_string = xpath;
	const XMLCh *content_name = GetContentName();
	Mp7JrsNodePtr me = m_myself.getPointer();
	Mp7JrsNodeCollectionPtr me_as_collection = me;
	Mp7JrsNodePtr parent = GetParent();

	if (XMLString::startsWith(work_string, X("/"))) {
		if (!parent)
			work_string++;		// consume leading slash
		else {
			Mp7JrsNodePtr root = parent;
			while (root->GetParent())
				root = root->GetParent();
			return root->NodeFromXPath(work_string); // EXIT: parse from root
		}
	}

	if (XMLString::stringLen(work_string) == 0)
		return me;

	if (XMLString::startsWith(work_string, X("..")))
		return UpwardWithXPath(work_string);

	XMLCh slash = X("/")[0];
	XMLCh bracket = X("[")[0];
	XMLCh close_bracket = X("]")[0];
	int next_slash = XMLString::indexOf(work_string, slash);
	int next_bracket = XMLString::indexOf(work_string, bracket);
	int next_close_bracket = XMLString::indexOf(work_string, close_bracket);

	// We're a collection -- find n, nth element, parse from there
	int n;
	if (!am_in_collection(next_slash, next_bracket, next_close_bracket))
	{
		if (!content_name) {
			// A collection can be a a choice too; there's a valid
			// test case where we needed to descend into a
			// collection for the xpath string "Name".  The
			// "!content_name" clause is just a futile attempt to
			// cut back the lossage a bit; I'm assuming that none
			// of these freak collections has it set.
			Mp7JrsNodeEnum children = me_as_collection->GetElements();
			while (children.HasNext()) {
				Mp7JrsNodePtr result = children.GetNext()->NodeFromXPath(xpath);
				if (result != Mp7JrsNodePtr())
					return result;	// EXIT: success!
			}
		}
		// fall-through case: no collection index found, malformed
		// xpath string or xpath string didn't match the children.
		return Mp7JrsNodePtr(); // EXIT
	}
	XMLCh *n_str = Mp7JrsUtil::allocate(next_close_bracket - next_bracket + 1);
	XMLString::moveChars(n_str, work_string + next_bracket + 1,
		next_close_bracket - next_bracket - 1);
	n = XMLString::parseInt(n_str) - 1;	// XPath indices start
	// with 1, ours start with
	// 0 -- adjust
	XMLString::release(&n_str);
	if (n >= 0 && n < (int)me_as_collection->size()) {
		// if the parent is a LocalType, our ContentName has to
		// match; otherwise, it's allowed to be unset.  if it's
		// set, it has to match.
		if ((content_name
				&& XMLString::startsWith(work_string, content_name)
				&& !XMLString::isAlphaNum(
					work_string[XMLString::stringLen(content_name)])))
		{
			if (next_slash != -1)
				if (dontSerializeWithContentName)
					return me_as_collection->elementAt(n)->NodeFromXPath(
						work_string);
				else
					return me_as_collection->elementAt(n)->NodeFromXPath(
						work_string + next_slash + 1);
			else
				// EXIT: success!  (xpath ended with ".../elemName[n]")
				return me_as_collection->elementAt(n);
		}
		else if (!content_name)
		{
			// It's probably a local collection (collection ->
			// LocalType -> Type) -- the name will be picked up by
			// the Type node, so pass in full work string here.
			return me_as_collection->elementAt(n)->NodeFromXPath(
				work_string); // EXIT
		}
		else
			return Mp7JrsNodePtr(); // EXIT: wrong content name
	} else
	return Mp7JrsNodePtr();	// EXIT: index out of range

}

