 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsPointOfViewType_ExtImplInclude.h



#include "Mp7JrsDType.h"
#include "Mp7JrsTextAnnotationType.h"
#include "Mp7JrsPointOfViewType_Importance_CollectionType.h"
#include "Mp7JrsPointOfViewType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsPointOfViewType_Importance_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Importance

#include <assert.h>
IMp7JrsPointOfViewType::IMp7JrsPointOfViewType()
{

// no includefile for extension defined 
// file Mp7JrsPointOfViewType_ExtPropInit.h

}

IMp7JrsPointOfViewType::~IMp7JrsPointOfViewType()
{
// no includefile for extension defined 
// file Mp7JrsPointOfViewType_ExtPropCleanup.h

}

Mp7JrsPointOfViewType::Mp7JrsPointOfViewType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsPointOfViewType::~Mp7JrsPointOfViewType()
{
	Cleanup();
}

void Mp7JrsPointOfViewType::Init()
{
	// Init base
	m_Base = CreateDType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_viewpoint = NULL; // String

	// Init elements (element, union sequence choice all any)
	
	m_SupplementalInformation = Mp7JrsTextAnnotationPtr(); // Class
	m_SupplementalInformation_Exist = false;
	m_Importance = Mp7JrsPointOfViewType_Importance_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsPointOfViewType_ExtMyPropInit.h

}

void Mp7JrsPointOfViewType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsPointOfViewType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_viewpoint); // String
	// Mp7JrsFactory::DeleteObject(m_SupplementalInformation);
	// Mp7JrsFactory::DeleteObject(m_Importance);
}

void Mp7JrsPointOfViewType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use PointOfViewTypePtr, since we
	// might need GetBase(), which isn't defined in IPointOfViewType
	const Mp7JrsPtr< Mp7JrsPointOfViewType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsPointOfViewType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_viewpoint); // String
		this->Setviewpoint(XMLString::replicate(tmp->Getviewpoint()));
	}
	if (tmp->IsValidSupplementalInformation())
	{
		// Mp7JrsFactory::DeleteObject(m_SupplementalInformation);
		this->SetSupplementalInformation(Mp7JrsFactory::CloneObject(tmp->GetSupplementalInformation()));
	}
	else
	{
		InvalidateSupplementalInformation();
	}
		// Mp7JrsFactory::DeleteObject(m_Importance);
		this->SetImportance(Mp7JrsFactory::CloneObject(tmp->GetImportance()));
}

Mp7JrsNodePtr Mp7JrsPointOfViewType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsPointOfViewType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDType > Mp7JrsPointOfViewType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsPointOfViewType::Getviewpoint() const
{
	return m_viewpoint;
}

void Mp7JrsPointOfViewType::Setviewpoint(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPointOfViewType::Setviewpoint().");
	}
	m_viewpoint = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsTextAnnotationPtr Mp7JrsPointOfViewType::GetSupplementalInformation() const
{
		return m_SupplementalInformation;
}

// Element is optional
bool Mp7JrsPointOfViewType::IsValidSupplementalInformation() const
{
	return m_SupplementalInformation_Exist;
}

Mp7JrsPointOfViewType_Importance_CollectionPtr Mp7JrsPointOfViewType::GetImportance() const
{
		return m_Importance;
}

void Mp7JrsPointOfViewType::SetSupplementalInformation(const Mp7JrsTextAnnotationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPointOfViewType::SetSupplementalInformation().");
	}
	if (m_SupplementalInformation != item || m_SupplementalInformation_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SupplementalInformation);
		m_SupplementalInformation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SupplementalInformation) m_SupplementalInformation->SetParent(m_myself.getPointer());
		if (m_SupplementalInformation && m_SupplementalInformation->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextAnnotationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SupplementalInformation->UseTypeAttribute = true;
		}
		m_SupplementalInformation_Exist = true;
		if(m_SupplementalInformation != Mp7JrsTextAnnotationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SupplementalInformation->SetContentName(XMLString::transcode("SupplementalInformation"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPointOfViewType::InvalidateSupplementalInformation()
{
	m_SupplementalInformation_Exist = false;
}
void Mp7JrsPointOfViewType::SetImportance(const Mp7JrsPointOfViewType_Importance_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPointOfViewType::SetImportance().");
	}
	if (m_Importance != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Importance);
		m_Importance = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Importance) m_Importance->SetParent(m_myself.getPointer());
		if(m_Importance != Mp7JrsPointOfViewType_Importance_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Importance->SetContentName(XMLString::transcode("Importance"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsPointOfViewType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; viewpoint,  T:http://www.w3.org/2001/XMLSchema:string, C:String 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("viewpoint")) == 0)
	{
		// viewpoint is simple attribute string
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "XMLCh *");
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SupplementalInformation")) == 0)
	{
		// SupplementalInformation is simple element TextAnnotationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSupplementalInformation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextAnnotationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextAnnotationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSupplementalInformation(p, client);
					if((p = GetSupplementalInformation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Importance")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Importance is item of type PointOfViewType_Importance_LocalType
		// in element collection PointOfViewType_Importance_CollectionType
		
		context->Found = true;
		Mp7JrsPointOfViewType_Importance_CollectionPtr coll = GetImportance();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePointOfViewType_Importance_CollectionType;
				SetImportance(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PointOfViewType_Importance_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPointOfViewType_Importance_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for PointOfViewType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "PointOfViewType");
	}
	return result;
}

XMLCh * Mp7JrsPointOfViewType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsPointOfViewType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsPointOfViewType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsPointOfViewType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("PointOfViewType"));
	// Attribute Serialization:
	// String
	if(m_viewpoint != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("viewpoint"), m_viewpoint);
	}
	// Element serialization:
	// Class
	
	if (m_SupplementalInformation != Mp7JrsTextAnnotationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SupplementalInformation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SupplementalInformation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SupplementalInformation->Serialize(doc, element, &elem);
	}
	if (m_Importance != Mp7JrsPointOfViewType_Importance_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Importance->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsPointOfViewType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("viewpoint")))
	{
		// Deserialize string type
		this->Setviewpoint(Mp7JrsConvert::TextToString(parent->getAttribute(X("viewpoint"))));
		* current = parent;
	}

	// Extensionbase is Mp7JrsDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SupplementalInformation"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SupplementalInformation")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateTextAnnotationType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSupplementalInformation(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("Importance"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("Importance")) == 0))
		{
			// Deserialize factory type
			Mp7JrsPointOfViewType_Importance_CollectionPtr tmp = CreatePointOfViewType_Importance_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetImportance(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsPointOfViewType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsPointOfViewType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("SupplementalInformation")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateTextAnnotationType;
	}
	this->SetSupplementalInformation(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Importance")) == 0))
  {
	Mp7JrsPointOfViewType_Importance_CollectionPtr tmp = CreatePointOfViewType_Importance_CollectionType;
	this->SetImportance(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsPointOfViewType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSupplementalInformation() != Mp7JrsNodePtr())
		result.Insert(GetSupplementalInformation());
	if (GetImportance() != Mp7JrsNodePtr())
		result.Insert(GetImportance());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsPointOfViewType_ExtMethodImpl.h


