 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsRatingType_ExtImplInclude.h



#include "Mp7JrsRatingType_RatingScheme_LocalType.h"
#include "Mp7JrsRatingType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsRatingType::IMp7JrsRatingType()
{

// no includefile for extension defined 
// file Mp7JrsRatingType_ExtPropInit.h

}

IMp7JrsRatingType::~IMp7JrsRatingType()
{
// no includefile for extension defined 
// file Mp7JrsRatingType_ExtPropCleanup.h

}

Mp7JrsRatingType::Mp7JrsRatingType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsRatingType::~Mp7JrsRatingType()
{
	Cleanup();
}

void Mp7JrsRatingType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_RatingValue = (0.0); // Value

	m_RatingScheme = Mp7JrsRatingType_RatingScheme_LocalPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsRatingType_ExtMyPropInit.h

}

void Mp7JrsRatingType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsRatingType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_RatingScheme);
}

void Mp7JrsRatingType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use RatingTypePtr, since we
	// might need GetBase(), which isn't defined in IRatingType
	const Mp7JrsPtr< Mp7JrsRatingType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		this->SetRatingValue(tmp->GetRatingValue());
		// Mp7JrsFactory::DeleteObject(m_RatingScheme);
		this->SetRatingScheme(Mp7JrsFactory::CloneObject(tmp->GetRatingScheme()));
}

Mp7JrsNodePtr Mp7JrsRatingType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsRatingType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


float Mp7JrsRatingType::GetRatingValue() const
{
		return m_RatingValue;
}

Mp7JrsRatingType_RatingScheme_LocalPtr Mp7JrsRatingType::GetRatingScheme() const
{
		return m_RatingScheme;
}

void Mp7JrsRatingType::SetRatingValue(float item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRatingType::SetRatingValue().");
	}
	if (m_RatingValue != item)
	{
		m_RatingValue = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRatingType::SetRatingScheme(const Mp7JrsRatingType_RatingScheme_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRatingType::SetRatingScheme().");
	}
	if (m_RatingScheme != item)
	{
		// Mp7JrsFactory::DeleteObject(m_RatingScheme);
		m_RatingScheme = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_RatingScheme) m_RatingScheme->SetParent(m_myself.getPointer());
		if (m_RatingScheme && m_RatingScheme->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RatingType_RatingScheme_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_RatingScheme->UseTypeAttribute = true;
		}
		if(m_RatingScheme != Mp7JrsRatingType_RatingScheme_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_RatingScheme->SetContentName(XMLString::transcode("RatingScheme"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsRatingType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("RatingScheme")) == 0)
	{
		// RatingScheme is simple element RatingType_RatingScheme_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRatingScheme()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RatingType_RatingScheme_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRatingType_RatingScheme_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRatingScheme(p, client);
					if((p = GetRatingScheme()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for RatingType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "RatingType");
	}
	return result;
}

XMLCh * Mp7JrsRatingType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsRatingType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsRatingType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("RatingType"));
	// Element serialization:
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_RatingValue);
		Mp7JrsUtil::SerializeTextNode(element, doc, tmp, X("urn:mpeg:mpeg7:schema:2004"), X("RatingValue"), false);
		XMLString::release(&tmp);
	}
	// Class
	
	if (m_RatingScheme != Mp7JrsRatingType_RatingScheme_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_RatingScheme->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("RatingScheme"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_RatingScheme->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsRatingType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
	// Deserialize value element
	if(Mp7JrsUtil::HasNodeName(parent, X("RatingValue"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		XMLCh * tmp = Mp7JrsUtil::GetElementText(parent);
		this->SetRatingValue(Mp7JrsConvert::TextToFloat(tmp));
		XMLString::release(&tmp);
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("RatingScheme"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("RatingScheme")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateRatingType_RatingScheme_LocalType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetRatingScheme(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsRatingType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsRatingType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("RatingScheme")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateRatingType_RatingScheme_LocalType;
	}
	this->SetRatingScheme(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsRatingType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetRatingScheme() != Mp7JrsNodePtr())
		result.Insert(GetRatingScheme());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsRatingType_ExtMethodImpl.h


