 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsRecordingRequestType_ExtImplInclude.h



#include "Mp7JrsUniqueIDType.h"
#include "Mp7JrsTextualType.h"
#include "Mp7JrsRecordingRequestType_RecordingPeriod_LocalType.h"
#include "Mp7JrsRecordingRequestType_RecordingLocationPreferences_CollectionType.h"
#include "Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_CollectionType.h"
#include "Mp7JrsRecordingRequestType_RecordAssociatedData_LocalType.h"
#include "Mp7JrsRecordingRequestType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:RecordingSourceTypePreferences

#include <assert.h>
IMp7JrsRecordingRequestType::IMp7JrsRecordingRequestType()
{

// no includefile for extension defined 
// file Mp7JrsRecordingRequestType_ExtPropInit.h

}

IMp7JrsRecordingRequestType::~IMp7JrsRecordingRequestType()
{
// no includefile for extension defined 
// file Mp7JrsRecordingRequestType_ExtPropCleanup.h

}

Mp7JrsRecordingRequestType::Mp7JrsRecordingRequestType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsRecordingRequestType::~Mp7JrsRecordingRequestType()
{
	Cleanup();
}

void Mp7JrsRecordingRequestType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_ContentIdentifier = Mp7JrsUniqueIDPtr(); // Class
	m_DisseminationSource = Mp7JrsTextualPtr(); // Class
	m_DisseminationSource_Exist = false;
	m_Title = Mp7JrsTextualPtr(); // Class
	m_Title_Exist = false;
	m_RecordingPeriod = Mp7JrsRecordingRequestType_RecordingPeriod_LocalPtr(); // Class
	m_RecordingLocationPreferences = Mp7JrsRecordingRequestType_RecordingLocationPreferences_CollectionPtr(); // Collection
	m_RecordingSourceTypePreferences = Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_CollectionPtr(); // Collection
	m_RecordAssociatedData = Mp7JrsRecordingRequestType_RecordAssociatedData_LocalPtr(); // Class
	m_RecordAssociatedData_Exist = false;


// no includefile for extension defined 
// file Mp7JrsRecordingRequestType_ExtMyPropInit.h

}

void Mp7JrsRecordingRequestType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsRecordingRequestType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_ContentIdentifier);
	// Mp7JrsFactory::DeleteObject(m_DisseminationSource);
	// Mp7JrsFactory::DeleteObject(m_Title);
	// Mp7JrsFactory::DeleteObject(m_RecordingPeriod);
	// Mp7JrsFactory::DeleteObject(m_RecordingLocationPreferences);
	// Mp7JrsFactory::DeleteObject(m_RecordingSourceTypePreferences);
	// Mp7JrsFactory::DeleteObject(m_RecordAssociatedData);
}

void Mp7JrsRecordingRequestType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use RecordingRequestTypePtr, since we
	// might need GetBase(), which isn't defined in IRecordingRequestType
	const Mp7JrsPtr< Mp7JrsRecordingRequestType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_ContentIdentifier);
		this->SetContentIdentifier(Mp7JrsFactory::CloneObject(tmp->GetContentIdentifier()));
	if (tmp->IsValidDisseminationSource())
	{
		// Mp7JrsFactory::DeleteObject(m_DisseminationSource);
		this->SetDisseminationSource(Mp7JrsFactory::CloneObject(tmp->GetDisseminationSource()));
	}
	else
	{
		InvalidateDisseminationSource();
	}
	if (tmp->IsValidTitle())
	{
		// Mp7JrsFactory::DeleteObject(m_Title);
		this->SetTitle(Mp7JrsFactory::CloneObject(tmp->GetTitle()));
	}
	else
	{
		InvalidateTitle();
	}
		// Mp7JrsFactory::DeleteObject(m_RecordingPeriod);
		this->SetRecordingPeriod(Mp7JrsFactory::CloneObject(tmp->GetRecordingPeriod()));
		// Mp7JrsFactory::DeleteObject(m_RecordingLocationPreferences);
		this->SetRecordingLocationPreferences(Mp7JrsFactory::CloneObject(tmp->GetRecordingLocationPreferences()));
		// Mp7JrsFactory::DeleteObject(m_RecordingSourceTypePreferences);
		this->SetRecordingSourceTypePreferences(Mp7JrsFactory::CloneObject(tmp->GetRecordingSourceTypePreferences()));
	if (tmp->IsValidRecordAssociatedData())
	{
		// Mp7JrsFactory::DeleteObject(m_RecordAssociatedData);
		this->SetRecordAssociatedData(Mp7JrsFactory::CloneObject(tmp->GetRecordAssociatedData()));
	}
	else
	{
		InvalidateRecordAssociatedData();
	}
}

Mp7JrsNodePtr Mp7JrsRecordingRequestType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsRecordingRequestType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsUniqueIDPtr Mp7JrsRecordingRequestType::GetContentIdentifier() const
{
		return m_ContentIdentifier;
}

Mp7JrsTextualPtr Mp7JrsRecordingRequestType::GetDisseminationSource() const
{
		return m_DisseminationSource;
}

// Element is optional
bool Mp7JrsRecordingRequestType::IsValidDisseminationSource() const
{
	return m_DisseminationSource_Exist;
}

Mp7JrsTextualPtr Mp7JrsRecordingRequestType::GetTitle() const
{
		return m_Title;
}

// Element is optional
bool Mp7JrsRecordingRequestType::IsValidTitle() const
{
	return m_Title_Exist;
}

Mp7JrsRecordingRequestType_RecordingPeriod_LocalPtr Mp7JrsRecordingRequestType::GetRecordingPeriod() const
{
		return m_RecordingPeriod;
}

Mp7JrsRecordingRequestType_RecordingLocationPreferences_CollectionPtr Mp7JrsRecordingRequestType::GetRecordingLocationPreferences() const
{
		return m_RecordingLocationPreferences;
}

Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_CollectionPtr Mp7JrsRecordingRequestType::GetRecordingSourceTypePreferences() const
{
		return m_RecordingSourceTypePreferences;
}

Mp7JrsRecordingRequestType_RecordAssociatedData_LocalPtr Mp7JrsRecordingRequestType::GetRecordAssociatedData() const
{
		return m_RecordAssociatedData;
}

// Element is optional
bool Mp7JrsRecordingRequestType::IsValidRecordAssociatedData() const
{
	return m_RecordAssociatedData_Exist;
}

void Mp7JrsRecordingRequestType::SetContentIdentifier(const Mp7JrsUniqueIDPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRecordingRequestType::SetContentIdentifier().");
	}
	if (m_ContentIdentifier != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ContentIdentifier);
		m_ContentIdentifier = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ContentIdentifier) m_ContentIdentifier->SetParent(m_myself.getPointer());
		if (m_ContentIdentifier && m_ContentIdentifier->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ContentIdentifier->UseTypeAttribute = true;
		}
		if(m_ContentIdentifier != Mp7JrsUniqueIDPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ContentIdentifier->SetContentName(XMLString::transcode("ContentIdentifier"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRecordingRequestType::SetDisseminationSource(const Mp7JrsTextualPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRecordingRequestType::SetDisseminationSource().");
	}
	if (m_DisseminationSource != item || m_DisseminationSource_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_DisseminationSource);
		m_DisseminationSource = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DisseminationSource) m_DisseminationSource->SetParent(m_myself.getPointer());
		if (m_DisseminationSource && m_DisseminationSource->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DisseminationSource->UseTypeAttribute = true;
		}
		m_DisseminationSource_Exist = true;
		if(m_DisseminationSource != Mp7JrsTextualPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DisseminationSource->SetContentName(XMLString::transcode("DisseminationSource"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRecordingRequestType::InvalidateDisseminationSource()
{
	m_DisseminationSource_Exist = false;
}
void Mp7JrsRecordingRequestType::SetTitle(const Mp7JrsTextualPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRecordingRequestType::SetTitle().");
	}
	if (m_Title != item || m_Title_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Title);
		m_Title = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Title) m_Title->SetParent(m_myself.getPointer());
		if (m_Title && m_Title->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Title->UseTypeAttribute = true;
		}
		m_Title_Exist = true;
		if(m_Title != Mp7JrsTextualPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Title->SetContentName(XMLString::transcode("Title"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRecordingRequestType::InvalidateTitle()
{
	m_Title_Exist = false;
}
void Mp7JrsRecordingRequestType::SetRecordingPeriod(const Mp7JrsRecordingRequestType_RecordingPeriod_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRecordingRequestType::SetRecordingPeriod().");
	}
	if (m_RecordingPeriod != item)
	{
		// Mp7JrsFactory::DeleteObject(m_RecordingPeriod);
		m_RecordingPeriod = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_RecordingPeriod) m_RecordingPeriod->SetParent(m_myself.getPointer());
		if (m_RecordingPeriod && m_RecordingPeriod->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RecordingRequestType_RecordingPeriod_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_RecordingPeriod->UseTypeAttribute = true;
		}
		if(m_RecordingPeriod != Mp7JrsRecordingRequestType_RecordingPeriod_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_RecordingPeriod->SetContentName(XMLString::transcode("RecordingPeriod"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRecordingRequestType::SetRecordingLocationPreferences(const Mp7JrsRecordingRequestType_RecordingLocationPreferences_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRecordingRequestType::SetRecordingLocationPreferences().");
	}
	if (m_RecordingLocationPreferences != item)
	{
		// Mp7JrsFactory::DeleteObject(m_RecordingLocationPreferences);
		m_RecordingLocationPreferences = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_RecordingLocationPreferences) m_RecordingLocationPreferences->SetParent(m_myself.getPointer());
		if(m_RecordingLocationPreferences != Mp7JrsRecordingRequestType_RecordingLocationPreferences_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_RecordingLocationPreferences->SetContentName(XMLString::transcode("RecordingLocationPreferences"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRecordingRequestType::SetRecordingSourceTypePreferences(const Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRecordingRequestType::SetRecordingSourceTypePreferences().");
	}
	if (m_RecordingSourceTypePreferences != item)
	{
		// Mp7JrsFactory::DeleteObject(m_RecordingSourceTypePreferences);
		m_RecordingSourceTypePreferences = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_RecordingSourceTypePreferences) m_RecordingSourceTypePreferences->SetParent(m_myself.getPointer());
		if(m_RecordingSourceTypePreferences != Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_RecordingSourceTypePreferences->SetContentName(XMLString::transcode("RecordingSourceTypePreferences"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRecordingRequestType::SetRecordAssociatedData(const Mp7JrsRecordingRequestType_RecordAssociatedData_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRecordingRequestType::SetRecordAssociatedData().");
	}
	if (m_RecordAssociatedData != item || m_RecordAssociatedData_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_RecordAssociatedData);
		m_RecordAssociatedData = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_RecordAssociatedData) m_RecordAssociatedData->SetParent(m_myself.getPointer());
		if (m_RecordAssociatedData && m_RecordAssociatedData->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RecordingRequestType_RecordAssociatedData_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_RecordAssociatedData->UseTypeAttribute = true;
		}
		m_RecordAssociatedData_Exist = true;
		if(m_RecordAssociatedData != Mp7JrsRecordingRequestType_RecordAssociatedData_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_RecordAssociatedData->SetContentName(XMLString::transcode("RecordAssociatedData"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRecordingRequestType::InvalidateRecordAssociatedData()
{
	m_RecordAssociatedData_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsRecordingRequestType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("ContentIdentifier")) == 0)
	{
		// ContentIdentifier is simple element UniqueIDType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetContentIdentifier()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUniqueIDPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetContentIdentifier(p, client);
					if((p = GetContentIdentifier()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("DisseminationSource")) == 0)
	{
		// DisseminationSource is simple element TextualType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDisseminationSource()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextualPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDisseminationSource(p, client);
					if((p = GetDisseminationSource()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Title")) == 0)
	{
		// Title is simple element TextualType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTitle()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextualPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTitle(p, client);
					if((p = GetTitle()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("RecordingPeriod")) == 0)
	{
		// RecordingPeriod is simple element RecordingRequestType_RecordingPeriod_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRecordingPeriod()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RecordingRequestType_RecordingPeriod_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRecordingRequestType_RecordingPeriod_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRecordingPeriod(p, client);
					if((p = GetRecordingPeriod()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("RecordingLocationPreferences")) == 0)
	{
		// Some primitive nodes types cannot be handled
		Mp7JrsXPathParseContext::ErrorUnsupportedType(context, "XMLCh *");
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("RecordingSourceTypePreferences")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:RecordingSourceTypePreferences is item of type RecordingRequestType_RecordingSourceTypePreferences_LocalType
		// in element collection RecordingRequestType_RecordingSourceTypePreferences_CollectionType
		
		context->Found = true;
		Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_CollectionPtr coll = GetRecordingSourceTypePreferences();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateRecordingRequestType_RecordingSourceTypePreferences_CollectionType;
				SetRecordingSourceTypePreferences(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), 2))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RecordingRequestType_RecordingSourceTypePreferences_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("RecordAssociatedData")) == 0)
	{
		// RecordAssociatedData is simple element RecordingRequestType_RecordAssociatedData_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRecordAssociatedData()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RecordingRequestType_RecordAssociatedData_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRecordingRequestType_RecordAssociatedData_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRecordAssociatedData(p, client);
					if((p = GetRecordAssociatedData()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for RecordingRequestType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "RecordingRequestType");
	}
	return result;
}

XMLCh * Mp7JrsRecordingRequestType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsRecordingRequestType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsRecordingRequestType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("RecordingRequestType"));
	// Element serialization:
	// Class
	
	if (m_ContentIdentifier != Mp7JrsUniqueIDPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ContentIdentifier->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ContentIdentifier"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ContentIdentifier->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_DisseminationSource != Mp7JrsTextualPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DisseminationSource->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DisseminationSource"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DisseminationSource->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Title != Mp7JrsTextualPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Title->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Title"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Title->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_RecordingPeriod != Mp7JrsRecordingRequestType_RecordingPeriod_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_RecordingPeriod->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("RecordingPeriod"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_RecordingPeriod->Serialize(doc, element, &elem);
	}
	if (m_RecordingLocationPreferences != Mp7JrsRecordingRequestType_RecordingLocationPreferences_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_RecordingLocationPreferences->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_RecordingSourceTypePreferences != Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_RecordingSourceTypePreferences->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_RecordAssociatedData != Mp7JrsRecordingRequestType_RecordAssociatedData_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_RecordAssociatedData->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("RecordAssociatedData"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_RecordAssociatedData->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsRecordingRequestType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ContentIdentifier"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ContentIdentifier")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateUniqueIDType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetContentIdentifier(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DisseminationSource"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DisseminationSource")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateTextualType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetDisseminationSource(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Title"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Title")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateTextualType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetTitle(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("RecordingPeriod"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("RecordingPeriod")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateRecordingRequestType_RecordingPeriod_LocalType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetRecordingPeriod(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("RecordingLocationPreferences"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("RecordingLocationPreferences")) == 0))
		{
			// Deserialize factory type
			Mp7JrsRecordingRequestType_RecordingLocationPreferences_CollectionPtr tmp = CreateRecordingRequestType_RecordingLocationPreferences_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetRecordingLocationPreferences(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("RecordingSourceTypePreferences"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("RecordingSourceTypePreferences")) == 0))
		{
			// Deserialize factory type
			Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_CollectionPtr tmp = CreateRecordingRequestType_RecordingSourceTypePreferences_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetRecordingSourceTypePreferences(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("RecordAssociatedData"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("RecordAssociatedData")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateRecordingRequestType_RecordAssociatedData_LocalType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetRecordAssociatedData(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsRecordingRequestType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsRecordingRequestType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("ContentIdentifier")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateUniqueIDType;
	}
	this->SetContentIdentifier(child);
  }
  if (XMLString::compareString(elementname, X("DisseminationSource")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateTextualType;
	}
	this->SetDisseminationSource(child);
  }
  if (XMLString::compareString(elementname, X("Title")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateTextualType;
	}
	this->SetTitle(child);
  }
  if (XMLString::compareString(elementname, X("RecordingPeriod")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateRecordingRequestType_RecordingPeriod_LocalType;
	}
	this->SetRecordingPeriod(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("RecordingLocationPreferences")) == 0))
  {
	Mp7JrsRecordingRequestType_RecordingLocationPreferences_CollectionPtr tmp = CreateRecordingRequestType_RecordingLocationPreferences_CollectionType;
	this->SetRecordingLocationPreferences(tmp);
	child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("RecordingSourceTypePreferences")) == 0))
  {
	Mp7JrsRecordingRequestType_RecordingSourceTypePreferences_CollectionPtr tmp = CreateRecordingRequestType_RecordingSourceTypePreferences_CollectionType;
	this->SetRecordingSourceTypePreferences(tmp);
	child = tmp;
  }
  if (XMLString::compareString(elementname, X("RecordAssociatedData")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateRecordingRequestType_RecordAssociatedData_LocalType;
	}
	this->SetRecordAssociatedData(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsRecordingRequestType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetContentIdentifier() != Mp7JrsNodePtr())
		result.Insert(GetContentIdentifier());
	if (GetDisseminationSource() != Mp7JrsNodePtr())
		result.Insert(GetDisseminationSource());
	if (GetTitle() != Mp7JrsNodePtr())
		result.Insert(GetTitle());
	if (GetRecordingPeriod() != Mp7JrsNodePtr())
		result.Insert(GetRecordingPeriod());
	if (GetRecordingLocationPreferences() != Mp7JrsNodePtr())
		result.Insert(GetRecordingLocationPreferences());
	if (GetRecordingSourceTypePreferences() != Mp7JrsNodePtr())
		result.Insert(GetRecordingSourceTypePreferences());
	if (GetRecordAssociatedData() != Mp7JrsNodePtr())
		result.Insert(GetRecordAssociatedData());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsRecordingRequestType_ExtMethodImpl.h


