 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsRightsType_ExtImplInclude.h



#include "Mp7JrsRightsType_RightsID_CollectionType.h"
#include "Mp7JrsRightsType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsUniqueIDType.h" // Element collection urn:mpeg:mpeg7:schema:2004:RightsID

#include <assert.h>
IMp7JrsRightsType::IMp7JrsRightsType()
{

// no includefile for extension defined 
// file Mp7JrsRightsType_ExtPropInit.h

}

IMp7JrsRightsType::~IMp7JrsRightsType()
{
// no includefile for extension defined 
// file Mp7JrsRightsType_ExtPropCleanup.h

}

Mp7JrsRightsType::Mp7JrsRightsType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsRightsType::~Mp7JrsRightsType()
{
	Cleanup();
}

void Mp7JrsRightsType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_RightsID = Mp7JrsRightsType_RightsID_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsRightsType_ExtMyPropInit.h

}

void Mp7JrsRightsType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsRightsType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_RightsID);
}

void Mp7JrsRightsType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use RightsTypePtr, since we
	// might need GetBase(), which isn't defined in IRightsType
	const Mp7JrsPtr< Mp7JrsRightsType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_RightsID);
		this->SetRightsID(Mp7JrsFactory::CloneObject(tmp->GetRightsID()));
}

Mp7JrsNodePtr Mp7JrsRightsType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsRightsType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsRightsType_RightsID_CollectionPtr Mp7JrsRightsType::GetRightsID() const
{
		return m_RightsID;
}

void Mp7JrsRightsType::SetRightsID(const Mp7JrsRightsType_RightsID_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRightsType::SetRightsID().");
	}
	if (m_RightsID != item)
	{
		// Mp7JrsFactory::DeleteObject(m_RightsID);
		m_RightsID = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_RightsID) m_RightsID->SetParent(m_myself.getPointer());
		if(m_RightsID != Mp7JrsRightsType_RightsID_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_RightsID->SetContentName(XMLString::transcode("RightsID"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsRightsType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("RightsID")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:RightsID is item of type UniqueIDType
		// in element collection RightsType_RightsID_CollectionType
		
		context->Found = true;
		Mp7JrsRightsType_RightsID_CollectionPtr coll = GetRightsID();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateRightsType_RightsID_CollectionType;
				SetRightsID(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUniqueIDPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for RightsType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "RightsType");
	}
	return result;
}

XMLCh * Mp7JrsRightsType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsRightsType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsRightsType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("RightsType"));
	// Element serialization:
	if (m_RightsID != Mp7JrsRightsType_RightsID_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_RightsID->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsRightsType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("RightsID"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("RightsID")) == 0))
		{
			// Deserialize factory type
			Mp7JrsRightsType_RightsID_CollectionPtr tmp = CreateRightsType_RightsID_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetRightsID(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsRightsType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsRightsType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("RightsID")) == 0))
  {
	Mp7JrsRightsType_RightsID_CollectionPtr tmp = CreateRightsType_RightsID_CollectionType;
	this->SetRightsID(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsRightsType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetRightsID() != Mp7JrsNodePtr())
		result.Insert(GetRightsID());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsRightsType_ExtMethodImpl.h


