
// Based on SchemaLocationMap_cpp.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include "Mp7JrsSchemaLocationMap.h"
#include "Mp7JrsUtil.h"

// TODO parameter consistency ! uri, prefix vs prefix, uri

Mp7JrsSchemaLocationMap::Mp7JrsSchemaLocationMap(void) {
	int n = 20;
	_locationMap = new Mp7JrsRefVectorOf<SchemaData>(n);
	_nsIter = 0;
	_newNs = true;
	_defaultNS = 0;
	_defaultNSPrefix = 0;
}

Mp7JrsSchemaLocationMap::~Mp7JrsSchemaLocationMap(void) {
	unsigned int i;
	for (i=0;i<_locationMap->size();i++) {
		SchemaData* sd = _locationMap->elementAt(i);
		SchemaProperties* sp = (SchemaProperties*) sd->getValue();
		sd->setValue(NULL);
		_DeleteSchemaProperties(&sp);
		
		// gcc 4 does not like this line, so we split it into two
		//XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&((XMLCh*) sd->getKey()));
			
		XMLCh* key = (XMLCh*) sd->getKey();
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&key);
	}
	_locationMap->removeAllElements();
	delete _locationMap;
	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&_defaultNS);
	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&_defaultNSPrefix);
}

void Mp7JrsSchemaLocationMap::SetSchemaLocation(const XMLCh* namespaceURI,const  XMLCh* prefix,const  XMLCh* schemaLocation) {
	// check if schema already in list
	SchemaData* sl = _GetSchemaData(namespaceURI);
	// if so, delete old location and set new
	if (sl) {
		SchemaProperties* oldProps = (SchemaProperties*) sl->getValue();
		SchemaProperties* sp = new SchemaProperties;
		int n = 3;
		sp->prefixes = new Mp7JrsRefVectorOf<XMLCh>(n);
		sp->prefixes->addElement(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(prefix));
		sp->location = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(schemaLocation);
		sp->prefix_it = 0;		
		sl->setValue(sp);
		_DeleteSchemaProperties(&oldProps);
	}
	// otherwise add new pair
	else {
		SchemaProperties* sp = new SchemaProperties;
		int n = 3;
		sp->prefixes = new Mp7JrsRefVectorOf<XMLCh>(n);
		sp->prefixes->addElement(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(prefix));
		sp->location = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(schemaLocation);
		sp->prefix_it = 0;
		sl = new SchemaData(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(namespaceURI),sp);
		_locationMap->addElement(sl);
	}
}

void Mp7JrsSchemaLocationMap::AddPrefixAlias(XMLCh* namespaceURI, XMLCh* prefix) {
	// check if schema already in list
	SchemaData* sl = _GetSchemaData(namespaceURI);
	if (sl) {
		SchemaProperties* sp = (SchemaProperties*) sl->getValue();
		sp->prefixes->addElement(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(prefix));	
	}
}

void Mp7JrsSchemaLocationMap::SetSchemaLocation(const char* namespaceURI, const char* prefix, const char* schemaLocation) {
	// check if schema already in list
	SchemaData* sl = _GetSchemaData(X(namespaceURI));
	// if so, delete old location and set new
	if (sl) {
		SchemaProperties* oldProps = (SchemaProperties*) sl->getValue();
		SchemaProperties* sp = new SchemaProperties;
		int n = 3;
		sp->prefixes = new Mp7JrsRefVectorOf<XMLCh>(n);
		sp->prefixes->addElement(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(prefix));
		sp->location = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(schemaLocation);
		sl->setValue(sp);
		_DeleteSchemaProperties(&oldProps);
	}
	// otherwise add new pair
	else {
		SchemaProperties* sp = new SchemaProperties;
		int n = 3;
		sp->prefixes = new Mp7JrsRefVectorOf<XMLCh>(n);
		sp->prefixes->addElement(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(prefix));
		sp->location = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(schemaLocation);
		sl = new SchemaData(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(namespaceURI),sp);
		_locationMap->addElement(sl);
	}
}

void Mp7JrsSchemaLocationMap::AddPrefixAlias(const char* namespaceURI, const char* prefix) {
	// check if schema already in list
	SchemaData* sl = _GetSchemaData(X(namespaceURI));
	if (sl) {
		// SchemaProperties* oldProps = (SchemaProperties*) sl->getValue();
		SchemaProperties* sp = new SchemaProperties;
		sp->prefixes->addElement(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(prefix));	
	}
}

const XMLCh* Mp7JrsSchemaLocationMap::GetSchemaLocation(const XMLCh* namespaceURI) {
	SchemaData* sl = _GetSchemaData(namespaceURI);
	if (!sl) return NULL;
	return sl->getValue()->location;
}

const XMLCh* Mp7JrsSchemaLocationMap::GetFirstSchemaPrefix(const XMLCh* namespaceURI) {
	SchemaData* sl = _GetSchemaData(namespaceURI);
	if (!sl) return NULL;
	if (sl->getValue()->prefixes->size() == 0) return NULL;
	sl->getValue()->prefix_it = 0;
	return sl->getValue()->prefixes->elementAt(0);
}

const XMLCh* Mp7JrsSchemaLocationMap::GetNextSchemaPrefix(const XMLCh* namespaceURI) {
	SchemaData* sl = _GetSchemaData(namespaceURI);
	if (!sl) return NULL;
	if (sl->getValue()->prefix_it +1 >= (int)sl->getValue()->prefixes->size()) return NULL;
	sl->getValue()->prefix_it++;
	return sl->getValue()->prefixes->elementAt(sl->getValue()->prefix_it);
}

/**
* Retrieves a key value list of namespaceURI and its corresponding schemalocation.
*
*/
XMLCh* Mp7JrsSchemaLocationMap::ToText() const {
	int nElems = _locationMap->size();
	int i;
	XMLCh* schemaList = NULL;
	
	for (i = 0; i < nElems; ++i) {
		SchemaData * sl = _locationMap->elementAt(i);
		// Do not include if location is empty
		if (XMLString::stringLen(sl->getValue()->location) != 0) {
		  Mp7JrsUtil::CatString(&schemaList, sl->getKey());
		  Mp7JrsUtil::CatString(&schemaList, X(" "));
			Mp7JrsUtil::CatString(&schemaList, sl->getValue()->location);
		}
		Mp7JrsUtil::CatString(&schemaList, X(" "));
	}
	return schemaList;
}

bool Mp7JrsSchemaLocationMap::GetFirstNamespace(const XMLCh* * prefix, const XMLCh* * ns) {
	_nsIter = 0;
	_newNs = true; // TODO _newNS is deprecated
	return GetNextNamespace(prefix,ns);
}

bool Mp7JrsSchemaLocationMap::GetNextNamespace(const XMLCh * * prefix, const XMLCh* * ns) {
	if (_nsIter>=_locationMap->size()) return false;
	SchemaData* sd = _locationMap->elementAt(_nsIter);
	if (!sd) return false;
	*ns = sd->getKey();
	*prefix = GetFirstSchemaPrefix(*ns);
	
	_nsIter++;
	return (*ns != 0);
}

void Mp7JrsSchemaLocationMap::SetDefaultNamespace(const char* namespaceURI, const char* prefix) {
	XMLCh* prefix1 = XMLString::transcode(prefix);
	XMLCh* ns1 = XMLString::transcode(namespaceURI);
	SetDefaultNamespace(ns1, prefix1);
	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&prefix1);
	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&ns1);
}


void Mp7JrsSchemaLocationMap::SetDefaultNamespace(const XMLCh* namespaceURI, const XMLCh* prefix) {
	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&_defaultNS);
	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&_defaultNSPrefix);
	_defaultNSPrefix = XMLString::replicate(prefix);
	_defaultNS = XMLString::replicate(namespaceURI);
}

bool Mp7JrsSchemaLocationMap::GetDefaultNamespace(const XMLCh* * namespaceURI, const XMLCh* * prefix) {
	if(_defaultNS != 0){
		*namespaceURI = _defaultNS;
		*prefix = _defaultNSPrefix;
	}
	return _defaultNS != 0;
}

//////////////////////////////////////////////////////////////////////////////////////////

SchemaData* Mp7JrsSchemaLocationMap::_GetSchemaData(const XMLCh* namespaceURI) {
	if(!namespaceURI) return NULL;
	int nElems = _locationMap->size();
	int i;
	for (i=0;i<nElems;i++) {
		SchemaData* sl = _locationMap->elementAt(i);
		if (sl && XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(sl->getKey(),namespaceURI)==0) return sl;
	}
	return NULL;
}


void Mp7JrsSchemaLocationMap::_DeleteSchemaProperties(SchemaProperties** sp) {
	// remove all XMLCh*s in prefixes list manually
	while ((*sp)->prefixes->size()>0) {
		XMLCh* pref = (*sp)->prefixes->orphanElementAt(0);
		XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&pref);
	}
	(*sp)->prefixes->removeAllElements();
	delete (*sp)->prefixes;
	XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&((*sp)->location));
	delete (*sp);
	*sp = NULL;
}
