 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSentencesType_LocalType_ExtImplInclude.h



#include "Mp7JrsSentencesType.h"
#include "Mp7JrsSyntacticConstituentType.h"
#include "Mp7JrsLinguisticDocumentType.h"
#include "Mp7JrsSentencesType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsSentencesType_LocalType::IMp7JrsSentencesType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsSentencesType_LocalType_ExtPropInit.h

}

IMp7JrsSentencesType_LocalType::~IMp7JrsSentencesType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsSentencesType_LocalType_ExtPropCleanup.h

}

Mp7JrsSentencesType_LocalType::Mp7JrsSentencesType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSentencesType_LocalType::~Mp7JrsSentencesType_LocalType()
{
	Cleanup();
}

void Mp7JrsSentencesType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_Sentences = Mp7JrsSentencesPtr(); // Class
	m_Sentence = Mp7JrsSyntacticConstituentPtr(); // Class
	m_Quotation = Mp7JrsLinguisticDocumentPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsSentencesType_LocalType_ExtMyPropInit.h

}

void Mp7JrsSentencesType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSentencesType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Sentences);
	// Mp7JrsFactory::DeleteObject(m_Sentence);
	// Mp7JrsFactory::DeleteObject(m_Quotation);
}

void Mp7JrsSentencesType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SentencesType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in ISentencesType_LocalType
	const Mp7JrsPtr< Mp7JrsSentencesType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_Sentences);
		this->SetSentences(Mp7JrsFactory::CloneObject(tmp->GetSentences()));
		// Mp7JrsFactory::DeleteObject(m_Sentence);
		this->SetSentence(Mp7JrsFactory::CloneObject(tmp->GetSentence()));
		// Mp7JrsFactory::DeleteObject(m_Quotation);
		this->SetQuotation(Mp7JrsFactory::CloneObject(tmp->GetQuotation()));
}

Mp7JrsNodePtr Mp7JrsSentencesType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsSentencesType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsSentencesPtr Mp7JrsSentencesType_LocalType::GetSentences() const
{
		return m_Sentences;
}

Mp7JrsSyntacticConstituentPtr Mp7JrsSentencesType_LocalType::GetSentence() const
{
		return m_Sentence;
}

Mp7JrsLinguisticDocumentPtr Mp7JrsSentencesType_LocalType::GetQuotation() const
{
		return m_Quotation;
}

// implementing setter for choice 
/* element */
void Mp7JrsSentencesType_LocalType::SetSentences(const Mp7JrsSentencesPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSentencesType_LocalType::SetSentences().");
	}
	if (m_Sentences != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Sentences);
		m_Sentences = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Sentences) m_Sentences->SetParent(m_myself.getPointer());
		if (m_Sentences && m_Sentences->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SentencesType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Sentences->UseTypeAttribute = true;
		}
		if(m_Sentences != Mp7JrsSentencesPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Sentences->SetContentName(XMLString::transcode("Sentences"));
		}
	// Mp7JrsFactory::DeleteObject(m_Sentence);
	m_Sentence = Mp7JrsSyntacticConstituentPtr();
	// Mp7JrsFactory::DeleteObject(m_Quotation);
	m_Quotation = Mp7JrsLinguisticDocumentPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsSentencesType_LocalType::SetSentence(const Mp7JrsSyntacticConstituentPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSentencesType_LocalType::SetSentence().");
	}
	if (m_Sentence != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Sentence);
		m_Sentence = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Sentence) m_Sentence->SetParent(m_myself.getPointer());
		if (m_Sentence && m_Sentence->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SyntacticConstituentType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Sentence->UseTypeAttribute = true;
		}
		if(m_Sentence != Mp7JrsSyntacticConstituentPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Sentence->SetContentName(XMLString::transcode("Sentence"));
		}
	// Mp7JrsFactory::DeleteObject(m_Sentences);
	m_Sentences = Mp7JrsSentencesPtr();
	// Mp7JrsFactory::DeleteObject(m_Quotation);
	m_Quotation = Mp7JrsLinguisticDocumentPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsSentencesType_LocalType::SetQuotation(const Mp7JrsLinguisticDocumentPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSentencesType_LocalType::SetQuotation().");
	}
	if (m_Quotation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Quotation);
		m_Quotation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Quotation) m_Quotation->SetParent(m_myself.getPointer());
		if (m_Quotation && m_Quotation->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:LinguisticDocumentType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Quotation->UseTypeAttribute = true;
		}
		if(m_Quotation != Mp7JrsLinguisticDocumentPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Quotation->SetContentName(XMLString::transcode("Quotation"));
		}
	// Mp7JrsFactory::DeleteObject(m_Sentences);
	m_Sentences = Mp7JrsSentencesPtr();
	// Mp7JrsFactory::DeleteObject(m_Sentence);
	m_Sentence = Mp7JrsSyntacticConstituentPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: SentencesType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsSentencesType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsSentencesType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSentencesType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsSentencesType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_Sentences != Mp7JrsSentencesPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Sentences->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Sentences"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Sentences->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Sentence != Mp7JrsSyntacticConstituentPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Sentence->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Sentence"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Sentence->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Quotation != Mp7JrsLinguisticDocumentPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Quotation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Quotation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Quotation->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsSentencesType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
	{
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Sentences"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Sentences")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateSentencesType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSentences(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Sentence"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Sentence")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateSyntacticConstituentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSentence(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Quotation"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Quotation")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateLinguisticDocumentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetQuotation(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	} while(false);
// no includefile for extension defined 
// file Mp7JrsSentencesType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSentencesType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Sentences")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateSentencesType;
	}
	this->SetSentences(child);
  }
  if (XMLString::compareString(elementname, X("Sentence")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateSyntacticConstituentType;
	}
	this->SetSentence(child);
  }
  if (XMLString::compareString(elementname, X("Quotation")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateLinguisticDocumentType;
	}
	this->SetQuotation(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSentencesType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSentences() != Mp7JrsNodePtr())
		result.Insert(GetSentences());
	if (GetSentence() != Mp7JrsNodePtr())
		result.Insert(GetSentence());
	if (GetQuotation() != Mp7JrsNodePtr())
		result.Insert(GetQuotation());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSentencesType_LocalType_ExtMethodImpl.h


