 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSeriesOfVectorType_ExtImplInclude.h



#include "Mp7JrsScalableSeriesType.h"
#include "Mp7JrsFloatMatrixType.h"
#include "Mp7JrsfloatVector.h"
#include "Mp7JrsScalableSeriesType_Scaling_CollectionType.h"
#include "Mp7JrsSeriesOfVectorType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsScalableSeriesType_Scaling_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Scaling

#include <assert.h>
IMp7JrsSeriesOfVectorType::IMp7JrsSeriesOfVectorType()
{

// no includefile for extension defined 
// file Mp7JrsSeriesOfVectorType_ExtPropInit.h

}

IMp7JrsSeriesOfVectorType::~IMp7JrsSeriesOfVectorType()
{
// no includefile for extension defined 
// file Mp7JrsSeriesOfVectorType_ExtPropCleanup.h

}

Mp7JrsSeriesOfVectorType::Mp7JrsSeriesOfVectorType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSeriesOfVectorType::~Mp7JrsSeriesOfVectorType()
{
	Cleanup();
}

void Mp7JrsSeriesOfVectorType::Init()
{
	// Init base
	m_Base = CreateScalableSeriesType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_vectorSize = 1; // Value
	m_vectorSize_Default = 1; // Default value
	m_vectorSize_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Raw = Mp7JrsFloatMatrixPtr(); // Class
	m_Raw_Exist = false;
	m_Min = Mp7JrsFloatMatrixPtr(); // Class
	m_Min_Exist = false;
	m_Max = Mp7JrsFloatMatrixPtr(); // Class
	m_Max_Exist = false;
	m_Mean = Mp7JrsFloatMatrixPtr(); // Class
	m_Mean_Exist = false;
	m_Random = Mp7JrsFloatMatrixPtr(); // Class
	m_Random_Exist = false;
	m_First = Mp7JrsFloatMatrixPtr(); // Class
	m_First_Exist = false;
	m_Last = Mp7JrsFloatMatrixPtr(); // Class
	m_Last_Exist = false;
	m_Variance = Mp7JrsFloatMatrixPtr(); // Class
	m_Variance_Exist = false;
	m_Covariance = Mp7JrsFloatMatrixPtr(); // Class
	m_Covariance_Exist = false;
	m_VarianceSummed = Mp7JrsfloatVectorPtr(); // Collection
	m_VarianceSummed_Exist = false;
	m_MaxSqDist = Mp7JrsfloatVectorPtr(); // Collection
	m_MaxSqDist_Exist = false;
	m_Weight = Mp7JrsfloatVectorPtr(); // Collection
	m_Weight_Exist = false;


// no includefile for extension defined 
// file Mp7JrsSeriesOfVectorType_ExtMyPropInit.h

}

void Mp7JrsSeriesOfVectorType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSeriesOfVectorType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Raw);
	// Mp7JrsFactory::DeleteObject(m_Min);
	// Mp7JrsFactory::DeleteObject(m_Max);
	// Mp7JrsFactory::DeleteObject(m_Mean);
	// Mp7JrsFactory::DeleteObject(m_Random);
	// Mp7JrsFactory::DeleteObject(m_First);
	// Mp7JrsFactory::DeleteObject(m_Last);
	// Mp7JrsFactory::DeleteObject(m_Variance);
	// Mp7JrsFactory::DeleteObject(m_Covariance);
	// Mp7JrsFactory::DeleteObject(m_VarianceSummed);
	// Mp7JrsFactory::DeleteObject(m_MaxSqDist);
	// Mp7JrsFactory::DeleteObject(m_Weight);
}

void Mp7JrsSeriesOfVectorType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SeriesOfVectorTypePtr, since we
	// might need GetBase(), which isn't defined in ISeriesOfVectorType
	const Mp7JrsPtr< Mp7JrsSeriesOfVectorType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsScalableSeriesPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSeriesOfVectorType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->ExistvectorSize())
	{
		this->SetvectorSize(tmp->GetvectorSize());
	}
	else
	{
		InvalidatevectorSize();
	}
	}
	if (tmp->IsValidRaw())
	{
		// Mp7JrsFactory::DeleteObject(m_Raw);
		this->SetRaw(Mp7JrsFactory::CloneObject(tmp->GetRaw()));
	}
	else
	{
		InvalidateRaw();
	}
	if (tmp->IsValidMin())
	{
		// Mp7JrsFactory::DeleteObject(m_Min);
		this->SetMin(Mp7JrsFactory::CloneObject(tmp->GetMin()));
	}
	else
	{
		InvalidateMin();
	}
	if (tmp->IsValidMax())
	{
		// Mp7JrsFactory::DeleteObject(m_Max);
		this->SetMax(Mp7JrsFactory::CloneObject(tmp->GetMax()));
	}
	else
	{
		InvalidateMax();
	}
	if (tmp->IsValidMean())
	{
		// Mp7JrsFactory::DeleteObject(m_Mean);
		this->SetMean(Mp7JrsFactory::CloneObject(tmp->GetMean()));
	}
	else
	{
		InvalidateMean();
	}
	if (tmp->IsValidRandom())
	{
		// Mp7JrsFactory::DeleteObject(m_Random);
		this->SetRandom(Mp7JrsFactory::CloneObject(tmp->GetRandom()));
	}
	else
	{
		InvalidateRandom();
	}
	if (tmp->IsValidFirst())
	{
		// Mp7JrsFactory::DeleteObject(m_First);
		this->SetFirst(Mp7JrsFactory::CloneObject(tmp->GetFirst()));
	}
	else
	{
		InvalidateFirst();
	}
	if (tmp->IsValidLast())
	{
		// Mp7JrsFactory::DeleteObject(m_Last);
		this->SetLast(Mp7JrsFactory::CloneObject(tmp->GetLast()));
	}
	else
	{
		InvalidateLast();
	}
	if (tmp->IsValidVariance())
	{
		// Mp7JrsFactory::DeleteObject(m_Variance);
		this->SetVariance(Mp7JrsFactory::CloneObject(tmp->GetVariance()));
	}
	else
	{
		InvalidateVariance();
	}
	if (tmp->IsValidCovariance())
	{
		// Mp7JrsFactory::DeleteObject(m_Covariance);
		this->SetCovariance(Mp7JrsFactory::CloneObject(tmp->GetCovariance()));
	}
	else
	{
		InvalidateCovariance();
	}
	if (tmp->IsValidVarianceSummed())
	{
		// Mp7JrsFactory::DeleteObject(m_VarianceSummed);
		this->SetVarianceSummed(Mp7JrsFactory::CloneObject(tmp->GetVarianceSummed()));
	}
	else
	{
		InvalidateVarianceSummed();
	}
	if (tmp->IsValidMaxSqDist())
	{
		// Mp7JrsFactory::DeleteObject(m_MaxSqDist);
		this->SetMaxSqDist(Mp7JrsFactory::CloneObject(tmp->GetMaxSqDist()));
	}
	else
	{
		InvalidateMaxSqDist();
	}
	if (tmp->IsValidWeight())
	{
		// Mp7JrsFactory::DeleteObject(m_Weight);
		this->SetWeight(Mp7JrsFactory::CloneObject(tmp->GetWeight()));
	}
	else
	{
		InvalidateWeight();
	}
}

Mp7JrsNodePtr Mp7JrsSeriesOfVectorType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsSeriesOfVectorType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsScalableSeriesType > Mp7JrsSeriesOfVectorType::GetBase() const
{
	return m_Base;
}

unsigned Mp7JrsSeriesOfVectorType::GetvectorSize() const
{
	if (this->ExistvectorSize()) {
		return m_vectorSize;
	} else {
		return m_vectorSize_Default;
	}
}

bool Mp7JrsSeriesOfVectorType::ExistvectorSize() const
{
	return m_vectorSize_Exist;
}
void Mp7JrsSeriesOfVectorType::SetvectorSize(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetvectorSize().");
	}
	m_vectorSize = item;
	m_vectorSize_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidatevectorSize()
{
	m_vectorSize_Exist = false;
}
Mp7JrsFloatMatrixPtr Mp7JrsSeriesOfVectorType::GetRaw() const
{
		return m_Raw;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidRaw() const
{
	return m_Raw_Exist;
}

Mp7JrsFloatMatrixPtr Mp7JrsSeriesOfVectorType::GetMin() const
{
		return m_Min;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidMin() const
{
	return m_Min_Exist;
}

Mp7JrsFloatMatrixPtr Mp7JrsSeriesOfVectorType::GetMax() const
{
		return m_Max;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidMax() const
{
	return m_Max_Exist;
}

Mp7JrsFloatMatrixPtr Mp7JrsSeriesOfVectorType::GetMean() const
{
		return m_Mean;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidMean() const
{
	return m_Mean_Exist;
}

Mp7JrsFloatMatrixPtr Mp7JrsSeriesOfVectorType::GetRandom() const
{
		return m_Random;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidRandom() const
{
	return m_Random_Exist;
}

Mp7JrsFloatMatrixPtr Mp7JrsSeriesOfVectorType::GetFirst() const
{
		return m_First;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidFirst() const
{
	return m_First_Exist;
}

Mp7JrsFloatMatrixPtr Mp7JrsSeriesOfVectorType::GetLast() const
{
		return m_Last;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidLast() const
{
	return m_Last_Exist;
}

Mp7JrsFloatMatrixPtr Mp7JrsSeriesOfVectorType::GetVariance() const
{
		return m_Variance;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidVariance() const
{
	return m_Variance_Exist;
}

Mp7JrsFloatMatrixPtr Mp7JrsSeriesOfVectorType::GetCovariance() const
{
		return m_Covariance;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidCovariance() const
{
	return m_Covariance_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfVectorType::GetVarianceSummed() const
{
		return m_VarianceSummed;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidVarianceSummed() const
{
	return m_VarianceSummed_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfVectorType::GetMaxSqDist() const
{
		return m_MaxSqDist;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidMaxSqDist() const
{
	return m_MaxSqDist_Exist;
}

Mp7JrsfloatVectorPtr Mp7JrsSeriesOfVectorType::GetWeight() const
{
		return m_Weight;
}

// Element is optional
bool Mp7JrsSeriesOfVectorType::IsValidWeight() const
{
	return m_Weight_Exist;
}

void Mp7JrsSeriesOfVectorType::SetRaw(const Mp7JrsFloatMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetRaw().");
	}
	if (m_Raw != item || m_Raw_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Raw);
		m_Raw = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Raw) m_Raw->SetParent(m_myself.getPointer());
		if (m_Raw && m_Raw->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Raw->UseTypeAttribute = true;
		}
		m_Raw_Exist = true;
		if(m_Raw != Mp7JrsFloatMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Raw->SetContentName(XMLString::transcode("Raw"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateRaw()
{
	m_Raw_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetMin(const Mp7JrsFloatMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetMin().");
	}
	if (m_Min != item || m_Min_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Min);
		m_Min = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Min) m_Min->SetParent(m_myself.getPointer());
		if (m_Min && m_Min->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Min->UseTypeAttribute = true;
		}
		m_Min_Exist = true;
		if(m_Min != Mp7JrsFloatMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Min->SetContentName(XMLString::transcode("Min"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateMin()
{
	m_Min_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetMax(const Mp7JrsFloatMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetMax().");
	}
	if (m_Max != item || m_Max_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Max);
		m_Max = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Max) m_Max->SetParent(m_myself.getPointer());
		if (m_Max && m_Max->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Max->UseTypeAttribute = true;
		}
		m_Max_Exist = true;
		if(m_Max != Mp7JrsFloatMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Max->SetContentName(XMLString::transcode("Max"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateMax()
{
	m_Max_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetMean(const Mp7JrsFloatMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetMean().");
	}
	if (m_Mean != item || m_Mean_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Mean);
		m_Mean = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Mean) m_Mean->SetParent(m_myself.getPointer());
		if (m_Mean && m_Mean->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Mean->UseTypeAttribute = true;
		}
		m_Mean_Exist = true;
		if(m_Mean != Mp7JrsFloatMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Mean->SetContentName(XMLString::transcode("Mean"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateMean()
{
	m_Mean_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetRandom(const Mp7JrsFloatMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetRandom().");
	}
	if (m_Random != item || m_Random_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Random);
		m_Random = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Random) m_Random->SetParent(m_myself.getPointer());
		if (m_Random && m_Random->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Random->UseTypeAttribute = true;
		}
		m_Random_Exist = true;
		if(m_Random != Mp7JrsFloatMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Random->SetContentName(XMLString::transcode("Random"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateRandom()
{
	m_Random_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetFirst(const Mp7JrsFloatMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetFirst().");
	}
	if (m_First != item || m_First_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_First);
		m_First = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_First) m_First->SetParent(m_myself.getPointer());
		if (m_First && m_First->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_First->UseTypeAttribute = true;
		}
		m_First_Exist = true;
		if(m_First != Mp7JrsFloatMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_First->SetContentName(XMLString::transcode("First"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateFirst()
{
	m_First_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetLast(const Mp7JrsFloatMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetLast().");
	}
	if (m_Last != item || m_Last_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Last);
		m_Last = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Last) m_Last->SetParent(m_myself.getPointer());
		if (m_Last && m_Last->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Last->UseTypeAttribute = true;
		}
		m_Last_Exist = true;
		if(m_Last != Mp7JrsFloatMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Last->SetContentName(XMLString::transcode("Last"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateLast()
{
	m_Last_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetVariance(const Mp7JrsFloatMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetVariance().");
	}
	if (m_Variance != item || m_Variance_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Variance);
		m_Variance = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Variance) m_Variance->SetParent(m_myself.getPointer());
		if (m_Variance && m_Variance->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Variance->UseTypeAttribute = true;
		}
		m_Variance_Exist = true;
		if(m_Variance != Mp7JrsFloatMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Variance->SetContentName(XMLString::transcode("Variance"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateVariance()
{
	m_Variance_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetCovariance(const Mp7JrsFloatMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetCovariance().");
	}
	if (m_Covariance != item || m_Covariance_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Covariance);
		m_Covariance = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Covariance) m_Covariance->SetParent(m_myself.getPointer());
		if (m_Covariance && m_Covariance->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Covariance->UseTypeAttribute = true;
		}
		m_Covariance_Exist = true;
		if(m_Covariance != Mp7JrsFloatMatrixPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Covariance->SetContentName(XMLString::transcode("Covariance"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateCovariance()
{
	m_Covariance_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetVarianceSummed(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetVarianceSummed().");
	}
	if (m_VarianceSummed != item || m_VarianceSummed_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_VarianceSummed);
		m_VarianceSummed = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VarianceSummed) m_VarianceSummed->SetParent(m_myself.getPointer());
		m_VarianceSummed_Exist = true;
		if(m_VarianceSummed != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VarianceSummed->SetContentName(XMLString::transcode("VarianceSummed"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateVarianceSummed()
{
	m_VarianceSummed_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetMaxSqDist(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetMaxSqDist().");
	}
	if (m_MaxSqDist != item || m_MaxSqDist_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_MaxSqDist);
		m_MaxSqDist = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_MaxSqDist) m_MaxSqDist->SetParent(m_myself.getPointer());
		m_MaxSqDist_Exist = true;
		if(m_MaxSqDist != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_MaxSqDist->SetContentName(XMLString::transcode("MaxSqDist"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateMaxSqDist()
{
	m_MaxSqDist_Exist = false;
}
void Mp7JrsSeriesOfVectorType::SetWeight(const Mp7JrsfloatVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetWeight().");
	}
	if (m_Weight != item || m_Weight_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Weight);
		m_Weight = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Weight) m_Weight->SetParent(m_myself.getPointer());
		m_Weight_Exist = true;
		if(m_Weight != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Weight->SetContentName(XMLString::transcode("Weight"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSeriesOfVectorType::InvalidateWeight()
{
	m_Weight_Exist = false;
}
unsigned Mp7JrsSeriesOfVectorType::GettotalNumOfSamples() const
{
	return GetBase()->GettotalNumOfSamples();
}

void Mp7JrsSeriesOfVectorType::SettotalNumOfSamples(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SettotalNumOfSamples().");
	}
	GetBase()->SettotalNumOfSamples(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsScalableSeriesType_Scaling_CollectionPtr Mp7JrsSeriesOfVectorType::GetScaling() const
{
	return GetBase()->GetScaling();
}

void Mp7JrsSeriesOfVectorType::SetScaling(const Mp7JrsScalableSeriesType_Scaling_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSeriesOfVectorType::SetScaling().");
	}
	GetBase()->SetScaling(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSeriesOfVectorType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 2 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; totalNumOfSamples,  T:http://www.w3.org/2001/XMLSchema:positiveInteger, C:Value 
//- N; vectorSize,  T:http://www.w3.org/2001/XMLSchema:positiveInteger, C:Value 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("vectorSize")) == 0)
	{
		// vectorSize is simple attribute positiveInteger
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "unsigned");
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Raw")) == 0)
	{
		// Raw is simple element FloatMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRaw()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFloatMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRaw(p, client);
					if((p = GetRaw()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Min")) == 0)
	{
		// Min is simple element FloatMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMin()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFloatMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMin(p, client);
					if((p = GetMin()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Max")) == 0)
	{
		// Max is simple element FloatMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMax()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFloatMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMax(p, client);
					if((p = GetMax()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Mean")) == 0)
	{
		// Mean is simple element FloatMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMean()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFloatMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMean(p, client);
					if((p = GetMean()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Random")) == 0)
	{
		// Random is simple element FloatMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRandom()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFloatMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRandom(p, client);
					if((p = GetRandom()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("First")) == 0)
	{
		// First is simple element FloatMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFirst()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFloatMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFirst(p, client);
					if((p = GetFirst()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Last")) == 0)
	{
		// Last is simple element FloatMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetLast()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFloatMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetLast(p, client);
					if((p = GetLast()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Variance")) == 0)
	{
		// Variance is simple element FloatMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetVariance()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFloatMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetVariance(p, client);
					if((p = GetVariance()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Covariance")) == 0)
	{
		// Covariance is simple element FloatMatrixType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetCovariance()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FloatMatrixType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFloatMatrixPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetCovariance(p, client);
					if((p = GetCovariance()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("VarianceSummed")) == 0)
	{
		// VarianceSummed is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetVarianceSummed()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetVarianceSummed(p, client);
					if((p = GetVarianceSummed()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("MaxSqDist")) == 0)
	{
		// MaxSqDist is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetMaxSqDist()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetMaxSqDist(p, client);
					if((p = GetMaxSqDist()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Weight")) == 0)
	{
		// Weight is simple element floatVector
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetWeight()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:floatVector")))) != empty)
			{
				// Is type allowed
				Mp7JrsfloatVectorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetWeight(p, client);
					if((p = GetWeight()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SeriesOfVectorType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SeriesOfVectorType");
	}
	return result;
}

XMLCh * Mp7JrsSeriesOfVectorType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSeriesOfVectorType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsSeriesOfVectorType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSeriesOfVectorType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SeriesOfVectorType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_vectorSize_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_vectorSize);
		element->setAttributeNS(X(""), X("vectorSize"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Raw != Mp7JrsFloatMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Raw->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Raw"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Raw->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Min != Mp7JrsFloatMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Min->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Min"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Min->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Max != Mp7JrsFloatMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Max->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Max"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Max->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Mean != Mp7JrsFloatMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Mean->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Mean"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Mean->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Random != Mp7JrsFloatMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Random->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Random"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Random->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_First != Mp7JrsFloatMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_First->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("First"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_First->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Last != Mp7JrsFloatMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Last->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Last"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Last->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Variance != Mp7JrsFloatMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Variance->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Variance"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Variance->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Covariance != Mp7JrsFloatMatrixPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Covariance->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Covariance"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Covariance->Serialize(doc, element, &elem);
	}
	if (m_VarianceSummed != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_VarianceSummed->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_MaxSqDist != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_MaxSqDist->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Weight != Mp7JrsfloatVectorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Weight->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsSeriesOfVectorType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("vectorSize")))
	{
		// deserialize value type
		this->SetvectorSize(Mp7JrsConvert::TextToUnsignedInt(parent->getAttribute(X("vectorSize"))));
		* current = parent;
	}

	// Extensionbase is Mp7JrsScalableSeriesType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Raw"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Raw")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFloatMatrixType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetRaw(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Min"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Min")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFloatMatrixType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetMin(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Max"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Max")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFloatMatrixType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetMax(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Mean"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Mean")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFloatMatrixType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetMean(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Random"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Random")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFloatMatrixType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetRandom(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("First"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("First")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFloatMatrixType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetFirst(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Last"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Last")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFloatMatrixType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetLast(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Variance"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Variance")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFloatMatrixType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetVariance(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Covariance"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Covariance")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateFloatMatrixType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetCovariance(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialise Element ValCollectionType
		if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("VarianceSummed")) == 0))
		{
			// Deserialize factory type
			Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
			tmp->Deserialize(doc, parent, current);
			this->SetVarianceSummed(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialise Element ValCollectionType
		if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("MaxSqDist")) == 0))
		{
			// Deserialize factory type
			Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
			tmp->Deserialize(doc, parent, current);
			this->SetMaxSqDist(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialise Element ValCollectionType
		if((parent != NULL) && (XMLString::compareString(parent->getNodeName()
			, X("Weight")) == 0))
		{
			// Deserialize factory type
			Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
			tmp->Deserialize(doc, parent, current);
			this->SetWeight(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsSeriesOfVectorType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSeriesOfVectorType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("Raw")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFloatMatrixType;
	}
	this->SetRaw(child);
  }
  if (XMLString::compareString(elementname, X("Min")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFloatMatrixType;
	}
	this->SetMin(child);
  }
  if (XMLString::compareString(elementname, X("Max")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFloatMatrixType;
	}
	this->SetMax(child);
  }
  if (XMLString::compareString(elementname, X("Mean")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFloatMatrixType;
	}
	this->SetMean(child);
  }
  if (XMLString::compareString(elementname, X("Random")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFloatMatrixType;
	}
	this->SetRandom(child);
  }
  if (XMLString::compareString(elementname, X("First")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFloatMatrixType;
	}
	this->SetFirst(child);
  }
  if (XMLString::compareString(elementname, X("Last")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFloatMatrixType;
	}
	this->SetLast(child);
  }
  if (XMLString::compareString(elementname, X("Variance")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFloatMatrixType;
	}
	this->SetVariance(child);
  }
  if (XMLString::compareString(elementname, X("Covariance")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateFloatMatrixType;
	}
	this->SetCovariance(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("VarianceSummed")) == 0))
  {
	Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
	this->SetVarianceSummed(tmp);
	child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("MaxSqDist")) == 0))
  {
	Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
	this->SetMaxSqDist(tmp);
	child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Weight")) == 0))
  {
	Mp7JrsfloatVectorPtr tmp = CreatefloatVector;
	this->SetWeight(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSeriesOfVectorType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetRaw() != Mp7JrsNodePtr())
		result.Insert(GetRaw());
	if (GetMin() != Mp7JrsNodePtr())
		result.Insert(GetMin());
	if (GetMax() != Mp7JrsNodePtr())
		result.Insert(GetMax());
	if (GetMean() != Mp7JrsNodePtr())
		result.Insert(GetMean());
	if (GetRandom() != Mp7JrsNodePtr())
		result.Insert(GetRandom());
	if (GetFirst() != Mp7JrsNodePtr())
		result.Insert(GetFirst());
	if (GetLast() != Mp7JrsNodePtr())
		result.Insert(GetLast());
	if (GetVariance() != Mp7JrsNodePtr())
		result.Insert(GetVariance());
	if (GetCovariance() != Mp7JrsNodePtr())
		result.Insert(GetCovariance());
	if (GetVarianceSummed() != Mp7JrsNodePtr())
		result.Insert(GetVarianceSummed());
	if (GetMaxSqDist() != Mp7JrsNodePtr())
		result.Insert(GetMaxSqDist());
	if (GetWeight() != Mp7JrsNodePtr())
		result.Insert(GetWeight());
	if (GetScaling() != Mp7JrsNodePtr())
		result.Insert(GetScaling());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSeriesOfVectorType_ExtMethodImpl.h


