 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsShape3DType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7Jrsunsigned4.h"
#include "Mp7JrsShape3DType_Spectrum_LocalType.h"
#include "Mp7Jrsunsigned12.h"
#include "Mp7JrsShape3DType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsShape3DType::IMp7JrsShape3DType()
{

// no includefile for extension defined 
// file Mp7JrsShape3DType_ExtPropInit.h

}

IMp7JrsShape3DType::~IMp7JrsShape3DType()
{
// no includefile for extension defined 
// file Mp7JrsShape3DType_ExtPropCleanup.h

}

Mp7JrsShape3DType::Mp7JrsShape3DType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsShape3DType::~Mp7JrsShape3DType()
{
	Cleanup();
}

void Mp7JrsShape3DType::Init()
{
	// Init base
	m_Base = CreateVisualDType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_bitsPerBin = Createunsigned4; // Create a valid class
	m_bitsPerBin->SetContent(0); // Use Value initialiser (xxx2)
	m_bitsPerBin_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Spectrum = Mp7JrsShape3DType_Spectrum_LocalPtr(); // Class
	m_PlanarSurfaces = Mp7Jrsunsigned12Ptr(); // Class with content 
	m_SingularSurfaces = Mp7Jrsunsigned12Ptr(); // Class with content 


// no includefile for extension defined 
// file Mp7JrsShape3DType_ExtMyPropInit.h

}

void Mp7JrsShape3DType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsShape3DType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_bitsPerBin);
	// Mp7JrsFactory::DeleteObject(m_Spectrum);
	// Mp7JrsFactory::DeleteObject(m_PlanarSurfaces);
	// Mp7JrsFactory::DeleteObject(m_SingularSurfaces);
}

void Mp7JrsShape3DType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use Shape3DTypePtr, since we
	// might need GetBase(), which isn't defined in IShape3DType
	const Mp7JrsPtr< Mp7JrsShape3DType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsVisualDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsShape3DType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_bitsPerBin);
	if (tmp->ExistbitsPerBin())
	{
		this->SetbitsPerBin(Mp7JrsFactory::CloneObject(tmp->GetbitsPerBin()));
	}
	else
	{
		InvalidatebitsPerBin();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Spectrum);
		this->SetSpectrum(Mp7JrsFactory::CloneObject(tmp->GetSpectrum()));
		// Mp7JrsFactory::DeleteObject(m_PlanarSurfaces);
		this->SetPlanarSurfaces(Mp7JrsFactory::CloneObject(tmp->GetPlanarSurfaces()));
		// Mp7JrsFactory::DeleteObject(m_SingularSurfaces);
		this->SetSingularSurfaces(Mp7JrsFactory::CloneObject(tmp->GetSingularSurfaces()));
}

Mp7JrsNodePtr Mp7JrsShape3DType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsShape3DType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDType > Mp7JrsShape3DType::GetBase() const
{
	return m_Base;
}

Mp7Jrsunsigned4Ptr Mp7JrsShape3DType::GetbitsPerBin() const
{
	if (this->ExistbitsPerBin()) {
		return m_bitsPerBin;
	} else {
		return m_bitsPerBin_Default;
	}
}

bool Mp7JrsShape3DType::ExistbitsPerBin() const
{
	return m_bitsPerBin_Exist;
}
void Mp7JrsShape3DType::SetbitsPerBin(const Mp7Jrsunsigned4Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsShape3DType::SetbitsPerBin().");
	}
	m_bitsPerBin = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_bitsPerBin) m_bitsPerBin->SetParent(m_myself.getPointer());
	m_bitsPerBin_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsShape3DType::InvalidatebitsPerBin()
{
	m_bitsPerBin_Exist = false;
}
Mp7JrsShape3DType_Spectrum_LocalPtr Mp7JrsShape3DType::GetSpectrum() const
{
		return m_Spectrum;
}

Mp7Jrsunsigned12Ptr Mp7JrsShape3DType::GetPlanarSurfaces() const
{
		return m_PlanarSurfaces;
}

Mp7Jrsunsigned12Ptr Mp7JrsShape3DType::GetSingularSurfaces() const
{
		return m_SingularSurfaces;
}

void Mp7JrsShape3DType::SetSpectrum(const Mp7JrsShape3DType_Spectrum_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsShape3DType::SetSpectrum().");
	}
	if (m_Spectrum != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Spectrum);
		m_Spectrum = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Spectrum) m_Spectrum->SetParent(m_myself.getPointer());
		if (m_Spectrum && m_Spectrum->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:Shape3DType_Spectrum_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Spectrum->UseTypeAttribute = true;
		}
		if(m_Spectrum != Mp7JrsShape3DType_Spectrum_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Spectrum->SetContentName(XMLString::transcode("Spectrum"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsShape3DType::SetPlanarSurfaces(const Mp7Jrsunsigned12Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsShape3DType::SetPlanarSurfaces().");
	}
	if (m_PlanarSurfaces != item)
	{
		// Mp7JrsFactory::DeleteObject(m_PlanarSurfaces);
		m_PlanarSurfaces = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_PlanarSurfaces) m_PlanarSurfaces->SetParent(m_myself.getPointer());
		if (m_PlanarSurfaces && m_PlanarSurfaces->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned12"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_PlanarSurfaces->UseTypeAttribute = true;
		}
		if(m_PlanarSurfaces != Mp7Jrsunsigned12Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_PlanarSurfaces->SetContentName(XMLString::transcode("PlanarSurfaces"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsShape3DType::SetSingularSurfaces(const Mp7Jrsunsigned12Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsShape3DType::SetSingularSurfaces().");
	}
	if (m_SingularSurfaces != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SingularSurfaces);
		m_SingularSurfaces = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SingularSurfaces) m_SingularSurfaces->SetParent(m_myself.getPointer());
		if (m_SingularSurfaces && m_SingularSurfaces->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned12"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SingularSurfaces->UseTypeAttribute = true;
		}
		if(m_SingularSurfaces != Mp7Jrsunsigned12Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SingularSurfaces->SetContentName(XMLString::transcode("SingularSurfaces"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsShape3DType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; bitsPerBin,  T:urn:mpeg:mpeg7:schema:2004:unsigned4, C:Class 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("bitsPerBin")) == 0)
	{
		// bitsPerBin is simple attribute unsigned4
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GetbitsPerBin()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned4")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned4Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetbitsPerBin(p, client);
					if((p = GetbitsPerBin()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Spectrum")) == 0)
	{
		// Spectrum is simple element Shape3DType_Spectrum_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSpectrum()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:Shape3DType_Spectrum_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsShape3DType_Spectrum_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSpectrum(p, client);
					if((p = GetSpectrum()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("PlanarSurfaces")) == 0)
	{
		// PlanarSurfaces is simple element unsigned12
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetPlanarSurfaces()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned12")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned12Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetPlanarSurfaces(p, client);
					if((p = GetPlanarSurfaces()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SingularSurfaces")) == 0)
	{
		// SingularSurfaces is simple element unsigned12
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSingularSurfaces()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned12")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned12Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSingularSurfaces(p, client);
					if((p = GetSingularSurfaces()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for Shape3DType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "Shape3DType");
	}
	return result;
}

XMLCh * Mp7JrsShape3DType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsShape3DType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsShape3DType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsShape3DType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("Shape3DType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_bitsPerBin_Exist)
	{
	// Class
	if (m_bitsPerBin != Mp7Jrsunsigned4Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_bitsPerBin->ToText();
		element->setAttributeNS(X(""), X("bitsPerBin"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Spectrum != Mp7JrsShape3DType_Spectrum_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Spectrum->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Spectrum"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Spectrum->Serialize(doc, element, &elem);
	}
	// Class with content		
	
	if (m_PlanarSurfaces != Mp7Jrsunsigned12Ptr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_PlanarSurfaces->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("PlanarSurfaces"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_PlanarSurfaces->Serialize(doc, element, &elem);
	}
	// Class with content		
	
	if (m_SingularSurfaces != Mp7Jrsunsigned12Ptr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SingularSurfaces->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SingularSurfaces"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SingularSurfaces->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsShape3DType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("bitsPerBin")))
	{
		// Deserialize class type
		Mp7Jrsunsigned4Ptr tmp = Createunsigned4;
		tmp->Parse(parent->getAttribute(X("bitsPerBin")));
		this->SetbitsPerBin(tmp);
		* current = parent;
	}

	// Extensionbase is Mp7JrsVisualDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Spectrum"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Spectrum")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateShape3DType_Spectrum_LocalType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSpectrum(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("PlanarSurfaces"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("PlanarSurfaces")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = Createunsigned12;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetPlanarSurfaces(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SingularSurfaces"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SingularSurfaces")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = Createunsigned12;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSingularSurfaces(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsShape3DType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsShape3DType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("Spectrum")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateShape3DType_Spectrum_LocalType;
	}
	this->SetSpectrum(child);
  }
  if (XMLString::compareString(elementname, X("PlanarSurfaces")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = Createunsigned12;
	}
	this->SetPlanarSurfaces(child);
  }
  if (XMLString::compareString(elementname, X("SingularSurfaces")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = Createunsigned12;
	}
	this->SetSingularSurfaces(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsShape3DType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSpectrum() != Mp7JrsNodePtr())
		result.Insert(GetSpectrum());
	if (GetPlanarSurfaces() != Mp7JrsNodePtr())
		result.Insert(GetPlanarSurfaces());
	if (GetSingularSurfaces() != Mp7JrsNodePtr())
		result.Insert(GetSingularSurfaces());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsShape3DType_ExtMethodImpl.h


