 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSoundModelType_ExtImplInclude.h



#include "Mp7JrsContinuousHiddenMarkovModelType.h"
#include "Mp7JrsTermUseType.h"
#include "Mp7JrsAudioSpectrumBasisType.h"
#include "Mp7JrsContinuousHiddenMarkovModelType_DescriptorModel_CollectionType.h"
#include "Mp7JrsContinuousHiddenMarkovModelType_ObservationDistribution_CollectionType.h"
#include "Mp7JrsProbabilityMatrixType.h"
#include "Mp7JrsStateTransitionModelType_State_CollectionType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsSoundModelType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsModelStateType.h" // Element collection urn:mpeg:mpeg7:schema:2004:State
#include "Mp7JrsDescriptorModelType.h" // Element collection urn:mpeg:mpeg7:schema:2004:DescriptorModel
#include "Mp7JrsContinuousDistributionType.h" // Element collection urn:mpeg:mpeg7:schema:2004:ObservationDistribution

#include <assert.h>
IMp7JrsSoundModelType::IMp7JrsSoundModelType()
{

// no includefile for extension defined 
// file Mp7JrsSoundModelType_ExtPropInit.h

}

IMp7JrsSoundModelType::~IMp7JrsSoundModelType()
{
// no includefile for extension defined 
// file Mp7JrsSoundModelType_ExtPropCleanup.h

}

Mp7JrsSoundModelType::Mp7JrsSoundModelType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSoundModelType::~Mp7JrsSoundModelType()
{
	Cleanup();
}

void Mp7JrsSoundModelType::Init()
{
	// Init base
	m_Base = CreateContinuousHiddenMarkovModelType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_SoundModelRef = NULL; // String
	m_SoundModelRef_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_SoundClassLabel = Mp7JrsTermUsePtr(); // Class
	m_SpectrumBasis = Mp7JrsAudioSpectrumBasisPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsSoundModelType_ExtMyPropInit.h

}

void Mp7JrsSoundModelType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSoundModelType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_SoundModelRef); // String
	// Mp7JrsFactory::DeleteObject(m_SoundClassLabel);
	// Mp7JrsFactory::DeleteObject(m_SpectrumBasis);
}

void Mp7JrsSoundModelType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SoundModelTypePtr, since we
	// might need GetBase(), which isn't defined in ISoundModelType
	const Mp7JrsPtr< Mp7JrsSoundModelType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsContinuousHiddenMarkovModelPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSoundModelType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_SoundModelRef); // String
	if (tmp->ExistSoundModelRef())
	{
		this->SetSoundModelRef(XMLString::replicate(tmp->GetSoundModelRef()));
	}
	else
	{
		InvalidateSoundModelRef();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_SoundClassLabel);
		this->SetSoundClassLabel(Mp7JrsFactory::CloneObject(tmp->GetSoundClassLabel()));
		// Mp7JrsFactory::DeleteObject(m_SpectrumBasis);
		this->SetSpectrumBasis(Mp7JrsFactory::CloneObject(tmp->GetSpectrumBasis()));
}

Mp7JrsNodePtr Mp7JrsSoundModelType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSoundModelType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsContinuousHiddenMarkovModelType > Mp7JrsSoundModelType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsSoundModelType::GetSoundModelRef() const
{
	return m_SoundModelRef;
}

bool Mp7JrsSoundModelType::ExistSoundModelRef() const
{
	return m_SoundModelRef_Exist;
}
void Mp7JrsSoundModelType::SetSoundModelRef(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetSoundModelRef().");
	}
	m_SoundModelRef = item;
	m_SoundModelRef_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::InvalidateSoundModelRef()
{
	m_SoundModelRef_Exist = false;
}
Mp7JrsTermUsePtr Mp7JrsSoundModelType::GetSoundClassLabel() const
{
		return m_SoundClassLabel;
}

Mp7JrsAudioSpectrumBasisPtr Mp7JrsSoundModelType::GetSpectrumBasis() const
{
		return m_SpectrumBasis;
}

void Mp7JrsSoundModelType::SetSoundClassLabel(const Mp7JrsTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetSoundClassLabel().");
	}
	if (m_SoundClassLabel != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SoundClassLabel);
		m_SoundClassLabel = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SoundClassLabel) m_SoundClassLabel->SetParent(m_myself.getPointer());
		if (m_SoundClassLabel && m_SoundClassLabel->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SoundClassLabel->UseTypeAttribute = true;
		}
		if(m_SoundClassLabel != Mp7JrsTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SoundClassLabel->SetContentName(XMLString::transcode("SoundClassLabel"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::SetSpectrumBasis(const Mp7JrsAudioSpectrumBasisPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetSpectrumBasis().");
	}
	if (m_SpectrumBasis != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SpectrumBasis);
		m_SpectrumBasis = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SpectrumBasis) m_SpectrumBasis->SetParent(m_myself.getPointer());
		if (m_SpectrumBasis && m_SpectrumBasis->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:AudioSpectrumBasisType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SpectrumBasis->UseTypeAttribute = true;
		}
		if(m_SpectrumBasis != Mp7JrsAudioSpectrumBasisPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SpectrumBasis->SetContentName(XMLString::transcode("SpectrumBasis"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsContinuousHiddenMarkovModelType_DescriptorModel_CollectionPtr Mp7JrsSoundModelType::GetDescriptorModel() const
{
	return GetBase()->GetDescriptorModel();
}

Mp7JrsContinuousHiddenMarkovModelType_ObservationDistribution_CollectionPtr Mp7JrsSoundModelType::GetObservationDistribution() const
{
	return GetBase()->GetObservationDistribution();
}

void Mp7JrsSoundModelType::SetDescriptorModel(const Mp7JrsContinuousHiddenMarkovModelType_DescriptorModel_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetDescriptorModel().");
	}
	GetBase()->SetDescriptorModel(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::SetObservationDistribution(const Mp7JrsContinuousHiddenMarkovModelType_ObservationDistribution_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetObservationDistribution().");
	}
	GetBase()->SetObservationDistribution(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsProbabilityMatrixPtr Mp7JrsSoundModelType::GetInitial() const
{
	return GetBase()->GetBase()->GetInitial();
}

// Element is optional
bool Mp7JrsSoundModelType::IsValidInitial() const
{
	return GetBase()->GetBase()->IsValidInitial();
}

Mp7JrsProbabilityMatrixPtr Mp7JrsSoundModelType::GetTransitions() const
{
	return GetBase()->GetBase()->GetTransitions();
}

Mp7JrsStateTransitionModelType_State_CollectionPtr Mp7JrsSoundModelType::GetState() const
{
	return GetBase()->GetBase()->GetState();
}

void Mp7JrsSoundModelType::SetInitial(const Mp7JrsProbabilityMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetInitial().");
	}
	GetBase()->GetBase()->SetInitial(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::InvalidateInitial()
{
	GetBase()->GetBase()->InvalidateInitial();
}
void Mp7JrsSoundModelType::SetTransitions(const Mp7JrsProbabilityMatrixPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetTransitions().");
	}
	GetBase()->GetBase()->SetTransitions(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::SetState(const Mp7JrsStateTransitionModelType_State_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetState().");
	}
	GetBase()->GetBase()->SetState(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

unsigned Mp7JrsSoundModelType::GetnumOfStates() const
{
	return GetBase()->GetBase()->GetBase()->GetnumOfStates();
}

void Mp7JrsSoundModelType::SetnumOfStates(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetnumOfStates().");
	}
	GetBase()->GetBase()->GetBase()->SetnumOfStates(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrszeroToOnePtr Mp7JrsSoundModelType::Getconfidence() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Getconfidence();
}

bool Mp7JrsSoundModelType::Existconfidence() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Existconfidence();
}
void Mp7JrsSoundModelType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::Setconfidence().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Setconfidence(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::Invalidateconfidence()
{
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Invalidateconfidence();
}
Mp7JrszeroToOnePtr Mp7JrsSoundModelType::Getreliability() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Getreliability();
}

bool Mp7JrsSoundModelType::Existreliability() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Existreliability();
}
void Mp7JrsSoundModelType::Setreliability(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::Setreliability().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Setreliability(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::Invalidatereliability()
{
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Invalidatereliability();
}
XMLCh * Mp7JrsSoundModelType::Getid() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Getid();
}

bool Mp7JrsSoundModelType::Existid() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Existid();
}
void Mp7JrsSoundModelType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::Setid().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::Invalidateid()
{
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsSoundModelType::GettimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsSoundModelType::ExisttimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsSoundModelType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SettimeBase().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::InvalidatetimeBase()
{
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsSoundModelType::GettimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsSoundModelType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsSoundModelType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SettimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsSoundModelType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsSoundModelType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsSoundModelType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsSoundModelType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsSoundModelType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsSoundModelType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSoundModelType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsSoundModelType::GetHeader() const
{
	return GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetHeader();
}

void Mp7JrsSoundModelType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSoundModelType::SetHeader().");
	}
	GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSoundModelType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 9 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; id,  T:http://www.w3.org/2001/XMLSchema:ID, C:String 
//- N; timeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; timeUnit,  T:urn:mpeg:mpeg7:schema:2004:durationType, C:Pattern 
//- N; mediaTimeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; mediaTimeUnit,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
//- N; confidence,  T:urn:mpeg:mpeg7:schema:2004:zeroToOneType, C:Class 
//- N; reliability,  T:urn:mpeg:mpeg7:schema:2004:zeroToOneType, C:Class 
//- N; numOfStates,  T:http://www.w3.org/2001/XMLSchema:nonNegativeInteger, C:Value 
//- N; SoundModelRef,  T:http://www.w3.org/2001/XMLSchema:anyURI, C:String 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("SoundModelRef")) == 0)
	{
		// SoundModelRef is simple attribute anyURI
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "XMLCh *");
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SoundClassLabel")) == 0)
	{
		// SoundClassLabel is simple element TermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSoundClassLabel()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSoundClassLabel(p, client);
					if((p = GetSoundClassLabel()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SpectrumBasis")) == 0)
	{
		// SpectrumBasis is simple element AudioSpectrumBasisType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSpectrumBasis()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:AudioSpectrumBasisType")))) != empty)
			{
				// Is type allowed
				Mp7JrsAudioSpectrumBasisPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSpectrumBasis(p, client);
					if((p = GetSpectrumBasis()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SoundModelType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SoundModelType");
	}
	return result;
}

XMLCh * Mp7JrsSoundModelType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSoundModelType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsSoundModelType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSoundModelType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SoundModelType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_SoundModelRef_Exist)
	{
	// String
	if(m_SoundModelRef != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("SoundModelRef"), m_SoundModelRef);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_SoundClassLabel != Mp7JrsTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SoundClassLabel->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SoundClassLabel"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SoundClassLabel->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_SpectrumBasis != Mp7JrsAudioSpectrumBasisPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SpectrumBasis->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SpectrumBasis"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SpectrumBasis->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsSoundModelType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("SoundModelRef")))
	{
		// Deserialize string type
		this->SetSoundModelRef(Mp7JrsConvert::TextToString(parent->getAttribute(X("SoundModelRef"))));
		* current = parent;
	}

	// Extensionbase is Mp7JrsContinuousHiddenMarkovModelType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SoundClassLabel"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SoundClassLabel")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateTermUseType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSoundClassLabel(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SpectrumBasis"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SpectrumBasis")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateAudioSpectrumBasisType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSpectrumBasis(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsSoundModelType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSoundModelType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("SoundClassLabel")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateTermUseType;
	}
	this->SetSoundClassLabel(child);
  }
  if (XMLString::compareString(elementname, X("SpectrumBasis")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateAudioSpectrumBasisType;
	}
	this->SetSpectrumBasis(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSoundModelType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSoundClassLabel() != Mp7JrsNodePtr())
		result.Insert(GetSoundClassLabel());
	if (GetSpectrumBasis() != Mp7JrsNodePtr())
		result.Insert(GetSpectrumBasis());
	if (GetDescriptorModel() != Mp7JrsNodePtr())
		result.Insert(GetDescriptorModel());
	if (GetObservationDistribution() != Mp7JrsNodePtr())
		result.Insert(GetObservationDistribution());
	if (GetInitial() != Mp7JrsNodePtr())
		result.Insert(GetInitial());
	if (GetTransitions() != Mp7JrsNodePtr())
		result.Insert(GetTransitions());
	if (GetState() != Mp7JrsNodePtr())
		result.Insert(GetState());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSoundModelType_ExtMethodImpl.h


