 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_ExtImplInclude.h



#include "Mp7JrsHeaderType.h"
#include "Mp7JrsSpokenContentHeaderType_CollectionType.h"
#include "Mp7JrsSpokenContentHeaderType_ConfusionInfo_CollectionType.h"
#include "Mp7JrsDescriptionMetadataType.h"
#include "Mp7JrsSpokenContentHeaderType_SpeakerInfo_CollectionType.h"
#include "Mp7JrsSpokenContentHeaderType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsSpokenContentHeaderType_LocalType.h" // Choice collection WordLexicon
#include "Mp7JrsWordLexiconType.h" // Choice collection element WordLexicon
#include "Mp7JrsPhoneLexiconType.h" // Choice collection element PhoneLexicon
#include "Mp7JrsConfusionCountType.h" // Element collection urn:mpeg:mpeg7:schema:2004:ConfusionInfo
#include "Mp7JrsSpeakerInfoType.h" // Element collection urn:mpeg:mpeg7:schema:2004:SpeakerInfo

#include <assert.h>
IMp7JrsSpokenContentHeaderType::IMp7JrsSpokenContentHeaderType()
{

// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_ExtPropInit.h

}

IMp7JrsSpokenContentHeaderType::~IMp7JrsSpokenContentHeaderType()
{
// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_ExtPropCleanup.h

}

Mp7JrsSpokenContentHeaderType::Mp7JrsSpokenContentHeaderType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSpokenContentHeaderType::~Mp7JrsSpokenContentHeaderType()
{
	Cleanup();
}

void Mp7JrsSpokenContentHeaderType::Init()
{
	// Init base
	m_Base = CreateHeaderType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_SpokenContentHeaderType_LocalType = Mp7JrsSpokenContentHeaderType_CollectionPtr(); // Collection
	m_ConfusionInfo = Mp7JrsSpokenContentHeaderType_ConfusionInfo_CollectionPtr(); // Collection
	m_DescriptionMetadata = Mp7JrsDescriptionMetadataPtr(); // Class
	m_DescriptionMetadata_Exist = false;
	m_SpeakerInfo = Mp7JrsSpokenContentHeaderType_SpeakerInfo_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_ExtMyPropInit.h

}

void Mp7JrsSpokenContentHeaderType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_SpokenContentHeaderType_LocalType);
	// Mp7JrsFactory::DeleteObject(m_ConfusionInfo);
	// Mp7JrsFactory::DeleteObject(m_DescriptionMetadata);
	// Mp7JrsFactory::DeleteObject(m_SpeakerInfo);
}

void Mp7JrsSpokenContentHeaderType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SpokenContentHeaderTypePtr, since we
	// might need GetBase(), which isn't defined in ISpokenContentHeaderType
	const Mp7JrsPtr< Mp7JrsSpokenContentHeaderType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsHeaderPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSpokenContentHeaderType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_SpokenContentHeaderType_LocalType);
		this->SetSpokenContentHeaderType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetSpokenContentHeaderType_LocalType()));
		// Mp7JrsFactory::DeleteObject(m_ConfusionInfo);
		this->SetConfusionInfo(Mp7JrsFactory::CloneObject(tmp->GetConfusionInfo()));
	if (tmp->IsValidDescriptionMetadata())
	{
		// Mp7JrsFactory::DeleteObject(m_DescriptionMetadata);
		this->SetDescriptionMetadata(Mp7JrsFactory::CloneObject(tmp->GetDescriptionMetadata()));
	}
	else
	{
		InvalidateDescriptionMetadata();
	}
		// Mp7JrsFactory::DeleteObject(m_SpeakerInfo);
		this->SetSpeakerInfo(Mp7JrsFactory::CloneObject(tmp->GetSpeakerInfo()));
}

Mp7JrsNodePtr Mp7JrsSpokenContentHeaderType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSpokenContentHeaderType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsHeaderType > Mp7JrsSpokenContentHeaderType::GetBase() const
{
	return m_Base;
}

Mp7JrsSpokenContentHeaderType_CollectionPtr Mp7JrsSpokenContentHeaderType::GetSpokenContentHeaderType_LocalType() const
{
		return m_SpokenContentHeaderType_LocalType;
}

Mp7JrsSpokenContentHeaderType_ConfusionInfo_CollectionPtr Mp7JrsSpokenContentHeaderType::GetConfusionInfo() const
{
		return m_ConfusionInfo;
}

Mp7JrsDescriptionMetadataPtr Mp7JrsSpokenContentHeaderType::GetDescriptionMetadata() const
{
		return m_DescriptionMetadata;
}

// Element is optional
bool Mp7JrsSpokenContentHeaderType::IsValidDescriptionMetadata() const
{
	return m_DescriptionMetadata_Exist;
}

Mp7JrsSpokenContentHeaderType_SpeakerInfo_CollectionPtr Mp7JrsSpokenContentHeaderType::GetSpeakerInfo() const
{
		return m_SpeakerInfo;
}

void Mp7JrsSpokenContentHeaderType::SetSpokenContentHeaderType_LocalType(const Mp7JrsSpokenContentHeaderType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentHeaderType::SetSpokenContentHeaderType_LocalType().");
	}
	if (m_SpokenContentHeaderType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SpokenContentHeaderType_LocalType);
		m_SpokenContentHeaderType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SpokenContentHeaderType_LocalType) m_SpokenContentHeaderType_LocalType->SetParent(m_myself.getPointer());
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpokenContentHeaderType::SetConfusionInfo(const Mp7JrsSpokenContentHeaderType_ConfusionInfo_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentHeaderType::SetConfusionInfo().");
	}
	if (m_ConfusionInfo != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ConfusionInfo);
		m_ConfusionInfo = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ConfusionInfo) m_ConfusionInfo->SetParent(m_myself.getPointer());
		if(m_ConfusionInfo != Mp7JrsSpokenContentHeaderType_ConfusionInfo_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ConfusionInfo->SetContentName(XMLString::transcode("ConfusionInfo"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpokenContentHeaderType::SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentHeaderType::SetDescriptionMetadata().");
	}
	if (m_DescriptionMetadata != item || m_DescriptionMetadata_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_DescriptionMetadata);
		m_DescriptionMetadata = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DescriptionMetadata) m_DescriptionMetadata->SetParent(m_myself.getPointer());
		if (m_DescriptionMetadata && m_DescriptionMetadata->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DescriptionMetadataType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DescriptionMetadata->UseTypeAttribute = true;
		}
		m_DescriptionMetadata_Exist = true;
		if(m_DescriptionMetadata != Mp7JrsDescriptionMetadataPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DescriptionMetadata->SetContentName(XMLString::transcode("DescriptionMetadata"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpokenContentHeaderType::InvalidateDescriptionMetadata()
{
	m_DescriptionMetadata_Exist = false;
}
void Mp7JrsSpokenContentHeaderType::SetSpeakerInfo(const Mp7JrsSpokenContentHeaderType_SpeakerInfo_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentHeaderType::SetSpeakerInfo().");
	}
	if (m_SpeakerInfo != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SpeakerInfo);
		m_SpeakerInfo = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SpeakerInfo) m_SpeakerInfo->SetParent(m_myself.getPointer());
		if(m_SpeakerInfo != Mp7JrsSpokenContentHeaderType_SpeakerInfo_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SpeakerInfo->SetContentName(XMLString::transcode("SpeakerInfo"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsSpokenContentHeaderType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsSpokenContentHeaderType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsSpokenContentHeaderType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentHeaderType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpokenContentHeaderType::Invalidateid()
{
	GetBase()->Invalidateid();
}

Mp7JrsNodeEnum Mp7JrsSpokenContentHeaderType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("WordLexicon")) == 0)
	{
		// WordLexicon is contained in itemtype SpokenContentHeaderType_LocalType
		// in choice collection SpokenContentHeaderType_CollectionType
		
		context->Found = true;
		Mp7JrsSpokenContentHeaderType_CollectionPtr coll = GetSpokenContentHeaderType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSpokenContentHeaderType_CollectionType;
				SetSpokenContentHeaderType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsSpokenContentHeaderType_LocalPtr)coll->elementAt(i))->GetWordLexicon()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsSpokenContentHeaderType_LocalPtr item = CreateSpokenContentHeaderType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:WordLexiconType")))) != empty)
			{
				// Is type allowed
				Mp7JrsWordLexiconPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsSpokenContentHeaderType_LocalPtr)coll->elementAt(i))->SetWordLexicon(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsSpokenContentHeaderType_LocalPtr)coll->elementAt(i))->GetWordLexicon()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("PhoneLexicon")) == 0)
	{
		// PhoneLexicon is contained in itemtype SpokenContentHeaderType_LocalType
		// in choice collection SpokenContentHeaderType_CollectionType
		
		context->Found = true;
		Mp7JrsSpokenContentHeaderType_CollectionPtr coll = GetSpokenContentHeaderType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSpokenContentHeaderType_CollectionType;
				SetSpokenContentHeaderType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsSpokenContentHeaderType_LocalPtr)coll->elementAt(i))->GetPhoneLexicon()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsSpokenContentHeaderType_LocalPtr item = CreateSpokenContentHeaderType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PhoneLexiconType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPhoneLexiconPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsSpokenContentHeaderType_LocalPtr)coll->elementAt(i))->SetPhoneLexicon(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsSpokenContentHeaderType_LocalPtr)coll->elementAt(i))->GetPhoneLexicon()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ConfusionInfo")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:ConfusionInfo is item of type ConfusionCountType
		// in element collection SpokenContentHeaderType_ConfusionInfo_CollectionType
		
		context->Found = true;
		Mp7JrsSpokenContentHeaderType_ConfusionInfo_CollectionPtr coll = GetConfusionInfo();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSpokenContentHeaderType_ConfusionInfo_CollectionType;
				SetConfusionInfo(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ConfusionCountType")))) != empty)
			{
				// Is type allowed
				Mp7JrsConfusionCountPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("DescriptionMetadata")) == 0)
	{
		// DescriptionMetadata is simple element DescriptionMetadataType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDescriptionMetadata()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DescriptionMetadataType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDescriptionMetadataPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDescriptionMetadata(p, client);
					if((p = GetDescriptionMetadata()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SpeakerInfo")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:SpeakerInfo is item of type SpeakerInfoType
		// in element collection SpokenContentHeaderType_SpeakerInfo_CollectionType
		
		context->Found = true;
		Mp7JrsSpokenContentHeaderType_SpeakerInfo_CollectionPtr coll = GetSpeakerInfo();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateSpokenContentHeaderType_SpeakerInfo_CollectionType;
				SetSpeakerInfo(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SpeakerInfoType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSpeakerInfoPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SpokenContentHeaderType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SpokenContentHeaderType");
	}
	return result;
}

XMLCh * Mp7JrsSpokenContentHeaderType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsSpokenContentHeaderType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsSpokenContentHeaderType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SpokenContentHeaderType"));
	// Element serialization:
	if (m_SpokenContentHeaderType_LocalType != Mp7JrsSpokenContentHeaderType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_SpokenContentHeaderType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_ConfusionInfo != Mp7JrsSpokenContentHeaderType_ConfusionInfo_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_ConfusionInfo->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_DescriptionMetadata != Mp7JrsDescriptionMetadataPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DescriptionMetadata->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DescriptionMetadata"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DescriptionMetadata->Serialize(doc, element, &elem);
	}
	if (m_SpeakerInfo != Mp7JrsSpokenContentHeaderType_SpeakerInfo_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_SpeakerInfo->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsSpokenContentHeaderType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsHeaderType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialise Choice Collection
		if((parent != NULL) && (
			// urn:mpeg:mpeg7:schema:2004:WordLexicon
			Mp7JrsUtil::HasNodeName(parent, X("WordLexicon"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD			(XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:WordLexicon")) == 0)
				||
			// urn:mpeg:mpeg7:schema:2004:PhoneLexicon
			Mp7JrsUtil::HasNodeName(parent, X("PhoneLexicon"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD			(XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:PhoneLexicon")) == 0)
				))
		{
			// Deserialize factory type
			Mp7JrsSpokenContentHeaderType_CollectionPtr tmp = CreateSpokenContentHeaderType_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetSpokenContentHeaderType_LocalType(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("ConfusionInfo"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("ConfusionInfo")) == 0))
		{
			// Deserialize factory type
			Mp7JrsSpokenContentHeaderType_ConfusionInfo_CollectionPtr tmp = CreateSpokenContentHeaderType_ConfusionInfo_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetConfusionInfo(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DescriptionMetadata"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DescriptionMetadata")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateDescriptionMetadataType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetDescriptionMetadata(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("SpeakerInfo"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("SpeakerInfo")) == 0))
		{
			// Deserialize factory type
			Mp7JrsSpokenContentHeaderType_SpeakerInfo_CollectionPtr tmp = CreateSpokenContentHeaderType_SpeakerInfo_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetSpeakerInfo(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSpokenContentHeaderType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  // Choice Collection
  if(
// TODO better use this	
//  (Mp7JrsUtil::HasNodeName(parent, X("WordLexicon"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
	(XMLString::compareString(elementname, X("WordLexicon")) == 0)
	||
// TODO better use this	
//  (Mp7JrsUtil::HasNodeName(parent, X("PhoneLexicon"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
	(XMLString::compareString(elementname, X("PhoneLexicon")) == 0)
	)
  {
	Mp7JrsSpokenContentHeaderType_CollectionPtr tmp = CreateSpokenContentHeaderType_CollectionType;
	this->SetSpokenContentHeaderType_LocalType(tmp);
	child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("ConfusionInfo")) == 0))
  {
	Mp7JrsSpokenContentHeaderType_ConfusionInfo_CollectionPtr tmp = CreateSpokenContentHeaderType_ConfusionInfo_CollectionType;
	this->SetConfusionInfo(tmp);
	child = tmp;
  }
  if (XMLString::compareString(elementname, X("DescriptionMetadata")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateDescriptionMetadataType;
	}
	this->SetDescriptionMetadata(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("SpeakerInfo")) == 0))
  {
	Mp7JrsSpokenContentHeaderType_SpeakerInfo_CollectionPtr tmp = CreateSpokenContentHeaderType_SpeakerInfo_CollectionType;
	this->SetSpeakerInfo(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSpokenContentHeaderType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSpokenContentHeaderType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetSpokenContentHeaderType_LocalType());
	if (GetConfusionInfo() != Mp7JrsNodePtr())
		result.Insert(GetConfusionInfo());
	if (GetDescriptionMetadata() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionMetadata());
	if (GetSpeakerInfo() != Mp7JrsNodePtr())
		result.Insert(GetSpeakerInfo());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_ExtMethodImpl.h


