 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSpokenContentIndexEntryType_ExtImplInclude.h



#include "Mp7Jrsunsigned16.h"
#include "Mp7JrsSpokenContentIndexEntryType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsSpokenContentIndexEntryType::IMp7JrsSpokenContentIndexEntryType()
{

// no includefile for extension defined 
// file Mp7JrsSpokenContentIndexEntryType_ExtPropInit.h

}

IMp7JrsSpokenContentIndexEntryType::~IMp7JrsSpokenContentIndexEntryType()
{
// no includefile for extension defined 
// file Mp7JrsSpokenContentIndexEntryType_ExtPropCleanup.h

}

Mp7JrsSpokenContentIndexEntryType::Mp7JrsSpokenContentIndexEntryType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSpokenContentIndexEntryType::~Mp7JrsSpokenContentIndexEntryType()
{
	Cleanup();
}

void Mp7JrsSpokenContentIndexEntryType::Init()
{

	// Init attributes
	m_node = Createunsigned16; // Create a valid class
	m_node->SetContent(0); // Use Value initialiser (xxx2)
	m_block = Createunsigned16; // Create a valid class
	m_block->SetContent(0); // Use Value initialiser (xxx2)
	m_lattice = NULL; // String
	m_lattice_Exist = false;



// no includefile for extension defined 
// file Mp7JrsSpokenContentIndexEntryType_ExtMyPropInit.h

}

void Mp7JrsSpokenContentIndexEntryType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSpokenContentIndexEntryType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_node);
	// Mp7JrsFactory::DeleteObject(m_block);
	XMLString::release(&m_lattice); // String
}

void Mp7JrsSpokenContentIndexEntryType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SpokenContentIndexEntryTypePtr, since we
	// might need GetBase(), which isn't defined in ISpokenContentIndexEntryType
	const Mp7JrsPtr< Mp7JrsSpokenContentIndexEntryType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	// Mp7JrsFactory::DeleteObject(m_node);
		this->Setnode(Mp7JrsFactory::CloneObject(tmp->Getnode()));
	}
	{
	// Mp7JrsFactory::DeleteObject(m_block);
		this->Setblock(Mp7JrsFactory::CloneObject(tmp->Getblock()));
	}
	{
	XMLString::release(&m_lattice); // String
	if (tmp->Existlattice())
	{
		this->Setlattice(XMLString::replicate(tmp->Getlattice()));
	}
	else
	{
		Invalidatelattice();
	}
	}
}

Mp7JrsNodePtr Mp7JrsSpokenContentIndexEntryType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsSpokenContentIndexEntryType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7Jrsunsigned16Ptr Mp7JrsSpokenContentIndexEntryType::Getnode() const
{
	return m_node;
}

void Mp7JrsSpokenContentIndexEntryType::Setnode(const Mp7Jrsunsigned16Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentIndexEntryType::Setnode().");
	}
	m_node = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_node) m_node->SetParent(m_myself.getPointer());
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7Jrsunsigned16Ptr Mp7JrsSpokenContentIndexEntryType::Getblock() const
{
	return m_block;
}

void Mp7JrsSpokenContentIndexEntryType::Setblock(const Mp7Jrsunsigned16Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentIndexEntryType::Setblock().");
	}
	m_block = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_block) m_block->SetParent(m_myself.getPointer());
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsSpokenContentIndexEntryType::Getlattice() const
{
	return m_lattice;
}

bool Mp7JrsSpokenContentIndexEntryType::Existlattice() const
{
	return m_lattice_Exist;
}
void Mp7JrsSpokenContentIndexEntryType::Setlattice(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentIndexEntryType::Setlattice().");
	}
	m_lattice = item;
	m_lattice_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpokenContentIndexEntryType::Invalidatelattice()
{
	m_lattice_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsSpokenContentIndexEntryType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  3, 3 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	// Only rudimentary expressions allowed for this type with no child elements
	return result;
}	

XMLCh * Mp7JrsSpokenContentIndexEntryType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSpokenContentIndexEntryType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsSpokenContentIndexEntryType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SpokenContentIndexEntryType"));
	// Attribute Serialization:
	// Class
	if (m_node != Mp7Jrsunsigned16Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_node->ToText();
		element->setAttributeNS(X(""), X("node"), tmp);
		XMLString::release(&tmp);
	}
	// Attribute Serialization:
	// Class
	if (m_block != Mp7Jrsunsigned16Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_block->ToText();
		element->setAttributeNS(X(""), X("block"), tmp);
		XMLString::release(&tmp);
	}
	// Attribute Serialization:
		

	// Optional attriute
	if(m_lattice_Exist)
	{
	// String
	if(m_lattice != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("lattice"), m_lattice);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsSpokenContentIndexEntryType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("node")))
	{
		// Deserialize class type
		Mp7Jrsunsigned16Ptr tmp = Createunsigned16;
		tmp->Parse(parent->getAttribute(X("node")));
		this->Setnode(tmp);
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("block")))
	{
		// Deserialize class type
		Mp7Jrsunsigned16Ptr tmp = Createunsigned16;
		tmp->Parse(parent->getAttribute(X("block")));
		this->Setblock(tmp);
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("lattice")))
	{
		// Deserialize string type
		this->Setlattice(Mp7JrsConvert::TextToString(parent->getAttribute(X("lattice"))));
		* current = parent;
	}

// no includefile for extension defined 
// file Mp7JrsSpokenContentIndexEntryType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSpokenContentIndexEntryType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSpokenContentIndexEntryType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSpokenContentIndexEntryType_ExtMethodImpl.h


