 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsStreamMaskType_ExtImplInclude.h



#include "Mp7JrsMaskType.h"
#include "Mp7JrsStreamMaskType_StreamSection_CollectionType.h"
#include "Mp7JrsStreamMaskType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsStreamSectionType.h" // Element collection urn:mpeg:mpeg7:schema:2004:StreamSection

#include <assert.h>
IMp7JrsStreamMaskType::IMp7JrsStreamMaskType()
{

// no includefile for extension defined 
// file Mp7JrsStreamMaskType_ExtPropInit.h

}

IMp7JrsStreamMaskType::~IMp7JrsStreamMaskType()
{
// no includefile for extension defined 
// file Mp7JrsStreamMaskType_ExtPropCleanup.h

}

Mp7JrsStreamMaskType::Mp7JrsStreamMaskType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsStreamMaskType::~Mp7JrsStreamMaskType()
{
	Cleanup();
}

void Mp7JrsStreamMaskType::Init()
{
	// Init base
	m_Base = CreateMaskType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_StreamSection = Mp7JrsStreamMaskType_StreamSection_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsStreamMaskType_ExtMyPropInit.h

}

void Mp7JrsStreamMaskType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsStreamMaskType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_StreamSection);
}

void Mp7JrsStreamMaskType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use StreamMaskTypePtr, since we
	// might need GetBase(), which isn't defined in IStreamMaskType
	const Mp7JrsPtr< Mp7JrsStreamMaskType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsMaskPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsStreamMaskType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_StreamSection);
		this->SetStreamSection(Mp7JrsFactory::CloneObject(tmp->GetStreamSection()));
}

Mp7JrsNodePtr Mp7JrsStreamMaskType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsStreamMaskType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsMaskType > Mp7JrsStreamMaskType::GetBase() const
{
	return m_Base;
}

Mp7JrsStreamMaskType_StreamSection_CollectionPtr Mp7JrsStreamMaskType::GetStreamSection() const
{
		return m_StreamSection;
}

void Mp7JrsStreamMaskType::SetStreamSection(const Mp7JrsStreamMaskType_StreamSection_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsStreamMaskType::SetStreamSection().");
	}
	if (m_StreamSection != item)
	{
		// Mp7JrsFactory::DeleteObject(m_StreamSection);
		m_StreamSection = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_StreamSection) m_StreamSection->SetParent(m_myself.getPointer());
		if(m_StreamSection != Mp7JrsStreamMaskType_StreamSection_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_StreamSection->SetContentName(XMLString::transcode("StreamSection"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsStreamMaskType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("StreamSection")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:StreamSection is item of type StreamSectionType
		// in element collection StreamMaskType_StreamSection_CollectionType
		
		context->Found = true;
		Mp7JrsStreamMaskType_StreamSection_CollectionPtr coll = GetStreamSection();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateStreamMaskType_StreamSection_CollectionType;
				SetStreamSection(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:StreamSectionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsStreamSectionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for StreamMaskType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "StreamMaskType");
	}
	return result;
}

XMLCh * Mp7JrsStreamMaskType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsStreamMaskType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsStreamMaskType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsStreamMaskType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("StreamMaskType"));
	// Element serialization:
	if (m_StreamSection != Mp7JrsStreamMaskType_StreamSection_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_StreamSection->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsStreamMaskType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsMaskType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("StreamSection"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("StreamSection")) == 0))
		{
			// Deserialize factory type
			Mp7JrsStreamMaskType_StreamSection_CollectionPtr tmp = CreateStreamMaskType_StreamSection_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetStreamSection(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsStreamMaskType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsStreamMaskType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("StreamSection")) == 0))
  {
	Mp7JrsStreamMaskType_StreamSection_CollectionPtr tmp = CreateStreamMaskType_StreamSection_CollectionType;
	this->SetStreamSection(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsStreamMaskType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetStreamSection() != Mp7JrsNodePtr())
		result.Insert(GetStreamSection());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsStreamMaskType_ExtMethodImpl.h


