 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTemporalInterpolationType_ExtImplInclude.h



#include "Mp7JrsTemporalInterpolationType_WholeInterval_LocalType.h"
#include "Mp7JrsTemporalInterpolationType_KeyTimePoint_LocalType.h"
#include "Mp7JrsTemporalInterpolationType_InterpolationFunctions_CollectionType.h"
#include "Mp7JrsTemporalInterpolationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsTemporalInterpolationType_InterpolationFunctions_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:InterpolationFunctions

#include <assert.h>
IMp7JrsTemporalInterpolationType::IMp7JrsTemporalInterpolationType()
{

// no includefile for extension defined 
// file Mp7JrsTemporalInterpolationType_ExtPropInit.h

}

IMp7JrsTemporalInterpolationType::~IMp7JrsTemporalInterpolationType()
{
// no includefile for extension defined 
// file Mp7JrsTemporalInterpolationType_ExtPropCleanup.h

}

Mp7JrsTemporalInterpolationType::Mp7JrsTemporalInterpolationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTemporalInterpolationType::~Mp7JrsTemporalInterpolationType()
{
	Cleanup();
}

void Mp7JrsTemporalInterpolationType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_WholeInterval = Mp7JrsTemporalInterpolationType_WholeInterval_LocalPtr(); // Class
	m_KeyTimePoint = Mp7JrsTemporalInterpolationType_KeyTimePoint_LocalPtr(); // Class
	m_InterpolationFunctions = Mp7JrsTemporalInterpolationType_InterpolationFunctions_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsTemporalInterpolationType_ExtMyPropInit.h

}

void Mp7JrsTemporalInterpolationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTemporalInterpolationType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_WholeInterval);
	// Mp7JrsFactory::DeleteObject(m_KeyTimePoint);
	// Mp7JrsFactory::DeleteObject(m_InterpolationFunctions);
}

void Mp7JrsTemporalInterpolationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TemporalInterpolationTypePtr, since we
	// might need GetBase(), which isn't defined in ITemporalInterpolationType
	const Mp7JrsPtr< Mp7JrsTemporalInterpolationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_WholeInterval);
		this->SetWholeInterval(Mp7JrsFactory::CloneObject(tmp->GetWholeInterval()));
		// Mp7JrsFactory::DeleteObject(m_KeyTimePoint);
		this->SetKeyTimePoint(Mp7JrsFactory::CloneObject(tmp->GetKeyTimePoint()));
		// Mp7JrsFactory::DeleteObject(m_InterpolationFunctions);
		this->SetInterpolationFunctions(Mp7JrsFactory::CloneObject(tmp->GetInterpolationFunctions()));
}

Mp7JrsNodePtr Mp7JrsTemporalInterpolationType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsTemporalInterpolationType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsTemporalInterpolationType_WholeInterval_LocalPtr Mp7JrsTemporalInterpolationType::GetWholeInterval() const
{
		return m_WholeInterval;
}

Mp7JrsTemporalInterpolationType_KeyTimePoint_LocalPtr Mp7JrsTemporalInterpolationType::GetKeyTimePoint() const
{
		return m_KeyTimePoint;
}

Mp7JrsTemporalInterpolationType_InterpolationFunctions_CollectionPtr Mp7JrsTemporalInterpolationType::GetInterpolationFunctions() const
{
		return m_InterpolationFunctions;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsTemporalInterpolationType::SetWholeInterval(const Mp7JrsTemporalInterpolationType_WholeInterval_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTemporalInterpolationType::SetWholeInterval().");
	}
	if (m_WholeInterval != item)
	{
		// Mp7JrsFactory::DeleteObject(m_WholeInterval);
		m_WholeInterval = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_WholeInterval) m_WholeInterval->SetParent(m_myself.getPointer());
		if (m_WholeInterval && m_WholeInterval->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalInterpolationType_WholeInterval_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_WholeInterval->UseTypeAttribute = true;
		}
		if(m_WholeInterval != Mp7JrsTemporalInterpolationType_WholeInterval_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_WholeInterval->SetContentName(XMLString::transcode("WholeInterval"));
		}
	// Mp7JrsFactory::DeleteObject(m_KeyTimePoint);
	m_KeyTimePoint = Mp7JrsTemporalInterpolationType_KeyTimePoint_LocalPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsTemporalInterpolationType::SetKeyTimePoint(const Mp7JrsTemporalInterpolationType_KeyTimePoint_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTemporalInterpolationType::SetKeyTimePoint().");
	}
	if (m_KeyTimePoint != item)
	{
		// Mp7JrsFactory::DeleteObject(m_KeyTimePoint);
		m_KeyTimePoint = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_KeyTimePoint) m_KeyTimePoint->SetParent(m_myself.getPointer());
		if (m_KeyTimePoint && m_KeyTimePoint->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalInterpolationType_KeyTimePoint_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_KeyTimePoint->UseTypeAttribute = true;
		}
		if(m_KeyTimePoint != Mp7JrsTemporalInterpolationType_KeyTimePoint_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_KeyTimePoint->SetContentName(XMLString::transcode("KeyTimePoint"));
		}
	// Mp7JrsFactory::DeleteObject(m_WholeInterval);
	m_WholeInterval = Mp7JrsTemporalInterpolationType_WholeInterval_LocalPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTemporalInterpolationType::SetInterpolationFunctions(const Mp7JrsTemporalInterpolationType_InterpolationFunctions_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTemporalInterpolationType::SetInterpolationFunctions().");
	}
	if (m_InterpolationFunctions != item)
	{
		// Mp7JrsFactory::DeleteObject(m_InterpolationFunctions);
		m_InterpolationFunctions = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_InterpolationFunctions) m_InterpolationFunctions->SetParent(m_myself.getPointer());
		if(m_InterpolationFunctions != Mp7JrsTemporalInterpolationType_InterpolationFunctions_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_InterpolationFunctions->SetContentName(XMLString::transcode("InterpolationFunctions"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsTemporalInterpolationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("WholeInterval")) == 0)
	{
		// WholeInterval is simple element TemporalInterpolationType_WholeInterval_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetWholeInterval()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalInterpolationType_WholeInterval_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalInterpolationType_WholeInterval_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetWholeInterval(p, client);
					if((p = GetWholeInterval()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("KeyTimePoint")) == 0)
	{
		// KeyTimePoint is simple element TemporalInterpolationType_KeyTimePoint_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetKeyTimePoint()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalInterpolationType_KeyTimePoint_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalInterpolationType_KeyTimePoint_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetKeyTimePoint(p, client);
					if((p = GetKeyTimePoint()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("InterpolationFunctions")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:InterpolationFunctions is item of type TemporalInterpolationType_InterpolationFunctions_LocalType
		// in element collection TemporalInterpolationType_InterpolationFunctions_CollectionType
		
		context->Found = true;
		Mp7JrsTemporalInterpolationType_InterpolationFunctions_CollectionPtr coll = GetInterpolationFunctions();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateTemporalInterpolationType_InterpolationFunctions_CollectionType;
				SetInterpolationFunctions(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), 15))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalInterpolationType_InterpolationFunctions_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalInterpolationType_InterpolationFunctions_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for TemporalInterpolationType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "TemporalInterpolationType");
	}
	return result;
}

XMLCh * Mp7JrsTemporalInterpolationType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsTemporalInterpolationType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsTemporalInterpolationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("TemporalInterpolationType"));
	// Element serialization:
	// Class
	
	if (m_WholeInterval != Mp7JrsTemporalInterpolationType_WholeInterval_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_WholeInterval->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("WholeInterval"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_WholeInterval->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_KeyTimePoint != Mp7JrsTemporalInterpolationType_KeyTimePoint_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_KeyTimePoint->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("KeyTimePoint"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_KeyTimePoint->Serialize(doc, element, &elem);
	}
	if (m_InterpolationFunctions != Mp7JrsTemporalInterpolationType_InterpolationFunctions_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_InterpolationFunctions->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsTemporalInterpolationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
	do // Deserialize choice just one time!
	{
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("WholeInterval"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("WholeInterval")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateTemporalInterpolationType_WholeInterval_LocalType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetWholeInterval(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("KeyTimePoint"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("KeyTimePoint")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateTemporalInterpolationType_KeyTimePoint_LocalType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetKeyTimePoint(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	} while(false);
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("InterpolationFunctions"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("InterpolationFunctions")) == 0))
		{
			// Deserialize factory type
			Mp7JrsTemporalInterpolationType_InterpolationFunctions_CollectionPtr tmp = CreateTemporalInterpolationType_InterpolationFunctions_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetInterpolationFunctions(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsTemporalInterpolationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTemporalInterpolationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("WholeInterval")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateTemporalInterpolationType_WholeInterval_LocalType;
	}
	this->SetWholeInterval(child);
  }
  if (XMLString::compareString(elementname, X("KeyTimePoint")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateTemporalInterpolationType_KeyTimePoint_LocalType;
	}
	this->SetKeyTimePoint(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("InterpolationFunctions")) == 0))
  {
	Mp7JrsTemporalInterpolationType_InterpolationFunctions_CollectionPtr tmp = CreateTemporalInterpolationType_InterpolationFunctions_CollectionType;
	this->SetInterpolationFunctions(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTemporalInterpolationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetInterpolationFunctions() != Mp7JrsNodePtr())
		result.Insert(GetInterpolationFunctions());
	if (GetWholeInterval() != Mp7JrsNodePtr())
		result.Insert(GetWholeInterval());
	if (GetKeyTimePoint() != Mp7JrsNodePtr())
		result.Insert(GetKeyTimePoint());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTemporalInterpolationType_ExtMethodImpl.h


