 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTermUseType_ExtImplInclude.h



#include "Mp7JrsInlineTermDefinitionType.h"
#include "Mp7JrstermReferenceType.h"
#include "Mp7JrsInlineTermDefinitionType_Name_CollectionType.h"
#include "Mp7JrsInlineTermDefinitionType_Definition_CollectionType.h"
#include "Mp7JrsInlineTermDefinitionType_Term_CollectionType.h"
#include "Mp7JrsTermUseType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsInlineTermDefinitionType_Name_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Name
#include "Mp7JrsTextualType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Definition
#include "Mp7JrsInlineTermDefinitionType_Term_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Term

#include <assert.h>
IMp7JrsTermUseType::IMp7JrsTermUseType()
{

// no includefile for extension defined 
// file Mp7JrsTermUseType_ExtPropInit.h

}

IMp7JrsTermUseType::~IMp7JrsTermUseType()
{
// no includefile for extension defined 
// file Mp7JrsTermUseType_ExtPropCleanup.h

}

Mp7JrsTermUseType::Mp7JrsTermUseType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTermUseType::~Mp7JrsTermUseType()
{
	Cleanup();
}

void Mp7JrsTermUseType::Init()
{
	// Init base
	m_Base = CreateInlineTermDefinitionType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_href = Mp7JrstermReferencePtr(); // Create emtpy class ptr
	m_href_Exist = false;



// no includefile for extension defined 
// file Mp7JrsTermUseType_ExtMyPropInit.h

}

void Mp7JrsTermUseType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTermUseType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_href);
}

void Mp7JrsTermUseType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TermUseTypePtr, since we
	// might need GetBase(), which isn't defined in ITermUseType
	const Mp7JrsPtr< Mp7JrsTermUseType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsInlineTermDefinitionPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsTermUseType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_href);
	if (tmp->Existhref())
	{
		this->Sethref(Mp7JrsFactory::CloneObject(tmp->Gethref()));
	}
	else
	{
		Invalidatehref();
	}
	}
}

Mp7JrsNodePtr Mp7JrsTermUseType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsTermUseType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsInlineTermDefinitionType > Mp7JrsTermUseType::GetBase() const
{
	return m_Base;
}

Mp7JrstermReferencePtr Mp7JrsTermUseType::Gethref() const
{
	return m_href;
}

bool Mp7JrsTermUseType::Existhref() const
{
	return m_href_Exist;
}
void Mp7JrsTermUseType::Sethref(const Mp7JrstermReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTermUseType::Sethref().");
	}
	m_href = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_href) m_href->SetParent(m_myself.getPointer());
	m_href_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTermUseType::Invalidatehref()
{
	m_href_Exist = false;
}
Mp7JrsInlineTermDefinitionType_Name_CollectionPtr Mp7JrsTermUseType::GetName() const
{
	return GetBase()->GetName();
}

Mp7JrsInlineTermDefinitionType_Definition_CollectionPtr Mp7JrsTermUseType::GetDefinition() const
{
	return GetBase()->GetDefinition();
}

Mp7JrsInlineTermDefinitionType_Term_CollectionPtr Mp7JrsTermUseType::GetTerm() const
{
	return GetBase()->GetTerm();
}

void Mp7JrsTermUseType::SetName(const Mp7JrsInlineTermDefinitionType_Name_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTermUseType::SetName().");
	}
	GetBase()->SetName(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTermUseType::SetDefinition(const Mp7JrsInlineTermDefinitionType_Definition_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTermUseType::SetDefinition().");
	}
	GetBase()->SetDefinition(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTermUseType::SetTerm(const Mp7JrsInlineTermDefinitionType_Term_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTermUseType::SetTerm().");
	}
	GetBase()->SetTerm(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsTermUseType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; href,  T:urn:mpeg:mpeg7:schema:2004:termReferenceType, C:Class 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("href")) == 0)
	{
		// href is simple attribute termReferenceType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = Gethref()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:termReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrstermReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					Sethref(p, client);
					if((p = Gethref()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for TermUseType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "TermUseType");
	}
	return result;
}

XMLCh * Mp7JrsTermUseType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsTermUseType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsTermUseType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsTermUseType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("TermUseType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_href_Exist)
	{
	// Class
	if (m_href != Mp7JrstermReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_href->ToText();
		element->setAttributeNS(X(""), X("href"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsTermUseType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("href")))
	{
		// Deserialize class type
		Mp7JrstermReferencePtr tmp = CreatetermReferenceType;
		tmp->Parse(parent->getAttribute(X("href")));
		this->Sethref(tmp);
		* current = parent;
	}

	// Extensionbase is Mp7JrsInlineTermDefinitionType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsTermUseType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTermUseType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTermUseType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetName() != Mp7JrsNodePtr())
		result.Insert(GetName());
	if (GetDefinition() != Mp7JrsNodePtr())
		result.Insert(GetDefinition());
	if (GetTerm() != Mp7JrsNodePtr())
		result.Insert(GetTerm());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTermUseType_ExtMethodImpl.h


