 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_LocalType_ExtImplInclude.h



#include "Mp7JrsTextualType.h"
#include "Mp7JrsStructuredAnnotationType.h"
#include "Mp7JrsDependencyStructureType.h"
#include "Mp7JrsKeywordAnnotationType.h"
#include "Mp7JrsTextAnnotationType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsTextAnnotationType_LocalType::IMp7JrsTextAnnotationType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_LocalType_ExtPropInit.h

}

IMp7JrsTextAnnotationType_LocalType::~IMp7JrsTextAnnotationType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_LocalType_ExtPropCleanup.h

}

Mp7JrsTextAnnotationType_LocalType::Mp7JrsTextAnnotationType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTextAnnotationType_LocalType::~Mp7JrsTextAnnotationType_LocalType()
{
	Cleanup();
}

void Mp7JrsTextAnnotationType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_FreeTextAnnotation = Mp7JrsTextualPtr(); // Class
	m_StructuredAnnotation = Mp7JrsStructuredAnnotationPtr(); // Class
	m_DependencyStructure = Mp7JrsDependencyStructurePtr(); // Class
	m_KeywordAnnotation = Mp7JrsKeywordAnnotationPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_LocalType_ExtMyPropInit.h

}

void Mp7JrsTextAnnotationType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_FreeTextAnnotation);
	// Mp7JrsFactory::DeleteObject(m_StructuredAnnotation);
	// Mp7JrsFactory::DeleteObject(m_DependencyStructure);
	// Mp7JrsFactory::DeleteObject(m_KeywordAnnotation);
}

void Mp7JrsTextAnnotationType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TextAnnotationType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in ITextAnnotationType_LocalType
	const Mp7JrsPtr< Mp7JrsTextAnnotationType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_FreeTextAnnotation);
		this->SetFreeTextAnnotation(Mp7JrsFactory::CloneObject(tmp->GetFreeTextAnnotation()));
		// Mp7JrsFactory::DeleteObject(m_StructuredAnnotation);
		this->SetStructuredAnnotation(Mp7JrsFactory::CloneObject(tmp->GetStructuredAnnotation()));
		// Mp7JrsFactory::DeleteObject(m_DependencyStructure);
		this->SetDependencyStructure(Mp7JrsFactory::CloneObject(tmp->GetDependencyStructure()));
		// Mp7JrsFactory::DeleteObject(m_KeywordAnnotation);
		this->SetKeywordAnnotation(Mp7JrsFactory::CloneObject(tmp->GetKeywordAnnotation()));
}

Mp7JrsNodePtr Mp7JrsTextAnnotationType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsTextAnnotationType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsTextualPtr Mp7JrsTextAnnotationType_LocalType::GetFreeTextAnnotation() const
{
		return m_FreeTextAnnotation;
}

Mp7JrsStructuredAnnotationPtr Mp7JrsTextAnnotationType_LocalType::GetStructuredAnnotation() const
{
		return m_StructuredAnnotation;
}

Mp7JrsDependencyStructurePtr Mp7JrsTextAnnotationType_LocalType::GetDependencyStructure() const
{
		return m_DependencyStructure;
}

Mp7JrsKeywordAnnotationPtr Mp7JrsTextAnnotationType_LocalType::GetKeywordAnnotation() const
{
		return m_KeywordAnnotation;
}

// implementing setter for choice 
/* element */
void Mp7JrsTextAnnotationType_LocalType::SetFreeTextAnnotation(const Mp7JrsTextualPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextAnnotationType_LocalType::SetFreeTextAnnotation().");
	}
	if (m_FreeTextAnnotation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_FreeTextAnnotation);
		m_FreeTextAnnotation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FreeTextAnnotation) m_FreeTextAnnotation->SetParent(m_myself.getPointer());
		if (m_FreeTextAnnotation && m_FreeTextAnnotation->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FreeTextAnnotation->UseTypeAttribute = true;
		}
		if(m_FreeTextAnnotation != Mp7JrsTextualPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FreeTextAnnotation->SetContentName(XMLString::transcode("FreeTextAnnotation"));
		}
	// Mp7JrsFactory::DeleteObject(m_StructuredAnnotation);
	m_StructuredAnnotation = Mp7JrsStructuredAnnotationPtr();
	// Mp7JrsFactory::DeleteObject(m_DependencyStructure);
	m_DependencyStructure = Mp7JrsDependencyStructurePtr();
	// Mp7JrsFactory::DeleteObject(m_KeywordAnnotation);
	m_KeywordAnnotation = Mp7JrsKeywordAnnotationPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsTextAnnotationType_LocalType::SetStructuredAnnotation(const Mp7JrsStructuredAnnotationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextAnnotationType_LocalType::SetStructuredAnnotation().");
	}
	if (m_StructuredAnnotation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_StructuredAnnotation);
		m_StructuredAnnotation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_StructuredAnnotation) m_StructuredAnnotation->SetParent(m_myself.getPointer());
		if (m_StructuredAnnotation && m_StructuredAnnotation->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:StructuredAnnotationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_StructuredAnnotation->UseTypeAttribute = true;
		}
		if(m_StructuredAnnotation != Mp7JrsStructuredAnnotationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_StructuredAnnotation->SetContentName(XMLString::transcode("StructuredAnnotation"));
		}
	// Mp7JrsFactory::DeleteObject(m_FreeTextAnnotation);
	m_FreeTextAnnotation = Mp7JrsTextualPtr();
	// Mp7JrsFactory::DeleteObject(m_DependencyStructure);
	m_DependencyStructure = Mp7JrsDependencyStructurePtr();
	// Mp7JrsFactory::DeleteObject(m_KeywordAnnotation);
	m_KeywordAnnotation = Mp7JrsKeywordAnnotationPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsTextAnnotationType_LocalType::SetDependencyStructure(const Mp7JrsDependencyStructurePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextAnnotationType_LocalType::SetDependencyStructure().");
	}
	if (m_DependencyStructure != item)
	{
		// Mp7JrsFactory::DeleteObject(m_DependencyStructure);
		m_DependencyStructure = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DependencyStructure) m_DependencyStructure->SetParent(m_myself.getPointer());
		if (m_DependencyStructure && m_DependencyStructure->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DependencyStructureType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DependencyStructure->UseTypeAttribute = true;
		}
		if(m_DependencyStructure != Mp7JrsDependencyStructurePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DependencyStructure->SetContentName(XMLString::transcode("DependencyStructure"));
		}
	// Mp7JrsFactory::DeleteObject(m_FreeTextAnnotation);
	m_FreeTextAnnotation = Mp7JrsTextualPtr();
	// Mp7JrsFactory::DeleteObject(m_StructuredAnnotation);
	m_StructuredAnnotation = Mp7JrsStructuredAnnotationPtr();
	// Mp7JrsFactory::DeleteObject(m_KeywordAnnotation);
	m_KeywordAnnotation = Mp7JrsKeywordAnnotationPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsTextAnnotationType_LocalType::SetKeywordAnnotation(const Mp7JrsKeywordAnnotationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextAnnotationType_LocalType::SetKeywordAnnotation().");
	}
	if (m_KeywordAnnotation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_KeywordAnnotation);
		m_KeywordAnnotation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_KeywordAnnotation) m_KeywordAnnotation->SetParent(m_myself.getPointer());
		if (m_KeywordAnnotation && m_KeywordAnnotation->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:KeywordAnnotationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_KeywordAnnotation->UseTypeAttribute = true;
		}
		if(m_KeywordAnnotation != Mp7JrsKeywordAnnotationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_KeywordAnnotation->SetContentName(XMLString::transcode("KeywordAnnotation"));
		}
	// Mp7JrsFactory::DeleteObject(m_FreeTextAnnotation);
	m_FreeTextAnnotation = Mp7JrsTextualPtr();
	// Mp7JrsFactory::DeleteObject(m_StructuredAnnotation);
	m_StructuredAnnotation = Mp7JrsStructuredAnnotationPtr();
	// Mp7JrsFactory::DeleteObject(m_DependencyStructure);
	m_DependencyStructure = Mp7JrsDependencyStructurePtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: TextAnnotationType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsTextAnnotationType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsTextAnnotationType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsTextAnnotationType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsTextAnnotationType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_FreeTextAnnotation != Mp7JrsTextualPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FreeTextAnnotation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FreeTextAnnotation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FreeTextAnnotation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_StructuredAnnotation != Mp7JrsStructuredAnnotationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_StructuredAnnotation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("StructuredAnnotation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_StructuredAnnotation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_DependencyStructure != Mp7JrsDependencyStructurePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DependencyStructure->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DependencyStructure"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DependencyStructure->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_KeywordAnnotation != Mp7JrsKeywordAnnotationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_KeywordAnnotation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("KeywordAnnotation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_KeywordAnnotation->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsTextAnnotationType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
	{
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FreeTextAnnotation"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FreeTextAnnotation")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateTextualType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetFreeTextAnnotation(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("StructuredAnnotation"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("StructuredAnnotation")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateStructuredAnnotationType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetStructuredAnnotation(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DependencyStructure"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DependencyStructure")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateDependencyStructureType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetDependencyStructure(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("KeywordAnnotation"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("KeywordAnnotation")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateKeywordAnnotationType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetKeywordAnnotation(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	} while(false);
// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTextAnnotationType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("FreeTextAnnotation")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateTextualType;
	}
	this->SetFreeTextAnnotation(child);
  }
  if (XMLString::compareString(elementname, X("StructuredAnnotation")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateStructuredAnnotationType;
	}
	this->SetStructuredAnnotation(child);
  }
  if (XMLString::compareString(elementname, X("DependencyStructure")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateDependencyStructureType;
	}
	this->SetDependencyStructure(child);
  }
  if (XMLString::compareString(elementname, X("KeywordAnnotation")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateKeywordAnnotationType;
	}
	this->SetKeywordAnnotation(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTextAnnotationType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetFreeTextAnnotation() != Mp7JrsNodePtr())
		result.Insert(GetFreeTextAnnotation());
	if (GetStructuredAnnotation() != Mp7JrsNodePtr())
		result.Insert(GetStructuredAnnotation());
	if (GetDependencyStructure() != Mp7JrsNodePtr())
		result.Insert(GetDependencyStructure());
	if (GetKeywordAnnotation() != Mp7JrsNodePtr())
		result.Insert(GetKeywordAnnotation());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_LocalType_ExtMethodImpl.h


