 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTextualBaseType_ExtImplInclude.h



#include "Mp7JrsTextualBaseType_phoneticTranscription_CollectionType.h"
#include "Mp7JrsTextualBaseType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsTextualBaseType::IMp7JrsTextualBaseType()
{

// no includefile for extension defined 
// file Mp7JrsTextualBaseType_ExtPropInit.h

}

IMp7JrsTextualBaseType::~IMp7JrsTextualBaseType()
{
// no includefile for extension defined 
// file Mp7JrsTextualBaseType_ExtPropCleanup.h

}

Mp7JrsTextualBaseType::Mp7JrsTextualBaseType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTextualBaseType::~Mp7JrsTextualBaseType()
{
	Cleanup();
}

void Mp7JrsTextualBaseType::Init()
{
	// Init base
	m_Base = XMLString::transcode("");

	// Init attributes
	m_lang = NULL; // String
	m_lang_Exist = false;
	m_phoneticTranscription = Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr(); // Collection
	m_phoneticTranscription_Exist = false;
	m_phoneticAlphabet = Mp7JrsphoneticAlphabetType::UninitializedEnumeration;
	m_phoneticAlphabet_Default = Mp7JrsphoneticAlphabetType::sampa; // Default enumeration
	m_phoneticAlphabet_Exist = false;



// no includefile for extension defined 
// file Mp7JrsTextualBaseType_ExtMyPropInit.h

}

void Mp7JrsTextualBaseType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTextualBaseType_ExtMyPropCleanup.h


	XMLString::release(&m_Base);
	XMLString::release(&m_lang); // String
	// Mp7JrsFactory::DeleteObject(m_phoneticTranscription);
}

void Mp7JrsTextualBaseType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TextualBaseTypePtr, since we
	// might need GetBase(), which isn't defined in ITextualBaseType
	const Mp7JrsPtr< Mp7JrsTextualBaseType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	XMLString::release(&m_Base);
	m_Base = XMLString::replicate(tmp->GetContent());
	{
	XMLString::release(&m_lang); // String
	if (tmp->Existlang())
	{
		this->Setlang(XMLString::replicate(tmp->Getlang()));
	}
	else
	{
		Invalidatelang();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_phoneticTranscription);
	if (tmp->ExistphoneticTranscription())
	{
		this->SetphoneticTranscription(Mp7JrsFactory::CloneObject(tmp->GetphoneticTranscription()));
	}
	else
	{
		InvalidatephoneticTranscription();
	}
	}
	{
	if (tmp->ExistphoneticAlphabet())
	{
		this->SetphoneticAlphabet(tmp->GetphoneticAlphabet());
	}
	else
	{
		InvalidatephoneticAlphabet();
	}
	}
}

Mp7JrsNodePtr Mp7JrsTextualBaseType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsTextualBaseType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


void Mp7JrsTextualBaseType::SetContent(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualBaseType::SetContent().");
	}
	if (m_Base != item)
	{
		XMLString::release(&m_Base);
		m_Base = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsTextualBaseType::GetContent() const
{
	return m_Base;
}
XMLCh * Mp7JrsTextualBaseType::Getlang() const
{
	return m_lang;
}

bool Mp7JrsTextualBaseType::Existlang() const
{
	return m_lang_Exist;
}
void Mp7JrsTextualBaseType::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualBaseType::Setlang().");
	}
	m_lang = item;
	m_lang_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualBaseType::Invalidatelang()
{
	m_lang_Exist = false;
}
Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr Mp7JrsTextualBaseType::GetphoneticTranscription() const
{
	return m_phoneticTranscription;
}

bool Mp7JrsTextualBaseType::ExistphoneticTranscription() const
{
	return m_phoneticTranscription_Exist;
}
void Mp7JrsTextualBaseType::SetphoneticTranscription(const Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualBaseType::SetphoneticTranscription().");
	}
	m_phoneticTranscription = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_phoneticTranscription) m_phoneticTranscription->SetParent(m_myself.getPointer());
	m_phoneticTranscription_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualBaseType::InvalidatephoneticTranscription()
{
	m_phoneticTranscription_Exist = false;
}
Mp7JrsphoneticAlphabetType::Enumeration Mp7JrsTextualBaseType::GetphoneticAlphabet() const
{
	if (this->ExistphoneticAlphabet()) {
		return m_phoneticAlphabet;
	} else {
		return m_phoneticAlphabet_Default;
	}
}

bool Mp7JrsTextualBaseType::ExistphoneticAlphabet() const
{
	return m_phoneticAlphabet_Exist;
}
void Mp7JrsTextualBaseType::SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualBaseType::SetphoneticAlphabet().");
	}
	m_phoneticAlphabet = item;
	m_phoneticAlphabet_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualBaseType::InvalidatephoneticAlphabet()
{
	m_phoneticAlphabet_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsTextualBaseType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  3, 3 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 3
//- N; lang,  T:http://www.w3.org/XML/1998/namespace:lang, C:String 
//- N; phoneticTranscription,  T:urn:mpeg:mpeg7:schema:2004:TextualBaseType_phoneticTranscription_CollectionType, C:Collection 
//- N; phoneticAlphabet,  T:urn:mpeg:mpeg7:schema:2004:phoneticAlphabetType, C:Enumeration 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("lang")) == 0)
	{
		// lang is simple attribute lang
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "XMLCh *");
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("phoneticTranscription")) == 0)
	{
		// phoneticTranscription is simple attribute TextualBaseType_phoneticTranscription_CollectionType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GetphoneticTranscription()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualBaseType_phoneticTranscription_CollectionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetphoneticTranscription(p, client);
					if((p = GetphoneticTranscription()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("phoneticAlphabet")) == 0)
	{
		// phoneticAlphabet is simple attribute phoneticAlphabetType
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "Mp7JrsphoneticAlphabetType::Enumeration");
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for TextualBaseType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "TextualBaseType");
	}
	return result;
}

XMLCh * Mp7JrsTextualBaseType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsTextualBaseType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsTextualBaseType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsTextualBaseType with workaround via Deserialise()
	return Mp7JrsXPathParseContext::ContentFromString(this, txt);
}
XMLCh* Mp7JrsTextualBaseType::ContentToString() const
{
	// Mp7JrsTextualBaseType with workaround via Serialise()
	return Mp7JrsXPathParseContext::ContentToString(this);
}

void Mp7JrsTextualBaseType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	element->appendChild(doc->createTextNode(m_Base));

	assert(element);
// no includefile for extension defined 
// file Mp7JrsTextualBaseType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("TextualBaseType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_lang_Exist)
	{
	// String
	if(m_lang != (XMLCh *)NULL)
	{
		element->setAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("xml:lang"), m_lang);
	}
		
	} // Optional attriute
	
	// Attribute Serialization:
		

	// Optional attriute
	if(m_phoneticTranscription_Exist)
	{
	// Collection
	if(m_phoneticTranscription != Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr())
	{
		XMLCh * tmp = m_phoneticTranscription->ToText();
		element->setAttributeNS(X(""), X("phoneticTranscription"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Attribute Serialization:
		

	// Optional attriute
	if(m_phoneticAlphabet_Exist)
	{
	// Enumeration
	if(m_phoneticAlphabet != Mp7JrsphoneticAlphabetType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsphoneticAlphabetType::ToText(m_phoneticAlphabet);
		element->setAttributeNS(X(""), X("phoneticAlphabet"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsTextualBaseType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// Qualified namespace handling required 
	if(parent->hasAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang")))
	{
		// Deserialize string type
		this->Setlang(Mp7JrsConvert::TextToString(parent->getAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang"))));
		* current = parent;
	}
	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("phoneticTranscription")))
	{
		// Deserialize collection type
		Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr tmp = CreateTextualBaseType_phoneticTranscription_CollectionType;
		tmp->Parse(parent->getAttribute(X("phoneticTranscription")));
		this->SetphoneticTranscription(tmp);
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("phoneticAlphabet")))
	{
		this->SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Parse(parent->getAttribute(X("phoneticAlphabet"))));
		* current = parent;
	}

	XMLCh * tmp = Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent));
	if(tmp != NULL)
	{
		XMLString::release(&m_Base);
		m_Base = tmp;
		found = true;
	}
// no includefile for extension defined 
// file Mp7JrsTextualBaseType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTextualBaseType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTextualBaseType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTextualBaseType_ExtMethodImpl.h


