 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsURIFragmentReferenceType_ExtImplInclude.h



#include "Mp7JrsFragmentReferenceType.h"
#include "Mp7JrsURIFragmentReferenceType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsURIFragmentReferenceType::IMp7JrsURIFragmentReferenceType()
{

// no includefile for extension defined 
// file Mp7JrsURIFragmentReferenceType_ExtPropInit.h

}

IMp7JrsURIFragmentReferenceType::~IMp7JrsURIFragmentReferenceType()
{
// no includefile for extension defined 
// file Mp7JrsURIFragmentReferenceType_ExtPropCleanup.h

}

Mp7JrsURIFragmentReferenceType::Mp7JrsURIFragmentReferenceType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsURIFragmentReferenceType::~Mp7JrsURIFragmentReferenceType()
{
	Cleanup();
}

void Mp7JrsURIFragmentReferenceType::Init()
{
	// Init base
	m_Base = CreateFragmentReferenceType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_href = NULL; // String



// no includefile for extension defined 
// file Mp7JrsURIFragmentReferenceType_ExtMyPropInit.h

}

void Mp7JrsURIFragmentReferenceType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsURIFragmentReferenceType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_href); // String
}

void Mp7JrsURIFragmentReferenceType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use URIFragmentReferenceTypePtr, since we
	// might need GetBase(), which isn't defined in IURIFragmentReferenceType
	const Mp7JrsPtr< Mp7JrsURIFragmentReferenceType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsFragmentReferencePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsURIFragmentReferenceType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_href); // String
		this->Sethref(XMLString::replicate(tmp->Gethref()));
	}
}

Mp7JrsNodePtr Mp7JrsURIFragmentReferenceType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsURIFragmentReferenceType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsFragmentReferenceType > Mp7JrsURIFragmentReferenceType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsURIFragmentReferenceType::Gethref() const
{
	return m_href;
}

void Mp7JrsURIFragmentReferenceType::Sethref(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsURIFragmentReferenceType::Sethref().");
	}
	m_href = item;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsURIFragmentReferenceType::GetisDeferred() const
{
	return GetBase()->GetisDeferred();
}

bool Mp7JrsURIFragmentReferenceType::ExistisDeferred() const
{
	return GetBase()->ExistisDeferred();
}
void Mp7JrsURIFragmentReferenceType::SetisDeferred(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsURIFragmentReferenceType::SetisDeferred().");
	}
	GetBase()->SetisDeferred(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsURIFragmentReferenceType::InvalidateisDeferred()
{
	GetBase()->InvalidateisDeferred();
}

Mp7JrsNodeEnum Mp7JrsURIFragmentReferenceType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 2 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; isDeferred,  T:http://www.w3.org/2001/XMLSchema:boolean, C:Value 
//- N; href,  T:http://www.w3.org/2001/XMLSchema:anyURI, C:String 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("href")) == 0)
	{
		// href is simple attribute anyURI
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "XMLCh *");
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for URIFragmentReferenceType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "URIFragmentReferenceType");
	}
	return result;
}

XMLCh * Mp7JrsURIFragmentReferenceType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsURIFragmentReferenceType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsURIFragmentReferenceType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsURIFragmentReferenceType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("URIFragmentReferenceType"));
	// Attribute Serialization:
	// String
	if(m_href != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("href"), m_href);
	}
	// Element serialization:

}

bool Mp7JrsURIFragmentReferenceType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("href")))
	{
		// Deserialize string type
		this->Sethref(Mp7JrsConvert::TextToString(parent->getAttribute(X("href"))));
		* current = parent;
	}

	// Extensionbase is Mp7JrsFragmentReferenceType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsURIFragmentReferenceType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsURIFragmentReferenceType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsURIFragmentReferenceType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsURIFragmentReferenceType_ExtMethodImpl.h


