 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsUsageHistoryType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsUserIdentifierType.h"
#include "Mp7JrsUsageHistoryType_UserActionHistory_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsUsageHistoryType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsUserActionHistoryType.h" // Element collection urn:mpeg:mpeg7:schema:2004:UserActionHistory

#include <assert.h>
IMp7JrsUsageHistoryType::IMp7JrsUsageHistoryType()
{

// no includefile for extension defined 
// file Mp7JrsUsageHistoryType_ExtPropInit.h

}

IMp7JrsUsageHistoryType::~IMp7JrsUsageHistoryType()
{
// no includefile for extension defined 
// file Mp7JrsUsageHistoryType_ExtPropCleanup.h

}

Mp7JrsUsageHistoryType::Mp7JrsUsageHistoryType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsUsageHistoryType::~Mp7JrsUsageHistoryType()
{
	Cleanup();
}

void Mp7JrsUsageHistoryType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_allowCollection = Mp7JrsuserChoiceType::UninitializedEnumeration;
	m_allowCollection_Default = Mp7JrsuserChoiceType::_false; // Default enumeration
	m_allowCollection_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_UserIdentifier = Mp7JrsUserIdentifierPtr(); // Class
	m_UserIdentifier_Exist = false;
	m_UserActionHistory = Mp7JrsUsageHistoryType_UserActionHistory_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsUsageHistoryType_ExtMyPropInit.h

}

void Mp7JrsUsageHistoryType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsUsageHistoryType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_UserIdentifier);
	// Mp7JrsFactory::DeleteObject(m_UserActionHistory);
}

void Mp7JrsUsageHistoryType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use UsageHistoryTypePtr, since we
	// might need GetBase(), which isn't defined in IUsageHistoryType
	const Mp7JrsPtr< Mp7JrsUsageHistoryType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsUsageHistoryType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->ExistallowCollection())
	{
		this->SetallowCollection(tmp->GetallowCollection());
	}
	else
	{
		InvalidateallowCollection();
	}
	}
	if (tmp->IsValidUserIdentifier())
	{
		// Mp7JrsFactory::DeleteObject(m_UserIdentifier);
		this->SetUserIdentifier(Mp7JrsFactory::CloneObject(tmp->GetUserIdentifier()));
	}
	else
	{
		InvalidateUserIdentifier();
	}
		// Mp7JrsFactory::DeleteObject(m_UserActionHistory);
		this->SetUserActionHistory(Mp7JrsFactory::CloneObject(tmp->GetUserActionHistory()));
}

Mp7JrsNodePtr Mp7JrsUsageHistoryType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsUsageHistoryType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsUsageHistoryType::GetBase() const
{
	return m_Base;
}

Mp7JrsuserChoiceType::Enumeration Mp7JrsUsageHistoryType::GetallowCollection() const
{
	if (this->ExistallowCollection()) {
		return m_allowCollection;
	} else {
		return m_allowCollection_Default;
	}
}

bool Mp7JrsUsageHistoryType::ExistallowCollection() const
{
	return m_allowCollection_Exist;
}
void Mp7JrsUsageHistoryType::SetallowCollection(Mp7JrsuserChoiceType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageHistoryType::SetallowCollection().");
	}
	m_allowCollection = item;
	m_allowCollection_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageHistoryType::InvalidateallowCollection()
{
	m_allowCollection_Exist = false;
}
Mp7JrsUserIdentifierPtr Mp7JrsUsageHistoryType::GetUserIdentifier() const
{
		return m_UserIdentifier;
}

// Element is optional
bool Mp7JrsUsageHistoryType::IsValidUserIdentifier() const
{
	return m_UserIdentifier_Exist;
}

Mp7JrsUsageHistoryType_UserActionHistory_CollectionPtr Mp7JrsUsageHistoryType::GetUserActionHistory() const
{
		return m_UserActionHistory;
}

void Mp7JrsUsageHistoryType::SetUserIdentifier(const Mp7JrsUserIdentifierPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageHistoryType::SetUserIdentifier().");
	}
	if (m_UserIdentifier != item || m_UserIdentifier_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_UserIdentifier);
		m_UserIdentifier = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_UserIdentifier) m_UserIdentifier->SetParent(m_myself.getPointer());
		if (m_UserIdentifier && m_UserIdentifier->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UserIdentifierType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_UserIdentifier->UseTypeAttribute = true;
		}
		m_UserIdentifier_Exist = true;
		if(m_UserIdentifier != Mp7JrsUserIdentifierPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_UserIdentifier->SetContentName(XMLString::transcode("UserIdentifier"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageHistoryType::InvalidateUserIdentifier()
{
	m_UserIdentifier_Exist = false;
}
void Mp7JrsUsageHistoryType::SetUserActionHistory(const Mp7JrsUsageHistoryType_UserActionHistory_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageHistoryType::SetUserActionHistory().");
	}
	if (m_UserActionHistory != item)
	{
		// Mp7JrsFactory::DeleteObject(m_UserActionHistory);
		m_UserActionHistory = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_UserActionHistory) m_UserActionHistory->SetParent(m_myself.getPointer());
		if(m_UserActionHistory != Mp7JrsUsageHistoryType_UserActionHistory_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_UserActionHistory->SetContentName(XMLString::transcode("UserActionHistory"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsUsageHistoryType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsUsageHistoryType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsUsageHistoryType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageHistoryType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageHistoryType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsUsageHistoryType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsUsageHistoryType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsUsageHistoryType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageHistoryType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageHistoryType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsUsageHistoryType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsUsageHistoryType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsUsageHistoryType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageHistoryType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageHistoryType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsUsageHistoryType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsUsageHistoryType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsUsageHistoryType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageHistoryType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageHistoryType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsUsageHistoryType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsUsageHistoryType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsUsageHistoryType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageHistoryType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageHistoryType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsUsageHistoryType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsUsageHistoryType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageHistoryType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsUsageHistoryType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 6 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; id,  T:http://www.w3.org/2001/XMLSchema:ID, C:String 
//- N; timeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; timeUnit,  T:urn:mpeg:mpeg7:schema:2004:durationType, C:Pattern 
//- N; mediaTimeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; mediaTimeUnit,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
//- N; allowCollection,  T:urn:mpeg:mpeg7:schema:2004:userChoiceType, C:Enumeration 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("allowCollection")) == 0)
	{
		// allowCollection is simple attribute userChoiceType
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "Mp7JrsuserChoiceType::Enumeration");
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("UserIdentifier")) == 0)
	{
		// UserIdentifier is simple element UserIdentifierType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetUserIdentifier()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UserIdentifierType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUserIdentifierPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetUserIdentifier(p, client);
					if((p = GetUserIdentifier()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("UserActionHistory")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:UserActionHistory is item of type UserActionHistoryType
		// in element collection UsageHistoryType_UserActionHistory_CollectionType
		
		context->Found = true;
		Mp7JrsUsageHistoryType_UserActionHistory_CollectionPtr coll = GetUserActionHistory();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateUsageHistoryType_UserActionHistory_CollectionType;
				SetUserActionHistory(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UserActionHistoryType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUserActionHistoryPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for UsageHistoryType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "UsageHistoryType");
	}
	return result;
}

XMLCh * Mp7JrsUsageHistoryType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsUsageHistoryType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsUsageHistoryType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsUsageHistoryType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("UsageHistoryType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_allowCollection_Exist)
	{
	// Enumeration
	if(m_allowCollection != Mp7JrsuserChoiceType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsuserChoiceType::ToText(m_allowCollection);
		element->setAttributeNS(X(""), X("allowCollection"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_UserIdentifier != Mp7JrsUserIdentifierPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_UserIdentifier->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("UserIdentifier"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_UserIdentifier->Serialize(doc, element, &elem);
	}
	if (m_UserActionHistory != Mp7JrsUsageHistoryType_UserActionHistory_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_UserActionHistory->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsUsageHistoryType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("allowCollection")))
	{
		this->SetallowCollection(Mp7JrsuserChoiceType::Parse(parent->getAttribute(X("allowCollection"))));
		* current = parent;
	}

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("UserIdentifier"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("UserIdentifier")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateUserIdentifierType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetUserIdentifier(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("UserActionHistory"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("UserActionHistory")) == 0))
		{
			// Deserialize factory type
			Mp7JrsUsageHistoryType_UserActionHistory_CollectionPtr tmp = CreateUsageHistoryType_UserActionHistory_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetUserActionHistory(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsUsageHistoryType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsUsageHistoryType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("UserIdentifier")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateUserIdentifierType;
	}
	this->SetUserIdentifier(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("UserActionHistory")) == 0))
  {
	Mp7JrsUsageHistoryType_UserActionHistory_CollectionPtr tmp = CreateUsageHistoryType_UserActionHistory_CollectionType;
	this->SetUserActionHistory(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsUsageHistoryType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetUserIdentifier() != Mp7JrsNodePtr())
		result.Insert(GetUserIdentifier());
	if (GetUserActionHistory() != Mp7JrsNodePtr())
		result.Insert(GetUserActionHistory());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsUsageHistoryType_ExtMethodImpl.h


