 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsUserActionListType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsTermUseType.h"
#include "Mp7JrsUserActionListType_UserAction_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsUserActionListType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsUserActionType.h" // Element collection urn:mpeg:mpeg7:schema:2004:UserAction

#include <assert.h>
IMp7JrsUserActionListType::IMp7JrsUserActionListType()
{

// no includefile for extension defined 
// file Mp7JrsUserActionListType_ExtPropInit.h

}

IMp7JrsUserActionListType::~IMp7JrsUserActionListType()
{
// no includefile for extension defined 
// file Mp7JrsUserActionListType_ExtPropCleanup.h

}

Mp7JrsUserActionListType::Mp7JrsUserActionListType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsUserActionListType::~Mp7JrsUserActionListType()
{
	Cleanup();
}

void Mp7JrsUserActionListType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_numOfInstances = 0; // Value
	m_numOfInstances_Exist = false;
	m_totalDuration = Mp7JrsdurationPtr(); // Pattern
	m_totalDuration_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_ActionType = Mp7JrsTermUsePtr(); // Class
	m_UserAction = Mp7JrsUserActionListType_UserAction_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsUserActionListType_ExtMyPropInit.h

}

void Mp7JrsUserActionListType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsUserActionListType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_totalDuration); // Pattern
	// Mp7JrsFactory::DeleteObject(m_ActionType);
	// Mp7JrsFactory::DeleteObject(m_UserAction);
}

void Mp7JrsUserActionListType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use UserActionListTypePtr, since we
	// might need GetBase(), which isn't defined in IUserActionListType
	const Mp7JrsPtr< Mp7JrsUserActionListType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsUserActionListType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->ExistnumOfInstances())
	{
		this->SetnumOfInstances(tmp->GetnumOfInstances());
	}
	else
	{
		InvalidatenumOfInstances();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_totalDuration); // Pattern
	if (tmp->ExisttotalDuration())
	{
		this->SettotalDuration(Mp7JrsFactory::CloneObject(tmp->GettotalDuration()));
	}
	else
	{
		InvalidatetotalDuration();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_ActionType);
		this->SetActionType(Mp7JrsFactory::CloneObject(tmp->GetActionType()));
		// Mp7JrsFactory::DeleteObject(m_UserAction);
		this->SetUserAction(Mp7JrsFactory::CloneObject(tmp->GetUserAction()));
}

Mp7JrsNodePtr Mp7JrsUserActionListType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsUserActionListType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsUserActionListType::GetBase() const
{
	return m_Base;
}

unsigned Mp7JrsUserActionListType::GetnumOfInstances() const
{
	return m_numOfInstances;
}

bool Mp7JrsUserActionListType::ExistnumOfInstances() const
{
	return m_numOfInstances_Exist;
}
void Mp7JrsUserActionListType::SetnumOfInstances(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionListType::SetnumOfInstances().");
	}
	m_numOfInstances = item;
	m_numOfInstances_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionListType::InvalidatenumOfInstances()
{
	m_numOfInstances_Exist = false;
}
Mp7JrsdurationPtr Mp7JrsUserActionListType::GettotalDuration() const
{
	return m_totalDuration;
}

bool Mp7JrsUserActionListType::ExisttotalDuration() const
{
	return m_totalDuration_Exist;
}
void Mp7JrsUserActionListType::SettotalDuration(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionListType::SettotalDuration().");
	}
	m_totalDuration = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_totalDuration) m_totalDuration->SetParent(m_myself.getPointer());
	m_totalDuration_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionListType::InvalidatetotalDuration()
{
	m_totalDuration_Exist = false;
}
Mp7JrsTermUsePtr Mp7JrsUserActionListType::GetActionType() const
{
		return m_ActionType;
}

Mp7JrsUserActionListType_UserAction_CollectionPtr Mp7JrsUserActionListType::GetUserAction() const
{
		return m_UserAction;
}

void Mp7JrsUserActionListType::SetActionType(const Mp7JrsTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionListType::SetActionType().");
	}
	if (m_ActionType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ActionType);
		m_ActionType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ActionType) m_ActionType->SetParent(m_myself.getPointer());
		if (m_ActionType && m_ActionType->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ActionType->UseTypeAttribute = true;
		}
		if(m_ActionType != Mp7JrsTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ActionType->SetContentName(XMLString::transcode("ActionType"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionListType::SetUserAction(const Mp7JrsUserActionListType_UserAction_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionListType::SetUserAction().");
	}
	if (m_UserAction != item)
	{
		// Mp7JrsFactory::DeleteObject(m_UserAction);
		m_UserAction = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_UserAction) m_UserAction->SetParent(m_myself.getPointer());
		if(m_UserAction != Mp7JrsUserActionListType_UserAction_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_UserAction->SetContentName(XMLString::transcode("UserAction"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsUserActionListType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsUserActionListType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsUserActionListType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionListType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionListType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsUserActionListType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsUserActionListType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsUserActionListType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionListType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionListType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsUserActionListType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsUserActionListType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsUserActionListType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionListType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionListType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsUserActionListType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsUserActionListType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsUserActionListType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionListType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionListType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsUserActionListType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsUserActionListType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsUserActionListType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionListType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionListType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsUserActionListType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsUserActionListType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionListType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsUserActionListType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  2, 7 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 2
//- N; id,  T:http://www.w3.org/2001/XMLSchema:ID, C:String 
//- N; timeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; timeUnit,  T:urn:mpeg:mpeg7:schema:2004:durationType, C:Pattern 
//- N; mediaTimeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; mediaTimeUnit,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
//- N; numOfInstances,  T:http://www.w3.org/2001/XMLSchema:nonNegativeInteger, C:Value 
//- N; totalDuration,  T:urn:mpeg:mpeg7:schema:2004:durationType, C:Pattern 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("numOfInstances")) == 0)
	{
		// numOfInstances is simple attribute nonNegativeInteger
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "unsigned");
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("totalDuration")) == 0)
	{
		// totalDuration is simple attribute durationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GettotalDuration()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:durationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsdurationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SettotalDuration(p, client);
					if((p = GettotalDuration()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ActionType")) == 0)
	{
		// ActionType is simple element TermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetActionType()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetActionType(p, client);
					if((p = GetActionType()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("UserAction")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:UserAction is item of type UserActionType
		// in element collection UserActionListType_UserAction_CollectionType
		
		context->Found = true;
		Mp7JrsUserActionListType_UserAction_CollectionPtr coll = GetUserAction();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateUserActionListType_UserAction_CollectionType;
				SetUserAction(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UserActionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUserActionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for UserActionListType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "UserActionListType");
	}
	return result;
}

XMLCh * Mp7JrsUserActionListType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsUserActionListType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsUserActionListType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsUserActionListType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("UserActionListType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_numOfInstances_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_numOfInstances);
		element->setAttributeNS(X(""), X("numOfInstances"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Attribute Serialization:
		

	// Optional attriute
	if(m_totalDuration_Exist)
	{
	// Pattern
	if (m_totalDuration != Mp7JrsdurationPtr())
	{
		XMLCh * tmp = m_totalDuration->ToText();
		element->setAttributeNS(X(""), X("totalDuration"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_ActionType != Mp7JrsTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ActionType->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ActionType"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ActionType->Serialize(doc, element, &elem);
	}
	if (m_UserAction != Mp7JrsUserActionListType_UserAction_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_UserAction->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsUserActionListType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("numOfInstances")))
	{
		// deserialize value type
		this->SetnumOfInstances(Mp7JrsConvert::TextToUnsignedInt(parent->getAttribute(X("numOfInstances"))));
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("totalDuration")))
	{
		Mp7JrsdurationPtr tmp = CreatedurationType;
		tmp->Parse(parent->getAttribute(X("totalDuration")));
		this->SettotalDuration(tmp);
		* current = parent;
	}

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ActionType"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ActionType")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateTermUseType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetActionType(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("UserAction"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("UserAction")) == 0))
		{
			// Deserialize factory type
			Mp7JrsUserActionListType_UserAction_CollectionPtr tmp = CreateUserActionListType_UserAction_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetUserAction(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsUserActionListType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsUserActionListType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("ActionType")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateTermUseType;
	}
	this->SetActionType(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("UserAction")) == 0))
  {
	Mp7JrsUserActionListType_UserAction_CollectionPtr tmp = CreateUserActionListType_UserAction_CollectionType;
	this->SetUserAction(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsUserActionListType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetActionType() != Mp7JrsNodePtr())
		result.Insert(GetActionType());
	if (GetUserAction() != Mp7JrsNodePtr())
		result.Insert(GetUserAction());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsUserActionListType_ExtMethodImpl.h


