 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtImplInclude.h



#include "Mp7JrsContentManagementType.h"
#include "Mp7JrsAgentType.h"
#include "Mp7JrsUserDescriptionType_UserPreferences_CollectionType.h"
#include "Mp7JrsUserDescriptionType_UsageHistory_CollectionType.h"
#include "Mp7JrsDescriptionMetadataType.h"
#include "Mp7JrsCompleteDescriptionType_Relationships_CollectionType.h"
#include "Mp7JrsCompleteDescriptionType_OrderingKey_CollectionType.h"
#include "Mp7JrsUserDescriptionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsGraphType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Relationships
#include "Mp7JrsOrderingKeyType.h" // Element collection urn:mpeg:mpeg7:schema:2004:OrderingKey
#include "Mp7JrsUserPreferencesType.h" // Element collection urn:mpeg:mpeg7:schema:2004:UserPreferences
#include "Mp7JrsUsageHistoryType.h" // Element collection urn:mpeg:mpeg7:schema:2004:UsageHistory

#include <assert.h>
IMp7JrsUserDescriptionType::IMp7JrsUserDescriptionType()
{

// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtPropInit.h

}

IMp7JrsUserDescriptionType::~IMp7JrsUserDescriptionType()
{
// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtPropCleanup.h

}

Mp7JrsUserDescriptionType::Mp7JrsUserDescriptionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsUserDescriptionType::~Mp7JrsUserDescriptionType()
{
	Cleanup();
}

void Mp7JrsUserDescriptionType::Init()
{
	// Init base
	m_Base = CreateContentManagementType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_User = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_User_Exist = false;
	m_UserPreferences = Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr(); // Collection
	m_UsageHistory = Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtMyPropInit.h

}

void Mp7JrsUserDescriptionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_User);
	// Mp7JrsFactory::DeleteObject(m_UserPreferences);
	// Mp7JrsFactory::DeleteObject(m_UsageHistory);
}

void Mp7JrsUserDescriptionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use UserDescriptionTypePtr, since we
	// might need GetBase(), which isn't defined in IUserDescriptionType
	const Mp7JrsPtr< Mp7JrsUserDescriptionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsContentManagementPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsUserDescriptionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidUser())
	{
		// Mp7JrsFactory::DeleteObject(m_User);
		this->SetUser(Mp7JrsFactory::CloneObject(tmp->GetUser()));
	}
	else
	{
		InvalidateUser();
	}
		// Mp7JrsFactory::DeleteObject(m_UserPreferences);
		this->SetUserPreferences(Mp7JrsFactory::CloneObject(tmp->GetUserPreferences()));
		// Mp7JrsFactory::DeleteObject(m_UsageHistory);
		this->SetUsageHistory(Mp7JrsFactory::CloneObject(tmp->GetUsageHistory()));
}

Mp7JrsNodePtr Mp7JrsUserDescriptionType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsUserDescriptionType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsContentManagementType > Mp7JrsUserDescriptionType::GetBase() const
{
	return m_Base;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsUserDescriptionType::GetUser() const
{
		return m_User;
}

// Element is optional
bool Mp7JrsUserDescriptionType::IsValidUser() const
{
	return m_User_Exist;
}

Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr Mp7JrsUserDescriptionType::GetUserPreferences() const
{
		return m_UserPreferences;
}

Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr Mp7JrsUserDescriptionType::GetUsageHistory() const
{
		return m_UsageHistory;
}

void Mp7JrsUserDescriptionType::SetUser(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserDescriptionType::SetUser().");
	}
	if (m_User != item || m_User_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_User);
		m_User = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_User) m_User->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_User) m_User->UseTypeAttribute = true;
		m_User_Exist = true;
		if(m_User != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_User->SetContentName(XMLString::transcode("User"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserDescriptionType::InvalidateUser()
{
	m_User_Exist = false;
}
void Mp7JrsUserDescriptionType::SetUserPreferences(const Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserDescriptionType::SetUserPreferences().");
	}
	if (m_UserPreferences != item)
	{
		// Mp7JrsFactory::DeleteObject(m_UserPreferences);
		m_UserPreferences = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_UserPreferences) m_UserPreferences->SetParent(m_myself.getPointer());
		if(m_UserPreferences != Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_UserPreferences->SetContentName(XMLString::transcode("UserPreferences"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserDescriptionType::SetUsageHistory(const Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserDescriptionType::SetUsageHistory().");
	}
	if (m_UsageHistory != item)
	{
		// Mp7JrsFactory::DeleteObject(m_UsageHistory);
		m_UsageHistory = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_UsageHistory) m_UsageHistory->SetParent(m_myself.getPointer());
		if(m_UsageHistory != Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_UsageHistory->SetContentName(XMLString::transcode("UsageHistory"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsDescriptionMetadataPtr Mp7JrsUserDescriptionType::GetDescriptionMetadata() const
{
	return GetBase()->GetBase()->GetDescriptionMetadata();
}

// Element is optional
bool Mp7JrsUserDescriptionType::IsValidDescriptionMetadata() const
{
	return GetBase()->GetBase()->IsValidDescriptionMetadata();
}

Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr Mp7JrsUserDescriptionType::GetRelationships() const
{
	return GetBase()->GetBase()->GetRelationships();
}

Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr Mp7JrsUserDescriptionType::GetOrderingKey() const
{
	return GetBase()->GetBase()->GetOrderingKey();
}

void Mp7JrsUserDescriptionType::SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserDescriptionType::SetDescriptionMetadata().");
	}
	GetBase()->GetBase()->SetDescriptionMetadata(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserDescriptionType::InvalidateDescriptionMetadata()
{
	GetBase()->GetBase()->InvalidateDescriptionMetadata();
}
void Mp7JrsUserDescriptionType::SetRelationships(const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserDescriptionType::SetRelationships().");
	}
	GetBase()->GetBase()->SetRelationships(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserDescriptionType::SetOrderingKey(const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserDescriptionType::SetOrderingKey().");
	}
	GetBase()->GetBase()->SetOrderingKey(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsUserDescriptionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("User")) == 0)
	{
		// User is simple abstract element AgentType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetUser()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsAgentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetUser(p, client);
					if((p = GetUser()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("UserPreferences")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:UserPreferences is item of type UserPreferencesType
		// in element collection UserDescriptionType_UserPreferences_CollectionType
		
		context->Found = true;
		Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr coll = GetUserPreferences();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateUserDescriptionType_UserPreferences_CollectionType;
				SetUserPreferences(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UserPreferencesType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUserPreferencesPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("UsageHistory")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:UsageHistory is item of type UsageHistoryType
		// in element collection UserDescriptionType_UsageHistory_CollectionType
		
		context->Found = true;
		Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr coll = GetUsageHistory();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateUserDescriptionType_UsageHistory_CollectionType;
				SetUsageHistory(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UsageHistoryType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUsageHistoryPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for UserDescriptionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "UserDescriptionType");
	}
	return result;
}

XMLCh * Mp7JrsUserDescriptionType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsUserDescriptionType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsUserDescriptionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("UserDescriptionType"));
	// Element serialization:
	// Class
	
	if (m_User != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_User->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("User"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_User->UseTypeAttribute = true;
		}
		m_User->Serialize(doc, element, &elem);
	}
	if (m_UserPreferences != Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_UserPreferences->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_UsageHistory != Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_UsageHistory->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsUserDescriptionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsContentManagementType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("User"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("User")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateAgentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetUser(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("UserPreferences"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("UserPreferences")) == 0))
		{
			// Deserialize factory type
			Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr tmp = CreateUserDescriptionType_UserPreferences_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetUserPreferences(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("UsageHistory"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("UsageHistory")) == 0))
		{
			// Deserialize factory type
			Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr tmp = CreateUserDescriptionType_UsageHistory_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetUsageHistory(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsUserDescriptionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("User")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateAgentType;
	}
	this->SetUser(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("UserPreferences")) == 0))
  {
	Mp7JrsUserDescriptionType_UserPreferences_CollectionPtr tmp = CreateUserDescriptionType_UserPreferences_CollectionType;
	this->SetUserPreferences(tmp);
	child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("UsageHistory")) == 0))
  {
	Mp7JrsUserDescriptionType_UsageHistory_CollectionPtr tmp = CreateUserDescriptionType_UsageHistory_CollectionType;
	this->SetUsageHistory(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsUserDescriptionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetUser() != Mp7JrsNodePtr())
		result.Insert(GetUser());
	if (GetUserPreferences() != Mp7JrsNodePtr())
		result.Insert(GetUserPreferences());
	if (GetUsageHistory() != Mp7JrsNodePtr())
		result.Insert(GetUsageHistory());
	if (GetDescriptionMetadata() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionMetadata());
	if (GetRelationships() != Mp7JrsNodePtr())
		result.Insert(GetRelationships());
	if (GetOrderingKey() != Mp7JrsNodePtr())
		result.Insert(GetOrderingKey());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsUserDescriptionType_ExtMethodImpl.h


