 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsVariationType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsMultimediaContentType.h"
#include "Mp7JrsVariationType_VariationRelationship_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsVariationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsVariationType_VariationRelationship_LocalType2.h" // Element collection urn:mpeg:mpeg7:schema:2004:VariationRelationship

#include <assert.h>
IMp7JrsVariationType::IMp7JrsVariationType()
{

// no includefile for extension defined 
// file Mp7JrsVariationType_ExtPropInit.h

}

IMp7JrsVariationType::~IMp7JrsVariationType()
{
// no includefile for extension defined 
// file Mp7JrsVariationType_ExtPropCleanup.h

}

Mp7JrsVariationType::Mp7JrsVariationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsVariationType::~Mp7JrsVariationType()
{
	Cleanup();
}

void Mp7JrsVariationType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_fidelity = CreatezeroToOneType; // Create a valid class
	m_fidelity->SetContent(0.0); // Use Value initialiser (xxx2)
	m_fidelity_Exist = false;
	m_priority = 0; // Value
	m_priority_Exist = false;
	m_timeOffset = Mp7JrsmediaDurationPtr(); // Pattern
	m_timeOffset_Exist = false;
	m_timeScale = CreatezeroToOneType; // Create a valid class
	m_timeScale->SetContent(0.0); // Use Value initialiser (xxx2)
	m_timeScale_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Source = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_Source_Exist = false;
	m_Content = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_VariationRelationship = Mp7JrsVariationType_VariationRelationship_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsVariationType_ExtMyPropInit.h

}

void Mp7JrsVariationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsVariationType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_fidelity);
	// Mp7JrsFactory::DeleteObject(m_timeOffset); // Pattern
	// Mp7JrsFactory::DeleteObject(m_timeScale);
	// Mp7JrsFactory::DeleteObject(m_Source);
	// Mp7JrsFactory::DeleteObject(m_Content);
	// Mp7JrsFactory::DeleteObject(m_VariationRelationship);
}

void Mp7JrsVariationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use VariationTypePtr, since we
	// might need GetBase(), which isn't defined in IVariationType
	const Mp7JrsPtr< Mp7JrsVariationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsVariationType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_fidelity);
	if (tmp->Existfidelity())
	{
		this->Setfidelity(Mp7JrsFactory::CloneObject(tmp->Getfidelity()));
	}
	else
	{
		Invalidatefidelity();
	}
	}
	{
	if (tmp->Existpriority())
	{
		this->Setpriority(tmp->Getpriority());
	}
	else
	{
		Invalidatepriority();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_timeOffset); // Pattern
	if (tmp->ExisttimeOffset())
	{
		this->SettimeOffset(Mp7JrsFactory::CloneObject(tmp->GettimeOffset()));
	}
	else
	{
		InvalidatetimeOffset();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_timeScale);
	if (tmp->ExisttimeScale())
	{
		this->SettimeScale(Mp7JrsFactory::CloneObject(tmp->GettimeScale()));
	}
	else
	{
		InvalidatetimeScale();
	}
	}
	if (tmp->IsValidSource())
	{
		// Mp7JrsFactory::DeleteObject(m_Source);
		this->SetSource(Mp7JrsFactory::CloneObject(tmp->GetSource()));
	}
	else
	{
		InvalidateSource();
	}
		// Mp7JrsFactory::DeleteObject(m_Content);
		this->SetContent(Mp7JrsFactory::CloneObject(tmp->GetContent()));
		// Mp7JrsFactory::DeleteObject(m_VariationRelationship);
		this->SetVariationRelationship(Mp7JrsFactory::CloneObject(tmp->GetVariationRelationship()));
}

Mp7JrsNodePtr Mp7JrsVariationType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsVariationType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsVariationType::GetBase() const
{
	return m_Base;
}

Mp7JrszeroToOnePtr Mp7JrsVariationType::Getfidelity() const
{
	return m_fidelity;
}

bool Mp7JrsVariationType::Existfidelity() const
{
	return m_fidelity_Exist;
}
void Mp7JrsVariationType::Setfidelity(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::Setfidelity().");
	}
	m_fidelity = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_fidelity) m_fidelity->SetParent(m_myself.getPointer());
	m_fidelity_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::Invalidatefidelity()
{
	m_fidelity_Exist = false;
}
unsigned Mp7JrsVariationType::Getpriority() const
{
	return m_priority;
}

bool Mp7JrsVariationType::Existpriority() const
{
	return m_priority_Exist;
}
void Mp7JrsVariationType::Setpriority(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::Setpriority().");
	}
	m_priority = item;
	m_priority_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::Invalidatepriority()
{
	m_priority_Exist = false;
}
Mp7JrsmediaDurationPtr Mp7JrsVariationType::GettimeOffset() const
{
	return m_timeOffset;
}

bool Mp7JrsVariationType::ExisttimeOffset() const
{
	return m_timeOffset_Exist;
}
void Mp7JrsVariationType::SettimeOffset(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::SettimeOffset().");
	}
	m_timeOffset = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_timeOffset) m_timeOffset->SetParent(m_myself.getPointer());
	m_timeOffset_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::InvalidatetimeOffset()
{
	m_timeOffset_Exist = false;
}
Mp7JrszeroToOnePtr Mp7JrsVariationType::GettimeScale() const
{
	return m_timeScale;
}

bool Mp7JrsVariationType::ExisttimeScale() const
{
	return m_timeScale_Exist;
}
void Mp7JrsVariationType::SettimeScale(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::SettimeScale().");
	}
	m_timeScale = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_timeScale) m_timeScale->SetParent(m_myself.getPointer());
	m_timeScale_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::InvalidatetimeScale()
{
	m_timeScale_Exist = false;
}
Mp7JrsPtr< Mp7JrsNode > Mp7JrsVariationType::GetSource() const
{
		return m_Source;
}

// Element is optional
bool Mp7JrsVariationType::IsValidSource() const
{
	return m_Source_Exist;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsVariationType::GetContent() const
{
		return m_Content;
}

Mp7JrsVariationType_VariationRelationship_CollectionPtr Mp7JrsVariationType::GetVariationRelationship() const
{
		return m_VariationRelationship;
}

void Mp7JrsVariationType::SetSource(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::SetSource().");
	}
	if (m_Source != item || m_Source_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Source);
		m_Source = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Source) m_Source->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_Source) m_Source->UseTypeAttribute = true;
		m_Source_Exist = true;
		if(m_Source != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Source->SetContentName(XMLString::transcode("Source"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::InvalidateSource()
{
	m_Source_Exist = false;
}
void Mp7JrsVariationType::SetContent(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::SetContent().");
	}
	if (m_Content != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Content);
		m_Content = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Content) m_Content->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_Content) m_Content->UseTypeAttribute = true;
		if(m_Content != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Content->SetContentName(XMLString::transcode("Content"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::SetVariationRelationship(const Mp7JrsVariationType_VariationRelationship_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::SetVariationRelationship().");
	}
	if (m_VariationRelationship != item)
	{
		// Mp7JrsFactory::DeleteObject(m_VariationRelationship);
		m_VariationRelationship = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VariationRelationship) m_VariationRelationship->SetParent(m_myself.getPointer());
		if(m_VariationRelationship != Mp7JrsVariationType_VariationRelationship_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VariationRelationship->SetContentName(XMLString::transcode("VariationRelationship"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsVariationType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsVariationType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsVariationType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsVariationType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsVariationType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsVariationType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsVariationType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsVariationType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsVariationType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsVariationType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsVariationType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsVariationType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsVariationType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsVariationType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsVariationType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVariationType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsVariationType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsVariationType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVariationType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsVariationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  4, 9 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 4
//- N; id,  T:http://www.w3.org/2001/XMLSchema:ID, C:String 
//- N; timeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; timeUnit,  T:urn:mpeg:mpeg7:schema:2004:durationType, C:Pattern 
//- N; mediaTimeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; mediaTimeUnit,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
//- N; fidelity,  T:urn:mpeg:mpeg7:schema:2004:zeroToOneType, C:Class 
//- N; priority,  T:http://www.w3.org/2001/XMLSchema:nonNegativeInteger, C:Value 
//- N; timeOffset,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
//- N; timeScale,  T:urn:mpeg:mpeg7:schema:2004:zeroToOneType, C:Class 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("fidelity")) == 0)
	{
		// fidelity is simple attribute zeroToOneType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = Getfidelity()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:zeroToOneType")))) != empty)
			{
				// Is type allowed
				Mp7JrszeroToOnePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					Setfidelity(p, client);
					if((p = Getfidelity()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("priority")) == 0)
	{
		// priority is simple attribute nonNegativeInteger
		Mp7JrsXPathParseContext::ErrorUnsupportedAttributeType(context, "unsigned");
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("timeOffset")) == 0)
	{
		// timeOffset is simple attribute mediaDurationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GettimeOffset()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaDurationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsmediaDurationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SettimeOffset(p, client);
					if((p = GettimeOffset()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Attribute ------------------------------------------------------------------
	else if(context->IsAttribute && XMLString::compareString(context->ElementName, X("timeScale")) == 0)
	{
		// timeScale is simple attribute zeroToOneType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GettimeScale()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:zeroToOneType")))) != empty)
			{
				// Is type allowed
				Mp7JrszeroToOnePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SettimeScale(p, client);
					if((p = GettimeScale()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Source")) == 0)
	{
		// Source is simple abstract element MultimediaContentType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSource()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsMultimediaContentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSource(p, client);
					if((p = GetSource()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Content")) == 0)
	{
		// Content is simple abstract element MultimediaContentType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetContent()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsMultimediaContentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetContent(p, client);
					if((p = GetContent()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("VariationRelationship")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:VariationRelationship is item of type VariationType_VariationRelationship_LocalType2
		// in element collection VariationType_VariationRelationship_CollectionType
		
		context->Found = true;
		Mp7JrsVariationType_VariationRelationship_CollectionPtr coll = GetVariationRelationship();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateVariationType_VariationRelationship_CollectionType;
				SetVariationRelationship(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:VariationType_VariationRelationship_LocalType2")))) != empty)
			{
				// Is type allowed
				Mp7JrsVariationType_VariationRelationship_Local2Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for VariationType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "VariationType");
	}
	return result;
}

XMLCh * Mp7JrsVariationType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsVariationType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsVariationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsVariationType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("VariationType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_fidelity_Exist)
	{
	// Class
	if (m_fidelity != Mp7JrszeroToOnePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_fidelity->ToText();
		element->setAttributeNS(X(""), X("fidelity"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Attribute Serialization:
		

	// Optional attriute
	if(m_priority_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_priority);
		element->setAttributeNS(X(""), X("priority"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Attribute Serialization:
		

	// Optional attriute
	if(m_timeOffset_Exist)
	{
	// Pattern
	if (m_timeOffset != Mp7JrsmediaDurationPtr())
	{
		XMLCh * tmp = m_timeOffset->ToText();
		element->setAttributeNS(X(""), X("timeOffset"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Attribute Serialization:
		

	// Optional attriute
	if(m_timeScale_Exist)
	{
	// Class
	if (m_timeScale != Mp7JrszeroToOnePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_timeScale->ToText();
		element->setAttributeNS(X(""), X("timeScale"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Source != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Source->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Source"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_Source->UseTypeAttribute = true;
		}
		m_Source->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Content != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Content->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Content"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_Content->UseTypeAttribute = true;
		}
		m_Content->Serialize(doc, element, &elem);
	}
	if (m_VariationRelationship != Mp7JrsVariationType_VariationRelationship_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_VariationRelationship->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsVariationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("fidelity")))
	{
		// Deserialize class type
		Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
		tmp->Parse(parent->getAttribute(X("fidelity")));
		this->Setfidelity(tmp);
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("priority")))
	{
		// deserialize value type
		this->Setpriority(Mp7JrsConvert::TextToUnsignedInt(parent->getAttribute(X("priority"))));
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("timeOffset")))
	{
		Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
		tmp->Parse(parent->getAttribute(X("timeOffset")));
		this->SettimeOffset(tmp);
		* current = parent;
	}

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("timeScale")))
	{
		// Deserialize class type
		Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
		tmp->Parse(parent->getAttribute(X("timeScale")));
		this->SettimeScale(tmp);
		* current = parent;
	}

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Source"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Source")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMultimediaContentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSource(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Content"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Content")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMultimediaContentType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetContent(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("VariationRelationship"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("VariationRelationship")) == 0))
		{
			// Deserialize factory type
			Mp7JrsVariationType_VariationRelationship_CollectionPtr tmp = CreateVariationType_VariationRelationship_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetVariationRelationship(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsVariationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsVariationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("Source")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMultimediaContentType;
	}
	this->SetSource(child);
  }
  if (XMLString::compareString(elementname, X("Content")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMultimediaContentType;
	}
	this->SetContent(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("VariationRelationship")) == 0))
  {
	Mp7JrsVariationType_VariationRelationship_CollectionPtr tmp = CreateVariationType_VariationRelationship_CollectionType;
	this->SetVariationRelationship(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsVariationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSource() != Mp7JrsNodePtr())
		result.Insert(GetSource());
	if (GetContent() != Mp7JrsNodePtr())
		result.Insert(GetContent());
	if (GetVariationRelationship() != Mp7JrsNodePtr())
		result.Insert(GetVariationRelationship());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsVariationType_ExtMethodImpl.h


