 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsViewDescriptionType_ExtImplInclude.h



#include "Mp7JrsContentAbstractionType.h"
#include "Mp7JrsViewDescriptionType_View_CollectionType.h"
#include "Mp7JrsViewDescriptionType_ViewSet_CollectionType.h"
#include "Mp7JrsViewDescriptionType_ViewDecomposition_CollectionType.h"
#include "Mp7JrsContentDescriptionType_Affective_CollectionType.h"
#include "Mp7JrsDescriptionMetadataType.h"
#include "Mp7JrsCompleteDescriptionType_Relationships_CollectionType.h"
#include "Mp7JrsCompleteDescriptionType_OrderingKey_CollectionType.h"
#include "Mp7JrsViewDescriptionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsGraphType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Relationships
#include "Mp7JrsOrderingKeyType.h" // Element collection urn:mpeg:mpeg7:schema:2004:OrderingKey
#include "Mp7JrsAffectiveType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Affective
#include "Mp7JrsViewType.h" // Element collection urn:mpeg:mpeg7:schema:2004:View
#include "Mp7JrsViewSetType.h" // Element collection urn:mpeg:mpeg7:schema:2004:ViewSet
#include "Mp7JrsViewDecompositionType.h" // Element collection urn:mpeg:mpeg7:schema:2004:ViewDecomposition

#include <assert.h>
IMp7JrsViewDescriptionType::IMp7JrsViewDescriptionType()
{

// no includefile for extension defined 
// file Mp7JrsViewDescriptionType_ExtPropInit.h

}

IMp7JrsViewDescriptionType::~IMp7JrsViewDescriptionType()
{
// no includefile for extension defined 
// file Mp7JrsViewDescriptionType_ExtPropCleanup.h

}

Mp7JrsViewDescriptionType::Mp7JrsViewDescriptionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsViewDescriptionType::~Mp7JrsViewDescriptionType()
{
	Cleanup();
}

void Mp7JrsViewDescriptionType::Init()
{
	// Init base
	m_Base = CreateContentAbstractionType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_View = Mp7JrsViewDescriptionType_View_CollectionPtr(); // Collection
	m_ViewSet = Mp7JrsViewDescriptionType_ViewSet_CollectionPtr(); // Collection
	m_ViewDecomposition = Mp7JrsViewDescriptionType_ViewDecomposition_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsViewDescriptionType_ExtMyPropInit.h

}

void Mp7JrsViewDescriptionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsViewDescriptionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_View);
	// Mp7JrsFactory::DeleteObject(m_ViewSet);
	// Mp7JrsFactory::DeleteObject(m_ViewDecomposition);
}

void Mp7JrsViewDescriptionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ViewDescriptionTypePtr, since we
	// might need GetBase(), which isn't defined in IViewDescriptionType
	const Mp7JrsPtr< Mp7JrsViewDescriptionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsContentAbstractionPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsViewDescriptionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_View);
		this->SetView(Mp7JrsFactory::CloneObject(tmp->GetView()));
		// Mp7JrsFactory::DeleteObject(m_ViewSet);
		this->SetViewSet(Mp7JrsFactory::CloneObject(tmp->GetViewSet()));
		// Mp7JrsFactory::DeleteObject(m_ViewDecomposition);
		this->SetViewDecomposition(Mp7JrsFactory::CloneObject(tmp->GetViewDecomposition()));
}

Mp7JrsNodePtr Mp7JrsViewDescriptionType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsViewDescriptionType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsContentAbstractionType > Mp7JrsViewDescriptionType::GetBase() const
{
	return m_Base;
}

Mp7JrsViewDescriptionType_View_CollectionPtr Mp7JrsViewDescriptionType::GetView() const
{
		return m_View;
}

Mp7JrsViewDescriptionType_ViewSet_CollectionPtr Mp7JrsViewDescriptionType::GetViewSet() const
{
		return m_ViewSet;
}

Mp7JrsViewDescriptionType_ViewDecomposition_CollectionPtr Mp7JrsViewDescriptionType::GetViewDecomposition() const
{
		return m_ViewDecomposition;
}

// implementing setter for choice 
/* element */
void Mp7JrsViewDescriptionType::SetView(const Mp7JrsViewDescriptionType_View_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDescriptionType::SetView().");
	}
	if (m_View != item)
	{
		// Mp7JrsFactory::DeleteObject(m_View);
		m_View = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_View) m_View->SetParent(m_myself.getPointer());
		if(m_View != Mp7JrsViewDescriptionType_View_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_View->SetContentName(XMLString::transcode("View"));
		}
	// Mp7JrsFactory::DeleteObject(m_ViewSet);
	m_ViewSet = Mp7JrsViewDescriptionType_ViewSet_CollectionPtr();
	// Mp7JrsFactory::DeleteObject(m_ViewDecomposition);
	m_ViewDecomposition = Mp7JrsViewDescriptionType_ViewDecomposition_CollectionPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsViewDescriptionType::SetViewSet(const Mp7JrsViewDescriptionType_ViewSet_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDescriptionType::SetViewSet().");
	}
	if (m_ViewSet != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ViewSet);
		m_ViewSet = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ViewSet) m_ViewSet->SetParent(m_myself.getPointer());
		if(m_ViewSet != Mp7JrsViewDescriptionType_ViewSet_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ViewSet->SetContentName(XMLString::transcode("ViewSet"));
		}
	// Mp7JrsFactory::DeleteObject(m_View);
	m_View = Mp7JrsViewDescriptionType_View_CollectionPtr();
	// Mp7JrsFactory::DeleteObject(m_ViewDecomposition);
	m_ViewDecomposition = Mp7JrsViewDescriptionType_ViewDecomposition_CollectionPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsViewDescriptionType::SetViewDecomposition(const Mp7JrsViewDescriptionType_ViewDecomposition_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDescriptionType::SetViewDecomposition().");
	}
	if (m_ViewDecomposition != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ViewDecomposition);
		m_ViewDecomposition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ViewDecomposition) m_ViewDecomposition->SetParent(m_myself.getPointer());
		if(m_ViewDecomposition != Mp7JrsViewDescriptionType_ViewDecomposition_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ViewDecomposition->SetContentName(XMLString::transcode("ViewDecomposition"));
		}
	// Mp7JrsFactory::DeleteObject(m_View);
	m_View = Mp7JrsViewDescriptionType_View_CollectionPtr();
	// Mp7JrsFactory::DeleteObject(m_ViewSet);
	m_ViewSet = Mp7JrsViewDescriptionType_ViewSet_CollectionPtr();
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsContentDescriptionType_Affective_CollectionPtr Mp7JrsViewDescriptionType::GetAffective() const
{
	return GetBase()->GetBase()->GetAffective();
}

void Mp7JrsViewDescriptionType::SetAffective(const Mp7JrsContentDescriptionType_Affective_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDescriptionType::SetAffective().");
	}
	GetBase()->GetBase()->SetAffective(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsDescriptionMetadataPtr Mp7JrsViewDescriptionType::GetDescriptionMetadata() const
{
	return GetBase()->GetBase()->GetBase()->GetDescriptionMetadata();
}

// Element is optional
bool Mp7JrsViewDescriptionType::IsValidDescriptionMetadata() const
{
	return GetBase()->GetBase()->GetBase()->IsValidDescriptionMetadata();
}

Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr Mp7JrsViewDescriptionType::GetRelationships() const
{
	return GetBase()->GetBase()->GetBase()->GetRelationships();
}

Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr Mp7JrsViewDescriptionType::GetOrderingKey() const
{
	return GetBase()->GetBase()->GetBase()->GetOrderingKey();
}

void Mp7JrsViewDescriptionType::SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDescriptionType::SetDescriptionMetadata().");
	}
	GetBase()->GetBase()->GetBase()->SetDescriptionMetadata(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewDescriptionType::InvalidateDescriptionMetadata()
{
	GetBase()->GetBase()->GetBase()->InvalidateDescriptionMetadata();
}
void Mp7JrsViewDescriptionType::SetRelationships(const Mp7JrsCompleteDescriptionType_Relationships_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDescriptionType::SetRelationships().");
	}
	GetBase()->GetBase()->GetBase()->SetRelationships(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewDescriptionType::SetOrderingKey(const Mp7JrsCompleteDescriptionType_OrderingKey_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDescriptionType::SetOrderingKey().");
	}
	GetBase()->GetBase()->GetBase()->SetOrderingKey(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsViewDescriptionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("View")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:View is item of abstract type ViewType
		// in element collection ViewDescriptionType_View_CollectionType
		
		context->Found = true;
		Mp7JrsViewDescriptionType_View_CollectionPtr coll = GetView();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateViewDescriptionType_View_CollectionType;
				SetView(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsViewPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ViewSet")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:ViewSet is item of type ViewSetType
		// in element collection ViewDescriptionType_ViewSet_CollectionType
		
		context->Found = true;
		Mp7JrsViewDescriptionType_ViewSet_CollectionPtr coll = GetViewSet();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateViewDescriptionType_ViewSet_CollectionType;
				SetViewSet(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ViewSetType")))) != empty)
			{
				// Is type allowed
				Mp7JrsViewSetPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ViewDecomposition")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:ViewDecomposition is item of abstract type ViewDecompositionType
		// in element collection ViewDescriptionType_ViewDecomposition_CollectionType
		
		context->Found = true;
		Mp7JrsViewDescriptionType_ViewDecomposition_CollectionPtr coll = GetViewDecomposition();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateViewDescriptionType_ViewDecomposition_CollectionType;
				SetViewDecomposition(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsViewDecompositionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ViewDescriptionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ViewDescriptionType");
	}
	return result;
}

XMLCh * Mp7JrsViewDescriptionType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsViewDescriptionType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsViewDescriptionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsViewDescriptionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ViewDescriptionType"));
	// Element serialization:
	if (m_View != Mp7JrsViewDescriptionType_View_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_View->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_ViewSet != Mp7JrsViewDescriptionType_ViewSet_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_ViewSet->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_ViewDecomposition != Mp7JrsViewDescriptionType_ViewDecomposition_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_ViewDecomposition->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsViewDescriptionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsContentAbstractionType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
	{
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("View"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("View")) == 0))
		{
			// Deserialize factory type
			Mp7JrsViewDescriptionType_View_CollectionPtr tmp = CreateViewDescriptionType_View_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetView(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("ViewSet"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("ViewSet")) == 0))
		{
			// Deserialize factory type
			Mp7JrsViewDescriptionType_ViewSet_CollectionPtr tmp = CreateViewDescriptionType_ViewSet_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetViewSet(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
		
		// Element Collection
		if((parent != NULL) && 
		Mp7JrsUtil::HasNodeName(parent, X("ViewDecomposition"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD		(XMLString::compareString(parent->getNodeName(), X("ViewDecomposition")) == 0))
		{
			// Deserialize factory type
			Mp7JrsViewDescriptionType_ViewDecomposition_CollectionPtr tmp = CreateViewDescriptionType_ViewDecomposition_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetViewDecomposition(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
			continue;	   // For choices
		}
	} while(false);
// no includefile for extension defined 
// file Mp7JrsViewDescriptionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsViewDescriptionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("View")) == 0))
  {
	Mp7JrsViewDescriptionType_View_CollectionPtr tmp = CreateViewDescriptionType_View_CollectionType;
	this->SetView(tmp);
	child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("ViewSet")) == 0))
  {
	Mp7JrsViewDescriptionType_ViewSet_CollectionPtr tmp = CreateViewDescriptionType_ViewSet_CollectionType;
	this->SetViewSet(tmp);
	child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("ViewDecomposition")) == 0))
  {
	Mp7JrsViewDescriptionType_ViewDecomposition_CollectionPtr tmp = CreateViewDescriptionType_ViewDecomposition_CollectionType;
	this->SetViewDecomposition(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsViewDescriptionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetView() != Mp7JrsNodePtr())
		result.Insert(GetView());
	if (GetViewSet() != Mp7JrsNodePtr())
		result.Insert(GetViewSet());
	if (GetViewDecomposition() != Mp7JrsNodePtr())
		result.Insert(GetViewDecomposition());
	if (GetAffective() != Mp7JrsNodePtr())
		result.Insert(GetAffective());
	if (GetDescriptionMetadata() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionMetadata());
	if (GetRelationships() != Mp7JrsNodePtr())
		result.Insert(GetRelationships());
	if (GetOrderingKey() != Mp7JrsNodePtr())
		result.Insert(GetOrderingKey());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsViewDescriptionType_ExtMethodImpl.h


