 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsViewSetType_ExtImplInclude.h



#include "Mp7JrsViewDecompositionType.h"
#include "Mp7JrsViewSetType_setProperty_LocalType2.h"
#include "Mp7JrsViewSetType_CollectionType.h"
#include "Mp7JrsSignalType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsViewSetType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsViewSetType_LocalType.h" // Choice collection View
#include "Mp7JrsViewType.h" // Choice collection element View
#include "Mp7JrsReferenceType.h" // Choice collection element ViewRef

#include <assert.h>
IMp7JrsViewSetType::IMp7JrsViewSetType()
{

// no includefile for extension defined 
// file Mp7JrsViewSetType_ExtPropInit.h

}

IMp7JrsViewSetType::~IMp7JrsViewSetType()
{
// no includefile for extension defined 
// file Mp7JrsViewSetType_ExtPropCleanup.h

}

Mp7JrsViewSetType::Mp7JrsViewSetType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsViewSetType::~Mp7JrsViewSetType()
{
	Cleanup();
}

void Mp7JrsViewSetType::Init()
{
	// Init base
	m_Base = CreateViewDecompositionType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_setProperty = Mp7JrsViewSetType_setProperty_Local2Ptr(); // Create emtpy class ptr
	m_setProperty_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_ViewSetType_LocalType = Mp7JrsViewSetType_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsViewSetType_ExtMyPropInit.h

}

void Mp7JrsViewSetType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsViewSetType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_setProperty);
	// Mp7JrsFactory::DeleteObject(m_ViewSetType_LocalType);
}

void Mp7JrsViewSetType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ViewSetTypePtr, since we
	// might need GetBase(), which isn't defined in IViewSetType
	const Mp7JrsPtr< Mp7JrsViewSetType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsViewDecompositionPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsViewSetType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_setProperty);
	if (tmp->ExistsetProperty())
	{
		this->SetsetProperty(Mp7JrsFactory::CloneObject(tmp->GetsetProperty()));
	}
	else
	{
		InvalidatesetProperty();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_ViewSetType_LocalType);
		this->SetViewSetType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetViewSetType_LocalType()));
}

Mp7JrsNodePtr Mp7JrsViewSetType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsViewSetType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsViewDecompositionType > Mp7JrsViewSetType::GetBase() const
{
	return m_Base;
}

Mp7JrsViewSetType_setProperty_Local2Ptr Mp7JrsViewSetType::GetsetProperty() const
{
	return m_setProperty;
}

bool Mp7JrsViewSetType::ExistsetProperty() const
{
	return m_setProperty_Exist;
}
void Mp7JrsViewSetType::SetsetProperty(const Mp7JrsViewSetType_setProperty_Local2Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::SetsetProperty().");
	}
	m_setProperty = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_setProperty) m_setProperty->SetParent(m_myself.getPointer());
	m_setProperty_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewSetType::InvalidatesetProperty()
{
	m_setProperty_Exist = false;
}
Mp7JrsViewSetType_CollectionPtr Mp7JrsViewSetType::GetViewSetType_LocalType() const
{
		return m_ViewSetType_LocalType;
}

void Mp7JrsViewSetType::SetViewSetType_LocalType(const Mp7JrsViewSetType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::SetViewSetType_LocalType().");
	}
	if (m_ViewSetType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ViewSetType_LocalType);
		m_ViewSetType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ViewSetType_LocalType) m_ViewSetType_LocalType->SetParent(m_myself.getPointer());
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsViewSetType::Getcomplete() const
{
	return GetBase()->Getcomplete();
}

bool Mp7JrsViewSetType::Existcomplete() const
{
	return GetBase()->Existcomplete();
}
void Mp7JrsViewSetType::Setcomplete(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::Setcomplete().");
	}
	GetBase()->Setcomplete(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewSetType::Invalidatecomplete()
{
	GetBase()->Invalidatecomplete();
}
bool Mp7JrsViewSetType::GetnonRedundant() const
{
	return GetBase()->GetnonRedundant();
}

bool Mp7JrsViewSetType::ExistnonRedundant() const
{
	return GetBase()->ExistnonRedundant();
}
void Mp7JrsViewSetType::SetnonRedundant(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::SetnonRedundant().");
	}
	GetBase()->SetnonRedundant(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewSetType::InvalidatenonRedundant()
{
	GetBase()->InvalidatenonRedundant();
}
Mp7JrsSignalPtr Mp7JrsViewSetType::GetSource() const
{
	return GetBase()->GetSource();
}

// Element is optional
bool Mp7JrsViewSetType::IsValidSource() const
{
	return GetBase()->IsValidSource();
}

void Mp7JrsViewSetType::SetSource(const Mp7JrsSignalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::SetSource().");
	}
	GetBase()->SetSource(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewSetType::InvalidateSource()
{
	GetBase()->InvalidateSource();
}
XMLCh * Mp7JrsViewSetType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsViewSetType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsViewSetType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewSetType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsViewSetType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsViewSetType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsViewSetType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewSetType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsViewSetType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsViewSetType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsViewSetType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewSetType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsViewSetType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsViewSetType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsViewSetType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewSetType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsViewSetType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsViewSetType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsViewSetType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewSetType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsViewSetType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsViewSetType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewSetType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsViewSetType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 8 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; id,  T:http://www.w3.org/2001/XMLSchema:ID, C:String 
//- N; timeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; timeUnit,  T:urn:mpeg:mpeg7:schema:2004:durationType, C:Pattern 
//- N; mediaTimeBase,  T:urn:mpeg:mpeg7:schema:2004:xPathRefType, C:Pattern 
//- N; mediaTimeUnit,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
//- N; complete,  T:http://www.w3.org/2001/XMLSchema:boolean, C:Value 
//- N; nonRedundant,  T:http://www.w3.org/2001/XMLSchema:boolean, C:Value 
//- N; setProperty,  T:urn:mpeg:mpeg7:schema:2004:ViewSetType_setProperty_LocalType2, C:Class 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("setProperty")) == 0)
	{
		// setProperty is simple attribute ViewSetType_setProperty_LocalType2
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = GetsetProperty()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ViewSetType_setProperty_LocalType2")))) != empty)
			{
				// Is type allowed
				Mp7JrsViewSetType_setProperty_Local2Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetsetProperty(p, client);
					if((p = GetsetProperty()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("View")) == 0)
	{
		// View is contained in itemtype ViewSetType_LocalType
		// in choice collection ViewSetType_CollectionType
		
		context->Found = true;
		Mp7JrsViewSetType_CollectionPtr coll = GetViewSetType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateViewSetType_CollectionType;
				SetViewSetType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsViewSetType_LocalPtr)coll->elementAt(i))->GetView()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsViewSetType_LocalPtr item = CreateViewSetType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsViewPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsViewSetType_LocalPtr)coll->elementAt(i))->SetView(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsViewSetType_LocalPtr)coll->elementAt(i))->GetView()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ViewRef")) == 0)
	{
		// ViewRef is contained in itemtype ViewSetType_LocalType
		// in choice collection ViewSetType_CollectionType
		
		context->Found = true;
		Mp7JrsViewSetType_CollectionPtr coll = GetViewSetType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateViewSetType_CollectionType;
				SetViewSetType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsViewSetType_LocalPtr)coll->elementAt(i))->GetViewRef()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsViewSetType_LocalPtr item = CreateViewSetType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsViewSetType_LocalPtr)coll->elementAt(i))->SetViewRef(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsViewSetType_LocalPtr)coll->elementAt(i))->GetViewRef()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ViewSetType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ViewSetType");
	}
	return result;
}

XMLCh * Mp7JrsViewSetType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsViewSetType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsViewSetType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsViewSetType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ViewSetType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_setProperty_Exist)
	{
	// Class
	if (m_setProperty != Mp7JrsViewSetType_setProperty_Local2Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_setProperty->ToText();
		element->setAttributeNS(X(""), X("setProperty"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_ViewSetType_LocalType != Mp7JrsViewSetType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_ViewSetType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsViewSetType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("setProperty")))
	{
		// Deserialize class type
		Mp7JrsViewSetType_setProperty_Local2Ptr tmp = CreateViewSetType_setProperty_LocalType2;
		tmp->Parse(parent->getAttribute(X("setProperty")));
		this->SetsetProperty(tmp);
		* current = parent;
	}

	// Extensionbase is Mp7JrsViewDecompositionType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialise Choice Collection
		if((parent != NULL) && (
			// urn:mpeg:mpeg7:schema:2004:View
			Mp7JrsUtil::HasNodeName(parent, X("View"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD			(XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:View")) == 0)
				||
			// urn:mpeg:mpeg7:schema:2004:ViewRef
			Mp7JrsUtil::HasNodeName(parent, X("ViewRef"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD			(XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:ViewRef")) == 0)
				))
		{
			// Deserialize factory type
			Mp7JrsViewSetType_CollectionPtr tmp = CreateViewSetType_CollectionType;
			tmp->Deserialize(doc, parent, current);
			this->SetViewSetType_LocalType(tmp);
			found = true;
			parent = * current; // For collections
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsViewSetType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsViewSetType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  // Choice Collection
  if(
// TODO better use this	
//  (Mp7JrsUtil::HasNodeName(parent, X("View"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
	(XMLString::compareString(elementname, X("View")) == 0)
	||
// TODO better use this	
//  (Mp7JrsUtil::HasNodeName(parent, X("ViewRef"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
	(XMLString::compareString(elementname, X("ViewRef")) == 0)
	)
  {
	Mp7JrsViewSetType_CollectionPtr tmp = CreateViewSetType_CollectionType;
	this->SetViewSetType_LocalType(tmp);
	child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsViewSetType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetViewSetType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetViewSetType_LocalType());
	if (GetSource() != Mp7JrsNodePtr())
		result.Insert(GetSource());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsViewSetType_ExtMethodImpl.h


