 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsVisualSummaryComponentType_ExtImplInclude.h



#include "Mp7JrsDType.h"
#include "Mp7JrsUniqueIDType.h"
#include "Mp7JrsTemporalSegmentLocatorType.h"
#include "Mp7JrsMediaTimeType.h"
#include "Mp7JrsImageLocatorType.h"
#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsRegionLocatorType.h"
#include "Mp7JrsVisualSummaryComponentType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsVisualSummaryComponentType::IMp7JrsVisualSummaryComponentType()
{

// no includefile for extension defined 
// file Mp7JrsVisualSummaryComponentType_ExtPropInit.h

}

IMp7JrsVisualSummaryComponentType::~IMp7JrsVisualSummaryComponentType()
{
// no includefile for extension defined 
// file Mp7JrsVisualSummaryComponentType_ExtPropCleanup.h

}

Mp7JrsVisualSummaryComponentType::Mp7JrsVisualSummaryComponentType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsVisualSummaryComponentType::~Mp7JrsVisualSummaryComponentType()
{
	Cleanup();
}

void Mp7JrsVisualSummaryComponentType::Init()
{
	// Init base
	m_Base = CreateDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_VideoSourceID = Mp7JrsUniqueIDPtr(); // Class
	m_VideoSourceID_Exist = false;
	m_VideoSourceLocator = Mp7JrsTemporalSegmentLocatorPtr(); // Class
	m_VideoSourceLocator_Exist = false;
	m_ComponentSourceTime = Mp7JrsMediaTimePtr(); // Class
	m_ComponentSourceTime_Exist = false;
	m_ImageLocator = Mp7JrsImageLocatorPtr(); // Class
	m_ImageLocator_Exist = false;
	m_SyncTime = Mp7JrsMediaTimePtr(); // Class
	m_SyncTime_Exist = false;
	m_FrameActivity = Mp7JrszeroToOnePtr(); // Class with content 
	m_FrameActivity_Exist = false;
	m_Region = Mp7JrsRegionLocatorPtr(); // Class
	m_Region_Exist = false;


// no includefile for extension defined 
// file Mp7JrsVisualSummaryComponentType_ExtMyPropInit.h

}

void Mp7JrsVisualSummaryComponentType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsVisualSummaryComponentType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_VideoSourceID);
	// Mp7JrsFactory::DeleteObject(m_VideoSourceLocator);
	// Mp7JrsFactory::DeleteObject(m_ComponentSourceTime);
	// Mp7JrsFactory::DeleteObject(m_ImageLocator);
	// Mp7JrsFactory::DeleteObject(m_SyncTime);
	// Mp7JrsFactory::DeleteObject(m_FrameActivity);
	// Mp7JrsFactory::DeleteObject(m_Region);
}

void Mp7JrsVisualSummaryComponentType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use VisualSummaryComponentTypePtr, since we
	// might need GetBase(), which isn't defined in IVisualSummaryComponentType
	const Mp7JrsPtr< Mp7JrsVisualSummaryComponentType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsVisualSummaryComponentType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidVideoSourceID())
	{
		// Mp7JrsFactory::DeleteObject(m_VideoSourceID);
		this->SetVideoSourceID(Mp7JrsFactory::CloneObject(tmp->GetVideoSourceID()));
	}
	else
	{
		InvalidateVideoSourceID();
	}
	if (tmp->IsValidVideoSourceLocator())
	{
		// Mp7JrsFactory::DeleteObject(m_VideoSourceLocator);
		this->SetVideoSourceLocator(Mp7JrsFactory::CloneObject(tmp->GetVideoSourceLocator()));
	}
	else
	{
		InvalidateVideoSourceLocator();
	}
	if (tmp->IsValidComponentSourceTime())
	{
		// Mp7JrsFactory::DeleteObject(m_ComponentSourceTime);
		this->SetComponentSourceTime(Mp7JrsFactory::CloneObject(tmp->GetComponentSourceTime()));
	}
	else
	{
		InvalidateComponentSourceTime();
	}
	if (tmp->IsValidImageLocator())
	{
		// Mp7JrsFactory::DeleteObject(m_ImageLocator);
		this->SetImageLocator(Mp7JrsFactory::CloneObject(tmp->GetImageLocator()));
	}
	else
	{
		InvalidateImageLocator();
	}
	if (tmp->IsValidSyncTime())
	{
		// Mp7JrsFactory::DeleteObject(m_SyncTime);
		this->SetSyncTime(Mp7JrsFactory::CloneObject(tmp->GetSyncTime()));
	}
	else
	{
		InvalidateSyncTime();
	}
	if (tmp->IsValidFrameActivity())
	{
		// Mp7JrsFactory::DeleteObject(m_FrameActivity);
		this->SetFrameActivity(Mp7JrsFactory::CloneObject(tmp->GetFrameActivity()));
	}
	else
	{
		InvalidateFrameActivity();
	}
	if (tmp->IsValidRegion())
	{
		// Mp7JrsFactory::DeleteObject(m_Region);
		this->SetRegion(Mp7JrsFactory::CloneObject(tmp->GetRegion()));
	}
	else
	{
		InvalidateRegion();
	}
}

Mp7JrsNodePtr Mp7JrsVisualSummaryComponentType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsVisualSummaryComponentType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDType > Mp7JrsVisualSummaryComponentType::GetBase() const
{
	return m_Base;
}

Mp7JrsUniqueIDPtr Mp7JrsVisualSummaryComponentType::GetVideoSourceID() const
{
		return m_VideoSourceID;
}

// Element is optional
bool Mp7JrsVisualSummaryComponentType::IsValidVideoSourceID() const
{
	return m_VideoSourceID_Exist;
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsVisualSummaryComponentType::GetVideoSourceLocator() const
{
		return m_VideoSourceLocator;
}

// Element is optional
bool Mp7JrsVisualSummaryComponentType::IsValidVideoSourceLocator() const
{
	return m_VideoSourceLocator_Exist;
}

Mp7JrsMediaTimePtr Mp7JrsVisualSummaryComponentType::GetComponentSourceTime() const
{
		return m_ComponentSourceTime;
}

// Element is optional
bool Mp7JrsVisualSummaryComponentType::IsValidComponentSourceTime() const
{
	return m_ComponentSourceTime_Exist;
}

Mp7JrsImageLocatorPtr Mp7JrsVisualSummaryComponentType::GetImageLocator() const
{
		return m_ImageLocator;
}

// Element is optional
bool Mp7JrsVisualSummaryComponentType::IsValidImageLocator() const
{
	return m_ImageLocator_Exist;
}

Mp7JrsMediaTimePtr Mp7JrsVisualSummaryComponentType::GetSyncTime() const
{
		return m_SyncTime;
}

// Element is optional
bool Mp7JrsVisualSummaryComponentType::IsValidSyncTime() const
{
	return m_SyncTime_Exist;
}

Mp7JrszeroToOnePtr Mp7JrsVisualSummaryComponentType::GetFrameActivity() const
{
		return m_FrameActivity;
}

// Element is optional
bool Mp7JrsVisualSummaryComponentType::IsValidFrameActivity() const
{
	return m_FrameActivity_Exist;
}

Mp7JrsRegionLocatorPtr Mp7JrsVisualSummaryComponentType::GetRegion() const
{
		return m_Region;
}

// Element is optional
bool Mp7JrsVisualSummaryComponentType::IsValidRegion() const
{
	return m_Region_Exist;
}

void Mp7JrsVisualSummaryComponentType::SetVideoSourceID(const Mp7JrsUniqueIDPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVisualSummaryComponentType::SetVideoSourceID().");
	}
	if (m_VideoSourceID != item || m_VideoSourceID_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_VideoSourceID);
		m_VideoSourceID = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VideoSourceID) m_VideoSourceID->SetParent(m_myself.getPointer());
		if (m_VideoSourceID && m_VideoSourceID->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_VideoSourceID->UseTypeAttribute = true;
		}
		m_VideoSourceID_Exist = true;
		if(m_VideoSourceID != Mp7JrsUniqueIDPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VideoSourceID->SetContentName(XMLString::transcode("VideoSourceID"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVisualSummaryComponentType::InvalidateVideoSourceID()
{
	m_VideoSourceID_Exist = false;
}
void Mp7JrsVisualSummaryComponentType::SetVideoSourceLocator(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVisualSummaryComponentType::SetVideoSourceLocator().");
	}
	if (m_VideoSourceLocator != item || m_VideoSourceLocator_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_VideoSourceLocator);
		m_VideoSourceLocator = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VideoSourceLocator) m_VideoSourceLocator->SetParent(m_myself.getPointer());
		if (m_VideoSourceLocator && m_VideoSourceLocator->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_VideoSourceLocator->UseTypeAttribute = true;
		}
		m_VideoSourceLocator_Exist = true;
		if(m_VideoSourceLocator != Mp7JrsTemporalSegmentLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VideoSourceLocator->SetContentName(XMLString::transcode("VideoSourceLocator"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVisualSummaryComponentType::InvalidateVideoSourceLocator()
{
	m_VideoSourceLocator_Exist = false;
}
void Mp7JrsVisualSummaryComponentType::SetComponentSourceTime(const Mp7JrsMediaTimePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVisualSummaryComponentType::SetComponentSourceTime().");
	}
	if (m_ComponentSourceTime != item || m_ComponentSourceTime_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ComponentSourceTime);
		m_ComponentSourceTime = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ComponentSourceTime) m_ComponentSourceTime->SetParent(m_myself.getPointer());
		if (m_ComponentSourceTime && m_ComponentSourceTime->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTimeType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ComponentSourceTime->UseTypeAttribute = true;
		}
		m_ComponentSourceTime_Exist = true;
		if(m_ComponentSourceTime != Mp7JrsMediaTimePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ComponentSourceTime->SetContentName(XMLString::transcode("ComponentSourceTime"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVisualSummaryComponentType::InvalidateComponentSourceTime()
{
	m_ComponentSourceTime_Exist = false;
}
void Mp7JrsVisualSummaryComponentType::SetImageLocator(const Mp7JrsImageLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVisualSummaryComponentType::SetImageLocator().");
	}
	if (m_ImageLocator != item || m_ImageLocator_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ImageLocator);
		m_ImageLocator = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ImageLocator) m_ImageLocator->SetParent(m_myself.getPointer());
		if (m_ImageLocator && m_ImageLocator->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ImageLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ImageLocator->UseTypeAttribute = true;
		}
		m_ImageLocator_Exist = true;
		if(m_ImageLocator != Mp7JrsImageLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ImageLocator->SetContentName(XMLString::transcode("ImageLocator"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVisualSummaryComponentType::InvalidateImageLocator()
{
	m_ImageLocator_Exist = false;
}
void Mp7JrsVisualSummaryComponentType::SetSyncTime(const Mp7JrsMediaTimePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVisualSummaryComponentType::SetSyncTime().");
	}
	if (m_SyncTime != item || m_SyncTime_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SyncTime);
		m_SyncTime = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SyncTime) m_SyncTime->SetParent(m_myself.getPointer());
		if (m_SyncTime && m_SyncTime->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTimeType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SyncTime->UseTypeAttribute = true;
		}
		m_SyncTime_Exist = true;
		if(m_SyncTime != Mp7JrsMediaTimePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SyncTime->SetContentName(XMLString::transcode("SyncTime"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVisualSummaryComponentType::InvalidateSyncTime()
{
	m_SyncTime_Exist = false;
}
void Mp7JrsVisualSummaryComponentType::SetFrameActivity(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVisualSummaryComponentType::SetFrameActivity().");
	}
	if (m_FrameActivity != item || m_FrameActivity_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_FrameActivity);
		m_FrameActivity = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FrameActivity) m_FrameActivity->SetParent(m_myself.getPointer());
		if (m_FrameActivity && m_FrameActivity->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:zeroToOneType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FrameActivity->UseTypeAttribute = true;
		}
		m_FrameActivity_Exist = true;
		if(m_FrameActivity != Mp7JrszeroToOnePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FrameActivity->SetContentName(XMLString::transcode("FrameActivity"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVisualSummaryComponentType::InvalidateFrameActivity()
{
	m_FrameActivity_Exist = false;
}
void Mp7JrsVisualSummaryComponentType::SetRegion(const Mp7JrsRegionLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVisualSummaryComponentType::SetRegion().");
	}
	if (m_Region != item || m_Region_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Region);
		m_Region = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Region) m_Region->SetParent(m_myself.getPointer());
		if (m_Region && m_Region->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RegionLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Region->UseTypeAttribute = true;
		}
		m_Region_Exist = true;
		if(m_Region != Mp7JrsRegionLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Region->SetContentName(XMLString::transcode("Region"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVisualSummaryComponentType::InvalidateRegion()
{
	m_Region_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsVisualSummaryComponentType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("VideoSourceID")) == 0)
	{
		// VideoSourceID is simple element UniqueIDType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetVideoSourceID()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UniqueIDType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUniqueIDPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetVideoSourceID(p, client);
					if((p = GetVideoSourceID()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("VideoSourceLocator")) == 0)
	{
		// VideoSourceLocator is simple element TemporalSegmentLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetVideoSourceLocator()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetVideoSourceLocator(p, client);
					if((p = GetVideoSourceLocator()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ComponentSourceTime")) == 0)
	{
		// ComponentSourceTime is simple element MediaTimeType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetComponentSourceTime()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTimeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaTimePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetComponentSourceTime(p, client);
					if((p = GetComponentSourceTime()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ImageLocator")) == 0)
	{
		// ImageLocator is simple element ImageLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetImageLocator()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ImageLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsImageLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetImageLocator(p, client);
					if((p = GetImageLocator()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SyncTime")) == 0)
	{
		// SyncTime is simple element MediaTimeType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSyncTime()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:MediaTimeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsMediaTimePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSyncTime(p, client);
					if((p = GetSyncTime()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("FrameActivity")) == 0)
	{
		// FrameActivity is simple element zeroToOneType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFrameActivity()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:zeroToOneType")))) != empty)
			{
				// Is type allowed
				Mp7JrszeroToOnePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFrameActivity(p, client);
					if((p = GetFrameActivity()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	// Element ------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Region")) == 0)
	{
		// Region is simple element RegionLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRegion()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RegionLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRegionLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRegion(p, client);
					if((p = GetRegion()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for VisualSummaryComponentType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "VisualSummaryComponentType");
	}
	return result;
}

XMLCh * Mp7JrsVisualSummaryComponentType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsVisualSummaryComponentType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsVisualSummaryComponentType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsVisualSummaryComponentType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("VisualSummaryComponentType"));
	// Element serialization:
	// Class
	
	if (m_VideoSourceID != Mp7JrsUniqueIDPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_VideoSourceID->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("VideoSourceID"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_VideoSourceID->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_VideoSourceLocator != Mp7JrsTemporalSegmentLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_VideoSourceLocator->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("VideoSourceLocator"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_VideoSourceLocator->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ComponentSourceTime != Mp7JrsMediaTimePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ComponentSourceTime->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ComponentSourceTime"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ComponentSourceTime->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ImageLocator != Mp7JrsImageLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ImageLocator->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ImageLocator"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ImageLocator->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_SyncTime != Mp7JrsMediaTimePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SyncTime->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SyncTime"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SyncTime->Serialize(doc, element, &elem);
	}
	// Class with content		
	
	if (m_FrameActivity != Mp7JrszeroToOnePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FrameActivity->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FrameActivity"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FrameActivity->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Region != Mp7JrsRegionLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Region->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Region"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Region->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsVisualSummaryComponentType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("VideoSourceID"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("VideoSourceID")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateUniqueIDType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetVideoSourceID(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("VideoSourceLocator"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("VideoSourceLocator")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateTemporalSegmentLocatorType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetVideoSourceLocator(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ComponentSourceTime"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ComponentSourceTime")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMediaTimeType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetComponentSourceTime(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ImageLocator"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ImageLocator")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateImageLocatorType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetImageLocator(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SyncTime"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SyncTime")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateMediaTimeType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetSyncTime(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FrameActivity"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FrameActivity")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreatezeroToOneType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetFrameActivity(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
		// Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Region"), X("urn:mpeg:mpeg7:schema:2004")))
//		if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Region")) == 0))
		{
			// We have to deal with xsi:type classes...
			// Deserialize factory type
			Mp7JrsPtr< Mp7JrsNode > tmp;
			if(parent->hasAttribute(X("xsi:type")))
			{
			  NamespaceHelper result;
			  Mp7JrsUtil::GetXsiType(parent, doc, &result);
			  XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//			  tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
			  if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
			}else 
			{
			  tmp = CreateRegionLocatorType;
			}
			tmp->Deserialize(doc, parent, current);
			
			if(parent->hasAttribute(X("xsi:type"))) {
					tmp->UseTypeAttribute = true;
			}
			this->SetRegion(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsVisualSummaryComponentType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsVisualSummaryComponentType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  if (XMLString::compareString(elementname, X("VideoSourceID")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateUniqueIDType;
	}
	this->SetVideoSourceID(child);
  }
  if (XMLString::compareString(elementname, X("VideoSourceLocator")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateTemporalSegmentLocatorType;
	}
	this->SetVideoSourceLocator(child);
  }
  if (XMLString::compareString(elementname, X("ComponentSourceTime")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMediaTimeType;
	}
	this->SetComponentSourceTime(child);
  }
  if (XMLString::compareString(elementname, X("ImageLocator")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateImageLocatorType;
	}
	this->SetImageLocator(child);
  }
  if (XMLString::compareString(elementname, X("SyncTime")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateMediaTimeType;
	}
	this->SetSyncTime(child);
  }
  if (XMLString::compareString(elementname, X("FrameActivity")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreatezeroToOneType;
	}
	this->SetFrameActivity(child);
  }
  if (XMLString::compareString(elementname, X("Region")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreateRegionLocatorType;
	}
	this->SetRegion(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsVisualSummaryComponentType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetVideoSourceID() != Mp7JrsNodePtr())
		result.Insert(GetVideoSourceID());
	if (GetVideoSourceLocator() != Mp7JrsNodePtr())
		result.Insert(GetVideoSourceLocator());
	if (GetComponentSourceTime() != Mp7JrsNodePtr())
		result.Insert(GetComponentSourceTime());
	if (GetImageLocator() != Mp7JrsNodePtr())
		result.Insert(GetImageLocator());
	if (GetSyncTime() != Mp7JrsNodePtr())
		result.Insert(GetSyncTime());
	if (GetFrameActivity() != Mp7JrsNodePtr())
		result.Insert(GetFrameActivity());
	if (GetRegion() != Mp7JrsNodePtr())
		result.Insert(GetRegion());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsVisualSummaryComponentType_ExtMethodImpl.h


