 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsVisualTimeSeriesType_ExtImplInclude.h



#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsVisualTimeSeriesType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsVisualTimeSeriesType::IMp7JrsVisualTimeSeriesType()
{

// no includefile for extension defined 
// file Mp7JrsVisualTimeSeriesType_ExtPropInit.h

}

IMp7JrsVisualTimeSeriesType::~IMp7JrsVisualTimeSeriesType()
{
// no includefile for extension defined 
// file Mp7JrsVisualTimeSeriesType_ExtPropCleanup.h

}

Mp7JrsVisualTimeSeriesType::Mp7JrsVisualTimeSeriesType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsVisualTimeSeriesType::~Mp7JrsVisualTimeSeriesType()
{
	Cleanup();
}

void Mp7JrsVisualTimeSeriesType::Init()
{

	// Init attributes
	m_offset = Mp7JrsmediaDurationPtr(); // Pattern
	m_offset_Default = CreatemediaDurationType; // Default pattern
	m_offset_Default->Parse(X("PT0S"));
	m_offset_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_TimeIncr = Mp7JrsmediaDurationPtr(); // Pattern


// no includefile for extension defined 
// file Mp7JrsVisualTimeSeriesType_ExtMyPropInit.h

}

void Mp7JrsVisualTimeSeriesType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsVisualTimeSeriesType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_offset); // Pattern
	// Mp7JrsFactory::DeleteObject(m_TimeIncr);
}

void Mp7JrsVisualTimeSeriesType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use VisualTimeSeriesTypePtr, since we
	// might need GetBase(), which isn't defined in IVisualTimeSeriesType
	const Mp7JrsPtr< Mp7JrsVisualTimeSeriesType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	// Mp7JrsFactory::DeleteObject(m_offset); // Pattern
	if (tmp->Existoffset())
	{
		this->Setoffset(Mp7JrsFactory::CloneObject(tmp->Getoffset()));
	}
	else
	{
		Invalidateoffset();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_TimeIncr);
		this->SetTimeIncr(Mp7JrsFactory::CloneObject(tmp->GetTimeIncr()));
}

Mp7JrsNodePtr Mp7JrsVisualTimeSeriesType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsVisualTimeSeriesType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsmediaDurationPtr Mp7JrsVisualTimeSeriesType::Getoffset() const
{
	if (this->Existoffset()) {
		return m_offset;
	} else {
		return m_offset_Default;
	}
}

bool Mp7JrsVisualTimeSeriesType::Existoffset() const
{
	return m_offset_Exist;
}
void Mp7JrsVisualTimeSeriesType::Setoffset(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVisualTimeSeriesType::Setoffset().");
	}
	m_offset = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_offset) m_offset->SetParent(m_myself.getPointer());
	m_offset_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsVisualTimeSeriesType::Invalidateoffset()
{
	m_offset_Exist = false;
}
Mp7JrsmediaDurationPtr Mp7JrsVisualTimeSeriesType::GetTimeIncr() const
{
		return m_TimeIncr;
}

void Mp7JrsVisualTimeSeriesType::SetTimeIncr(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsVisualTimeSeriesType::SetTimeIncr().");
	}
	if (m_TimeIncr != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TimeIncr);
		m_TimeIncr = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TimeIncr) m_TimeIncr->SetParent(m_myself.getPointer());
		if (m_TimeIncr && m_TimeIncr->GetClassId() 
	!= Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaDurationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TimeIncr->UseTypeAttribute = true;
		}
		if(m_TimeIncr != Mp7JrsmediaDurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TimeIncr->SetContentName(XMLString::transcode("TimeIncr"));
		}
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsVisualTimeSeriesType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  1, 1 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
//-A*T*T*R*I*B*S: 1
//- N; offset,  T:urn:mpeg:mpeg7:schema:2004:mediaDurationType, C:Pattern 
	// Attribute ------------------------------------------------------------------
	if(context->IsAttribute && XMLString::compareString(context->ElementName, X("offset")) == 0)
	{
		// offset is simple attribute mediaDurationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleAttribute(context);

		Mp7JrsNodePtr p;
		if((p = Getoffset()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleAttribute(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaDurationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsmediaDurationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					Setoffset(p, client);
					if((p = Getoffset()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetAttribute(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
	}
	// Element ------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("TimeIncr")) == 0)
	{
		// TimeIncr is simple element mediaDurationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTimeIncr()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaDurationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsmediaDurationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTimeIncr(p, client);
					if((p = GetTimeIncr()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for VisualTimeSeriesType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "VisualTimeSeriesType");
	}
	return result;
}

XMLCh * Mp7JrsVisualTimeSeriesType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsVisualTimeSeriesType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsVisualTimeSeriesType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("VisualTimeSeriesType"));
	// Attribute Serialization:
		

	// Optional attriute
	if(m_offset_Exist)
	{
	// Pattern
	if (m_offset != Mp7JrsmediaDurationPtr())
	{
		XMLCh * tmp = m_offset->ToText();
		element->setAttributeNS(X(""), X("offset"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if(m_TimeIncr != Mp7JrsmediaDurationPtr()) // Pattern FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Pattern
		// CON contentname already set in element setter -- not needed here
//		XMLCh * contentname = XMLString::transcode("TimeIncr");
//		m_TimeIncr->SetContentName(contentname);
		m_TimeIncr->UseTypeAttribute = this->UseTypeAttribute;
		m_TimeIncr->Serialize(doc, element);
	}

}

bool Mp7JrsVisualTimeSeriesType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
	if(parent->hasAttribute(X("offset")))
	{
		Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
		tmp->Parse(parent->getAttribute(X("offset")));
		this->Setoffset(tmp);
		* current = parent;
	}

	parent = Mp7JrsUtil::GetNextChild(parent);
		// Deserialize pattern element
	if(Mp7JrsUtil::HasNodeName(parent, X("TimeIncr"), X("urn:mpeg:mpeg7:schema:2004")))
//			if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TimeIncr")) == 0))
		{
			Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
			tmp->Deserialize(doc, parent, current);
			this->SetTimeIncr(tmp);
			found = true;
			* current = parent; // We must adjust it!
			parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
		}
// no includefile for extension defined 
// file Mp7JrsVisualTimeSeriesType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsVisualTimeSeriesType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("TimeIncr")) == 0) {
	// We check for xsi:type
	if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
	  child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
	} else { // no xsi:type
	  child = CreatemediaDurationType;
	}
	this->SetTimeIncr(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsVisualTimeSeriesType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTimeIncr() != Mp7JrsNodePtr())
		result.Insert(GetTimeIncr());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsVisualTimeSeriesType_ExtMethodImpl.h


