
// Based on PatternType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include <stdio.h>
#include <xercesc/util/regx/RegularExpression.hpp>
#include <xercesc/util/regx/Match.hpp>
#include <assert.h>
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrsbasicDurationType_ExtImplInclude.h


#include "Mp7JrsbasicDurationType.h"

Mp7JrsbasicDurationType::Mp7JrsbasicDurationType()
	: m_nsURI(X("urn:mpeg:mpeg7:schema:2004"))
{
		Init();
}

Mp7JrsbasicDurationType::~Mp7JrsbasicDurationType()
{
		Cleanup();
}

void Mp7JrsbasicDurationType::Init()
{	   
		m_DurationSign = 1;
		m_Days = -1;
		m_Hours = -1;
		m_Minutes = -1;
		m_Seconds = -1;
		m_NumberOfFractions = -1;
		m_DecimalFractions = -1;
		m_Time_Exist = false;
		m_FractionsPerSecond = -1;
		m_TimezoneSign = 1;
		m_TimezoneHours = 0;
		m_TimezoneMinutes = 0;
		m_Timezone_Exist = false;

// no includefile for extension defined 
// file Mp7JrsbasicDurationType_ExtPropInit.h

}

void Mp7JrsbasicDurationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsbasicDurationType_ExtPropCleanup.h


}

void Mp7JrsbasicDurationType::DeepCopy(const Mp7JrsNodePtr &original)
{
		Mp7JrsPtr< Mp7JrsbasicDurationType > tmp = original;
		if (tmp == NULL) return; // EXIT: wrong type passed
		Cleanup();
		// FIXME: this will fail for derived classes
  
		this->Parse(tmp->ToText());
		
		this->m_ContentName = NULL;
}

Mp7JrsNodePtr Mp7JrsbasicDurationType::GetBaseRoot()
{
		return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsbasicDurationType::GetBaseRoot() const
{
		return m_myself.getPointer();
}

int Mp7JrsbasicDurationType::GetDurationSign() const
{
		return m_DurationSign;
}

void Mp7JrsbasicDurationType::SetDurationSign(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetDurationSign().");
		}
		m_DurationSign = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsbasicDurationType::GetDays() const
{
		return m_Days;
}

void Mp7JrsbasicDurationType::SetDays(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetDays().");
		}
		m_Days = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsbasicDurationType::GetHours() const
{
		return m_Hours;
}

void Mp7JrsbasicDurationType::SetHours(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetHours().");
		}
		m_Hours = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsbasicDurationType::GetMinutes() const
{
		return m_Minutes;
}

void Mp7JrsbasicDurationType::SetMinutes(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetMinutes().");
		}
		m_Minutes = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsbasicDurationType::GetSeconds() const
{
		return m_Seconds;
}

void Mp7JrsbasicDurationType::SetSeconds(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetSeconds().");
		}
		m_Seconds = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsbasicDurationType::GetNumberOfFractions() const
{
		return m_NumberOfFractions;
}

void Mp7JrsbasicDurationType::SetNumberOfFractions(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetNumberOfFractions().");
		}
		m_NumberOfFractions = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsbasicDurationType::GetDecimalFractions() const
{
		return m_DecimalFractions;
}

void Mp7JrsbasicDurationType::SetDecimalFractions(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetDecimalFractions().");
		}
		m_DecimalFractions = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
bool Mp7JrsbasicDurationType::GetTime_Exist() const
{
		return m_Time_Exist;
}

void Mp7JrsbasicDurationType::SetTime_Exist(bool val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetTime_Exist().");
		}
		m_Time_Exist = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsbasicDurationType::GetFractionsPerSecond() const
{
		return m_FractionsPerSecond;
}

void Mp7JrsbasicDurationType::SetFractionsPerSecond(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetFractionsPerSecond().");
		}
		m_FractionsPerSecond = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsbasicDurationType::GetTimezoneSign() const
{
		return m_TimezoneSign;
}

void Mp7JrsbasicDurationType::SetTimezoneSign(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetTimezoneSign().");
		}
		m_TimezoneSign = val;
		m_Timezone_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
unsigned int Mp7JrsbasicDurationType::GetTimezoneHours() const
{
		return m_TimezoneHours;
}

void Mp7JrsbasicDurationType::SetTimezoneHours(unsigned int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetTimezoneHours().");
		}
		m_TimezoneHours = val;
		m_Timezone_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
unsigned int Mp7JrsbasicDurationType::GetTimezoneMinutes() const
{
		return m_TimezoneMinutes;
}

void Mp7JrsbasicDurationType::SetTimezoneMinutes(unsigned int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetTimezoneMinutes().");
		}
		m_TimezoneMinutes = val;
		m_Timezone_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
bool Mp7JrsbasicDurationType::GetTimezone_Exist() const
{
		return m_Timezone_Exist;
}

void Mp7JrsbasicDurationType::SetTimezone_Exist(bool val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsbasicDurationType::SetTimezone_Exist().");
		}
		m_Timezone_Exist = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}


XMLCh * Mp7JrsbasicDurationType::ToText() const
{
		XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
		XMLCh * tmp = NULL;
		
		if(m_DurationSign < 0)
		{
				Mp7JrsUtil::CatString(&buf, X("-"));
		}

		if(m_Days >= 0)
		{
			tmp = Mp7JrsConvert::BinToText(m_Days);
			Mp7JrsUtil::CatString(&buf, tmp);
			Mp7JrsUtil::CatString(&buf, X("D"));
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
		}

		if(m_Time_Exist)
		{
			Mp7JrsUtil::CatString(&buf, X("T"));

			if(m_Hours >= 0)
			{
				tmp = Mp7JrsConvert::BinToText(m_Hours);
				Mp7JrsUtil::CatString(&buf, tmp);
				Mp7JrsUtil::CatString(&buf, X("H"));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_Minutes >= 0)
			{
				tmp = Mp7JrsConvert::BinToText(m_Minutes);
				Mp7JrsUtil::CatString(&buf, tmp);
				Mp7JrsUtil::CatString(&buf, X("M"));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_Seconds >= 0)
			{
				tmp = Mp7JrsConvert::BinToText(m_Seconds);
				Mp7JrsUtil::CatString(&buf, tmp);
				Mp7JrsUtil::CatString(&buf, X("S"));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_NumberOfFractions >= 0)
			{
				tmp = Mp7JrsConvert::BinToText(m_NumberOfFractions);
				Mp7JrsUtil::CatString(&buf, tmp);
				Mp7JrsUtil::CatString(&buf, X("N"));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_DecimalFractions >= 0)
			{
				XMLCh* tmp1 = Mp7JrsConvert::BinToText(m_DecimalFractions);
				tmp = Mp7JrsUtil::PadLeft(tmp1, X("0"), 2);
				Mp7JrsUtil::CatString(&buf, tmp);
				Mp7JrsUtil::CatString(&buf, X("f"));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp1);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}
		}

		if(m_FractionsPerSecond >= 0)
		{
			tmp = Mp7JrsConvert::BinToText(m_FractionsPerSecond);
			Mp7JrsUtil::CatString(&buf, tmp);
			Mp7JrsUtil::CatString(&buf, X("F"));
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
		}

		if(m_Timezone_Exist)
		{
			if(m_TimezoneSign > 0)
			{
				Mp7JrsUtil::CatString(&buf, X("+"));
			}
			else if(m_TimezoneSign < 0)
			{
				Mp7JrsUtil::CatString(&buf, X("-"));
			}
			XMLCh* tmp1 = Mp7JrsConvert::BinToText(m_TimezoneHours);
			tmp = Mp7JrsUtil::PadLeft(tmp1, X("0"), 2);
			Mp7JrsUtil::CatString(&buf, tmp);
			Mp7JrsUtil::CatString(&buf, X(":"));
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp1);
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			tmp1 = Mp7JrsConvert::BinToText(m_TimezoneMinutes);
			tmp = Mp7JrsUtil::PadLeft(tmp1, X("0"), 2);
			Mp7JrsUtil::CatString(&buf, tmp);
			Mp7JrsUtil::CatString(&buf, X("Z"));
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp1);
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
		}
		return buf;
}


/////////////////////////////////////////////////////////////////

bool Mp7JrsbasicDurationType::Parse(const XMLCh * const txt)
{
		Match m;
		XMLCh substring [128] = {0};
		RegularExpression r("^\\-?P(\\d+D)?(T(\\d+H)?(\\d+M)?(\\d+S)?(\\d+N)?(\\d{2}f)?)?(\\d+F)?((\\-|\\+)\\d{2}:\\d{2}Z)?$");
		// This is for Xerces 3.1.1+
		if(r.matches(txt, &m))
		{
			for(int i = 0; i < m.getNoGroups(); i++)
			{
				int i1 = m.getStartPos(i);
				int i2 = Mp7JrsUtil::GetNextEndPos(m, i);
				if(i1 == i2) continue; // ignore missing optional or wrong group
				switch(i)
				{
					case 0:
						if(i1 >= 0 && i2 - i1 > 1)
						{
							// \\-?P
							m_DurationSign = -1;
						}
					break;

					case 1:
						if(i1 >= 0)
						{
							// (\\d+D)?
							XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
							m_Days = Mp7JrsConvert::TextToUnsignedInt(substring);
						}
					break;

					case 2:
						if(i1 >= 0)
						{
							// T
							// Nothing to do
						}
					break;

					case 3:
						if(i1 >= 0)
						{
							// (\\d+H)?
							XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
							m_Hours = Mp7JrsConvert::TextToUnsignedInt(substring);
							m_Time_Exist = true;
						}
					break;

					case 4:
						if(i1 >= 0)
						{
							// (\\d+M)?
							XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
							m_Minutes = Mp7JrsConvert::TextToUnsignedInt(substring);
							m_Time_Exist = true;
						}
					break;

					case 5:
						if(i1 >= 0)
						{
							// (\\d+S)?
							XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
							m_Seconds = Mp7JrsConvert::TextToUnsignedInt(substring);
							m_Time_Exist = true;
						}
					break;

					case 6:
						if(i1 >= 0)
						{
							// (\\d+N)?
							XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
							m_NumberOfFractions = Mp7JrsConvert::TextToUnsignedInt(substring);
							m_Time_Exist = true;
						}
					break;

					case 7:
						if(i1 >= 0)
						{
							// (\\d{2}+f)?
							XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
							m_DecimalFractions = Mp7JrsConvert::TextToUnsignedInt(substring);
							m_Time_Exist = true;
						}
					break;

					case 8:
						if(i1 >= 0)
						{
							// (\\d+F)?
							XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
							m_FractionsPerSecond = Mp7JrsConvert::TextToUnsignedInt(substring);
						}
					break;

					case 9:
						if(i1 >= 0)
						{
							// ((\\-|\\+)\\d{2}:\\d{2}Z)?
							XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i1 + 1);
							m_TimezoneSign = Mp7JrsConvert::TextToSign(substring);

							XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i1 + 3);
							m_TimezoneHours = Mp7JrsConvert::TextToUnsignedInt(substring);

							XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 4, i2 - 1);
							m_TimezoneMinutes = Mp7JrsConvert::TextToUnsignedInt(substring);

							m_Timezone_Exist = true;
						}
					break;

					default: return true;
					break;
				}
			}
		}
		else return false;
		return true;
}

bool Mp7JrsbasicDurationType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsbasicDurationType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsbasicDurationType::ContentToString() const
{
	// Mp7JrsbasicDurationType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrsbasicDurationType::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
		// The element we serialize into
		XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

		XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
		if (!newElem) newElem = &localNewElem;

		if (*newElem) {
				element = *newElem;
		} else if(parent == NULL) {
				element = doc->getDocumentElement();
				*newElem = element;
		} else {
				if(this->GetContentName() != (XMLCh*)NULL) {
//  OLD					  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
						DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
						parent->appendChild(elem);
						*newElem = elem;
						element = elem;
				} else
						assert(false);
		}
		assert(element);

		if(this->UseTypeAttribute && ! element->hasAttribute(X("xsi:type")))
		{
			element->setAttribute(X("xsi:type"), X(Mp7JrsFactory::GetTypeName(Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:basicDurationType")))));
		}

// no includefile for extension defined 
// file Mp7JrsbasicDurationType_ExtPreImplementCleanup.h


		XMLCh * buf = this->ToText();
		if(buf != NULL)
		{
				element->appendChild(doc->createTextNode(buf));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&buf);
		}
}

bool Mp7JrsbasicDurationType::Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent,  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current)
{
// no includefile for extension defined 
// file Mp7JrsbasicDurationType_ExtPostDeserialize.h


		bool found = this->Parse(Mp7JrsUtil::GetElementText(parent));
		if(found)
		{
				* current = parent;
		}
		return found;   
}

// no includefile for extension defined 
// file Mp7JrsbasicDurationType_ExtMethodImpl.h








