
// Based on PatternType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include <stdio.h>
#include <xercesc/util/regx/RegularExpression.hpp>
#include <xercesc/util/regx/Match.hpp>
#include <assert.h>
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrsdurationType_ExtImplInclude.h


#include "Mp7JrsdurationType.h"

Mp7JrsdurationType::Mp7JrsdurationType()
	: m_nsURI(X("urn:mpeg:mpeg7:schema:2004"))
{
		Init();
}

Mp7JrsdurationType::~Mp7JrsdurationType()
{
		Cleanup();
}

void Mp7JrsdurationType::Init()
{	   
		m_DurationSign = 1;
		m_Days = -1;
		m_Hours = -1;
		m_Minutes = -1;
		m_Seconds = -1;
		m_NumberOfFractions = -1;
		m_Time_Exist = false;
		m_FractionsPerSecond = -1;
		m_TimezoneSign = +1;
		m_TimezoneHours = -1;
		m_TimezoneMinutes = -1;
		m_Timezone_Exist = false;

// no includefile for extension defined 
// file Mp7JrsdurationType_ExtPropInit.h

}

void Mp7JrsdurationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsdurationType_ExtPropCleanup.h


}

void Mp7JrsdurationType::DeepCopy(const Mp7JrsNodePtr &original)
{
		Mp7JrsPtr< Mp7JrsdurationType > tmp = original;
		if (tmp == NULL) return; // EXIT: wrong type passed
		Cleanup();
		// FIXME: this will fail for derived classes
  
		this->Parse(tmp->ToText());
		
		this->m_ContentName = NULL;
}

Mp7JrsNodePtr Mp7JrsdurationType::GetBaseRoot()
{
		return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsdurationType::GetBaseRoot() const
{
		return m_myself.getPointer();
}

int Mp7JrsdurationType::GetDurationSign() const
{
		return m_DurationSign;
}

void Mp7JrsdurationType::SetDurationSign(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetDurationSign().");
		}
		m_DurationSign = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsdurationType::GetDays() const
{
		return m_Days;
}

void Mp7JrsdurationType::SetDays(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetDays().");
		}
		m_Days = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsdurationType::GetHours() const
{
		return m_Hours;
}

void Mp7JrsdurationType::SetHours(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetHours().");
		}
		m_Hours = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsdurationType::GetMinutes() const
{
		return m_Minutes;
}

void Mp7JrsdurationType::SetMinutes(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetMinutes().");
		}
		m_Minutes = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsdurationType::GetSeconds() const
{
		return m_Seconds;
}

void Mp7JrsdurationType::SetSeconds(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetSeconds().");
		}
		m_Seconds = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsdurationType::GetNumberOfFractions() const
{
		return m_NumberOfFractions;
}

void Mp7JrsdurationType::SetNumberOfFractions(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetNumberOfFractions().");
		}
		m_NumberOfFractions = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
bool Mp7JrsdurationType::GetTime_Exist() const
{
		return m_Time_Exist;
}

void Mp7JrsdurationType::SetTime_Exist(bool val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetTime_Exist().");
		}
		m_Time_Exist = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsdurationType::GetFractionsPerSecond() const
{
		return m_FractionsPerSecond;
}

void Mp7JrsdurationType::SetFractionsPerSecond(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetFractionsPerSecond().");
		}
		m_FractionsPerSecond = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsdurationType::GetTimezoneSign() const
{
		return m_TimezoneSign;
}

void Mp7JrsdurationType::SetTimezoneSign(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetTimezoneSign().");
		}
		m_TimezoneSign = val;
		m_Timezone_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsdurationType::GetTimezoneHours() const
{
		return m_TimezoneHours;
}

void Mp7JrsdurationType::SetTimezoneHours(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetTimezoneHours().");
		}
		m_TimezoneHours = val;
		m_Timezone_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsdurationType::GetTimezoneMinutes() const
{
		return m_TimezoneMinutes;
}

void Mp7JrsdurationType::SetTimezoneMinutes(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetTimezoneMinutes().");
		}
		m_TimezoneMinutes = val;
		m_Timezone_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
bool Mp7JrsdurationType::GetTimezone_Exist() const
{
		return m_Timezone_Exist;
}

void Mp7JrsdurationType::SetTimezone_Exist(bool val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsdurationType::SetTimezone_Exist().");
		}
		m_Timezone_Exist = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}


XMLCh * Mp7JrsdurationType::ToText() const
{
		XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
		XMLCh * tmp = NULL;
		
		if(m_DurationSign < 0)
		{
			Mp7JrsUtil::CatString(&buf, X("-"));
		}

		Mp7JrsUtil::CatString(&buf, X("P"));

		if(m_Days >= 0)
		{
			tmp = Mp7JrsConvert::BinToText(m_Days);
			Mp7JrsUtil::CatString(&buf, tmp);
			Mp7JrsUtil::CatString(&buf, X("D"));
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
		}

		if(m_Time_Exist)
		{
			Mp7JrsUtil::CatString(&buf, X("T"));

			if(m_Hours >= 0)
			{
				tmp = Mp7JrsConvert::BinToText(m_Hours);
				Mp7JrsUtil::CatString(&buf, tmp);
				Mp7JrsUtil::CatString(&buf, X("H"));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_Minutes >= 0)
			{
				tmp = Mp7JrsConvert::BinToText(m_Minutes);
				Mp7JrsUtil::CatString(&buf, tmp);
				Mp7JrsUtil::CatString(&buf, X("M"));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_Seconds >= 0)
			{
				tmp = Mp7JrsConvert::BinToText(m_Seconds);
				Mp7JrsUtil::CatString(&buf, tmp);
				Mp7JrsUtil::CatString(&buf, X("S"));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_NumberOfFractions >= 0)
			{
				tmp = Mp7JrsConvert::BinToText(m_NumberOfFractions);
				Mp7JrsUtil::CatString(&buf, tmp);
				Mp7JrsUtil::CatString(&buf, X("N"));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}
		}

		if(m_FractionsPerSecond >= 0)
		{
			tmp = Mp7JrsConvert::BinToText(m_FractionsPerSecond);
			Mp7JrsUtil::CatString(&buf, tmp);
			Mp7JrsUtil::CatString(&buf, X("F"));
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
		}

		if(m_Timezone_Exist)
		{
			if(m_TimezoneSign > 0)
			{
				Mp7JrsUtil::CatString(&buf, X("+"));
			}
			else if(m_TimezoneSign < 0)
			{
				Mp7JrsUtil::CatString(&buf, X("-"));
			}

			XMLCh* tmp1 = Mp7JrsConvert::BinToText(m_TimezoneHours);
			tmp = Mp7JrsUtil::PadLeft(tmp1, X("0"), 2);
			Mp7JrsUtil::CatString(&buf, tmp);
			Mp7JrsUtil::CatString(&buf, X(":"));
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp1);
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			tmp1 = Mp7JrsConvert::BinToText(m_TimezoneMinutes);
			tmp = Mp7JrsUtil::PadLeft(tmp1, X("0"), 2);
			Mp7JrsUtil::CatString(&buf, tmp);
			Mp7JrsUtil::CatString(&buf, X("Z"));
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp1);
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
		}
		return buf;
}


/////////////////////////////////////////////////////////////////

bool Mp7JrsdurationType::Parse(const XMLCh * const txt)
{
		Match m;
		XMLCh substring [128] = {0};
		RegularExpression r("^\\-?P(\\d+D)?(T(\\d+H)?(\\d+M)?(\\d+S)?(\\d+N)?)?(\\d+F)?((\\-|\\+)\\d{2}:\\d{2}Z)?$");
		// This is for Xerces 3.1.1+
		if(r.matches(txt, &m))
		{
				for(int i = 0; i < m.getNoGroups(); i++)
				{
						int i1 = m.getStartPos(i);
						int i2 = Mp7JrsUtil::GetNextEndPos(m, i);
						if(i1 == i2) continue; // ignore missing optional or wrong group
						switch(i)
						{
								case 0:
										if(i1 >= 0 && i2 - i1 > 1)
										{
												// \\-?P
												m_DurationSign = -1;
										}
								break;

								case 1:
										if(i1 >= 0)
										{
												// (\\d+D)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
												m_Days = Mp7JrsConvert::TextToUnsignedInt(substring);
										}
								break;

								case 2:
										if(i1 >= 0)
										{
												// T
												// Nothing to do
										}
								break;

								case 3:
										if(i1 >= 0)
										{
												// (\\d+H)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
												m_Hours = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Time_Exist = true;
										}
								break;

								case 4:
										if(i1 >= 0)
										{
												// (\\d+M)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
												m_Minutes = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Time_Exist = true;
										}
								break;

								case 5:
										if(i1 >= 0)
										{
												// (\\d+S)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
												m_Seconds = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Time_Exist = true;
										}
								break;

								case 6:
										if(i1 >= 0)
										{
												// (\\d+N)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
												m_NumberOfFractions = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Time_Exist = true;
										}
								break;

								case 7:
										if(i1 >= 0)
										{
												// (\\d+F)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
												m_FractionsPerSecond = Mp7JrsConvert::TextToUnsignedInt(substring);
										}
								break;

								case 8:
										if(i1 >= 0)
										{
												// ((\\-|\\+)\\d{2}:\\d{2}Z)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i1 + 1);
												m_TimezoneSign = Mp7JrsConvert::TextToSign(substring);

												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i1 + 3);
												m_TimezoneHours = Mp7JrsConvert::TextToUnsignedInt(substring);

												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 4, i2 - 1);
												m_TimezoneMinutes = Mp7JrsConvert::TextToUnsignedInt(substring);

												m_Timezone_Exist = true;
										}
								break;

								default: return true;
								break;
						}
				}
		}
		else return false;
		return true;
}

bool Mp7JrsdurationType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsdurationType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsdurationType::ContentToString() const
{
	// Mp7JrsdurationType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrsdurationType::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
		// The element we serialize into
		XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

		XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
		if (!newElem) newElem = &localNewElem;

		if (*newElem) {
				element = *newElem;
		} else if(parent == NULL) {
				element = doc->getDocumentElement();
				*newElem = element;
		} else {
				if(this->GetContentName() != (XMLCh*)NULL) {
//  OLD					  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
						DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
						parent->appendChild(elem);
						*newElem = elem;
						element = elem;
				} else
						assert(false);
		}
		assert(element);

		if(this->UseTypeAttribute && ! element->hasAttribute(X("xsi:type")))
		{
			element->setAttribute(X("xsi:type"), X(Mp7JrsFactory::GetTypeName(Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:durationType")))));
		}

// no includefile for extension defined 
// file Mp7JrsdurationType_ExtPreImplementCleanup.h


		XMLCh * buf = this->ToText();
		if(buf != NULL)
		{
				element->appendChild(doc->createTextNode(buf));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&buf);
		}
}

bool Mp7JrsdurationType::Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent,  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current)
{
// no includefile for extension defined 
// file Mp7JrsdurationType_ExtPostDeserialize.h


		bool found = this->Parse(Mp7JrsUtil::GetElementText(parent));
		if(found)
		{
				* current = parent;
		}
		return found;   
}

// begin extension included
// file Mp7JrsdurationType_ExtMethodImpl.h


int IMp7JrsdurationType::GetTotalNrFractions() {
	unsigned int sum = 0;
	// Days and months missing!
	int val = GetHours() > 0 ? GetHours() : 0;
	sum = 60 * (val + sum);
	val = GetMinutes() > 0 ? GetMinutes() : 0;
	sum = 60 * (val + sum);
	val = GetSeconds() > 0 ? GetSeconds() : 0;
	int val1 = GetFractionsPerSecond() > 0 ? GetFractionsPerSecond() : 0;
	sum = val1 * (val + sum);
	val = GetNumberOfFractions() > 0 ? GetNumberOfFractions() : 0;
	sum += val; 
	return (int) sum;
}

void IMp7JrsdurationType::SetTotalNrFractions(int fractions, Mp7JrsClientID client) {
	unsigned int remainder = (unsigned int) fractions;
	SetNumberOfFractions(remainder % GetFractionsPerSecond(),client);
	remainder /= GetFractionsPerSecond();
	SetSeconds(remainder % 60,client);
	remainder /= 60;
	SetMinutes(remainder % 60,client);
	remainder /= 60;
	SetHours(remainder,client);
	// Days and months ignored!!!!
}

// end extension included








