
// Based on PatternType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include <stdio.h>
#include <xercesc/util/regx/RegularExpression.hpp>
#include <xercesc/util/regx/Match.hpp>
#include <assert.h>
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrsmediaTimePointType_ExtImplInclude.h


#include "Mp7JrsmediaTimePointType.h"

Mp7JrsmediaTimePointType::Mp7JrsmediaTimePointType()
	: m_nsURI(X("urn:mpeg:mpeg7:schema:2004"))
{
		Init();
}

Mp7JrsmediaTimePointType::~Mp7JrsmediaTimePointType()
{
		Cleanup();
}

void Mp7JrsmediaTimePointType::Init()
{	   
		m_TimepointSign = +1;
		m_Year = -1;
		m_Month = -1;
		m_Day = -1;
		m_Date_Exist = false;
		m_Hours = -1;
		m_Minutes = -1;
		m_Seconds = -1;
		m_NumberOfFractions = -1;
		m_Time_Exist = false;
		m_FractionsPerSecond = -1;

// no includefile for extension defined 
// file Mp7JrsmediaTimePointType_ExtPropInit.h

}

void Mp7JrsmediaTimePointType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsmediaTimePointType_ExtPropCleanup.h


}

void Mp7JrsmediaTimePointType::DeepCopy(const Mp7JrsNodePtr &original)
{
		Mp7JrsPtr< Mp7JrsmediaTimePointType > tmp = original;
		if (tmp == NULL) return; // EXIT: wrong type passed
		Cleanup();
		// FIXME: this will fail for derived classes
  
		this->Parse(tmp->ToText());
		
		this->m_ContentName = NULL;
}

Mp7JrsNodePtr Mp7JrsmediaTimePointType::GetBaseRoot()
{
		return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsmediaTimePointType::GetBaseRoot() const
{
		return m_myself.getPointer();
}

int Mp7JrsmediaTimePointType::GetTimepointSign() const
{
		return m_TimepointSign;
}

void Mp7JrsmediaTimePointType::SetTimepointSign(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetTimepointSign().");
		}
		m_TimepointSign = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsmediaTimePointType::GetYear() const
{
		return m_Year;
}

void Mp7JrsmediaTimePointType::SetYear(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetYear().");
		}
		m_Year = val;
		m_Date_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsmediaTimePointType::GetMonth() const
{
		return m_Month;
}

void Mp7JrsmediaTimePointType::SetMonth(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetMonth().");
		}
		m_Month = val;
		m_Date_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsmediaTimePointType::GetDay() const
{
		return m_Day;
}

void Mp7JrsmediaTimePointType::SetDay(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetDay().");
		}
		m_Day = val;
		m_Date_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
bool Mp7JrsmediaTimePointType::GetDate_Exist() const
{
		return m_Date_Exist;
}

void Mp7JrsmediaTimePointType::SetDate_Exist(bool val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetDate_Exist().");
		}
		m_Date_Exist = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsmediaTimePointType::GetHours() const
{
		return m_Hours;
}

void Mp7JrsmediaTimePointType::SetHours(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetHours().");
		}
		m_Hours = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsmediaTimePointType::GetMinutes() const
{
		return m_Minutes;
}

void Mp7JrsmediaTimePointType::SetMinutes(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetMinutes().");
		}
		m_Minutes = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsmediaTimePointType::GetSeconds() const
{
		return m_Seconds;
}

void Mp7JrsmediaTimePointType::SetSeconds(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetSeconds().");
		}
		m_Seconds = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsmediaTimePointType::GetNumberOfFractions() const
{
		return m_NumberOfFractions;
}

void Mp7JrsmediaTimePointType::SetNumberOfFractions(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetNumberOfFractions().");
		}
		m_NumberOfFractions = val;
		m_Time_Exist = true;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
bool Mp7JrsmediaTimePointType::GetTime_Exist() const
{
		return m_Time_Exist;
}

void Mp7JrsmediaTimePointType::SetTime_Exist(bool val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetTime_Exist().");
		}
		m_Time_Exist = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}
int Mp7JrsmediaTimePointType::GetFractionsPerSecond() const
{
		return m_FractionsPerSecond;
}

void Mp7JrsmediaTimePointType::SetFractionsPerSecond(int val, Mp7JrsClientID client)
{
		if (!IsLocked(client)) {
				throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimePointType::SetFractionsPerSecond().");
		}
		m_FractionsPerSecond = val;
		Mp7JrsClientManager::SendUpdateNotification(
				m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
				client, m_myself.getPointer());
}


XMLCh * Mp7JrsmediaTimePointType::ToText() const
{
		XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
		XMLCh * tmp = NULL;
		
		if(m_TimepointSign < 0)
		{
			Mp7JrsUtil::CatString(&buf, X("-"));
		}

		if(m_Date_Exist)
		{
			if(m_Year >= 0)
			{
				tmp = Mp7JrsConvert::BinToText(m_Year);
				Mp7JrsUtil::CatString(&buf, tmp);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_Month >= 0)
			{
				XMLCh* tmp1 = Mp7JrsConvert::BinToText(m_Month);
				tmp = Mp7JrsUtil::PadLeft(tmp1, X("0"), 2);
				Mp7JrsUtil::CatString(&buf, X("-"));
				Mp7JrsUtil::CatString(&buf, tmp);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp1);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_Day >= 0)
			{
				XMLCh* tmp1 = Mp7JrsConvert::BinToText(m_Day);
				tmp = Mp7JrsUtil::PadLeft(tmp1, X("0"), 2);
				Mp7JrsUtil::CatString(&buf, X("-"));
				Mp7JrsUtil::CatString(&buf, tmp);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp1);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}
		}

		if(m_Time_Exist)
		{
			if(m_Hours >= 0)
			{
				Mp7JrsUtil::CatString(&buf, X("T"));
				XMLCh* tmp1 = Mp7JrsConvert::BinToText(m_Hours);
				tmp = Mp7JrsUtil::PadLeft(tmp1, X("0"), 2);
				Mp7JrsUtil::CatString(&buf, tmp);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp1);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_Minutes >= 0)
			{
				Mp7JrsUtil::CatString(&buf, X(":"));
				XMLCh* tmp1 = Mp7JrsConvert::BinToText(m_Minutes);
				tmp = Mp7JrsUtil::PadLeft(tmp1, X("0"), 2);
				Mp7JrsUtil::CatString(&buf, tmp);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp1);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_Seconds >= 0)
			{
				Mp7JrsUtil::CatString(&buf, X(":"));
				XMLCh* tmp1 = Mp7JrsConvert::BinToText(m_Seconds);
				tmp = Mp7JrsUtil::PadLeft(tmp1, X("0"), 2);
				Mp7JrsUtil::CatString(&buf, tmp);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp1);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}

			if(m_NumberOfFractions >= 0)
			{
				tmp = Mp7JrsConvert::BinToText(m_NumberOfFractions);
				Mp7JrsUtil::CatString(&buf, X(":"));
				Mp7JrsUtil::CatString(&buf, tmp);
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
			}
		}

		if(m_FractionsPerSecond >= 0)
		{
			tmp = Mp7JrsConvert::BinToText(m_FractionsPerSecond);
			Mp7JrsUtil::CatString(&buf, X("F"));
			Mp7JrsUtil::CatString(&buf, tmp);
			XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
		}
		return buf;
}


/////////////////////////////////////////////////////////////////

bool Mp7JrsmediaTimePointType::Parse(const XMLCh * const txt)
{
		Match m;
		XMLCh substring [128] = {0};
		RegularExpression r("^(\\-?\\d+(\\-\\d{2}(\\-\\d{2})?)?)?(T\\d{2}(:\\d{2}(:\\d{2}(:\\d+)?)?)?)?(F\\d+)?$");
		// This is for Xerces 3.1.1+
		if(r.matches(txt, &m))
		{
				for(int i = 1; i < m.getNoGroups(); i++)
				{
						int i1 = m.getStartPos(i);
						int i2 = Mp7JrsUtil::GetNextEndPos(m, i);
						if(i1 == i2) continue; // ignore missing optional or wrong group
						switch(i)
						{
								case 0:
								//\\-?
												// Nothing to do here...
								break;

								case 1:
										if(i1 >= 0)
										{
												// (\\d+...)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2);
												m_TimepointSign = Mp7JrsConvert::TextToSign(substring);
												if(m_TimepointSign < 0)
												{
														XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
												}
												m_Year = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Date_Exist = true;
										}
								break;

								case 2:
										if(i1 >= 0)
										{
												// (\\-\\d{2}...)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
												m_Month = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Date_Exist = true;
										}
								break;

								case 3:
										if(i1 >= 0)
										{
												// (\\-\\d{2})?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
												m_Day = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Date_Exist = true;
										}
								break;

								case 4:
										if(i1 >= 0)
										{
												// (T\\d{2}...)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
												m_Hours = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Time_Exist = true;
										}
								break;


								case 5:
										if(i1 >= 0)
										{
												// (:\\d{2}...)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
												m_Minutes = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Time_Exist = true;
										}
								break;

								case 6:
										if(i1 >= 0)
										{
												// (:\\d{2}...)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
												m_Seconds = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Time_Exist = true;
										}
								break;

								case 7:
										if(i1 >= 0)
										{
												// (:\\d+...)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
												m_NumberOfFractions = Mp7JrsConvert::TextToUnsignedInt(substring);
												m_Time_Exist = true;
										}
								break;

								case 8:
										if(i1 >= 0)
										{
												// (F\\d+)?
												XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
												m_FractionsPerSecond = Mp7JrsConvert::TextToUnsignedInt(substring);
										}
								break;

								default: return true;
								break;
						}
				}
		}
		else return false;
		return true;
}

bool Mp7JrsmediaTimePointType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsmediaTimePointType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsmediaTimePointType::ContentToString() const
{
	// Mp7JrsmediaTimePointType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrsmediaTimePointType::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
		// The element we serialize into
		XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

		XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
		if (!newElem) newElem = &localNewElem;

		if (*newElem) {
				element = *newElem;
		} else if(parent == NULL) {
				element = doc->getDocumentElement();
				*newElem = element;
		} else {
				if(this->GetContentName() != (XMLCh*)NULL) {
//  OLD					  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
						DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
						parent->appendChild(elem);
						*newElem = elem;
						element = elem;
				} else
						assert(false);
		}
		assert(element);

		if(this->UseTypeAttribute && ! element->hasAttribute(X("xsi:type")))
		{
			element->setAttribute(X("xsi:type"), X(Mp7JrsFactory::GetTypeName(Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaTimePointType")))));
		}

// no includefile for extension defined 
// file Mp7JrsmediaTimePointType_ExtPreImplementCleanup.h


		XMLCh * buf = this->ToText();
		if(buf != NULL)
		{
				element->appendChild(doc->createTextNode(buf));
				XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&buf);
		}
}

bool Mp7JrsmediaTimePointType::Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent,  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current)
{
// no includefile for extension defined 
// file Mp7JrsmediaTimePointType_ExtPostDeserialize.h


		bool found = this->Parse(Mp7JrsUtil::GetElementText(parent));
		if(found)
		{
				* current = parent;
		}
		return found;   
}

// begin extension included
// file Mp7JrsmediaTimePointType_ExtMethodImpl.h
#include "Mp7JrsmediaDurationType.h"

void IMp7JrsmediaTimePointType::Add(Mp7JrsmediaDurationPtr duration, Mp7JrsClientID client) {
	int nFractions = (int)(GetTotalNrFractions()+(duration->GetTotalNrFractions()*1.0/duration->GetFractionsPerSecond())*GetFractionsPerSecond());
	SetTotalNrFractions(nFractions,client);
}

Mp7JrsmediaDurationPtr IMp7JrsmediaTimePointType::Subtract(Mp7JrsmediaTimePointPtr timePoint, Mp7JrsClientID client) {
	int nFractions = (int)(GetTotalNrFractions()-(timePoint->GetTotalNrFractions()*1.0/timePoint->GetFractionsPerSecond())*GetFractionsPerSecond());
	Mp7JrsmediaDurationPtr duration = CreatemediaDurationType;
	duration->SetFractionsPerSecond(GetFractionsPerSecond(),client);
	duration->SetTotalNrFractions(nFractions,client);
	return duration;
}

int IMp7JrsmediaTimePointType::GetTotalNrFractions() {
	int sum = 0;
	// Days and months missing!
	int val = GetHours() > 0 ? GetHours() : 0;
	sum = 60 * (val + sum);
	val = GetMinutes() > 0 ? GetMinutes() : 0;
	sum = 60 * (val + sum);
	val = GetSeconds() > 0 ? GetSeconds() : 0;
	int val1 = GetFractionsPerSecond() > 0 ? GetFractionsPerSecond() : 0;
	sum = val1 * (val + sum);
	val = GetNumberOfFractions() > 0 ? GetNumberOfFractions() : 0;
	sum += val; 
	return sum;
}

void IMp7JrsmediaTimePointType::SetTotalNrFractions(int fractions, Mp7JrsClientID client) {
	int remainder = fractions;
	SetNumberOfFractions(remainder % GetFractionsPerSecond(),client);
	remainder /= GetFractionsPerSecond();
	SetSeconds (remainder % 60,client);
	remainder /= 60;
	SetMinutes (remainder % 60,client);
	remainder /= 60;
	SetHours (remainder,client);
	// Days and months ignored!!!!
}

// end extension included








