 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2013, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsscaleType_ExtImplInclude.h



#include "Mp7JrsfloatVector.h"
#include "Mp7JrsscaleType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
#include "Mp7JrsXPathParseContext.h" // XPath stuff

#include <assert.h>
IMp7JrsscaleType::IMp7JrsscaleType()
{

// no includefile for extension defined 
// file Mp7JrsscaleType_ExtPropInit.h

}

IMp7JrsscaleType::~IMp7JrsscaleType()
{
// no includefile for extension defined 
// file Mp7JrsscaleType_ExtPropCleanup.h

}

Mp7JrsscaleType::Mp7JrsscaleType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsscaleType::~Mp7JrsscaleType()
{
	Cleanup();
}

void Mp7JrsscaleType::Init()
{
	// Init base
	m_Base = CreatefloatVector;
	m_Base->m_myself = m_myself.getPointer();
	




// no includefile for extension defined 
// file Mp7JrsscaleType_ExtMyPropInit.h

}

void Mp7JrsscaleType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsscaleType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
}

void Mp7JrsscaleType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use scaleTypePtr, since we
	// might need GetBase(), which isn't defined in IscaleType
	const Mp7JrsPtr< Mp7JrsscaleType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//	  m_Base = Mp7JrsfloatVectorPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsscaleType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
}

Mp7JrsNodePtr Mp7JrsscaleType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsscaleType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsfloatVector > Mp7JrsscaleType::GetBase() const
{
	return m_Base;
}

Mp7JrsPtr< Mp7JrsfloatVector > Mp7JrsscaleType::GetBaseCollection() const
{
	return GetBase();
}

void Mp7JrsscaleType::Initialize(unsigned int maxElems)
{
	GetBase()->Initialize(maxElems);
}

void Mp7JrsscaleType::addElement(const float &toAdd, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsscaleType::addElement().");
	}
	GetBase()->addElement(toAdd);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsscaleType::setElementAt(const float &toSet, unsigned int setAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsscaleType::setElementAt().");
	}
	GetBase()->setElementAt(toSet, setAt);
	Mp7JrsClientManager::SendUpdateNotification(
	m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
	client, m_myself.getPointer());
}

void Mp7JrsscaleType::insertElementAt(const float &toInsert, unsigned int insertAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsscaleType::insertElementAt().");
	}
	GetBase()->insertElementAt(toInsert, insertAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsscaleType::removeAllElements(Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsscaleType::removeAllElements().");
	}
	GetBase()->removeAllElements();
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsscaleType::removeElementAt(unsigned int removeAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsscaleType::removeElementAt().");
	}
	GetBase()->removeElementAt(removeAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsscaleType::containsElement(const float &toCheck)
{
	return GetBase()->containsElement(toCheck);
}

unsigned int Mp7JrsscaleType::curCapacity() const
{
	return GetBase()->curCapacity();
}

const float Mp7JrsscaleType::elementAt(unsigned int getAt) const
{
	return GetBase()->elementAt(getAt);
}

float Mp7JrsscaleType::elementAt(unsigned int getAt)
{
	return GetBase()->elementAt(getAt);
}


unsigned int Mp7JrsscaleType::size() const
{
	return GetBase()->size();
}

void Mp7JrsscaleType::ensureExtraCapacity(unsigned int length)
{
	GetBase()->ensureExtraCapacity(length);
}

int Mp7JrsscaleType::elementIndexOf(const float &toCheck) const
{
	return GetBase()->elementIndexOf(toCheck);
}


Mp7JrsNodeEnum Mp7JrsscaleType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{

// ********* Number of Attributes:  0, 0 ************


	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for scaleType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "scaleType");
	}
	return result;
}

XMLCh * Mp7JrsscaleType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsscaleType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsscaleType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsscaleType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsscaleType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


	// pass newElem to base class, so that it contains new node after return
	m_Base->Serialize(doc, element, newElem);
	element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsscaleType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("scaleType"));
	// Element serialization:

}

bool Mp7JrsscaleType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

  // Extensionbase is Mp7JrsfloatVector
  // Deserialize cce factory type
  if(m_Base->Deserialize(doc, parent, current))
  {
	found = true;
	parent = * current;
	parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
  }
// no includefile for extension defined 
// file Mp7JrsscaleType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsscaleType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
	child = Mp7JrsNodePtr();
  } else {
	return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsscaleType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsscaleType_ExtMethodImpl.h


